Version: 1.1.0
Type: Package
Title: Fractional Response Regressions
Package: fracreg
Authors@R: person(given = c("Sulman", "Olieko"),
                   family = "Owili",
                   role = c("aut", "cre"),
                   email = "oliekosulman@gmail.com",
                   comment = c(ORCID = "0000-0001-7401-5326"))
License: GPL-3
Description: Provides routines for the estimation and specification analysis of fractional response models. Includes univariate one-part, two-part, and double-inflated three-part fractional models. Further incorporates estimators for panel data settings and addresses unobserved heterogeneity and endogeneity via correlated random effects and control function approaches. Extends fractional methodology to multivariate data via fractional multinomial logit models and handles high-dimensional multicollinear data via fractional ridge regression. Calculates analytical partial effects across all model types and includes generalised goodness-of-functional-form (GGOFF) and Regression Equation Specification Error Test (RESET) hypothesis tests. Methods are described in Papke and Wooldridge (1996) <doi:10.1002/(SICI)1099-1255(199611)11:6%3C619::AID-JAE418%3E3.0.CO;2-1>, Papke and Wooldridge (2008) <doi:10.1016/j.jeconom.2008.05.009>, Buis (2008) <http://maartenbuis.nl/software/likelihoodFmlogit.pdf>, Ramalho, Ramalho and Murteira (2011) <doi:10.1111/j.1467-6419.2009.00602.x>, Fang and Ma (2013) <doi:10.1080/02664763.2012.758246>, Mullahy (2015) <doi:10.1515/jem-2012-0006>, Murteira and Ramalho (2016) <doi:10.1080/07474938.2013.806849>, and Rokem and Kay (2020) <doi:10.1093/gigascience/giaa133>.
Language: en-GB
Date: 2026-08-08
Packaged: 2026-08-20 07:02:02 UTC; macbookpro
NeedsCompilation: no
Repository: CRAN
URL: https://sulmanolieko.github.io/fracreg/
Depends: R (>= 3.5.0)
Imports: stats, maxLik, ggplot2, grid, MASS
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), tibble, gtsummary
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Config/testthat/edition: 3
Encoding: UTF-8
Author: Sulman Olieko Owili [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7401-5326>)
Maintainer: Sulman Olieko Owili <oliekosulman@gmail.com>
Date/Publication: 2026-08-20 13:00:02 UTC
Built: R 4.6.0; ; 2026-08-20 16:04:35 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(broom::glance,fracreg)
S3method(broom::glance,fracreghet)
S3method(broom::glance,fracregmlogit)
S3method(broom::glance,fracregpd)
S3method(broom::glance,fracregridge)
S3method(broom::tidy,fracreg)
S3method(broom::tidy,fracreg.ggoff)
S3method(broom::tidy,fracreg.pe)
S3method(broom::tidy,fracreg.ptest)
S3method(broom::tidy,fracreg.reset)
S3method(broom::tidy,fracreghet)
S3method(broom::tidy,fracreghet.pe)
S3method(broom::tidy,fracreghet.reset)
S3method(broom::tidy,fracregmlogit)
S3method(broom::tidy,fracregmlogit.pe)
S3method(broom::tidy,fracregmlogit.wtp)
S3method(broom::tidy,fracregpd)
S3method(broom::tidy,fracregpd.pe)
S3method(broom::tidy,fracregridge)
S3method(broom::tidy,fracregridge.pe)
S3method(coef,fracreg)
S3method(coef,fracreghet)
S3method(coef,fracregmlogit)
S3method(coef,fracregpd)
S3method(coef,fracregridge)
S3method(fitted,fracreg)
S3method(fitted,fracreghet)
S3method(fitted,fracregmlogit)
S3method(fitted,fracregpd)
S3method(fitted,fracregridge)
S3method(gtsummary::tbl_regression,fracreg)
S3method(logLik,fracreg)
S3method(logLik,fracreghet)
S3method(logLik,fracregmlogit)
S3method(logLik,fracregpd)
S3method(logLik,fracregridge)
S3method(nobs,fracreg)
S3method(nobs,fracreghet)
S3method(nobs,fracregmlogit)
S3method(nobs,fracregpd)
S3method(nobs,fracregridge)
S3method(plot,fracregmlogit)
S3method(plot,fracregmlogit.pe)
S3method(predict,fracreg)
S3method(predict,fracreghet)
S3method(predict,fracregmlogit)
S3method(predict,fracregpd)
S3method(predict,fracregridge)
S3method(print,fracreg)
S3method(print,fracreg.ggoff)
S3method(print,fracreg.pe)
S3method(print,fracreg.ptest)
S3method(print,fracreg.reset)
S3method(print,fracreghet)
S3method(print,fracreghet.pe)
S3method(print,fracreghet.reset)
S3method(print,fracregmlogit)
S3method(print,fracregmlogit.pe)
S3method(print,fracregmlogit.wtp)
S3method(print,fracregpd)
S3method(print,fracregridge)
S3method(print,fracregridge.pe)
S3method(print,summary.fracreg)
S3method(print,summary.fracreg.ggoff)
S3method(print,summary.fracreg.pe)
S3method(print,summary.fracreg.ptest)
S3method(print,summary.fracreg.reset)
S3method(print,summary.fracreghet)
S3method(print,summary.fracreghet.pe)
S3method(print,summary.fracreghet.reset)
S3method(print,summary.fracregmlogit)
S3method(print,summary.fracregmlogit.pe)
S3method(print,summary.fracregmlogit.wtp)
S3method(print,summary.fracregpd)
S3method(print,summary.fracregridge)
S3method(print,summary.fracregridge.pe)
S3method(residuals,fracreg)
S3method(residuals,fracreghet)
S3method(residuals,fracregmlogit)
S3method(residuals,fracregpd)
S3method(residuals,fracregridge)
S3method(stats::family,fracreg)
S3method(stats::family,fracreghet)
S3method(stats::family,fracregmlogit)
S3method(stats::family,fracregpd)
S3method(stats::family,fracregridge)
S3method(stats::formula,fracreg)
S3method(stats::formula,fracreghet)
S3method(stats::formula,fracregmlogit)
S3method(stats::formula,fracregpd)
S3method(stats::formula,fracregridge)
S3method(stats::model.frame,fracreg)
S3method(stats::model.frame,fracreghet)
S3method(stats::model.frame,fracregmlogit)
S3method(stats::model.frame,fracregpd)
S3method(stats::model.frame,fracregridge)
S3method(stats::model.matrix,fracreg)
S3method(stats::model.matrix,fracreghet)
S3method(stats::model.matrix,fracregmlogit)
S3method(stats::model.matrix,fracregpd)
S3method(stats::model.matrix,fracregridge)
S3method(stats::terms,fracreg)
S3method(stats::terms,fracreghet)
S3method(stats::terms,fracregmlogit)
S3method(stats::terms,fracregpd)
S3method(stats::terms,fracregridge)
S3method(summary,fracreg)
S3method(summary,fracreg.ggoff)
S3method(summary,fracreg.pe)
S3method(summary,fracreg.ptest)
S3method(summary,fracreg.reset)
S3method(summary,fracreghet)
S3method(summary,fracreghet.pe)
S3method(summary,fracreghet.reset)
S3method(summary,fracregmlogit)
S3method(summary,fracregmlogit.pe)
S3method(summary,fracregmlogit.wtp)
S3method(summary,fracregpd)
S3method(summary,fracregridge)
S3method(summary,fracregridge.pe)
S3method(vcov,fracreg)
S3method(vcov,fracreghet)
S3method(vcov,fracregmlogit)
S3method(vcov,fracregpd)
S3method(vcov,fracregridge)
export(fracreg)
export(fracreg.ggoff)
export(fracreg.pe)
export(fracreg.ptest)
export(fracreg.reset)
export(fracreghet)
export(fracreghet.pe)
export(fracreghet.reset)
export(fracregmlogit)
export(fracregmlogit.pe)
export(fracregpd)
export(fracregpd.pe)
export(fracregridge)
export(fracregridge.pe)
export(wtp)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,scale_color_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(graphics,par)
importFrom(grid,grid.layout)
importFrom(grid,pushViewport)
importFrom(grid,viewport)
importFrom(maxLik,maxLik)
importFrom(stats,Gamma)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,dcauchy)
importFrom(stats,dlogis)
importFrom(stats,dnorm)
importFrom(stats,fitted)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,na.omit)
importFrom(stats,nlminb)
importFrom(stats,nobs)
importFrom(stats,pcauchy)
importFrom(stats,pchisq)
importFrom(stats,plogis)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,pt)
importFrom(stats,qcauchy)
importFrom(stats,qlogis)
importFrom(stats,qnorm)
importFrom(stats,quasibinomial)
importFrom(stats,residuals)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(stats,vcov)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  eval      = TRUE
)

local({
  hook_output <- knitr::knit_hooks$get("output")
  knitr::knit_hooks$set(output = function(x, options) {
    paste0("\n<details><summary>Toggle to see the output</summary>\n\n",
           hook_output(x, options),
           "\n</details>\n")
  })
})

library(fracreg)

## ----load-package-------------------------------------------------------------
library(fracreg)

## ----fracreg------------------------------------------------------------------
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age,  
           totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole) 
 
# 1P Model 
mod <- fracreg(y, X, type="1P", linkfrac="logit") 
summary(mod)

# 1P Model reporting odds ratios and 99% confidence intervals
mod <- fracreg(y, X, type="1P", linkfrac="logit", or=TRUE, level=0.99)
summary(mod)
 
# 2P Model (modelling mass at 1) 
mod <- fracreg(y, X, type="2P", inflation=1, linkbin="logit", linkfrac="logit") 
summary(mod)
 
# 3P Model (inject artificial 0s for demonstration) 
y_3p <- y; y_3p[1:50] <- 0 
mod <- fracreg(y_3p, X, type="3P", linkbin=c("logit","logit"), linkfrac="logit") 
summary(mod)
 
### Simulated Examples 
 
set.seed(123) 
N <- 1000 
x1 <- rnorm(N) 
x2 <- runif(N) 
 
# Generating a fractional dependent variable with inflation at 0 and 1 
XB <- -0.5 + 0.8 * x1 + 1.2 * x2 + rnorm(N) 
y_latent <- exp(XB) / (1 + exp(XB)) 
 
y <- y_latent 
# Inflate at boundaries 
y[y_latent < 0.2] <- 0 
y[y_latent > 0.8] <- 1 
 
X <- cbind(x1 = x1, x2 = x2) 
 
# fracreg estimation of a logit fractional response model 
mod <- fracreg(y, X, type="1P", linkfrac="logit") 
summary(mod)
 

## ----fracreg_2pbin------------------------------------------------------------
# Estimate the binary logit component 
mod <- fracreg(y, X, type="2Pbin", inflation=0, linkbin="logit") 
summary(mod)
 

## ----fracreg_2pfrac-----------------------------------------------------------
# Estimate the fractional component using a probit link 
mod <- fracreg(y, X, type="2Pfrac", inflation=0, linkfrac="probit") 
summary(mod)
 

## ----fracreg_2p_joint---------------------------------------------------------
# Estimate both components jointly 
mod <- fracreg(y, X, type="2P", inflation=0, linkbin="cloglog", linkfrac="logit") 
summary(mod)
 

## ----fracreg_3p_simulated-----------------------------------------------------
# Three-part double-inflated model 
mod <- fracreg(y, X, type="3P", linkbin=c("logit","probit"), linkfrac="logit") 
summary(mod)

## ----fracreg-pe---------------------------------------------------------------
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age,  
           totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole) 
 
m <- fracreg(y, X, type="1P", linkfrac="logit") 
pe_res <- fracreg.pe(m) 
summary(pe_res)
 
### Simulated Examples 
 
N <- 250 
u <- rnorm(N) 
 
X <- cbind(rnorm(N),rnorm(N)) 
dimnames(X)[[2]] <- c("X1","X2") 
 
ym <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u)) 
y <- rbeta(N,ym*20,20*(1-ym)) 
y[y > 0.9] <- 1 
 
#Computing average partial effects for a logit fractional response model 
mod <- fracreg(y,X,linkfrac="logit",table=FALSE) 
pe_res <- fracreg.pe(mod) 
summary(pe_res)
 

## ----fracreg_pe_2p------------------------------------------------------------
# Compute average partial effects for a binary logit + fractional probit two-part model 
mod <- fracreg(y,X,linkbin="logit",linkfrac="probit",type="2P",inf=1,table=FALSE) 
pe_res <- fracreg.pe(mod) 
summary(pe_res)
 

## ----fracreg_pe_cpe-----------------------------------------------------------
# Compute conditional partial effects for X2 at median values 
mod <- fracreg(y,X,linkfrac="logit",type="2Pfrac",inf=1,table=FALSE) 
pe_res <- fracreg.pe(mod,APE=FALSE,CPE=TRUE,at="median",which.x="X2") 
summary(pe_res)
 

## ----fracreg_pe_3p------------------------------------------------------------
# Compute average partial effects for a three-part double-inflated model 
y3p <- y 
y3p[1:20] <- 0 
y3p[21:40] <- 1 
res3p <- fracreg(y3p,X,linkbin=c("logit","probit"),linkfrac="logit",type="3P",table=FALSE) 
pe_res <- fracreg.pe(res3p) 
summary(pe_res)

## ----fracreg-ggoff------------------------------------------------------------
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age,  
           totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole) 
 
m <- fracreg(y, X, type="1P", linkfrac="logit") 
ggoff_res <- fracreg.ggoff(m) 
summary(ggoff_res)
 
### Simulated Examples 
 
N <- 250 
u <- rnorm(N) 
 
X <- cbind(rnorm(N),rnorm(N)) 
dimnames(X)[[2]] <- c("X1","X2") 
 
ym <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u)) 
y <- rbeta(N,ym*20,20*(1-ym)) 
y[y > 0.9] <- 1 
 
#Testing the logit specification of a standard fractional response model 
#using LM and Wald versions of the GGOFF test, based on 1 or 2 fitted powers of 
#the linear predictor 
mod <- fracreg(y,X,linkfrac="logit",table=FALSE) 
ggoff_res <- fracreg.ggoff(mod,c("Wald","LM")) 
summary(ggoff_res)
 

## ----fracreg_ggoff_2pbin------------------------------------------------------
# Test the probit specification of the binary component 
mod <- fracreg(y,X,linkbin="probit",type="2Pbin",inf=1,table=FALSE) 
ggoff_res <- fracreg.ggoff(mod,"LR") 
summary(ggoff_res)

## ----fracreg-reset------------------------------------------------------------
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age,  
           totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole) 
 
m <- fracreg(y, X, type="1P", linkfrac="logit") 
reset_res <- fracreg.reset(m) 
summary(reset_res)
 
### Simulated Examples 
 
N <- 250 
u <- rnorm(N) 
 
X <- cbind(rnorm(N),rnorm(N)) 
dimnames(X)[[2]] <- c("X1","X2") 
 
ym <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u)) 
y <- rbeta(N,ym*20,20*(1-ym)) 
y[y > 0.9] <- 1 
 
#Testing the logit specification of a standard fractional response model 
#using LM and Wald versions of the RESET test, based on 1 or 2 fitted powers of 
#the linear predictor 
mod <- fracreg(y,X,linkfrac="logit",table=FALSE) 
reset_res <- fracreg.reset(mod,2:3,c("Wald","LM")) 
summary(reset_res)
 

## ----fracreg_reset_2pbin------------------------------------------------------
# Test the probit specification of the binary component using LR RESET 
mod <- fracreg(y,X,linkbin="probit",type="2Pbin",inf=1,table=FALSE) 
reset_res <- fracreg.reset(mod,3,"LR") 
summary(reset_res)

## ----fracreg-ptest------------------------------------------------------------
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age,  
           totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole) 
 
m1 <- fracreg(y, X, type="1P", linkfrac="logit") 
m2 <- fracreg(y, X, type="1P", linkfrac="probit") 
ptest_res <- fracreg.ptest(m1, m2) 
summary(ptest_res)
 
### Simulated Examples 
 
N <- 250 
u <- rnorm(N) 
 
X <- cbind(rnorm(N),rnorm(N)) 
dimnames(X)[[2]] <- c("X1","X2") 
 
ym <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u)) 
y <- rbeta(N,ym*20,20*(1-ym)) 
y[y > 0.9] <- 1 
 
#Testing logit versus loglog specifications for standard fractional 
#regression models using a LM version of the P test 
res1 <- fracreg(y,X,linkfrac="logit",table=FALSE) 
res2 <- fracreg(y,X,linkfrac="loglog",table=FALSE) 
ptest_res <- fracreg.ptest(res1,res2,"LM") 
summary(ptest_res)
 

## ----fracreg_ptest_1p_vs_2p---------------------------------------------------
# Test 1P logit versus 2P logit-probit using Wald P-test 
res1 <- fracreg(y,X,linkfrac="logit",table=FALSE) 
res2 <- fracreg(y,X,linkbin="logit",linkfrac="probit",type="2P",inf=1,table=FALSE) 
ptest_res <- fracreg.ptest(res1,res2,"Wald") 
summary(ptest_res)

## ----fracreghet---------------------------------------------------------------
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X_het <- cbind(mrate = fracreg_k401k$mrate, ltotemp = fracreg_k401k$ltotemp)
 
# fracreghet estimators do not allow exact 1s or 0s
y_adj <- y
y_adj[y_adj == 1] <- 0.999

# Instrument mrate using age

Z_emp <- cbind(age = fracreg_k401k$age, ltotemp = fracreg_k401k$ltotemp) 
mod <- fracreghet(y_adj, X_het, Z_emp, var.endog = X_het[, "mrate"], type="QMLxv", link="logit") 
summary(mod)

# Compute the same QMLxv estimator reporting Odds Ratios with 90% confidence intervals
mod <- fracreghet(y_adj, X_het, Z_emp, var.endog = X_het[, "mrate"], type="QMLxv", link="logit", or=TRUE, level=0.90)
summary(mod)
 
### Simulated Examples 
 
set.seed(123) 
N <- 1000 
x1 <- rnorm(N) 
 
# Simulating an endogenous variable (var.endog) and an instrument (z1) 
z1 <- rnorm(N) 
u <- 0.5 * z1 + rnorm(N) 
var.endog <- 0.8 * z1 + u 
y_endog <- exp(0.5 * x1 + 1.2 * var.endog + u) / (1 + exp(0.5 * x1 + 1.2 * var.endog + u)) 
 
# Avoid exact 0 or 1 boundaries for some estimators 
y_endog[y_endog <= 0] <- 0.01 
y_endog[y_endog >= 1] <- 0.99 
 
X <- cbind(x1 = x1, var.endog = var.endog) 
Z <- cbind(x1 = x1, z1 = z1) 
 
# Exogeneity (assuming var.endog is exogenous for comparison), GMMx estimator 
mod <- fracreghet(y = y_endog, x = X, type = "GMMx", link = "logit") 
summary(mod)
 

## ----fracreghet_gmmz----------------------------------------------------------
# Endogeneity, GMMz estimator 
mod <- fracreghet(y = y_endog, x = X, z = Z, type = "GMMz", link = "logit") 
summary(mod)
 

## ----fracreghet_gmmxv---------------------------------------------------------
# Endogeneity, GMMxv estimator 
mod <- fracreghet(y = y_endog, x = X, z = Z, var.endog = var.endog, type = "GMMxv", link = "logit") 
summary(mod)
 

## ----fracreghet_qmlxv---------------------------------------------------------
# Endogeneity, QMLxv control function approach 
mod <- fracreghet(y = y_endog, x = X, z = Z, var.endog = var.endog, type = "QMLxv", link = "logit") 
summary(mod)

## ----fracreghet-pe------------------------------------------------------------
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X_het <- cbind(mrate = fracreg_k401k$mrate, ltotemp = fracreg_k401k$ltotemp)
 
# fracreghet estimators do not allow exact 1s or 0s
y_adj <- y
y_adj[y_adj == 1] <- 0.999

# Instrument mrate using age

Z_emp <- cbind(age = fracreg_k401k$age, ltotemp = fracreg_k401k$ltotemp) 
res_emp <- fracreghet(y_adj, X_het, Z_emp, var.endog = X_het[, "mrate"], type="QMLxv", link="logit", table=FALSE) 
pe_res <- fracreghet.pe(res_emp, which.x="mrate")
summary(pe_res)

### Simulated Examples 
 
N <- 250 
u <- rnorm(N) 
 
X <- cbind(rnorm(N),rnorm(N)) 
dimnames(X)[[2]] <- c("X1","X2") 
 
Z <- cbind(rnorm(N),rnorm(N),rnorm(N)) 
dimnames(Z)[[2]] <- c("Z1","Z2","Z3") 
 
y <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u)) 
 
mod <- fracreghet(y,X,type="GMMx",table=FALSE) 
 
#Smearing estimator of average partial effects for variable X1 
pe_res <- fracreghet.pe(mod,which.x="X1") 
summary(pe_res)
 

## ----fracreghet_pe_cpe--------------------------------------------------------
# Naive estimator of CPE evaluated at fixed values 
pe_res <- fracreghet.pe(mod,smearing=FALSE,APE=FALSE,CPE=TRUE,at=c(1,-1)) 
summary(pe_res)

## ----fracreghet-reset---------------------------------------------------------
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X_het <- cbind(mrate = fracreg_k401k$mrate, ltotemp = fracreg_k401k$ltotemp)
 
# fracreghet estimators do not allow exact 1s or 0s
y_adj <- y
y_adj[y_adj == 1] <- 0.999

# Instrument mrate using age

Z_emp <- cbind(age = fracreg_k401k$age, ltotemp = fracreg_k401k$ltotemp) 
res_emp <- fracreghet(y_adj, X_het, type="GMMx", link="logit", table=FALSE) 
reset_res <- fracreghet.reset(res_emp)
summary(reset_res)

### Simulated Examples 
 
N <- 250 
u <- rnorm(N) 
 
X <- cbind(rnorm(N),rnorm(N)) 
dimnames(X)[[2]] <- c("X1","X2") 
 
Z <- cbind(rnorm(N),rnorm(N),rnorm(N)) 
dimnames(Z)[[2]] <- c("Z1","Z2","Z3") 
 
y <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u)) 
 
mod <- fracreghet(y,X,type="GMMx",table=FALSE) 
 
#LM and Wald versions of the RESET test, based on 1 or 2 fitted powers of xb 
reset_res <- fracreghet.reset(mod,2:3,c("Wald","LM")) 
summary(reset_res)

## ----fracregpd----------------------------------------------------------------
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age,  
           totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole) 
 
# Artificial panel data structure for demonstration 
N_emp <- nrow(X) 
id_emp <- rep(1:(N_emp/2), each=2) 
time_emp <- rep(1:2, times=N_emp/2) 
mod <- fracregpd(id_emp, time_emp, y, X, type="QMLcre", link="probit") 
summary(mod)
 
### Simulated Examples 
 
set.seed(123) 
# Simulating Panel Data 
N <- 100 
T_periods <- 5 
id <- rep(1:N, each = T_periods) 
time <- rep(1:T_periods, times = N) 
x_panel <- rnorm(N * T_periods) 
 
# Unobserved individual effect (CRE) 
c_i <- rep(rnorm(N), each = T_periods)  
y_panel <- exp(x_panel + c_i) / (1 + exp(x_panel + c_i)) 
 
X <- cbind(x_panel = x_panel) 
 
# Endogenous variable and instrument simulation 
z_panel <- rnorm(N * T_periods) 
u_panel <- 0.5 * z_panel + rnorm(N * T_periods) 
var_endog <- 0.8 * z_panel + u_panel 
y_endog <- exp(x_panel + 1.2 * var_endog + c_i + u_panel) /  
             (1 + exp(x_panel + 1.2 * var_endog + c_i + u_panel)) 
 
X_endog <- cbind(x_panel = x_panel, var_endog = var_endog) 
Z_inst <- cbind(x_panel = x_panel, z_panel = z_panel) 
 
# Estimate a Correlated Random Effects (CRE) Model 
mod <- fracregpd(id=id, time=time, y=y_panel, x=X, type="QMLcre", link="probit") 
summary(mod)
 

## ----fracregpd_gmmbgw---------------------------------------------------------
# Exogeneity, GMMbgw estimator 
mod <- fracregpd(id=id, time=time, y=y_panel, x=X, type="GMMbgw") 
summary(mod)


## ----fracregpd_gmmww----------------------------------------------------------
# Estimate GMMww estimator with odds ratios
mod <- fracregpd(id=id, time=time, y=y_panel, x=X, type="GMMww", or=TRUE, level=0.99)
summary(mod)
 

## ----fracregpd_gmmww_lags-----------------------------------------------------
# Lagged covariates and instruments 
mod <- fracregpd(id=id, time=time, y=y_panel, x=X, lags=TRUE, type="GMMww", var.type="robust") 
summary(mod)
 

## ----fracregpd_gmmpfe---------------------------------------------------------
# Endogeneity, time dummies, GMMpfe estimator 
mod <- fracregpd(id=id, time=time, y=y_endog, x=X_endog, z=Z_inst,  
          x.exogenous=FALSE, type="GMMpfe", tdummies=TRUE) 
summary(mod)

## ----fracregridge-empirical---------------------------------------------------
data("fracreg_k401k")
y_401k <- fracreg_k401k$prate
X_401k <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age,
                totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole)

# Fit fractional ridge regression
mod_401k <- fracregridge(y = y_401k, x = X_401k, fracs = seq(0.2, 1.0, by = 0.2))

# View full detailed summary showing the chosen alphas
summary(mod_401k)

# Compute Average Partial Effects for Ridge
pe_401k <- fracregridge.pe(mod_401k)
summary(pe_401k)

## ----fracregridge-simulated---------------------------------------------------
# Generate random data
set.seed(123)
n <- 100
p <- 10
y_sim <- rnorm(n)
X_sim <- matrix(rnorm(n * p), n, p)
colnames(X_sim) <- paste0("X", 1:p)

# Fit Fractional Ridge Regression for 30%, 50%, and 80% fractions
mod_sim <- fracregridge(y = y_sim, x = X_sim, fracs = c(0.3, 0.5, 0.8))

# View brief summary
print(mod_sim)

# Compute Partial Effects
pe_sim <- fracregridge.pe(mod_sim)
summary(pe_sim)

## ----fracregmlogit_example----------------------------------------------------
# Load the empirical spending data
data("fracreg_spending")

# Define covariates and fractional responses
X <- fracreg_spending[, c("houseval", "popdens", "noleft", "minorityleft", "tot")]
y <- fracreg_spending[, c("governing", "safety", "education", "recreation", "social", "urbanplanning")]

# Fit the Fractional Multinomial Logit model
mn_fit <- fracregmlogit(y, X)

# View estimates
summary(mn_fit)

# Compute Average Partial Effects (discrete)
mn_pe <- fracregmlogit.pe(mn_fit, effect = "discrete", varlist = c("noleft", "minorityleft"))
summary(mn_pe)

## ----fracregmlogit_wtp--------------------------------------------------------
# Calculate Willingness to Pay for the 'noleft' variable using a hypothetical WTP vector
# Assuming WTP = 1, 2, 3, 4, 5, 6 for each of the 6 choices
wtp_est <- wtp(mn_pe, wtp.vec = 1:6, varlist = "noleft")
summary(wtp_est)

# Plot the Willingness to Pay effect across observations
plot(mn_fit, wtp.vec = 1:6, varlist = "noleft")

#' Plot Marginal or Discrete Effects of Willingness to Pay
#' 
#' Plot marginal or discrete effects of willingness to pay, potentially against another variable.
#' 
#' @param x A "fracregmlogit" object.
#' @param wtp.vec A numeric vector for willingness to pay.
#' @param varlist A string vector which provides the names of variables to plot the effect for.
#'  If missing, all variables in the object will be plotted.
#' @param against A vector with the same length as the number of observations in the model, or the name of a variable. 
#' Serves as the x-axis in the plots.
#' @param mfrow A numeric vector with two elements. Specifies the number of rows and columns in a panel.
#' Similar to par(mfrow=c()). Default to NULL, and the program will choose a square panel. 
#' @param t Number of points to be used for smoothing.
#' @param effect The type of effect ("marginal" or "discrete").
#' @param type Plot type.
#' @param plot.show If TRUE, the plot will be created. Otherwise, the function returns raw data that can be
#' used to create user-specified (custom) plots. 
#' @param ... Additional arguments.
#' @return Panel plots of effects vs. chosen variables.
#' @details 
#' This function provides a visualisation tool for potentially heterogeneous marginal and discrete effects of willingness to pay.
#' The function allows the user to plot marginal effects to detect any patterns in the effects, in itself
#' and against other variables. The plot also allows visualisation of sub-groups in data, which can be
#' very useful to visualise categorical and dummy variables. 
#' 
#' The function takes a \code{fracregmlogit} object, and internally calls \code{wtp} and \code{fracregmlogit.pe} 
#' to compute the willingness to pay at different data points.
#' 
#' Additional parameters include \code{varlist}, a vector of string variable names to be plotted. 
#'  
#' \code{against} allows a different variable to be chosen
#' as the x-axis. \code{against} can supply the column name of a variable in the original dataset to plot against. 
#'  
#' @seealso \code{\link{wtp}}, \code{\link{fracregmlogit.pe}}
#' @examples
#' data("fracreg_spending")
#' X = fracreg_spending[,2:5]
#' y = fracreg_spending[,6:11]
#' results1 = fracregmlogit(y, X)
#' 
#' # Define a willingness to pay vector
#' wtp.vec = c(1, 1, 1, 1, 1, 1)
#' 
#' # Plot WTP for 'popdens'
#' plot(results1, wtp.vec=wtp.vec, varlist="popdens")
#' @exportS3Method plot fracregmlogit
plot.fracregmlogit = function(x, wtp.vec=NULL, varlist=NULL, against=NULL, mfrow=NULL, t=500, effect=c("discrete","marginal"), type=NULL, plot.show=TRUE, ...){
  effect = match.arg(effect)
  object <- x
  K = ncol(object$X); j = ncol(object$y); N = nrow(object$X); 
  Xnames = colnames(object$X) ; ynames = colnames(object$y)
  X = object$X; y=object$y
  
  # determine variable list
  var_colNo = which(Xnames %in% varlist)
  k = length(var_colNo)
  
  if(is.null(mfrow)){
    js = ceiling(sqrt(k))
    jr = ifelse(js*(js-1)>=k,js-1,js)
  }else{
    jr = mfrow[1]; js = mfrow[2]
  }
  
  if(!is.null(against)) {
    ag_No = which(Xnames == against)
    if(length(ag_No)==0) stop(paste("The against vector specified,",against,
                                    "is not in the list of explanatory variables. Please check again."))
    ag_min = min(X[,ag_No]); ag_max = max(X[,ag_No])
    ag_vec = seq(ag_min,ag_max,length.out = t)
    wtp_mat = matrix(nrow=t,ncol=k)
    colnames(wtp_mat) = varlist
    for(i in 1:t){
      newdata = colMeans(X[,-K])
      newdata[ag_No] = ag_vec[i]
      wtp_mat[i,] = wtp(fracregmlogit.pe(object,effect=effect,se=FALSE,varlist=varlist,at=newdata),wtp.vec)[[1]]
    }
  }else{
    against="ObsNo"
    ag_vec=1:N
    wtp_mat = matrix(nrow=N,ncol=k)
    colnames(wtp_mat) = varlist
    for(i in 1:N){
      newdata = X[i,-K]
      wtp_mat[i,] = wtp(fracregmlogit.pe(object,effect=effect,se=FALSE,varlist=varlist,at=newdata),wtp.vec)[[1]]
    }
  }
  # plotting
  if(plot.show){
    par(mfrow=c(jr,js))
    if(is.null(type)){type="l"} # default to line plot. 
    for(i in 1:k){
      plot(ag_vec,wtp_mat[,i],xlab=against,ylab=paste(effect,"effect of", varlist[i]),...)
    }}
  return(list(ag_vec,wtp_mat))
}
#' Plot Marginal or Discrete Effects
#' 
#' Plot the desired effect at each observed value for each choice.
#' 
#' @param x A "fracregmlogit.pe" object.
#' @param X A matrix of independent variables.
#' @param y A matrix of dependent variables.
#' @param varlist A string vector which provides the names of variables to plot the effect for.
#'  If missing, all variables in the object will be plotted.
#' @param against A vector with the same length as the number of observations in the model. 
#' Serves as the x-axis in the plots.
#' @param against.x A character string, supply the column name in the X matrix to plot against.
#' @param against.y A character string, supply the column name in the y matrix to plot against.
#' @param group.x A character string. Supply the column name in the X matrix to group upon. 
#' @param group.algebra A character string. Supply additional algebra imposed on the group variable. 
#' @param mfrow A numeric vector with two elements. Specifies the number of rows and columns in a panel.
#' Similar to par(mfrow=c()). Default to NULL, and the program will choose a square panel. 
#' @param ... Additional arguments.
#' @return Panel plots of effects vs. chosen variables.
#' @details 
#' This function provides a visualisation tool for potentially heterogeneous marginal and discrete effects.
#' The function allows the user to plot marginal effects to detect any patterns in the effects, in itself
#' and against other variables. The plot also allows visualisation of sub-groups in data, which can be
#' very useful to visualise categorical and dummy variables. 
#' 
#' The function takes a \code{fracregmlogit.pe} object, created by the \code{fracregmlogit.pe()} function. Note that since 
#' the plotting requires marginal effects for all observations, the object should be created by choosing 
#' \code{marg.type="aveacr"}, the average across method for effects calculation. 
#' 
#' Additional parameters include \code{varlist}, a vector of string variable names to be plotted. \code{X}
#' and \code{y} are the dependent and independent variable matrices in the original regression model. 
#'  
#' \code{against}, \code{against.x}, and \code{against.y} allow different variables to be chosen
#' as the x-axis. \code{against} directly supplies the vector to be plotted against, whereas \code{against.x}
#' and \code{against.y} supply variable names in the original dataset. Note that the user has to provide
#' \code{X} and \code{y} in order to use the column name options, respectively. 
#'  
#' \code{group.x} supplies the column name in the X matrix to group by. The plot will be able to 
#' differentiate different groups by colours. Additionally, the user can supply a string to \code{group.algebra},
#' which provides an algebra operation that will be evaluated on the group vector. For example, choosing 
#' \code{group.x = "a"} and \code{group.algebra = ">0"} will create two groups, one with X$a > 0, and one with X$a <= 0.
#' 
#' @seealso \code{\link{fracregmlogit.pe}}
#' 
#' @examples
#' \donttest{
#' data("fracreg_spending")
#' X = fracreg_spending[,2:5]
#' y = fracreg_spending[,6:11]
#' results1 = fracregmlogit(y, X)
#' 
#' # Calculate marginal effects with marg.type="aveacr" (no standard errors for speed)
#' effect1 = fracregmlogit.pe(results1, effect="marginal", marg.type="aveacr", se=FALSE)
#' 
#' # Plot effects
#' plot(effect1, X=results1$X, against.x = "houseval", group.x = "popdens", group.algebra = ">10")
#' }
#' @exportS3Method plot fracregmlogit.pe
plot.fracregmlogit.pe = function(x, varlist=NULL, X=NULL, y=NULL, 
                                against=NULL,against.x=NULL,against.y=NULL,
                                group.x=NULL, group.algebra=NULL,
                                mfrow=NULL, ...){
  object <- x
  requireNamespace("ggplot2", quietly = TRUE)
  requireNamespace("grid", quietly = TRUE)
  
  if(is.null(object[["marg.list"]])) stop("Please choose marg.type=aveacr when calculating effects")
  k = ncol(object$effects); j = nrow(object$effects); N = nrow(object$marg.list[[1]]); 
  Xnames = colnames(object$effects) ; ynames = rownames(object$effects)
  # X = object$X; y=object$y
  
  # determine variable list
  if(length(varlist)==0){
    varlist=Xnames
    var_colNo = 1:k
  }else{
    var_colNo = which(Xnames %in% varlist)
    k = length(var_colNo)
  }
  if(k==0) stop("Variable list not matched. Please check your varlist input.")
  
  # determine panel size
  if(is.null(mfrow)){
    js = ceiling(sqrt(j))
    jr = ifelse(js*(js-1)>j,js-1,js)
  }else{
    jr = mfrow[1]; js = mfrow[2]
  }
  
  # determine plotting x axis. 
  if(is.null(against) & is.null(against.x) & is.null(against.y)){
    M.against=1:N 
    ag.name = "ObsNo"
  }else if(is.null(against.x)==FALSE){
    M.against = X[,against.x]
    if(is.null(M.against)){
      stop("against.x not found in variable list. Please double check your spelling")
    }
    ag.name = against.x
  }else if(is.null(against.y)==FALSE){
    M.against = y[,against.y] 
    ag.name = against.y
  }else{M.against=against}
  
  
  # determine group variables
  if(is.null(group.x) & is.null(group.algebra)) {M.group=NULL; g.name=NULL}
  if(is.null(group.x)==FALSE) {M.group = X[,group.x]; g.name.display <- g.name <- group.x;}
  if(is.null(group.algebra)==FALSE) {
    M.group = eval(parse(text=paste("X[,",'"',group.x,'"',"]",group.algebra,sep="")))
    M.group = ifelse(M.group,"Yes","No")
    g.name = group.x
    g.name.display = paste(group.x,group.algebra,sep="")
    }
  
  for(c in var_colNo){
    ggplot()
    pushViewport(viewport(layout = grid.layout(jr, js)))
    temp.data = cbind(object$marg.list[[c]],M.against)
    temp.data = as.data.frame(temp.data)
    colnames(temp.data) = c(colnames(object$marg.list[[c]]),ag.name)
    if(is.null(M.group)==FALSE){
      temp.data = cbind(temp.data,as.factor(M.group))
      colnames(temp.data)[ncol(temp.data)] = g.name
    }
    for(i in 1:j){
      g <- ggplot(temp.data,aes_string(ag.name,ynames[i],color=g.name)) + geom_point() 
      g <- g + geom_hline(yintercept = 0) + theme_classic() + ggtitle(paste("Effects on", Xnames[c]))
      if(is.null(M.group)==FALSE) g <- g + theme(legend.title = element_text(colour="black"))+
        scale_color_discrete(name=g.name.display)
      print(g,vp = viewport(layout.pos.row = ifelse(i%%jr==0,jr,i%%jr), layout.pos.col = (i-1) %/%js + 1) )
  }
}}

#' @title Extract Model Coefficients for fracregridge
#' @description Extracts the estimated coefficients from a fitted \code{fracregridge} model.
#' @param object A fitted model object of class \code{fracregridge}.
#' @param ... Further arguments passed to or from other methods.
#' @return A matrix or array of coefficients.
#' @exportS3Method coef fracregridge
coef.fracregridge <- function(object, ...) {
    return(object$coef)
}

#' @title Extract Fitted Values for fracregridge
#' @description Extracts the fitted conditional mean values from a \code{fracregridge} model.
#' @param object A fitted model object of class \code{fracregridge}.
#' @param ... Further arguments passed to or from other methods.
#' @return A matrix or array of fitted values.
#' @exportS3Method fitted fracregridge
fitted.fracregridge <- function(object, ...) {
    return(object$yhat)
}

#' @title Extract Model Residuals for fracregridge
#' @description Extracts the response residuals from a fitted \code{fracregridge} model.
#' @param object A fitted model object of class \code{fracregridge}.
#' @param ... Further arguments passed to or from other methods.
#' @return A matrix or array of residuals.
#' @exportS3Method residuals fracregridge
residuals.fracregridge <- function(object, ...) {
    y <- object$y
    yhat <- fitted(object)
    
    if (length(dim(yhat)) == 2) {
        # yhat is N x F
        res <- yhat
        y_vec <- as.numeric(y)
        for(f in 1:ncol(yhat)) {
            res[, f] <- y_vec - yhat[, f]
        }
        return(res)
    }
    
    if (!is.matrix(y)) {
        y <- as.matrix(y)
    }
    
    # y is now N x J
    # yhat is N x F x J
    res <- yhat
    for(i in 1:dim(yhat)[3]) {
        for(f in 1:dim(yhat)[2]) {
            res[, f, i] <- y[, i] - yhat[, f, i]
        }
    }
    return(res)
}

#' @title Extract the Number of Observations for fracregridge
#' @description Extracts the number of observations used to estimate a \code{fracregridge} model.
#' @param object A fitted model object of class \code{fracregridge}.
#' @param ... Further arguments passed to or from other methods.
#' @return An integer denoting the number of observations.
#' @exportS3Method nobs fracregridge
nobs.fracregridge <- function(object, ...) {
    return(object$stats.info[[1]]$n_obs)
}

#' @title Extract Log-Likelihood for fracregridge
#' @description Extracts the log-likelihood from a fitted \code{fracregridge} model.
#' @param object A fitted model object of class \code{fracregridge}.
#' @param ... Further arguments passed to or from other methods.
#' @return An object of class \code{logLik}.
#' @exportS3Method logLik fracregridge
logLik.fracregridge <- function(object, ...) {
    warning("logLik() is not applicable for fracregridge models because they are estimated via an analytical SVD penalty solver (Ridge), not MLE.")
    return(NA)
}

#' @title Extract Covariance Matrix for fracregridge
#' @description Extracts the estimated variance-covariance matrices of the parameters.
#' @param object A fitted model object of class \code{fracregridge}.
#' @param ... Further arguments passed to or from other methods.
#' @return A matrix of the estimated covariances.
#' @exportS3Method vcov fracregridge
vcov.fracregridge <- function(object, ...) {
    warning("vcov() is not natively stored for fracregridge models across all fractions. See summary(object) for individual fraction standard errors.")
    return(NULL)
}

#' @title Predict Method for fracregridge
#' @description Predicts conditional mean values from a fitted \code{fracregridge} model.
#' @param object A fitted model object of class \code{fracregridge}.
#' @param newdata An optional data frame or matrix in which to look for variables with which to predict. If omitted, the fitted values are used.
#' @param ... Further arguments passed to or from other methods.
#' @return A matrix or array of predicted values.
#' @exportS3Method predict fracregridge
predict.fracregridge <- function(object, newdata = NULL, ...) {
    if (is.null(newdata)) {
        return(fitted(object))
    }
    
    # Simple check to map data frame to matrix
    x.names <- colnames(object$x)
    if(is.data.frame(newdata)) {
        if("(Intercept)" %in% x.names && !"(Intercept)" %in% colnames(newdata)) {
            newdata <- cbind(1, as.matrix(newdata))
            colnames(newdata)[1] <- "(Intercept)"
        } else {
            newdata <- as.matrix(newdata)
        }
    } else if (is.matrix(newdata)) {
        if("(Intercept)" %in% x.names && !"(Intercept)" %in% colnames(newdata)) {
            newdata <- cbind(1, newdata)
            colnames(newdata)[1] <- "(Intercept)"
        }
    }
    
    # Ensure correct ordering and matching
    if(!all(x.names %in% colnames(newdata))) {
        stop("newdata does not contain all the necessary columns")
    }
    
    X <- newdata[, x.names, drop=FALSE]
    beta <- coef(object)
    
    if (is.matrix(beta)) {
        # Single equation
        return(X %*% beta)
    } else if (is.array(beta)) {
        # Multivariate
        pred <- array(NA, dim = c(nrow(X), dim(beta)[2], dim(beta)[3]))
        for(i in 1:dim(beta)[3]) {
            pred[,,i] <- X %*% beta[,,i]
        }
        return(pred)
    }
}

#' @title Extract Model Coefficients for fracreghet
#' @description Extracts the estimated coefficients from a fitted \code{fracreghet} model.
#' @param object A fitted model object of class \code{fracreghet}.
#' @param ... Further arguments passed to or from other methods.
#' @return A named vector of coefficients.
#' @exportS3Method coef fracreghet
coef.fracreghet <- function(object, ...) {
    if(!is.null(object$p)) {
        return(object$p)
    } else if (!is.null(object$table.info$p)) {
        return(object$table.info$p)
    }
    return(NULL)
}

#' @title Extract Fitted Values for fracreghet
#' @description Extracts the fitted conditional mean values from a \code{fracreghet} model.
#' @param object A fitted model object of class \code{fracreghet}.
#' @param ... Further arguments passed to or from other methods.
#' @return A numeric vector of fitted values.
#' @exportS3Method fitted fracreghet
fitted.fracreghet <- function(object, ...) {
    link <- object$link
    yhat <- fracreghet.links(link)$linkinv(object$xbhat)
    return(yhat)
}

#' @title Extract Model Residuals for fracreghet
#' @description Extracts the response residuals from a fitted \code{fracreghet} model.
#' @param object A fitted model object of class \code{fracreghet}.
#' @param ... Further arguments passed to or from other methods.
#' @return A numeric vector of residuals.
#' @exportS3Method residuals fracreghet
residuals.fracreghet <- function(object, ...) {
    type <- object$type
    link <- object$link
    Hy <- object$Hy
    adjust <- object$adjust
    
    if (type == "QMLxv") {
        y <- Hy
    } else if (type %in% c("LINx", "LINz", "LINxv")) {
        # Hy = linkfun(y_adj) => y_adj = linkinv(Hy) => y = linkinv(Hy) - adjust
        y_adj <- fracreghet.links(link)$linkinv(Hy)
        if (is.numeric(adjust)) {
            y <- y_adj - adjust
        } else {
            y <- y_adj
        }
    } else {
        # GMM estimators
        if (link == "logit") {
            y <- Hy / (1 + Hy)
        } else if (link == "cloglog") {
            y <- 1 - exp(-Hy)
        } else {
            y <- Hy # Fallback
        }
    }
    
    return(y - fitted(object))
}

#' @title Extract the Number of Observations for fracreghet
#' @description Extracts the number of observations used to estimate a \code{fracreghet} model.
#' @param object A fitted model object of class \code{fracreghet}.
#' @param ... Further arguments passed to or from other methods.
#' @return An integer denoting the number of observations.
#' @exportS3Method nobs fracreghet
nobs.fracreghet <- function(object, ...) {
    return(length(object$Hy))
}

#' @title Extract Log-Likelihood for fracreghet
#' @description Extracts the log-pseudolikelihood or log-likelihood from a fitted \code{fracreghet} model.
#' @param object A fitted model object of class \code{fracreghet}.
#' @param ... Further arguments passed to or from other methods.
#' @return An object of class \code{logLik}.
#' @exportS3Method logLik fracreghet
logLik.fracreghet <- function(object, ...) {
    if (object$type != "QMLxv") {
        return(NA) # GMM/LIN do not have log-likelihood
    }
    yhat <- fitted(object)
    y <- object$Hy # Because for QMLxv, Hy is y
    eps <- 1e-16
    LL <- sum(ifelse(y > 0, y * log(pmax(yhat, eps)), 0) + ifelse(y < 1, (1-y) * log(pmax(1-yhat, eps)), 0))
    
    attr(LL, "df") <- length(coef(object))
    attr(LL, "nobs") <- nobs(object)
    class(LL) <- "logLik"
    return(LL)
}

#' @title Extract Covariance Matrix for fracreghet
#' @description Extracts the estimated variance-covariance matrix of the parameters.
#' @param object A fitted model object of class \code{fracreghet}.
#' @param ... Further arguments passed to or from other methods.
#' @return A matrix of the estimated covariances.
#' @exportS3Method vcov fracreghet
vcov.fracreghet <- function(object, ...) {
    if(!is.null(object$p.var)) {
        return(object$p.var)
    }
    return(NULL)
}

#' @title Predict Method for fracreghet
#' @description Predicts conditional mean values from a fitted \code{fracreghet} model.
#' @param object A fitted model object of class \code{fracreghet}.
#' @param newdata An optional data frame or matrix in which to look for variables with which to predict. If omitted, the fitted values are used.
#' @param ... Further arguments passed to or from other methods.
#' @return A numeric vector of predicted values.
#' @exportS3Method predict fracreghet
predict.fracreghet <- function(object, newdata = NULL, ...) {
    if (is.null(newdata)) {
        return(fitted(object))
    }
    
    stop("predict() with newdata is currently not natively supported for fracreghet due to the secondary variance equation. Fitted values are available without newdata.")
}

fracregStartupMessage <- function() {
  art_lines <- c(
    "     ,...                                                ",
    "  .d' \"\"                                                ",
    "  dM`                                                   ",
    " mMMmm`7Mb,od8 ,6\"Yb.  ,p6\"bo `7Mb,od8 .gP\"Ya   .P\"Ybmmm",
    "  MM    MM' \"'8)   MM 6M'  OO   MM' \"',M'   Yb :MI  I8  ",
    "  MM    MM     ,pm9MM 8M        MM    8M\"\"\"\"\"\"  WmmmP\"  ",
    "  MM    MM    8M   MM YM.    ,  MM    YM.    , 8M       ",
    ".JMML..JMML.  `Moo9^Yo.YMbmd' .JMML.   `Mbmmd'  YMMMMMb ",
    "                                               6'     dP",
    "                                               Ybmmmd'  "
  )
  base_msg <- paste0(
    "\n\n* Please cite the 'fracreg' package as:\n",
    "Owili, SO. (2026). fracreg: Fractional Response Regressions. R package version 1.0.1.\n\n",
    "See also: citation(\"fracreg\")\n\n",
    "* For any questions, suggestions, or comments on the 'fracreg' package, you can contact the authors directly or visit:\n",
    "  https://github.com/SulmanOlieko/fracreg/issues\n"
  )

  console_width <- getOption("width")
  art_width <- max(nchar(art_lines))

  if (console_width < art_width) {
    # 1. Truncate all lines to fit the console width
    trimmed_art <- substr(art_lines, 1, console_width - 3)

    # 2. Add the ASCII ellipsis strictly to the last row
    ellipsis <- "..."
    e_len <- nchar(ellipsis)
    cut_point <- max(0, console_width - e_len)

    last_line_index <- length(trimmed_art)
    trimmed_art[last_line_index] <- paste0(
      substr(art_lines[last_line_index], 1, cut_point),
      ellipsis
    )

    # 3. Set the version alignment target
    align_width <- console_width
  } else {
    trimmed_art <- art_lines
    align_width <- art_width
  }

  art_msg <- paste(trimmed_art, collapse = "\n")

  # 4. Pad the version text so it hits the exact right edge
  version_msg <- paste0(
    "\n",
    sprintf(paste0("%", align_width, "s"), "version 1.0.1")
  )

  return(paste0(art_msg, version_msg, base_msg))
}

.onAttach <- function(libname, pkgname) {
  # We still skip printing during the CRAN check to keep the test logs clean
  in_chk <- Sys.getenv("_R_CHECK_PACKAGE_NAME_") != ""

  if (!in_chk) {
    msg <- fracregStartupMessage()
  } else {
    msg <- paste0(
      "\n* Please cite the 'fracreg' package as:\n",
      "Owili, SO. (2026). fracreg: Fractional Response Regressions. R package version 1.0.1.\n\n",
      "See also: citation(\"fracreg\")\n\n",
      "* For any questions, suggestions, or comments on the 'fracreg' package, you can contact the authors directly or visit:\n",
      "  https://github.com/SulmanOlieko/fracreg/issues\n"
    )
  }

  packageStartupMessage(msg)
  invisible()
}

fracreghet.links <- function(link) 
{
	switch(link,
		logit = {
			linkfun <- function(mu) qlogis(mu)
			linkinv <- function(eta) plogis(eta)
			mu.eta <- function(eta) dlogis(eta)
			H1 <- function(dep) dep/(1-dep)
			G2 <- function(eta) exp(eta)
			g2 <- function(eta) exp(eta)
			H2 <- function(uu) log(uu)
			gd <- function(eta) exp(eta)*(1-exp(eta))/((1+exp(eta))^3)
			valideta <- function(eta) TRUE
		},

		probit = {
			linkfun <- function(mu) qnorm(mu)
			linkinv <- function(eta) {
				thresh <- -qnorm(.Machine$double.eps)
				eta <- pmin(pmax(eta, -thresh), thresh)
	 			pnorm(eta)
			}
			mu.eta <- function(eta) pmax(dnorm(eta),.Machine$double.eps)
			H1 <- NULL
			G2 <- NULL
			g2 <- NULL
			H2 <- NULL
			gd <- function(eta) -eta*dnorm(eta)
			valideta <- function(eta) TRUE
		},

		cauchit = {
		  	linkfun <- function(mu) qcauchy(mu)
			linkinv <- function(eta) {
				thresh <- -qcauchy(.Machine$double.eps)
				eta <- pmin(pmax(eta, -thresh), thresh)
				pcauchy(eta)
			}
			mu.eta <- function(eta) pmax(dcauchy(eta),.Machine$double.eps)
			H1 <- NULL
			G2 <- NULL
			g2 <- NULL
			H2 <- NULL
			gd <- function(eta) -2*eta/(pi*(eta^2+1)^2)
			valideta <- function(eta) TRUE
		},

		cloglog = {
			linkfun <- function(mu) log(-log(1 - mu))
			linkinv <- function(eta) pmax(pmin(-expm1(-exp(eta)),1-.Machine$double.eps),.Machine$double.eps)
			mu.eta <- function(eta) {
				eta <- pmin(eta, 700)
				pmax(exp(eta) * exp(-exp(eta)),.Machine$double.eps)
			}
			H1 <- function(dep) -log(1-dep)
			G2 <- function(eta) exp(eta)
			g2 <- function(eta) -exp(-eta)
			H2 <- function(uu) log(uu)
			gd <- function(eta) (exp(-exp(eta))*exp(eta))*(1-exp(eta))
			valideta <- function(eta) TRUE
		},

		loglog = {
			linkfun <- function(mu) -log(-log(mu))
			linkinv <- function(eta) exp(-exp(-eta))
			mu.eta <- function(eta) exp(-exp(-eta)-eta)
			H1 <- NULL
			G2 <- NULL
			g2 <- NULL
			H2 <- NULL
			gd <- function(eta) (exp(-exp(-eta))*exp(-eta))*(exp(-eta)-1)
			valideta <- function(eta) TRUE
		}
	)

	structure(list(linkfun=linkfun,linkinv=linkinv,mu.eta=mu.eta,H1=H1,G2=G2,g2=g2,H2=H2,gd=gd,valideta=valideta,name=link),class="link-glm")
}

fracreghet.gi <- function(type,z,Hy,XB,link)
{
	if(type!="QMLxv")
	{
		G2 <- fracreghet.links(link)$G2(XB)
		u <- Hy/G2-1
	}
	if(type=="QMLxv")
	{
		yhat <- fracreghet.links(link)$linkinv(XB)
		g <- fracreghet.links(link)$mu.eta(XB)
		u <- Hy-yhat
		u <- g*u/(yhat*(1-yhat))
	}

	gi <- t(z*u)

	return(list(gi=gi,u=u))
}

fracreghet.Gn <- function(type,x,z,z.in,u,bet,p)
{
	N <- length(u)

	if(all(type!=c("GMMxv","LINxv","QMLxv"))) Gn <- -(1/N)*t(z*bet)%*%x
	if(any(type==c("GMMxv","LINxv","QMLxv")))
	{
		k <- ncol(x)
		
		G11 <- -(1/N)*t(z*bet)%*%x

		G12 <- (1/N)*t(x*(bet*p[k]))%*%z.in
		G12[k,] <- G12[k,]-(1/N)*apply(u*z.in,2,sum)

		G21 <- matrix(0,nrow=ncol(z.in),ncol=k)
		G22 <- -(1/N)*t(z.in)%*%z.in

		Gn <- rbind(cbind(G11,G12),cbind(G21,G22))
	}

	return(Gn)
}

fracreghet.est <- function(type,x,z,link,start,Hy,variance,var.type,var.cluster,gixv,vhat,offset=NULL,...)
{
	if(any(type==c("GMMxv","QMLxv")))
	{
		z.in <- z
		z <- x
	}

	GMM.est <- TRUE
	if(any(type==c("GMMx","GMMxv")) & !any(Hy==0))
	{
		results <- tryCatch(glm(Hy ~ x-1,family=Gamma(link=log),maxit=100,offset=offset),error=function(e) return(NULL))
		if(any(is.null(results))) converged <- FALSE
		else
		{
			p <- results$coefficients
			XB <- results$linear.predictors
			converged <- results$converged*(1-results$boundary)
		}

		if(converged==TRUE) GMM.est <- FALSE
	}
	if(GMM.est==TRUE)
	{
		GMMn <- function(p)
		{
			XB <- as.vector(x%*%p)
			if (!is.null(offset)) XB <- XB + offset
			gi <- fracreghet.gi(type,z,Hy,XB,link)$gi

			gn <- as.matrix(apply(gi,1,mean))
			Qn <- t(gn)%*%S%*%gn

			return(Qn)
		}

		S <- diag(ncol(z))
		results <- nlminb(start=start,objective=GMMn,...)
		p <- results$par
		XB <- as.vector(x%*%p)
		if (!is.null(offset)) XB <- XB + offset

		if(type=="GMMz" & ncol(z)>ncol(x))
		{
			fi.inv <- fracreghet.var(type,p,XB,x,z,link,Hy,var.type,var.cluster,TRUE,gixv,vhat)$fi.inv
			if(!is.character(fi.inv))
			{
				S <- fi.inv
				results <- nlminb(start=start,objective=GMMn,...)
				p <- results$par
				XB <- as.vector(x%*%p)
				if (!is.null(offset)) XB <- XB + offset
			}

			Qn <- results$objective
		}

		converged <- ifelse(results$convergence==0,TRUE,FALSE)
	}

	ret.list <- list(p=p,XB=XB,converged=converged)
	if(type=="GMMz" & ncol(z)>ncol(x)) ret.list[["Qn"]] <- Qn
	if (any(type == c("QMLxv", "QMLz"))) {
		yhat <- fracreghet.links(link)$linkinv(XB)
		eps <- 1e-16
		LL <- sum(ifelse(Hy > 0, Hy * log(pmax(yhat, eps)), 0) + ifelse(Hy < 1, (1-Hy) * log(pmax(1-yhat, eps)), 0))
		ret.list[["LL"]] <- LL
	}

	if(variance==FALSE | converged==FALSE) return(ret.list)

	if(any(type==c("GMMxv","QMLxv"))) z <- z.in
	p.var <- fracreghet.var(type,p,XB,x,z,link,Hy,var.type,var.cluster,FALSE,gixv,vhat)$p.var
	ret.list[["p.var"]] <- p.var

	return(ret.list)
}

fracreghet.var <- function(type,p,XB,x,z,link,Hy,var.type,id,step.one,gixv,vhat)
{
	N <- length(Hy)

	if(any(type==c("GMMxv","LINxv","QMLxv")))
	{
		z.in <- z
		z <- x
	}
	else z.in <- z

	if(any(type==c("LINx","LINxv","LINz")))
	{
		u <- Hy-XB
		gi <- t(z*u)
		bet <- rep(1,N)
	}
	if(any(type==c("GMMx","GMMz","GMMxv","QMLxv")))
	{
		results <- fracreghet.gi(type,z,Hy,XB,link)
		gi <- results$gi
		u <- results$u

		if(type!="QMLxv")
		{
			G2 <- fracreghet.links(link)$G2(XB)
			g2 <- fracreghet.links(link)$g2(XB)
			bet <- Hy*g2/(G2^2)
		}
		if(type=="QMLxv")
		{
			yhat <- fracreghet.links(link)$linkinv(XB)
			g <- fracreghet.links(link)$mu.eta(XB)
			bet <- (g^2)/(yhat*(1-yhat))
		}
	}

	if(any(type==c("GMMxv","LINxv","QMLxv"))) gi <- rbind(gi,gixv)

	if(var.type=="robust") fi <- (1/N)*gi%*%t(gi)

	if(var.type=="cluster")
	{
		fi <- 0

		for(j in unique(id))
		{
			Zi <- matrix(z[id==j,],ncol=ncol(z))
			ui <- u[id==j]
			zu <- t(Zi)%*%ui

			if(any(type==c("GMMxv","LINxv","QMLxv")))
			{
				Zi <- matrix(z.in[id==j,],ncol=ncol(z.in))
				vi <- vhat[id==j]
				zu2 <- t(Zi)%*%vi
				zu <- rbind(zu,zu2)
			}

			fi <- fi+zu%*%t(zu)
		}

		fi <- fi/N
	}

	fi.inv <- tryCatch(solve(fi),error=function(e) NaN)
	if(any(is.nan(fi.inv))) fi.inv <- "singular"

	if(step.one==TRUE) return(list(fi.inv=fi.inv))

	Gn <- fracreghet.Gn(type,x,z,z.in,u,bet,p)

	if(is.numeric(fi.inv))
	{
		sigma <- N*t(Gn)%*%fi.inv%*%Gn
		p.var <- tryCatch(solve(sigma),error=function(e) NaN)
		if(any(is.nan(p.var))) p.var <- "singular"
	}
	else p.var <- "singular"

	if(is.character(p.var))
	{
		if(ncol(z)>ncol(x)) p.var <- "singular"
		else
		{
			Gn.inv <- tryCatch(solve(Gn),error=function(e) NaN)
			if(any(is.nan(Gn.inv))) p.var <- "singular"
			else p.var <- (1/N)*Gn.inv%*%fi%*%t(Gn.inv)
		}
	}

	ret.list <- list(p.var=p.var)

	return(ret.list)
}

#' @title Fitting Fractional Response Regressions under Unobserved Heterogeneity
#'
#' @description
#' \code{fracreghet} is used to fit fractional response models under unobserved heterogeneity, i.e. regression models for proportions, percentages or fractions that suffer from neglected heterogeneity and/or endogeneity issues.
#' @param y a numeric vector containing the values of the response variable.
#' @param x a numeric matrix, with column names, containing the values of all covariates (exogenous and endogenous).
#' @param z a numeric matrix, with column names, containing the values of all exogenous variables (covariates and instrumental  variables). Defaults to \code{x}.
#' @param var.endog a numeric vector containing the values of the endogenous covariate (or of some transformation of it), which will be used as dependent variable in the linear reduced form assumed for application of xv-type estimators.
#' @param start a numeric vector containing the initial values for the parameters to be optimised. Optional.
#' @param type a description of the estimator to compute: \code{GMMx} (the default), \code{GMMxv}, \code{GMMz}, \code{LINx}, \code{LINxv}, \code{LINz} or \code{QMLxv}.
#' @param link a description of the link function to use. Available options for all estimators: \code{logit} and \code{cloglog}. Additional available options for QML and LIN estimators: \code{probit}, \code{cauchit} and \code{loglog}.
#' @param intercept a logical value indicating whether the model should include a constant term or not.
#' @param table a logical value indicating whether a summary table with the regression results should be printed.
#' @param variance a logical value indicating whether the variance of the estimated parameters should be calculated. Defaults to \code{TRUE} whenever \code{table = TRUE}.
#' @param var.type a description of the type of variance of the estimated parameters to be calculated. Options are \code{robust}, the default, and \code{cluster}.
#' @param var.cluster a numeric vector containing the values of the variable that specifies to which cluster each observation belongs.
#' @param adjust the numeric value to be added to the response variable in case of boundary observations when the LIN estimators are applied or the string \code{drop}, which implies that the boundary observations are dropped.
#' @param offset an optional numeric vector containing an offset. It must be of the same dimension as the response variable. It specifies that the variable should be included in the model with its coefficient constrained to 1.
#' @param or a logical value indicating whether to report odds ratios. Only valid when the link function is \code{"logit"}. Defaults to \code{FALSE}.
#' @param level a numeric value between 0 and 1 indicating the confidence level for the confidence intervals. Defaults to \code{0.95}.
#' @param na.action A function specifying how to handle missing values, default is \code{stats::na.omit}. If \code{NULL}, no action is taken.
#' @param \dots Arguments to pass to \link[stats]{nlminb}.
#'
#' @details
#' \code{fracreghet} computes the GMM estimators proposed in Ramalho and Ramalho (2017) for fractional response models with unobserved heterogeneity: GMMx, which allows for neglected heterogeneity but not for endogeneity; GMMxv, which allows both issues and assumes a linear reduced form for the endogeneous covariate (or for a transformation of it); and GMMz, which also allows for both issues but does not require the assumption of a reduced form for the endogenous covariate. In addition, \code{fracreghet} also computes three linearised estimators (LINx, LINxv and LINz) that have similar features to their GMM counterparts. It also provides a QML estimator (QMLxv) that addresses endogeneity using a Control Function (CF) approach, which includes the first-stage reduced-form residuals as an additional regressor in the main fractional equation, providing a Hausman-type test for endogeneity. 
#' 
#' \strong{Control Function (CF) Approach - QMLxv:}
#' When a continuous regressor \eqn{y_{2i}} is endogenous, the CF approach (Papke and Wooldridge, 2008; Terza et al., 2008) uses a two-stage procedure. First, a linear reduced form is estimated:
#' \deqn{y_{2i} = z_i \pi + v_i}
#' where \eqn{z_i} includes all exogenous variables and external instruments. The residuals \eqn{\hat{v}_i} are then included in the fractional response model:
#' \deqn{E(y_{1i} | z_i, y_{2i}, v_i) = G(x_i \beta + \gamma \hat{v}_i)}
#' A test of \eqn{H_0: \gamma = 0} serves as a robust Hausman-type test for endogeneity.
#' 
#' \strong{Generalised Method of Moments (GMM):}
#' For estimators like GMMz, which do not strictly require a linear reduced form, the estimation relies on population orthogonality conditions between the instruments \eqn{Z_i} and the model residuals:
#' \deqn{E[Z_{i} (y_i - G(x_i \beta))] = 0}
#' or via specific transformations of the dependent variable to eliminate unobserved heterogeneity (Ramalho and Ramalho, 2017).
#' 
#' For overidentified models, \code{fracreghet} calculates Hansen's J statistic. For \code{GMMx} and \code{LINx}, \code{fracreghet} stores the information needed to implement the RESET test (\link{fracreghet.reset}). For all estimators, \code{fracreghet} stores the information needed to calculate partial effects (\link{fracreghet.pe}).
#'
#' @return
#' \code{fracreghet} returns a list with the following elements:
#'   \item{class}{"fracreghet".
#' }
#'   \item{formula}{the model formula.
#' }
#'   \item{type}{the name of the estimator computed.
#' }
#'   \item{link}{the name of the specified link.
#' }
#'   \item{adjust}{The value or the type of the adjustment applied to LIN estimators.
#' }
#'   \item{p}{a named vector of coefficients.
#' }
#'   \item{Hy}{the transformed values of the response variable when GMM or LIN estimators are computed or the 
#'    values of the response variable in the QML case.
#' }
#'   \item{xbhat}{the fitted mean values of the linear predictor (for xv-type estimators, includes the term relative to the first-stage residual).
#' }
#'   \item{converged}{logical. Was the algorithm judged to have converged?
#' }
#'   \item{x.names}{a vector containing the names of the covariates.
#' }
#' 
#' In case of an overidentifying model, the following element is also returned:
#'   \item{J}{the result of Hansen's J test of overidentifying moment conditions.
#' }
#' 
#' If \code{variance = TRUE} or \code{table = TRUE} and the algorithm converged successfully, the previous list also contains the following elements:
#' 
#'   \item{p.var}{a named covariance matrix.
#' }
#'   \item{var.type}{covariance matrix type.
#' }
#' 
#' If \code{var.type = "cluster"}, the list also contains the following element:
#'   \item{var.cluster}{the variable that specifies to which cluster each observation belongs.
#' }
#'
#' @section Odds Ratios:
#' When \code{or=TRUE} and the fractional link function (\code{linkfrac} or \code{link}) is \code{"logit"}, the model additionally computes odds ratios for the coefficients. 
#' Odds Ratios are exponentiated coefficients.
#' The corresponding standard errors for the odds ratios are calculated using the Delta method.
#' The confidence intervals for the odds ratios are calculated using the adjusted standard errors and the specified \code{level} (defaulting to 95\%).
#' Odds ratios are particularly useful in fractional logit models as they provide a direct multiplicative interpretation of the independent variable on the odds of the fractional outcome.
#'
#' @references
#' Papke, L. E. and Wooldridge, J. M. (2008), "Panel data methods for fractional response variables with an application to test pass rates", \emph{Journal of Econometrics}, 145, 121-133.
#' 
#' Ramalho, E. A., & Ramalho, J. J. S. (2017), "Moment-based estimation of nonlinear regression models with boundary outcomes and endogeneity, with applications to nonnegative and fractional responses", \emph{Econometric Reviews}, 36(4), 397-420.
#' 
#' Terza, J. V., Basu, A., and Rathouz, P. J. (2008), "Two-stage residual inclusion estimation: addressing endogeneity in health econometric modeling", \emph{Journal of Health Economics}, 27(3), 531-543.
#'
#' @author Sulman Olieko Owili <oliekosulman@gmail.com>
#'
#' @seealso
#' \code{\link{fracreghet.reset}}, for the RESET test.\cr
#' \code{\link{fracreghet.pe}}, for computing partial effects.\cr
#' \code{\link{fracreg}}, for fitting standard cross-sectional fractional response models.\cr
#' \code{\link{fracregpd}}, for fitting panel data fractional response models.
#'
#' @examples
#' ### Empirical 401(k) Examples 
#' data("fracreg_k401k") 
#' y <- fracreg_k401k$prate 
#' X_het <- cbind(mrate = fracreg_k401k$mrate, ltotemp = fracreg_k401k$ltotemp)
#'  
#' # fracreghet estimators do not allow exact 1s or 0s
#' y_adj <- y
#' y_adj[y_adj == 1] <- 0.999
#' 
#' # Instrument mrate using age
#' 
#' Z_emp <- cbind(age = fracreg_k401k$age, ltotemp = fracreg_k401k$ltotemp) 
#' mod <- fracreghet(y_adj, X_het, Z_emp, var.endog = X_het[, "mrate"], type="QMLxv", link="logit")
#' summary(mod)
#' 
#' # Compute the same QMLxv estimator reporting Odds Ratios with 90% confidence intervals
#' mod <- fracreghet(y_adj, X_het, Z_emp, var.endog = X_het[, "mrate"], type="QMLxv", 
#'            link="logit", or=TRUE, level=0.90) 
#'  
#' ### Simulated Examples 
#' 
#' set.seed(123)
#' N <- 1000
#' x1 <- rnorm(N)
#' 
#' # Simulating an endogenous variable (var.endog) and an instrument (z1)
#' z1 <- rnorm(N)
#' u <- 0.5 * z1 + rnorm(N)
#' var.endog <- 0.8 * z1 + u
#' y_endog <- exp(0.5 * x1 + 1.2 * var.endog + u) / (1 + exp(0.5 * x1 + 1.2 * var.endog + u))
#' 
#' # Avoid exact 0 or 1 boundaries for some estimators
#' y_endog[y_endog <= 0] <- 0.01
#' y_endog[y_endog >= 1] <- 0.99
#' 
#' X <- cbind(x1 = x1, var.endog = var.endog)
#' Z <- cbind(x1 = x1, z1 = z1)
#' 
#' # Exogeneity (assuming var.endog is exogenous for comparison), GMMx estimator
#' mod <- fracreghet(y = y_endog, x = X, type = "GMMx", link = "logit")
#' summary(mod)
#' 
#' # Endogeneity, GMMz estimator (does not require reduced form for endog)
#' mod <- fracreghet(y = y_endog, x = X, z = Z, type = "GMMz", link = "logit")
#' summary(mod)
#' 
#' # Endogeneity, GMMxv estimator (assumes linear reduced form for var.endog)
#' mod <- fracreghet(y = y_endog, x = X, z = Z, var.endog = var.endog, type = "GMMxv", link = "logit")
#' summary(mod)
#' 
#' # Endogeneity, QMLxv control function approach
#' mod <- fracreghet(y = y_endog, x = X, z = Z, var.endog = var.endog, type = "QMLxv", link = "logit")
#' summary(mod)
#' @export
fracreghet <- function(y,x,z=x,var.endog,start,type="GMMx",link="logit",intercept=TRUE,table=TRUE,variance=TRUE,var.type="robust",var.cluster,adjust=0,offset=NULL,or=FALSE,level=0.95,na.action=stats::na.omit,...)
{
	cl <- match.call()
	LL <- NULL
	
	if (!missing(y) && !missing(x)) {
	    args_to_clean <- list(y=y, x=x, offset=offset)
	    if (!missing(z)) args_to_clean$z <- z
	    if (!missing(var.endog)) args_to_clean$var.endog <- var.endog
	    if (!missing(var.cluster)) args_to_clean$var.cluster <- var.cluster
	    
	    args_to_clean$na.action <- na.action
	    cleaned <- do.call(fracreg_clean_data, args_to_clean)
	    
	    y <- cleaned$y
	    x <- cleaned$x
	    offset <- cleaned$offset
	    if (!missing(z)) z <- cleaned$z
	    if (!missing(var.endog)) var.endog <- cleaned$var.endog
	    if (!missing(var.cluster)) var.cluster <- cleaned$var.cluster
	}
	
	### 1. Error and warning messages

	if(missing(y)) stop("dependent variable is missing")
	if(missing(x)) stop("explanatory variables are missing")
	if(any(y>1) | any(y<0)) stop("The dependent variable has values outside the unit interval")
	if((any(y==1) | any(y==0)) & adjust==0 & any(type==c("LINx","LINz","LINxv"))) stop("0/1 values for the response variable: LIN estimators require adjustment")
	if(all(link!=c("logit","probit","cauchit","cloglog","loglog"))) stop(sQuote(link)," - link not recognised")
	if(all(type!=c("GMMx","GMMz","GMMxv","LINx","LINz","LINxv","QMLxv"))) stop(sQuote(type)," - type not recognised")
	if(any(type==c("GMMx","GMMz","GMMxv")) & all(link!=c("logit","cloglog"))) stop("type and link not compatible")
	if(any(type==c("GMMx","GMMz","GMMxv")) & any(y==1)) stop("estimator does not allow y = 1")
	if(!is.numeric(adjust) & adjust!="drop") stop("adjust not defined properly")
	if(any(type==c("GMMxv","LINxv","QMLxv")) & missing(var.endog)) stop(sQuote(type)," requires var.endog to be specified")
	if(all(type!=c("GMMxv","LINxv","QMLxv")) & !missing(var.endog)) stop("var.endog should not be specified for this estimator")
	if(table==TRUE & variance==FALSE)
	{
		variance <- TRUE
		warning("option variance changed from FALSE to TRUE, as required by table=TRUE")
	}
	if(all(var.type!=c("robust","cluster"))) stop(sQuote(var.type)," - var.type not recognised")
	if(var.type=="cluster" & missing(var.cluster)) stop("option cluster for covariance matrix but no var.cluster supplied")
	if(var.type=="robust" & !missing(var.cluster)) stop("option robust for covariance matrix but var.cluster supplied")

	if(!is.logical(intercept)) stop("non-logical value assigned to option intercept")
	if(!is.logical(table)) stop("non-logical value assigned to option table")
	if(!is.logical(variance)) stop("non-logical value assigned to option variance")

	### 2. Data and variables preparation

	y <- as.vector(y)

	if(is.data.frame(x)) x <- as.matrix(x)
	if(is.data.frame(z)) z <- as.matrix(z)

	if(!is.matrix(x)) stop("x is not a matrix")
	if(!is.matrix(z)) stop("z is not a matrix")

	x.names <- dimnames(x)[[2]]
	z.names <- dimnames(z)[[2]]

	if(is.null(x.names)) stop("x has no column names")
	if(is.null(z.names)) stop("z has no column names")

	if(intercept==TRUE)
	{
		x <- cbind(1,x)
		z <- cbind(1,z)
		x.names <- c("(Intercept)",x.names)
		z.names <- c("(Intercept)",z.names)
	}

	if(length(x.names)!=length(unique(x.names))) stop("some covariate names in x are identical")
	if(length(z.names)!=length(unique(z.names))) stop("some instrument names in z are identical")
	if(identical(x.names,z.names) & any(type==c("GMMz","GMMxv","LINz","LINxv","QMLxv"))) stop("instruments and covariates are identical")
	if(!identical(x.names,z.names) & any(type==c("GMMx","LINx"))) stop("instruments should not be specified for this estimator")
	if(length(x.names)>length(z.names)) stop("number of instruments not enough")

	if(adjust=="drop")
	{
		x <- as.matrix(x[y>0 & y<1,])
		z <- as.matrix(z[y>0 & y<1,])
		if(var.type=="cluster") var.cluster <- var.cluster[y>0 & y<1]
		if(!is.null(offset)) offset <- offset[y>0 & y<1]
		if(!missing(var.endog)) var.endog <- var.endog[y>0 & y<1]
		y <- y[y>0 & y<1]
	}

	if(any(length(y)!=c(nrow(x),nrow(z)))) stop("the number of observations for y, x and/or z is different")
	if(var.type=="cluster")
	{
		if(length(y)!=length(var.cluster)) stop("var.cluster does not have the appropriate dimension")
	}
	if(!missing(var.endog))
	{
		if(length(y)!=length(var.endog)) stop("var.endog does not have the appropriate dimension")
	}
	if(!is.null(offset) && length(y)!=length(offset)) stop("offset does not have the appropriate dimension")

	class <- "fracreghet"

	### 3. Estimation

	if(any(type==c("GMMx","LINx"))) z <- x

	if(any(type==c("GMMxv","LINxv","QMLxv")))
	{
		results <- lm(var.endog ~ 0+z)
		PIhat <- results$coefficients
		vhat <- results$residuals

		gixv <- t(z*vhat)
		x <- cbind(x,vhat)
	}
	else
	{
		gixv <- NA
		vhat <- NA
	}

	k <- ncol(x)
	kz <- ncol(z)
	N <- nrow(x)
	J <- NA
	dfJ <- kz-k

	if(any(type==c("GMMx","GMMz","GMMxv","QMLxv")))
	{
		if(any(type==c("GMMx","GMMz","GMMxv"))) Hy <- fracreghet.links(link)$H1(y)
		if(type=="QMLxv") Hy <- y

		if(missing(start)) start <- rep(0,k)
		if(length(start)!=k) stop("start is not of the same dimension as the covariate vector (including vhat, in case of GMMxv or QMLxv)")
		
		results <- fracreghet.est(type,x,z,link,start,Hy,variance,var.type,var.cluster,gixv,vhat,offset=offset,...)
		p <- results$p
		converged <- results$converged

		XB <- results$XB
		if(any(type=="GMMz") & dfJ>0)
		{
			Qn <- results$Qn
			J <- N*Qn
		}
	}

	if(any(type==c("LINx","LINz","LINxv")))
	{
		if(is.numeric(adjust))
		{
			y <- y+adjust
			if(any(y>=1) | any(y<=0)) stop("After adjustment, the dependent variable has values outside or at the boundaries of the unit interval")
		}

		Hy <- fracreghet.links(link)$linkfun(y)

		if(any(type==c("LINx","LINxv")))
		{
			results <- lm(Hy ~ 0+x, offset=offset)
			p <- results$coefficients

			XB <- results$fitted.values
		}
		if(type==c("LINz"))
		{
			Hy_adj <- Hy
			if (!is.null(offset)) Hy_adj <- Hy - offset
			XZ <- t(x)%*%z
			p <- solve(XZ%*%t(XZ))%*%(XZ%*%(t(z)%*%Hy_adj))
			if(kz>k)
			{
				u <- as.vector(Hy_adj-x%*%p)

				gi <- t(z*u)
				fi <- (1/N)*gi%*%t(gi)
				fi.inv <- solve(fi)
 				p <- as.vector(solve(XZ%*%fi.inv%*%t(XZ))%*%(XZ%*%fi.inv%*%(t(z)%*%Hy_adj)))
			}

			XB <- as.vector(x%*%p)
			if(!is.null(offset)) XB <- XB + offset

			if(kz>k) J <- N*t(Hy-XB)%*%z%*%fi.inv%*%t(z)%*%(Hy-XB)
		}

		converged <- TRUE
		if(variance==TRUE) results <- fracreghet.var(type,p,XB,x,z,link,Hy,var.type,var.cluster,FALSE,gixv,vhat)
	}

	if(variance==TRUE & converged==TRUE) p.var <- results$p.var
	else p.var <- "singular"

	x.names.in <- x.names

	if(any(type==c("GMMxv","LINxv","QMLxv")))
	{
		p <- c(p,PIhat)
		x.names <- c(x.names,"vhat",paste("Z",z.names,sep="_"))
	}
	names(p) <- x.names

	if(table==TRUE) fracreghet.table(p,p.var,x.names,type,link,converged,N,var.type,adjust,k,J,dfJ,LL=LL,or=or,level=level)

	formula <- y ~ x - 1

	res <- list(class=class,formula=formula,type=type,link=link,adjust=adjust,p=p,Hy=Hy,xbhat=as.vector(XB),converged=converged,x.names=x.names.in)
	if(!is.null(offset)) res[["offset"]] <- offset
	if(any(type==c("GMMz","LINz")) & kz>k) res[["J"]] <- J

	if(variance==TRUE & converged==TRUE)
	{
		if(is.character(p.var)) p.var <- matrix(NA,nrow=length(p),ncol=length(p))
		dimnames(p.var) <- list(x.names,x.names)
		res[["p.var"]] <- p.var
		res[["var.type"]] <- var.type
		if(var.type=="cluster") res[["var.cluster"]] <- var.cluster
	}

	### 4. Return results

	class(res) <- "fracreghet"
	return(invisible(res))
}

library(testthat)
library(fracreg)

test_that("fracreghet QMLxv estimator works", {
  set.seed(42)
  N <- 500
  x1 <- rnorm(N)
  z <- rnorm(N)
  u <- 0.5 * z + rnorm(N)
  var.endog <- 0.8 * z + u
  xb <- 0.5 * x1 + 1.2 * var.endog + u
  y_endog <- exp(xb) / (1 + exp(xb))
  
  X <- cbind(x1 = x1, var.endog = var.endog)
  Z <- cbind(x1 = x1, z = z)
  
  model_het <- fracreghet(
    y = y_endog, 
    x = X, 
    z = Z, 
    var.endog = var.endog,
    type = "QMLxv", 
    link = "logit",
    table = FALSE
  )
  
  expect_type(model_het, "list")
  expect_true(all(c("p", "p.var", "Hy", "xbhat") %in% names(model_het)))
})

library(testthat)
library(fracreg)

test_that("S3 Extractors return expected classes and dimensions across models", {
  N <- 100
  set.seed(42)
  X <- cbind(x1 = rnorm(N), x2 = runif(N))
  y <- exp(X[,1] - X[,2]) / (1 + exp(X[,1] - X[,2]))
  
  # 1. fracreg
  mod1 <- fracreg(y, X, type="1P", linkfrac="logit", table=FALSE)
  expect_equal(length(coef(mod1)), 3)
  expect_equal(length(fitted(mod1)), 100)
  expect_equal(length(residuals(mod1)), 100)
  expect_equal(nobs(mod1), 100)
  expect_true(is.numeric(logLik(mod1)))
  expect_equal(length(predict(mod1)), 100)
  
  # 2. fracregpd
  id <- rep(1:20, each=5)
  time <- rep(1:5, 20)
  mod2 <- fracregpd(id, time, y, X, type="GMMpfe", link="logit", table=FALSE)
  expect_equal(length(coef(mod2)), 2)
  expect_equal(length(fitted(mod2)), 100)
  expect_equal(length(residuals(mod2)), 100)
  expect_equal(nobs(mod2), 100)
  expect_true(is.na(logLik(mod2)))
  expect_equal(length(predict(mod2)), 100)
  
  # Bounded y to prevent intermediate glm(Gamma) optimization NaNs 
  y_bounded <- pmin(pmax(y, 0.1), 0.9)

  # 3. fracreghet
  mod3 <- fracreghet(y_bounded, X, type="GMMx", link="logit", table=FALSE)
  expect_equal(length(coef(mod3)), 3)
  expect_equal(length(fitted(mod3)), 100)
  expect_equal(length(residuals(mod3)), 100)
  expect_equal(nobs(mod3), 100)
  expect_true(is.na(logLik(mod3)))
  expect_equal(length(predict(mod3)), 100)
  
  # 4. fracregridge
  mod4 <- fracregridge(y, X, fracs=0.5)
  expect_true(is.array(coef(mod4)))
  expect_true(is.array(fitted(mod4)))
  expect_true(is.array(residuals(mod4)))
  expect_equal(nobs(mod4), 100)
  
  # 5. fracregmlogit
  y_ml <- cbind(y, 1-y)
  mod5 <- fracregmlogit(y_ml, X)
  expect_true(is.matrix(coef(mod5)))
  expect_s3_class(fitted(mod5), "data.frame")
  expect_s3_class(residuals(mod5), "data.frame")
  expect_s3_class(predict(mod5), "data.frame")
  expect_equal(nobs(mod5), 100)
  expect_true(is.numeric(logLik(mod5)))
})

library(testthat)
library(fracreg)

test_that("global na.omit wrapper correctly drops NAs across all models", {
  N <- 100
  set.seed(42)
  x1 <- rnorm(N)
  x2 <- runif(N)
  y_latent <- exp(x1 - x2) / (1 + exp(x1 - x2))
  y <- y_latent
  
  y[c(5, 10)] <- NA
  x1[15] <- NA
  X <- cbind(x1, x2)
  
  # fracreg
  mod1 <- fracreg(y, X, type="1P", linkfrac="logit", table=FALSE)
  expect_equal(nobs(mod1), 97)
  
  # fracreghet
  y_bounded <- pmin(pmax(y, 0.1), 0.9)
  mod2 <- fracreghet(y_bounded, X, type="GMMx", link="logit", table=FALSE)
  expect_equal(nobs(mod2), 97)
  
  # fracregridge
  mod3 <- fracregridge(y, X, fracs=0.5)
  expect_equal(nobs(mod3), 97)
  
  # fracregmlogit
  y_ml <- cbind(y, 1-y)
  mod4 <- fracregmlogit(y_ml, X)
  expect_equal(nobs(mod4), 97)
  
  # fracregpd
  id <- rep(1:20, each=5)
  time <- rep(1:5, 20)
  mod5 <- fracregpd(id, time, y, X, type="GMMpfe", link="logit", table=FALSE)
  # For panel data, length is also 97 but nobs might not be fully reliable for pd, let's check nrow(mod5$xbase)
  expect_equal(nobs(mod5), 97)
})

#' @title Fractional Ridge Regression
#'
#' @description
#' \code{fracregridge} implements Fractional Ridge Regression (Rokem & Kay, 2020), which is an approach to regularized linear regression.
#' Unlike standard ridge regression where the penalty term \eqn{\alpha} is chosen directly, \code{fracregridge} allows you to specify the desired \emph{fraction} of the unregularized OLS coefficient vector length. The algorithm then automatically determines the corresponding \eqn{\alpha} penalties.
#' @param y A numeric vector or matrix of the dependent variable(s).
#' @param x A numeric matrix of the explanatory variables.
#' @param fracs A numeric vector indicating the desired fractions of the unregularized coefficient vector length. Default is \code{seq(0.1, 1.0, by=0.1)}. Values must be sorted in ascending order.
#' @param tol A numeric tolerance under which singular values of the \code{x} matrix are considered to be zero. Default is \code{1e-10}.
#' @param intercept logical. If \code{TRUE}, an intercept is included in the model. Default is \code{TRUE}.
#' @param na.action A function specifying how to handle missing values, default is \code{stats::na.omit}. If \code{NULL}, no action is taken.
#' @param ... further arguments passed to or from other methods.
#'
#' @details
#' Standard ridge regression minimizes the following objective function:
#' \deqn{L = (y - X\beta)'(y - X\beta) + \alpha \beta'\beta}
#' 
#' The penalty \eqn{\alpha} shrinks the coefficients towards zero, reducing the length of the coefficient vector \eqn{||\beta||_2}. However, choosing \eqn{\alpha} can be unintuitive. 
#' \code{fracregridge} re-parameterizes the problem so the user specifies \code{fracs}, the fraction of the unregularized Ordinary Least Squares (OLS) vector length \eqn{\gamma = \frac{||\beta(\alpha)||_2}{||\beta(0)||_2}}.
#' The function automatically determines the \eqn{\alpha} values corresponding to these fractions.
#'
#' @return
#' An object of class \code{fracregridge} containing:
#' \item{coef}{The estimated ridge coefficients for each requested fraction.}
#' \item{alphas}{The corresponding \eqn{\alpha} penalty values.}
#' \item{fracs}{The grid of fractions.}
#' \item{call}{The matched call.}
#'
#' @references
#' Rokem, A., & Kay, K. (2020). Fractional ridge regression: a fast, interpretable reparameterization of ridge regression. \emph{GigaScience}, 9(12).
#' 
#' Rokem, A., and Kay, K., fracridge: Fractional Ridge Regression. Package repository. <https://github.com/nrdg/fracridge>.
#'
#' @author Sulman Olieko Owili <oliekosulman@gmail.com>
#'
#' @seealso
#' \code{\link{fracreg}}
#'
#' @examples
#' # Empirical 401(k) Example
#' data("fracreg_k401k")
#' y_401k <- fracreg_k401k$prate
#' X_401k <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age,
#'                 totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole)
#' 
#' # Fit fractional ridge regression for the 401(k) participation rates
#' mod_401k <- fracregridge(y = y_401k, x = X_401k, fracs = seq(0.2, 1.0, by = 0.2))
#' 
#' # View full detailed summary
#' summary(mod_401k)
#' 
#' # Compute Average Partial Effects for Ridge
#' pe_401k <- fracregridge.pe(mod_401k)
#' summary(pe_401k)
#' 
#' # Simulated Data Example
#' set.seed(123)
#' n <- 100
#' p <- 10
#' y <- rnorm(n)
#' X <- matrix(rnorm(n * p), n, p)
#' colnames(X) <- paste0("X", 1:p)
#' 
#' # Fit Fractional Ridge Regression
#' # We want the coefficients that correspond to 30\\%, 50\\%, and 80\\% of the OLS length
#' mod_sim <- fracregridge(y, X, fracs = c(0.3, 0.5, 0.8))
#' 
#' # View brief summary
#' print(mod_sim)
#' 
#' # Compute Partial Effects
#' pe_sim <- fracregridge.pe(mod_sim)
#' summary(pe_sim)
#' @export
#'
#' @examples
#' set.seed(123)
#' y <- rnorm(100)
#' x <- matrix(rnorm(1000), 100, 10)
#' colnames(x) <- paste0("x", 1:10)
#' 
#' # Fit fractional ridge regression
#' mod <- fracregridge(y, x, fracs = c(0.3, 0.5, 0.8))
#' print(mod)
#' summary(mod)
fracregridge <- function(y, x, fracs = seq(0.1, 1.0, by=0.1), tol = 1e-10, intercept = TRUE, na.action = stats::na.omit, ...)
{
    if (!missing(y) && !missing(x)) {
        cleaned <- fracreg_clean_data(y=y, x=x, na.action=na.action)
        y <- cleaned$y
        x <- cleaned$x
    }
	cl <- match.call()

	if(missing(y)) stop("dependent variable is missing")
	if(missing(x)) stop("explanatory variables are missing")

	if(is.data.frame(x)) x <- as.matrix(x)
	if(!is.matrix(x)) stop("x is not a matrix")

	x.names <- dimnames(x)[[2]]
	if(is.null(x.names)) stop("x has no column names")

	if(intercept==TRUE)
	{
		x <- cbind(1,x)
		x.names <- c("(Intercept)",x.names)
	}

	if(length(x.names)!=length(unique(x.names))) stop("some covariate names in x are identical")
	if(NROW(y)!=nrow(x)) stop("the number of observations for y and x are different")
	
	if(any(diff(fracs) < 0)) stop("The 'fracs' input must be sorted in ascending order.")
	
	nn <- nrow(x)
	pp <- ncol(x)
	
	y_mat <- as.matrix(y)
	bb <- ncol(y_mat)
	ff <- length(fracs)
	
	# SVD Rotation
	if (nn > pp) {
		s <- svd(crossprod(x))
		uu <- s$u
		ss <- s$d
		v_t <- t(s$v)
		selt <- sqrt(pmax(ss, 0))
		
		if (bb >= nn) {
			ynew <- (diag(1/selt) %*% v_t %*% t(x)) %*% y_mat
		} else {
			ynew <- diag(1/selt) %*% v_t %*% (t(x) %*% y_mat)
		}
	} else {
		s <- svd(x)
		uu <- s$u
		selt <- s$d
		v_t <- t(s$v)
		ynew <- t(uu) %*% y_mat
	}
	
	ols_coef <- t(t(ynew) / selt)
	
	isbad <- selt < tol
	if (any(isbad)) {
		warning("Some eigenvalues are being treated as 0")
		ols_coef[isbad, ] <- 0
	}
	
	BIG_BIAS <- 10e3
	SMALL_BIAS <- 10e-3
	BIAS_STEP <- 0.2
	
	val1 <- BIG_BIAS * (selt[1]^2)
	val2 <- SMALL_BIAS * (selt[length(selt)]^2)
	
	alphagrid <- c(0, 10^seq(floor(log10(val2)), ceiling(log10(val1)), by = BIAS_STEP))
	
	seltsq <- selt^2
	
	sclg <- outer(seltsq, alphagrid, function(s, a) s / (s + a))
	sclg_sq <- sclg^2
	
	first_dim <- min(nn, pp)
	
	coef_res <- array(NA, dim = c(first_dim, ff, bb))
	alphas_res <- matrix(NA, nrow = ff, ncol = bb)
	
	for (ii in seq_len(bb)) {
		newlen <- sqrt(as.numeric(t(sclg_sq) %*% (ols_coef[, ii]^2)))
		newlen <- newlen / newlen[1]
		
		# interpolate (log transformed space)
		temp <- stats::approx(x = rev(newlen), y = rev(log(1 + alphagrid)), xout = fracs, rule = 2)$y
		
		targetalphas <- exp(temp) - 1
		alphas_res[, ii] <- targetalphas
		
		sc <- outer(seltsq, targetalphas, function(s, a) s / (s + a))
		
		c_mat <- sc * ols_coef[, ii]
		coef_res[,, ii] <- c_mat
	}
	
	coef_flat <- matrix(coef_res, nrow = first_dim, ncol = ff * bb)
	coef_unrot <- t(v_t) %*% coef_flat
	
	coef_final <- array(coef_unrot, dim = c(pp, ff, bb))
	
	if (bb == 1) {
		coef_final <- matrix(coef_final[,,1], nrow = pp, ncol = ff)
		colnames(coef_final) <- paste0("frac_", fracs)
		rownames(coef_final) <- x.names
		alphas_res <- as.numeric(alphas_res[, 1])
		names(alphas_res) <- paste0("frac_", fracs)
	} else {
		dimnames(coef_final) <- list(x.names, paste0("frac_", fracs), colnames(y_mat))
	}
	
	yhat <- array(NA, dim = c(nn, ff, bb))
	xbhat <- array(NA, dim = c(nn, ff, bb))
	se <- array(NA, dim = c(pp, ff, bb))
	table.info <- list()
	stats.info <- list()
	
	V2 <- t(v_t)^2
	warned_np <- FALSE
	
	for (ii in seq_len(bb)) {
	    for (jj in seq_len(ff)) {
	        if (bb == 1) {
	            beta <- coef_final[, jj]
	        } else {
	            beta <- coef_final[, jj, ii]
	        }
	    
	        y_pred <- x %*% beta
	        yhat[, jj, ii] <- y_pred
	        xbhat[, jj, ii] <- y_pred
	        
	        if (bb == 1) {
	            alpha <- alphas_res[jj]
	        } else {
	            alpha <- alphas_res[jj, ii]
	        }
	        
	        df_alpha <- sum(seltsq / (seltsq + alpha))
	        
	        if (nn > df_alpha) {
	            sigma2 <- sum((y_mat[, ii] - y_pred)^2) / (nn - df_alpha)
	            D <- seltsq / (seltsq + alpha)^2
	            var_diag <- V2 %*% D
	            std_err <- sqrt(as.numeric(sigma2 * var_diag))
	        } else {
	            if (!warned_np) {
	                warning("Degrees of freedom (n - df(alpha)) is less than or equal to 0. Standard errors cannot be computed and will be NA.")
	                warned_np <- TRUE
	            }
	            std_err <- rep(NA, pp)
	        }
	        
	        se[, jj, ii] <- std_err
	        
	        est <- beta
	        zval <- est / std_err
	        pval <- 2 * pnorm(-abs(zval))
	        
	        tab <- cbind(Coefficient = est, `Std. Error` = std_err, `z value` = zval, `Pr(>|z|)` = pval)
	        rownames(tab) <- x.names
	        
	        if (nn > df_alpha && !is.na(sigma2) && sigma2 > 0) {
	            W <- tryCatch(as.numeric(t(beta) %*% t(v_t) %*% diag(1/D) %*% v_t %*% beta / sigma2), error = function(e) NA)
	            p_W <- if(!is.na(W)) 1 - pchisq(W, df = pp) else NA
	        } else {
	            W <- NA
	            p_W <- NA
	        }
	        
	        list_name <- paste0("target_", colnames(y_mat)[ii], "_frac_", fracs[jj])
	        if (bb == 1) list_name <- paste0("frac_", fracs[jj])
	        
	        table.info[[list_name]] <- tab
	        stats.info[[list_name]] <- list(n_obs = nn, R2 = cor(y_mat[, ii], y_pred)^2, df_alpha = df_alpha, W = W, p_W = p_W, df_W = pp)
	    }
	}
	
	p.var <- se^2
	if (bb == 1) {
	    p.var <- matrix(p.var[,,1], nrow = pp, ncol = ff)
	    colnames(p.var) <- paste0("frac_", fracs)
	    rownames(p.var) <- x.names
	    
	    yhat_out <- matrix(yhat[,,1], nrow = nn, ncol = ff)
	    colnames(yhat_out) <- paste0("frac_", fracs)
	    
	    xbhat_out <- matrix(xbhat[,,1], nrow = nn, ncol = ff)
	    colnames(xbhat_out) <- paste0("frac_", fracs)
	} else {
	    dimnames(p.var) <- list(x.names, paste0("frac_", fracs), colnames(y_mat))
	    dimnames(yhat) <- list(NULL, paste0("frac_", fracs), colnames(y_mat))
	    dimnames(xbhat) <- list(NULL, paste0("frac_", fracs), colnames(y_mat))
	    yhat_out <- yhat
	    xbhat_out <- xbhat
	}
	
	res <- list(call = cl, 
	            y = y_mat,
	            coef = coef_final, 
	            alphas = alphas_res, 
	            fracs = fracs, 
	            x.names = x.names,
	            type = "fracregridge",
	            class = "fracregridge",
	            yhat = yhat_out,
	            xbhat = xbhat_out,
	            p.var = p.var,
	            table.info = table.info,
	            stats.info = stats.info)
	class(res) <- "fracregridge"
	
	return(res)
}

fracreghet.tests.table <- function(test.which,S,Sp,ver,title1,title2)
{
	res <- cbind(`Statistic` = S[-1], `p-value` = Sp[-1])
	rownames(res) <- ver[-1]

	cat("\n")
	cat(.fracreg.sep(), "\n")
	cat(.fracreg.center(paste(test.which, "test")), "\n")
	cat(.fracreg.sep(), "\n")
	cat(.fracreg.center(title1), "\n")
	cat(.fracreg.sep(), "\n")
	
	cat("H0:", title2, "\n")
	cat(.fracreg.sep(), "\n")
	printCoefmat(res, P.values = TRUE, has.Pvalue = TRUE, digits = 4, signif.legend = TRUE)
	
	cat(.fracreg.sep(), "\n")
	cat(.fracreg.center(paste("Run Date:", format(Sys.time(), "%Y-%m-%d %H:%M:%S"))), "\n")
	cat(.fracreg.sep(), "\n")
}

fracreghet.table <- function(p,p.var,x.names,type,link,converged,N,var.type,adjust,k,J,dfJ,LL=NULL,or=FALSE,level=0.95)
{
	if(converged==TRUE)
	{
		stars <- rep("",length(p))

		if(!is.character(p.var))
		{
			p.sd <- diag(p.var)^0.5
			z.ratio <- p/p.sd
			p.value <- 2*pnorm(abs(z.ratio), lower.tail=FALSE)
			qval <- qnorm((1 + level) / 2)
			ci.low <- p - qval * p.sd
			ci.high <- p + qval * p.sd

			if(or && link == "logit") {
				p <- exp(p)
				p.sd <- p.sd * p
				ci.low <- exp(ci.low)
				ci.high <- exp(ci.high)
			}
			
			conf_str <- paste0("[", round(level * 100), "% Conf.")
			res <- cbind(Coefficient = p, `Std.Err.` = p.sd, `z value` = z.ratio, `ci_low` = ci.low, `ci_high` = ci.high, `Pr(>|z|)` = p.value)
			colnames(res)[4] <- conf_str
			colnames(res)[5] <- "Interval]"
			
			if(or && link == "logit") {
				colnames(res)[1] <- "Odds Ratio"
			}
			
			se_label <- var.type
			if(se_label == "standard") se_label <- "EIM"
			colnames(res)[2] <- paste0(tools::toTitleCase(se_label), " Std.Err.")
		}
		else
		{
			conf_str <- paste0("[", round(level * 100), "% Conf.")
			res <- cbind(Coefficient = p, `Std.Err.` = rep(NA,length(p)), `z value` = rep(NA,length(p)), `ci_low` = rep(NA,length(p)), `ci_high` = rep(NA,length(p)), `Pr(>|z|)` = rep(NA,length(p)))
			colnames(res)[4] <- conf_str
			colnames(res)[5] <- "Interval]"

			if(or && link == "logit") {
				res[,"Coefficient"] <- exp(p)
				colnames(res)[1] <- "Odds Ratio"
			}
			se_label <- var.type
			if(se_label == "standard") se_label <- "EIM"
			colnames(res)[2] <- paste0(tools::toTitleCase(se_label), " Std.Err.")
		}
		rownames(res) <- x.names
		rownames(res)[rownames(res) == "(Intercept)"] <- "(Intercept)"

		Wald <- NA
		p_wald <- NA
		df_wald <- NA
		if(!is.character(p.var)) {
			if(length(p) > 1 && x.names[1] == "(Intercept)") {
				p_idx <- 2:length(p)
				W <- tryCatch(t(p[p_idx]) %*% solve(p.var[p_idx, p_idx]) %*% p[p_idx], error = function(e) NA)
				if (!is.na(W)) {
					Wald <- as.numeric(W)
					df_wald <- length(p_idx)
					p_wald <- 1 - pchisq(Wald, df = df_wald)
				}
			} else if(length(p) > 0 && x.names[1] != "(Intercept)") {
				W <- tryCatch(t(p) %*% solve(p.var) %*% p, error = function(e) NA)
				if (!is.na(W)) {
					Wald <- as.numeric(W)
					df_wald <- length(p)
					p_wald <- 1 - pchisq(Wald, df = df_wald)
				}
			}
		}

		cat("\n")
		cat(.fracreg.sep(), "\n")
		cat(.fracreg.center(paste("Fractional", link, "regression with heteroscedasticity/endogeneity")), "\n")
		cat(.fracreg.sep(), "\n")
		
		.fracreg.cat.right("Data type:", "Cross-sectional")
		.fracreg.cat.right("Estimator:", type)
		.fracreg.cat.right("Convergence:", "Successful")
		.fracreg.cat.right("Number of observations:", N)
		if(!is.null(LL)) .fracreg.cat.right("Log pseudolikelihood:", round(LL, 4))
		
		if(any(type==c("GMMz","LINz")) & dfJ>0)
		{
			p.value <- 1-pchisq(J,dfJ)
			.fracreg.cat.right("J test (p-value):", paste0(round(J, 4), " (", round(p.value, 4), ")"))
		}
		if(!is.na(Wald)) {
			.fracreg.cat.right(paste0("Wald chi2(", df_wald, "):"), round(Wald, 4))
			.fracreg.cat.right("Prob > chi2:", sprintf("%.4f", p_wald))
		}
		
		if(adjust!=0)
		{
			if(is.numeric(adjust)) .fracreg.cat.right("Adjustment:", paste(adjust,"added to all observations"))
			else .fracreg.cat.right("Adjustment:", "all boundary observations dropped")
		}
		
		if(var.type == "robust") {
			.fracreg.cat.right("Standard errors:", "HC0")
		} else if(var.type == "cluster") {
			.fracreg.cat.right("Standard errors:", "CRVE")
		} else {
			.fracreg.cat.right("Standard errors:", var.type)
		}
		
		cat(.fracreg.sep(), "\n")
		type_full <- type
		type_full <- sub("^QML", "Quasi-Maximum Likelihood ", type_full)
		type_full <- sub("^ML", "Maximum Likelihood ", type_full)
		type_full <- trimws(type_full)
		cat(.fracreg.center(paste("Final", type_full, "estimates")), "\n")
		cat(.fracreg.sep(), "\n")
		
		if(all(type!=c("GMMxv","LINxv","QMLxv"))) printCoefmat(res, P.values = TRUE, has.Pvalue = TRUE, digits = 4, signif.legend = TRUE, na.print = ".")
		if(any(type==c("GMMxv","LINxv","QMLxv")))
		{
			printCoefmat(res[1:k, , drop = FALSE], P.values = TRUE, has.Pvalue = TRUE, digits = 4, signif.legend = TRUE, na.print = ".")
			cat("\n")
			cat(.fracreg.center("Reduced form:"), "\n")
			cat(.fracreg.sep(), "\n")
			printCoefmat(res[-(1:k), , drop = FALSE], P.values = TRUE, has.Pvalue = TRUE, digits = 4, signif.legend = TRUE, na.print = ".")
		}
		cat(.fracreg.sep(), "\n")
		cat(.fracreg.center(paste("Run Date:", format(Sys.time(), "%Y-%m-%d %H:%M:%S"))), "\n")
		cat(.fracreg.sep(), "\n")
	}
	else {
		cat(.fracreg.sep(), "\n")
		.fracreg.cat.right("Convergence:", "FAILED")
		cat(.fracreg.sep(), "\n")
		cat(.fracreg.center(paste("Run Date:", format(Sys.time(), "%Y-%m-%d %H:%M:%S"))), "\n")
		cat(.fracreg.sep(), "\n")
	}
	cat("\n")
}

#' @title RESET Test for Fractional Response Regressions
#'
#' @description
#' \code{fracreg.reset} is used to perform the Regression Equation Specification Error Test (RESET) to check the functional form and specification of fractional response models.
#' @param object an object containing the results of an \code{fracreg} command.
#' @param lastpower.vec a numeric vector containing the maximum powers of the linear predictors to be used in RESET tests.
#' @param version a vector containing the test versions to use. Available options: \code{Wald}, \code{LM} (the default) and, only for the binary component of two-part models, \code{LR}. More than one option may be chosen.
#' @param table a logical value indicating whether a summary table with the test results should be printed.
#' @param \dots Arguments to pass to \link[stats]{glm}, which is used to estimate the model under the alternative hypothesis when \code{version} is a vector containing \code{"Wald"} or \code{"LR"}.
#'
#' @details
#' \code{fracreg.reset} applies the RESET test statistic to fractional response
#' models estimated via \code{fracreg}. \code{fracreg.reset} may be used to test the link specification of: (i) one-part fractional response models; (ii) the binary
#' components of two-part and three-part fractional response models; and (iii) the fractional components of two-part and three-part fractional response models. 
#' 
#' \strong{RESET Test Framework:}
#' The Regression Equation Specification Error Test (RESET) assesses whether the link function \eqn{G(\cdot)} and the linear index \eqn{x\beta} are correctly specified. It tests the null hypothesis \eqn{H_0: \gamma = 0} in the augmented model:
#' \deqn{E(y|x) = G(x\beta + \sum_{k=2}^P \gamma_k (x\hat{\beta})^k)}
#' where \eqn{P} is the maximum power of the linear predictor (specified by \code{lastpower.vec}) and \eqn{\hat{\beta}} are the estimated parameters from the baseline model.
#' 
#' When the \code{Wald} version is implemented, it is taken into account the option that was chosen for computing standard errors in the model under evaluation. For the \code{LM} version, a robust version is computed in cases (i) and (iii) and a conventional version in case (ii). See Ramalho, Ramalho and Murteira (2011) for details on the application of the RESET test in the fractional response framework.
#'
#' @return
#' \code{fracreg.reset} returns a named vector with the test results.
#'
#' @references
#' Ramalho, E.A., J.J.S. Ramalho and J.M.R. Murteira (2011), "Alternative
#' estimating and testing empirical strategies for fractional response models",
#' \emph{Journal of Economic Surveys}, 25(1), 19-68.
#' 
#' Ramsey, J.B. (1969), "Tests for Specification Errors in Classical Linear Least-Squares Regression Analysis", \emph{Journal of the Royal Statistical Society: Series B (Methodological)}, 31(2), 350-371.
#'
#' @author Sulman Olieko Owili <oliekosulman@gmail.com>
#'
#' @seealso
#' \code{\link{fracreg}}, for fitting fractional response models.\cr
#' \code{\link{fracreg.ggoff}}, for asymptotically equivalent specification tests.\cr
#' \code{\link{fracreg.ptest}}, for non-nested hypothesis tests.\cr
#' \code{\link{fracreg.pe}}, for computing partial effects.
#'
#' @examples
#' ### Empirical 401(k) Examples
#' data("fracreg_k401k")
#' y <- fracreg_k401k$prate
#' X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age, 
#'            totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole)
#' 
#' m <- fracreg(y, X, type="1P", linkfrac="logit")
#' reset_res <- fracreg.reset(m)
#' summary(reset_res)
#' 
#' ### Simulated Examples
#' 
#' N <- 250
#' u <- rnorm(N)
#' 
#' X <- cbind(rnorm(N),rnorm(N))
#' dimnames(X)[[2]] <- c("X1","X2")
#' 
#' ym <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u))
#' y <- rbeta(N,ym*20,20*(1-ym))
#' y[y > 0.9] <- 1
#' 
#' #Testing the logit specification of a standard fractional response model
#' #using LM and Wald versions of the RESET test, based on 1 or 2 fitted powers of
#' #the linear predictor
#' mod <- fracreg(y,X,linkfrac="logit")
#' reset_res <- fracreg.reset(mod,2:3,c("Wald","LM"))
#' summary(reset_res)
#' 
#' #Testing the probit specification of the binary component of a two-part fractional
#' #regression model using LR-based RESET tests with quadratic and cubic fitted 
#' #powers of the linear predictor
#' mod <- fracreg(y,X,linkbin="probit",type="2Pbin",inf=1)
#' reset_res <- fracreg.reset(mod,3,"LR")
#' summary(reset_res)
#' @export
fracreg.reset <- function(object,lastpower.vec=3,version="LM",table=FALSE,...)
{
	### 1. Error and warning messages

	if(missing(object)) stop("object is missing")
	if(is.null(object$class)) stop("object is not the output of an fracreg command")
	if(object$class!="fracreg") stop("object is not the output of an fracreg command")
	if(object$type=="2P" | object$type=="3P") stop("The RESET test is not applicable to two-part or three-part models")
	if(object$converged==0) stop("object is not the output of a successful (converged) fracreg command")
	if(object$method!="ML" & any(version=="LR")) stop("LR tests require ML estimation")
	if(any(lastpower.vec<2)) stop(sQuote(lastpower.vec)," - lastpower.vec contains elements lower than 2")
	if(all(version!="LM") & all(version!="Wald") & all(version!="LR")) stop("test version not correctly specified")
	if(!is.logical(table)) stop("non-logical value assigned to option table")

	### 2. Recovering definitions and estimates

	mf <- model.frame(object$formula)
	y <- model.response(mf)
	x <- model.matrix(object$formula)

	yhat <- object$yhat
	xbhat <- object$xbhat
	method <- object$method
	link <- object$link
	type <- object$type

	if(any(version=="Wald"))
	{
		if(is.null(object$var.type)) stop("Wald test required but fracreg command was run with variance = FALSE")
		var.type <- object$var.type
		var.eim <- object$var.eim
		dfc <- object$dfc
		if(var.type=="cluster") var.cluster <- object$var.cluster
	}
	if(any(version=="LR")) LL0 <- object$LL
	if(type=="1P") title <- paste("Fractional",link,"regression")
	if(type=="2Pbin") title <- paste("Binary",link,"component of a two-part regression")
	if(type=="2Pfrac") title <- paste("Fractional",link,"component of a two-part regression")

	### 3. Tests

	lastpower.vec <- round(lastpower.vec,0)

	g <- fracreg.links(link)$mu.eta(xbhat)
	gx <- g*x

	z.all <- as.matrix(xbhat^2)
	if(max(lastpower.vec)>2) for(i in 3:max(lastpower.vec)) z.all <- cbind(z.all,xbhat^i)

	ver <- NA
	S <- NA
	Sp <- NA

	for(j in lastpower.vec)
	{
		df <- j-1
		z <- z.all[,1:(j-1)]
		gz <- g*z

		if(any(version=="LM"))
		{
			name <- paste("LM(",j,")",sep="")
			ver <- c(ver,name)

			results <- fracreg.lm(y,yhat,gx,gz,type)

			Sj <- results$LM
			S <- c(S,Sj)
			Sp <- c(Sp,1-pchisq(Sj,df))
		}
		if(any(version=="LR") | any(version=="Wald"))
		{
			if(any(version=="Wald")) results <- fracreg.est(y,cbind(x,z),link,method,TRUE,var.type,var.eim,var.cluster,dfc,...)
			else results <- fracreg.est(y,cbind(x,z),link,method,FALSE,...)

			if(any(version=="LR"))
			{
				name <- paste("LR(",j,")",sep="")
				ver <- c(ver,name)

				if(results$converged==TRUE)
				{
					LL1 <- results$LL

					Sj <- 2*(LL1-LL0)
					S <- c(S,Sj)
					Sp <- c(Sp,1-pchisq(Sj,df))
				}
				else
				{
					S <- c(S,NA)
					Sp <- c(Sp,NA)
				}
			}
			if(any(version=="Wald"))
			{
				name <- paste("Wald(",j,")",sep="")
				ver <- c(ver,name)

				if(results$converged==TRUE)
				{
					p1 <- results$p
					p.var1 <- results$p.var

					p.n <- length(p1)
					p1 <- p1[(p.n-df+1):p.n]
					p.var1 <- p.var1[(p.n-df+1):p.n,(p.n-df+1):p.n]

					Sj <- t(p1)%*%solve(p.var1)%*%p1
					S <- c(S,Sj)
					Sp <- c(Sp,1-pchisq(Sj,df))
				}
				else
				{
					S <- c(S,NA)
					Sp <- c(Sp,NA)
				}
			}
		}
	}

	table.info <- list(test.which="RESET",S=S,Sp=Sp,ver=ver,title1=title)
	if(table==TRUE) do.call(fracreg.tests.table, table.info)

	### 4. Return results

	statistics <- S[-1]
	names(statistics) <- ver[-1]
	
	class(statistics) <- c("fracreg.reset", "numeric")
	attr(statistics, "table.info") <- table.info

	return(invisible(statistics))
}

library(testthat)
library(fracreg)

# Prepare data for testing
data("fracreg_k401k")
y <- fracreg_k401k$prate
X <- cbind(
  mrate = fracreg_k401k$mrate, 
  age = fracreg_k401k$age, 
  totemp = fracreg_k401k$totemp, 
  sole = fracreg_k401k$sole
)

test_that("fracreg.reset works", {
  model_1p <- fracreg(y = y, x = X, type = "1P", linkfrac = "logit")
  reset_res <- fracreg.reset(model_1p, table = FALSE)
  
  expect_type(reset_res, "double")
  expect_true(length(reset_res) > 0)
})

test_that("fracreg.ggoff works", {
  model_1p <- fracreg(y = y, x = X, type = "1P", linkfrac = "logit")
  ggoff_res <- fracreg.ggoff(model_1p, version = "LM", table = FALSE)
  
  expect_type(ggoff_res, "double")
  expect_true(length(ggoff_res) > 0)
})

test_that("fracreg.ptest works", {
  model_1p_logit <- fracreg(y = y, x = X, type = "1P", linkfrac = "logit")
  model_1p_clog <- fracreg(y = y, x = X, type = "1P", linkfrac = "cloglog")
  
  ptest_res <- fracreg.ptest(model_1p_logit, model_1p_clog, table = FALSE)
  
  expect_type(ptest_res, "double")
  expect_true(length(ptest_res) > 0)
})

#' Fractional Multinomial Logit Average Partial Effects
#' 
#' Calculate average partial effects (APE) of independent variables from a fractional multinomial logit model. 
#' 
#' @param object A "fracregmlogit" object.
#' @param effect Can be "marginal", for marginal effects; or "discrete", for discrete changes from
#' the min to the max. 
#' @param marg.type Type of marginal or discrete effects to be computed. Default to "atmean", the effect at 
#' the mean of all covariates. Also takes "aveacr", the averaged effects across all observations. See details. 
#' @param se Whether to calculate standard errors for those margins. See details. 
#' @param varlist A string vector which provides the names of variables to calculate 
#' the marginal effect for. If missing, all variables except the constant will be calculated. 
#' Use "constant" if you wish to compute the marginal effect of the constant. 
#' @param at Specify values of the X-matrix at which the partial effect will be retrieved. Expects a vector input
#' of length K-1. Only supported for \code{marg.type="atmean"}. See \code{predict.fracregmlogit(newdata)}. 
#' @param R Number of times to sample for the Krinsky-Robb standard error. Default to 1000. 
#' @details This module calculates the average partial effects (APEs) from a fractional multinomial logit model.
#' Partial effects are the counterpart of the marginal effects in a linear model setting. In linear models, 
#' usually the parameter estimate itself represents the marginal effect (if the variable in question is continuous). 
#' In logit models, however, the parameter estimate at hand is the effect on the log-ratio between the choice variable
#' and the baseline variable. This function is intended to extract APEs from the 
#' coefficient estimates computed from the fractional multinomial logit models.
#' 
#' This function allows for two types of partial effects: marginal effects, and discrete effects.
#' A marginal effect represents how a unit change in one continuous variable x may influence the choice variable y. 
#' The estimation of marginal effects is very straightforward. However, special care is needed when averaging 
#' the marginal effect across observations to acquire the APE. One approach is to use the estimate of the marginal effect while setting
#' other explanatory variables at the mean. We call this the marginal effect at the mean (MEM), which corresponds
#' to the option \code{marg.type="atmean"}. Another approach is to take the average of marginal effects for each
#' individual. We call this the average marginal effect (AME), which corresponds to the option \code{marg.type="aveacr"}. 
#' 
#' The discrete effect represents how a discrete change in one specific x, discrete or continuous, influences the choice variable y. 
#' This is more useful for categorical variables, as calculating the "marginal effect" makes little sense
#' for them. In this function, we calculate the discrete effect by changing the explanatory variable from 
#' its minimum to its maximum. For a binary variable, this is just the difference between 0 and 1. Similar 
#' to the marginal effect case, we also have the discrete effect at the mean (DEM), corresponding to \code{marg.type="atmean"}
#' and the average discrete effect (ADE), corresponding to \code{marg.type="aveacr"}.
#' 
#' Standard errors are provided for the effects by using the Krinsky-Robb (KR) method. Krinsky-Robb is a simulation-based
#' method that calculates the empirical value of a function given a known distribution of its variables. Here 
#' we provide Krinsky-Robb standard errors for MEM and DEM, and the user can specify how many times of 
#' simulation \code{R} the Krinsky-Robb algorithm should run. 
#' 
#' The user can also specify a subset of explanatory variables when calculating effects. This is done through
#' specifying string vectors containing the column names of the explanatory variables to \code{varlist}. As the
#' KR standard error can be computationally intensive, it is advised to calculate it only for the variables of interest. 
#' 
#' @return The function returns an object of class "fracregmlogit.pe". It contains the following components:
#' @return \code{effects} A matrix of calculated effects.
#' @return \code{se} A matrix of standard errors corresponding to the effects. Shows up if se=TRUE for the 
#' input parameter.
#' @return \code{ztable} A list of matrices containing effects, standard errors, z-stats and p-values.
#' @return \code{R} Number of simulation times for Krinsky-Robb standard error calculation. Null if se=FALSE.  
#' @return \code{expl} String message explaining the effects calculated.    
#' 
#' @seealso \code{\link{fracregmlogit}} for the model estimation, \code{\link{plot.fracregmlogit.pe}} for plotting effects.
#' 
#' @examples
#' data("fracreg_spending")
#' X = fracreg_spending[,2:5]
#' y = fracreg_spending[,6:11]
#' results1 = fracregmlogit(y, X)
#' 
#' # Calculate marginal effects at the mean (without standard errors for speed)
#' pe_marg = fracregmlogit.pe(results1, effect="marginal", se=FALSE)
#' 
#' # Calculate discrete effects for specific variables with standard errors
#' pe_disc = fracregmlogit.pe(results1, effect="discrete", 
#'                            varlist = colnames(results1$X)[c(1,3)], 
#'                            se=TRUE, R=50)
#' summary(pe_disc)
#' @export fracregmlogit.pe
fracregmlogit.pe<-function(object,effect=c("marginal","discrete"),
                          marg.type="atmean",se=FALSE,varlist = NULL,at=NULL,R=1000){
  effect = match.arg(effect)
  j=length(object$estimates)+1; K=dim(object$estimates[[1]])[1]; N=dim(object$y)[1]
  betamat = object$coefficient
  R = R # for Krinsky-Robb sampling
  # determine variables
  Xnames = colnames(object$X); ynames = colnames(object$y)
  if(length(varlist)==0){
    varlist=Xnames[-K]
    var_colNo = c(1:(K-1))
    k = length(var_colNo)
  }else{
    var_colNo = unlist(lapply(varlist, function(x) {which(Xnames == x)}))
    if(length(varlist) != length(var_colNo)) stop("Unrecognized varlist input. Please double check your spelling")
    k = length(var_colNo)
  }
  
  xmarg = matrix(ncol=k,nrow=j)
  se_mat = matrix(ncol=k,nrow=j)
  marg_list = list()
  
  # Pre-draw Multivariate Normal vectors for Krinsky-Robb sampling if required
  if (se == TRUE) {
    # Initialize a 3D array: R simulations x J choices x (K+1) parameters
    betamat_R = array(0, dim=c(R, j, K))
    for (i in 2:j) {
      # Use MASS::mvrnorm to capture covariances between variables within each equation
      betamat_R[, i, ] = MASS::mvrnorm(R, betamat[i, ], object$vcov[[i]])
    }
    # Note: betamat_R[, 1, ] remains perfectly 0 as required for the baseline choice.
  }

  if(effect == "marginal"){
    # calculate marginal effects
    yhat = predict(object); yhat = as.matrix(yhat)
    for(c in var_colNo){
      c1 = which(var_colNo == c)
      if(marg.type == "aveacr"){
        # this is the average marginal effect for all observations
        beta_bar = as.vector(yhat %*% betamat[,c])
        betak_long = matrix(rep(betamat[,c],N),nrow=N,byrow=TRUE)
        marg_mat =  yhat * (betak_long-beta_bar)
        xmarg[,c1] = colMeans(marg_mat)
        marg_list[[c1]] = marg_mat
      }
      if(marg.type == "atmean"){
        # this is the marginal effect at the mean
        if(is.null(at)) at = colMeans(object$X[,-K])
        yhat_mean = predict(object,newdata=at)
        beta_bar = sum(yhat_mean * betamat[,c])
        betak = betamat[,c]
        marg_vec = yhat_mean * (betak - beta_bar)
        xmarg[,c1] = as.numeric(marg_vec) 
      }
    }
    
    if (se == TRUE) {
      # Vectorized Multivariate Krinsky-Robb standard error calculation for marginal effects
      x_mean = c(colMeans(object$X[,-K]), 1)
      
      Xbeta_R = matrix(0, nrow=R, ncol=j)
      for (i in 2:j) {
        Xbeta_R[, i] = betamat_R[, i, ] %*% x_mean
      }
      exp_Xbeta_R = exp(Xbeta_R)
      yhat_R = exp_Xbeta_R / rowSums(exp_Xbeta_R) # R x j
      
      for (c in var_colNo) {
        c1 = which(var_colNo == c)
        
        # Calculate the beta_bar (weighted average parameter) for each of the R draws
        beta_bar_R = rowSums(yhat_R * betamat_R[, , c])
        
        # Calculate the marginal effect spread across the R draws for each choice j
        for (i in 1:j) {
          marg_matrix_i = yhat_R[, i] * (betamat_R[, i, c] - beta_bar_R)
          se_mat[i, c1] = sd(marg_matrix_i)
        }
      }
    }
  }
  
  if(effect=="discrete"){
    for(c in var_colNo){
      c1 = which(var_colNo == c)
      if(marg.type == "aveacr"){
        Xmin <- Xmax <- object$X[,-K]
        Xmin[,c] = min(object$X[,c])
        Xmax[,c] = max(object$X[,c])
        yhat_min = predict(object,newdata=Xmin)
        yhat_max = predict(object,newdata=Xmax)
        ydisc = yhat_max - yhat_min
        xmarg[,c1] = colMeans(ydisc)
        marg_list[[c1]] = ydisc
      }
      if(marg.type == "atmean"){
        if(is.null(at)) at = colMeans(object$X[,-K])
        Xmin <- Xmax <- at
        Xmin[c] = min(object$X[,c])
        Xmax[c] = max(object$X[,c])
        yhat_min = predict(object,newdata=Xmin)
        yhat_max = predict(object,newdata=Xmax)
        ydisc = yhat_max - yhat_min
        xmarg[,c1] = as.numeric(ydisc)
      }
    }
    
    if (se == TRUE) {
      # Vectorized Multivariate Krinsky-Robb standard error calculation for discrete effects
      for (c in var_colNo) {
        c1 = which(var_colNo == c)
        
        Xmin <- Xmax <- c(colMeans(object$X[,-K]), 1)
        Xmin[c] = min(object$X[,c])
        Xmax[c] = max(object$X[,c])
        
        Xbeta_min_R = matrix(0, nrow=R, ncol=j)
        Xbeta_max_R = matrix(0, nrow=R, ncol=j)
        for (i in 2:j) {
          Xbeta_min_R[, i] = betamat_R[, i, ] %*% Xmin
          Xbeta_max_R[, i] = betamat_R[, i, ] %*% Xmax
        }
        
        yhat_min_R = exp(Xbeta_min_R) / rowSums(exp(Xbeta_min_R))
        yhat_max_R = exp(Xbeta_max_R) / rowSums(exp(Xbeta_max_R))
        
        ydisc_R = yhat_max_R - yhat_min_R # R x j
        for (i in 1:j) {
          se_mat[i, c1] = sd(ydisc_R[, i])
        }
      }
    }
  }
  
  # generating hypothesis testing tables.
  listmat = list()
  if(se){
    for(i in 1:k){
      tabout = matrix(ncol=4,nrow=j)
      tabout[,1:2] = cbind(xmarg[,i],se_mat[,i])
      tabout[,3] = tabout[,1] / tabout[,2]
      tabout[,4] = 2*(1-pnorm(abs(tabout[,3])))
      colnames(tabout) = c("estimate","std","z","p-value")
      rownames(tabout) = ynames
      listmat[[i]] = tabout
    }
    names(listmat)=varlist
  }
  
  colnames(xmarg) <- colnames(se_mat) <- varlist
  rownames(xmarg) <- rownames(se_mat) <-colnames(object$y)
  outlist=list()
  outlist$effects = xmarg
  if(se==TRUE){outlist$se = se_mat; outlist$ztable = listmat}
  if(marg.type=="aveacr") {names(marg_list)=varlist; outlist$marg.list = marg_list}
  marg.type.out = ifelse(marg.type=="atmean","at the mean,","average across observations,")

  # please include this in the file
  outlist$R = ifelse(se,R,0)

  outlist$expl = paste(effect,"effect",marg.type.out,
                       ifelse(se==TRUE,"Krinsky-Robb standard error calculated","standard error not computed"))
  return(structure(outlist,class="fracregmlogit.pe"))
}

#' @title Extract Model Coefficients for fracregpd
#' @description Extracts the estimated coefficients from a fitted \code{fracregpd} model.
#' @param object A fitted model object of class \code{fracregpd}.
#' @param ... Further arguments passed to or from other methods.
#' @return A named vector of coefficients.
#' @exportS3Method coef fracregpd
coef.fracregpd <- function(object, ...) {
    if(!is.null(object$p)) {
        return(object$p)
    } else if (!is.null(object$table.info$p)) {
        return(object$table.info$p)
    }
    return(NULL)
}

#' @title Extract Fitted Values for fracregpd
#' @description Extracts the fitted conditional mean values from a \code{fracregpd} model.
#' @param object A fitted model object of class \code{fracregpd}.
#' @param ... Further arguments passed to or from other methods.
#' @return A numeric vector of fitted values.
#' @exportS3Method fitted fracregpd
fitted.fracregpd <- function(object, ...) {
    link <- object$link
    yhat <- fracregpd.links(link)$linkinv(object$xbhat)
    return(yhat)
}

#' @title Extract Model Residuals for fracregpd
#' @description Extracts the response residuals from a fitted \code{fracregpd} model.
#' @param object A fitted model object of class \code{fracregpd}.
#' @param ... Further arguments passed to or from other methods.
#' @return A numeric vector of residuals.
#' @exportS3Method residuals fracregpd
residuals.fracregpd <- function(object, ...) {
    type <- object$type
    link <- object$link
    Hy <- object$Hy
    
    if (type == "QMLcre") {
        y <- Hy
    } else {
        if (link == "logit") {
            y <- Hy / (1 + Hy)
        } else if (link == "cloglog") {
            y <- 1 - exp(-Hy)
        } else {
            y <- Hy # Fallback
        }
    }
    
    return(y - fitted(object))
}

#' @title Extract the Number of Observations for fracregpd
#' @description Extracts the number of observations used to estimate a \code{fracregpd} model.
#' @param object A fitted model object of class \code{fracregpd}.
#' @param ... Further arguments passed to or from other methods.
#' @return An integer denoting the number of observations.
#' @exportS3Method nobs fracregpd
nobs.fracregpd <- function(object, ...) {
    return(length(object$Hy))
}

#' @title Extract Log-Likelihood for fracregpd
#' @description Extracts the log-pseudolikelihood or log-likelihood from a fitted \code{fracregpd} model.
#' @param object A fitted model object of class \code{fracregpd}.
#' @param ... Further arguments passed to or from other methods.
#' @return An object of class \code{logLik}.
#' @exportS3Method logLik fracregpd
logLik.fracregpd <- function(object, ...) {
    if(is.null(object$table.info$LL)) {
        return(NA)
    }
    val <- object$table.info$LL
    attr(val, "df") <- length(coef(object))
    attr(val, "nobs") <- nobs(object)
    class(val) <- "logLik"
    return(val)
}

#' @title Extract Covariance Matrix for fracregpd
#' @description Extracts the estimated variance-covariance matrix of the parameters.
#' @param object A fitted model object of class \code{fracregpd}.
#' @param ... Further arguments passed to or from other methods.
#' @return A matrix of the estimated covariances.
#' @exportS3Method vcov fracregpd
vcov.fracregpd <- function(object, ...) {
    if(!is.null(object$p.var)) {
        return(object$p.var)
    }
    return(NULL)
}

#' @title Predict Method for fracregpd
#' @description Predicts conditional mean values from a fitted \code{fracregpd} model.
#' @param object A fitted model object of class \code{fracregpd}.
#' @param newdata An optional data frame or matrix in which to look for variables with which to predict. If omitted, the fitted values are used.
#' @param ... Further arguments passed to or from other methods.
#' @return A numeric vector of predicted values.
#' @exportS3Method predict fracregpd
predict.fracregpd <- function(object, newdata = NULL, ...) {
    if (is.null(newdata)) {
        return(fitted(object))
    }
    
    # We must construct the linear predictor
    x.names <- object$table.info$x.names
    
    # Simple check to map data frame to matrix
    if(is.data.frame(newdata)) {
        # Check if intercept is needed
        if("(Intercept)" %in% x.names && !"(Intercept)" %in% colnames(newdata)) {
            newdata <- cbind(1, as.matrix(newdata))
            colnames(newdata)[1] <- "(Intercept)"
        } else {
            newdata <- as.matrix(newdata)
        }
    } else if (is.matrix(newdata)) {
        if("(Intercept)" %in% x.names && !"(Intercept)" %in% colnames(newdata)) {
            newdata <- cbind(1, newdata)
            colnames(newdata)[1] <- "(Intercept)"
        }
    }
    
    # Ensure correct ordering and matching
    if(!all(x.names %in% colnames(newdata))) {
        stop("newdata does not contain all the necessary columns")
    }
    
    X <- newdata[, x.names, drop=FALSE]
    beta <- coef(object)
    
    xb <- as.vector(X %*% beta)
    
    link <- object$link
    pred <- fracregpd.links(link)$linkinv(xb)
    return(pred)
}

#' @title Clean Data for Fractional Regression Models
#' @description An internal helper to gracefully drop missing values across an arbitrary number of vectors and matrices, replicating the functionality of na.action = na.omit for multi-array model inputs.
#' @param ... A variable number of vectors, matrices, or data frames.
#' @param na.action A function specifying how to handle missing values, default is \code{stats::na.omit}. If \code{NULL}, no action is taken.
#' @return A named list containing the subsets of the provided arrays without missing values.
fracreg_clean_data <- function(..., na.action = stats::na.omit) {
  if (is.null(na.action)) return(list(...))
  
  args <- list(...)
  non_null <- !sapply(args, is.null)
  
  if (sum(non_null) > 0) {
    # 1. Determine the target number of observations (N)
    # We look for the first matrix, data.frame, or vector that likely represents the dataset.
    N_target <- 0
    for (i in seq_along(args)) {
        if (!is.null(args[[i]])) {
            if (is.matrix(args[[i]]) || is.data.frame(args[[i]])) {
                N_target <- max(N_target, nrow(args[[i]]))
            } else if (is.vector(args[[i]]) && !is.character(args[[i]])) {
                N_target <- max(N_target, length(args[[i]]))
            }
        }
    }
    
    # If no data structure of length > 1 is found, return as-is
    if (N_target <= 1) return(args)
    
    # 2. Extract ONLY arrays/vectors that match the target N exactly
    valid_idx <- sapply(args, function(x) {
        if (is.null(x)) return(FALSE)
        if (is.matrix(x) || is.data.frame(x)) return(nrow(x) == N_target)
        if (is.vector(x)) return(length(x) == N_target)
        return(FALSE)
    })
    
    if (sum(valid_idx) > 0) {
      valid_args <- args[valid_idx]
      valid_rows <- do.call(stats::complete.cases, valid_args)
      
      if (!all(valid_rows)) {
        for (i in seq_along(args)) {
          if (valid_idx[i]) {
            if (is.matrix(args[[i]]) || is.data.frame(args[[i]])) {
              args[[i]] <- args[[i]][valid_rows, , drop = FALSE]
            } else {
              args[[i]] <- args[[i]][valid_rows]
            }
          }
        }
      }
    }
  }
  return(args)
}

fracreg.links <- function(link) 
{
	switch(link,
		logit = {
			linkfun <- function(mu) qlogis(mu)
			linkinv <- function(eta) plogis(eta)
			mu.eta <- function(eta) dlogis(eta)
			gd <- function(eta) exp(eta)*(1-exp(eta))/((1+exp(eta))^3)
			valideta <- function(eta) TRUE
		},

		probit = {
			linkfun <- function(mu) qnorm(mu)
			linkinv <- function(eta) {
				thresh <- -qnorm(.Machine$double.eps)
				eta <- pmin(pmax(eta, -thresh), thresh)
	 			pnorm(eta)
			}
			mu.eta <- function(eta) pmax(dnorm(eta),.Machine$double.eps)
			gd <- function(eta) -eta*dnorm(eta)
			valideta <- function(eta) TRUE
		},

		cauchit = {
		  	linkfun <- function(mu) qcauchy(mu)
			linkinv <- function(eta) {
				thresh <- -qcauchy(.Machine$double.eps)
				eta <- pmin(pmax(eta, -thresh), thresh)
				pcauchy(eta)
			}
			mu.eta <- function(eta) pmax(dcauchy(eta),.Machine$double.eps)
			gd <- function(eta) -2*eta/(pi*(eta^2+1)^2)
			valideta <- function(eta) TRUE
		},

		cloglog = {
			linkfun <- function(mu) log(-log(1 - mu))
			linkinv <- function(eta) pmax(pmin(-expm1(-exp(eta)),1-.Machine$double.eps),.Machine$double.eps)
			mu.eta <- function(eta) {
				eta <- pmin(eta, 700)
				pmax(exp(eta) * exp(-exp(eta)),.Machine$double.eps)
			}
			gd <- function(eta) (exp(-exp(eta))*exp(eta))*(1-exp(eta))
			valideta <- function(eta) TRUE
		},

		loglog = {
			linkfun <- function(mu) -log(-log(mu))
			linkinv <- function(eta) exp(-exp(-eta))
			mu.eta <- function(eta) exp(-exp(-eta)-eta)
			gd <- function(eta) (exp(-exp(-eta))*exp(-eta))*(exp(-eta)-1)
			valideta <- function(eta) TRUE
		},

		stop(sQuote(link), " - link not recognised")
	)

	structure(list(linkfun=linkfun,linkinv=linkinv,mu.eta=mu.eta,gd=gd,valideta=valideta,name=link),class="link-glm")
}

.fracreg.sep <- function(width = 80) paste0(rep("-", width), collapse = "")

.fracreg.center <- function(text, width = 80) {
    spaces <- max(0, floor((width - nchar(text)) / 2))
    paste0(paste0(rep(" ", spaces), collapse = ""), text)
}

.fracreg.cat.right <- function(label, value, width = 80) {
    val_str <- as.character(value)
    spaces <- max(1, width - nchar(label) - nchar(val_str) - 2)
    cat(label, paste0(rep(" ", spaces), collapse=""), val_str, "\n")
}

fracreg.est <- function(y,x,link,method,variance,var.type,var.eim,var.cluster,dfc,...)
{
	if(method=="ML") results <- glm(y ~ x-1,family=binomial(link=fracreg.links(link)),...)
	if(method=="QML") results <- glm(y ~ x-1,family=quasibinomial(link=fracreg.links(link)),...)
	p <- results$coefficients
	xbhat <- results$linear.predictors
	yhat <- results$fitted.values
	converged <- results$converged*(1-results$boundary)
	if(method=="ML") LL <- as.numeric(logLik(results))
	if(method=="QML") {
		eps <- 1e-16
		LL <- sum(ifelse(y > 0, y * log(pmax(yhat, eps)), 0) + ifelse(y < 1, (1-y) * log(pmax(1-yhat, eps)), 0))
	}

	ret.list <- list(p=p,yhat=yhat,xbhat=xbhat,converged=converged)
	ret.list[["LL"]] <- LL

	if(variance==FALSE) return(ret.list)

	p.var <- fracreg.var(y,x,yhat,xbhat,link,var.type,var.eim,var.cluster,dfc)$var
	ret.list[["p.var"]] <- p.var

	return(ret.list)
}

fracreg.var <- function(y,x,yhat,xbhat,link,var.type,var.eim,var.cluster,dfc)
{
	n <- nrow(x)
	uhat <- y-yhat
	g <- fracreg.links(link)$mu.eta(xbhat)
	gd <- fracreg.links(link)$gd(xbhat)

	A <- 0
	B <- 0

	for(jj in 1:n)
	{
		xx <- x[jj,]%*%t(x[jj,])
		A1 <- xx*(g[jj]^2)/(yhat[jj]*(1-yhat[jj]))
		if(var.eim==TRUE) A <- A+A1
		if(var.eim==FALSE)
		{
			div <- (yhat[jj]*(1-yhat[jj]))
			A2 <- -xx*uhat[jj]*gd[jj]/div
			A3 <- xx*uhat[jj]*(g[jj]^2)/(div*yhat[jj])
			A4 <- -xx*uhat[jj]*(g[jj]^2)/(div*(1-yhat[jj]))

			A <- A+A1+A2+A3+A4
		}
		if(var.type=="robust") B <- B+xx*(uhat[jj]^2)*(g[jj]^2)/((yhat[jj]*(1-yhat[jj]))^2)
	}

	if(var.type=="cluster")
	{
		id <- var.cluster
		id.uni <- unique(id)

		for(j in id.uni)
		{
			Xi <- matrix(x[id==j,],ncol=ncol(x))
			yhati <- yhat[id==j]
			gi <- g[id==j]
			ui <- uhat[id==j]

			ugGi <- ui*gi/(yhati*(1-yhati))

			B <- B+t(Xi)%*%ugGi%*%t(ugGi)%*%Xi
		}
	}

	if(dfc==TRUE)
	{
		if(any(var.type==c("standard","robust"))) df <- n/(n-1)
		if(var.type=="cluster") df <- length(id.uni)/(length(id.uni)-1)
	}
	else df <- 1

	A.inv <- solve(A)

	if(var.type=="standard") var <- df*A.inv
	if(var.type!="standard") var <- df*A.inv%*%B%*%A.inv


	return(list(var=var))
}

fracreg.lm <- function(y,yhat,gx,gz,type)
{
	u <- y-yhat
	w <- as.vector(sqrt(yhat*(1-yhat)))

	uw <- u/w
	gxw <- gx/w

	gzw <- gz/w

	if(type=="2Pbin")
	{
		gxzw <- cbind(gxw,gzw)

		res <- lm(uw ~ gxzw-1)

		LM <- t(res$fitted.values)%*%uw

	}
	else
	{
		res <- lm(gzw ~ gxw-1)
		ur <- as.matrix(res$residuals)

		n <- length(u)
		ones <- rep(1,n)

		uwr <- as.matrix(uw*ur)
		res <- lm(ones ~ uwr-1)
		uu <- res$residuals
		RSS <- sum(uu^2)

		LM <- n-RSS
	}

	return(list(LM=LM))
}

fracreg.pe.var <- function(x,npar,which.x,x.names,xvar.names,type,pa,xbhata,ga,linka,pa.var,pb=NA,xbhatb=NA,gb=NA,linkb=NA,pb.var=NA,yhata=NA,yhatb=NA,pc=NA,xbhatc=NA,gc=NA,linkc=NA,pc.var=NA,yhatc=NA)
{
	gda <- fracreg.links(linka)$gd(xbhata)
	PE1.sd <- matrix(NA,nrow=npar,ncol=npar)

	if(type!="2P" & type!="3P")
	{
		for(i in 1:npar)
		{
			for(j in 1:npar) PE1.sd[i,j] <- mean(pa[i]*gda*x[,j]+(i==j)*ga)
		}

		PE.sd <- PE1.sd%*%pa.var%*%t(PE1.sd)
	}
	if(type=="3P")
	{
		gdb <- fracreg.links(linkb)$gd(xbhatb)
		gdc <- fracreg.links(linkc)$gd(xbhatc)
		PE2.sd <- matrix(NA,nrow=npar,ncol=npar)
		PE3.sd <- matrix(NA,nrow=npar,ncol=npar)

		for(i in 1:npar)
		{
			for(j in 1:npar)
			{
				H <- yhatb + (1-yhatb)*yhatc
				PE1.sd[i,j] <- mean( (i==j)*ga*H + pa[i]*H*gda*x[,j] + pb[i]*gb*ga*(1-yhatc)*x[,j] + pc[i]*gc*ga*(1-yhatb)*x[,j] )
				PE2.sd[i,j] <- mean( pa[i]*ga*gb*(1-yhatc)*x[,j] + (i==j)*gb*yhata*(1-yhatc) + pb[i]*gdb*yhata*(1-yhatc)*x[,j] - pc[i]*gc*yhata*gb*x[,j] )
				PE3.sd[i,j] <- mean( pa[i]*ga*gc*(1-yhatb)*x[,j] - pb[i]*gb*yhata*gc*x[,j] + (i==j)*gc*yhata*(1-yhatb) + pc[i]*gdc*yhata*(1-yhatb)*x[,j] )
			}
		}

		PE.sd <- PE1.sd%*%pa.var%*%t(PE1.sd) + PE2.sd%*%pb.var%*%t(PE2.sd) + PE3.sd%*%pc.var%*%t(PE3.sd)
	}
	if(type=="2P")
	{
		gdb <- fracreg.links(linkb)$gd(xbhatb)
		PE2.sd <- matrix(NA,nrow=npar,ncol=npar)

		for(i in 1:npar)
		{
			for(j in 1:npar)
			{
				PE1.sd[i,j] <- mean(pb[i]*gb*ga*x[,j]+(i==j)*ga*yhatb+pa[i]*gda*yhatb*x[,j])
				PE2.sd[i,j] <- mean(pa[i]*ga*gb*x[,j]+(i==j)*gb*yhata+pb[i]*gdb*yhata*x[,j])
			}
		}

		PE.sd <- PE1.sd%*%pa.var%*%t(PE1.sd)+PE2.sd%*%pb.var%*%t(PE2.sd)
	}

	PE.sd <- diag(PE.sd)^0.5
	if(any(x.names=="(Intercept)")) PE.sd <- PE.sd[-1]

	names(PE.sd) <- xvar.names
	PE.sd <- PE.sd[which.x]

	return(PE.sd)
}

#' @export
summary.fracreg <- function(object, ...) {
    if(!is.null(object$resBIN0) && !is.null(object$resBIN0$table.info)) do.call(fracreg.table, object$resBIN0$table.info)
    if(!is.null(object$resBIN1) && !is.null(object$resBIN1$table.info)) do.call(fracreg.table, object$resBIN1$table.info)
    if(!is.null(object$resBIN) && !is.null(object$resBIN$table.info)) do.call(fracreg.table, object$resBIN$table.info)
    if(!is.null(object$resFRAC) && !is.null(object$resFRAC$table.info)) do.call(fracreg.table, object$resFRAC$table.info)
    if(!is.null(object$table.info)) do.call(fracreg.table, object$table.info)
    invisible(object)
}

#' @export
print.fracreg <- function(x, ...) {
        title <- "Fractional response regression"
    if(!is.null(x$type)) {
        if(x$type == "1P") title <- paste("Fractional", x$link, "regression")
        if(x$type == "2Pbin") title <- paste("Part 1: Binary", x$link, "regression")
        if(x$type == "2Pfrac") title <- paste("Part 2: Fractional", x$link, "regression")
        if(x$type == "3Pbin0") title <- paste("Part 1: Binary", x$link, "regression")
        if(x$type == "3Pbin1") title <- paste("Part 2: Binary", x$link, "regression")
        if(x$type == "3Pfrac") title <- paste("Part 3: Fractional", x$link, "regression")
        if(x$type == "2P") title <- paste("Two-part fractional regression: binary", x$link[1], "+ fractional", x$link[2])
        if(x$type == "3P") title <- paste("Three-part fractional regression: binary", x$link[1], ", binary", x$link[2], "+ fractional", x$link[3])
    }
    cat(paste0("\n", title, "\n"))
    if(!is.null(x$call)) {
        cat("\nCall:\n")
        print(x$call)
    }
    
    if(!is.null(x$resBIN0)) {
        cat("\nComponent 1 (Binary 0):\nCoefficients:\n")
        print.default(x$resBIN0$p)
    }
    if(!is.null(x$resBIN1)) {
        cat("\nComponent 2 (Binary 1):\nCoefficients:\n")
        print.default(x$resBIN1$p)
    }
    if(!is.null(x$resBIN)) {
        cat("\nBinary Component:\nCoefficients:\n")
        print.default(x$resBIN$p)
    }
    if(!is.null(x$resFRAC)) {
        cat("\nFractional Component:\nCoefficients:\n")
        print.default(x$resFRAC$p)
    }
    if(is.null(x$resBIN0) && is.null(x$resBIN1) && is.null(x$resBIN) && is.null(x$resFRAC) && !is.null(x$p)) {
        cat("\nCoefficients:\n")
        print.default(x$p)
    }
    cat("\n")
    invisible(x)
}

#' @export
summary.fracregpd <- function(object, ...) {
    if(!is.null(object$table.info)) do.call(fracregpd.table, object$table.info)
    invisible(object)
}

#' @export
print.fracregpd <- function(x, ...) {
        model_desc <- "panel data"
    if (!is.null(x$type) && x$type %in% c("QMLcre", "GMMcre")) model_desc <- "correlated random effects"
    link_str <- if(!is.null(x$link)) x$link else ""
    title <- paste("Fractional", link_str, model_desc, "regression")
    cat(paste0("\n", title, "\n"))
    if(!is.null(x$call)) {
        cat("\nCall:\n")
        print(x$call)
    }
    if(!is.null(x$p)) {
        cat("\nCoefficients:\n")
        print.default(x$p)
    }
    cat("\n")
    invisible(x)
}

#' @export
summary.fracreghet <- function(object, ...) {
    if(!is.null(object$table.info)) do.call(fracreghet.table, object$table.info)
    invisible(object)
}

#' @export
print.fracreghet <- function(x, ...) {
        link_str <- if(!is.null(x$link)) x$link else ""
    title <- paste("Fractional", link_str, "regression with heteroscedasticity/endogeneity")
    cat(paste0("\n", title, "\n"))
    if(!is.null(x$call)) {
        cat("\nCall:\n")
        print(x$call)
    }
    if(!is.null(x$p)) {
        cat("\nCoefficients:\n")
        print.default(x$p)
    }
    cat("\n")
    invisible(x)
}

#' @export
summary.fracreg.pe <- function(object, ...) {
    if(!is.null(object$table.info)) {
        do.call(fracreg.pe.table, object$table.info)
    } else if(!is.null(object$ape) || !is.null(object$cpe)) {
        if(!is.null(object$ape$table.info)) do.call(fracreg.pe.table, object$ape$table.info)
        if(!is.null(object$cpe$table.info)) do.call(fracreg.pe.table, object$cpe$table.info)
    }
    invisible(object)
}

#' @export
print.fracreg.pe <- function(x, ...) {
    cat("\nPartial Effects for Fractional Regression\n")
    if(!is.null(x$ape)) {
        cat("\nAverage Partial Effects (APE):\n")
        print.default(x$ape$table.info$PE.p)
    }
    if(!is.null(x$cpe)) {
        cat("\nConditional Partial Effects (CPE):\n")
        print.default(x$cpe$table.info$PE.p)
    }
    if(!is.null(x$table.info)) {
        if(x$table.info$PE.type == "APE") cat("\nAverage Partial Effects (APE):\n")
        else cat("\nConditional Partial Effects (CPE):\n")
        print.default(x$table.info$PE.p)
    }
    cat("\n")
    invisible(x)
}

#' @export
summary.fracregpd.pe <- function(object, ...) {
    if(!is.null(object$table.info)) {
        do.call(fracreg.pe.table, object$table.info)
    } else if(!is.null(object$ape) || !is.null(object$cpe)) {
        if(!is.null(object$ape$table.info)) do.call(fracreg.pe.table, object$ape$table.info)
        if(!is.null(object$cpe$table.info)) do.call(fracreg.pe.table, object$cpe$table.info)
    }
    invisible(object)
}

#' @export
print.fracregpd.pe <- function(x, ...) {
    cat("\nPartial Effects for Fractional Panel Data Regression\n")
    if(!is.null(x$ape)) {
        cat("\nAverage Partial Effects (APE):\n")
        print.default(x$ape$table.info$PE.p)
    }
    if(!is.null(x$cpe)) {
        cat("\nConditional Partial Effects (CPE):\n")
        print.default(x$cpe$table.info$PE.p)
    }
    if(!is.null(x$table.info)) {
        if(x$table.info$PE.type == "APE") cat("\nAverage Partial Effects (APE):\n")
        else cat("\nConditional Partial Effects (CPE):\n")
        print.default(x$table.info$PE.p)
    }
    cat("\n")
    invisible(x)
}

#' @export
summary.fracreghet.pe <- function(object, ...) {
    if(!is.null(object$table.info)) {
        do.call(fracreghet.pe.table, object$table.info)
    } else if(!is.null(object$ape) || !is.null(object$cpe)) {
        if(!is.null(object$ape$table.info)) do.call(fracreghet.pe.table, object$ape$table.info)
        if(!is.null(object$cpe$table.info)) do.call(fracreghet.pe.table, object$cpe$table.info)
    }
    invisible(object)
}

#' @export
print.fracreghet.pe <- function(x, ...) {
    cat("\nPartial Effects for Fractional Regression with Heteroskedasticity\n")
    if(!is.null(x$ape)) {
        cat("\nAverage Partial Effects (APE):\n")
        print.default(x$ape$table.info$PE.p)
    }
    if(!is.null(x$cpe)) {
        cat("\nConditional Partial Effects (CPE):\n")
        print.default(x$cpe$table.info$PE.p)
    }
    if(!is.null(x$table.info)) {
        if(x$table.info$PE.type == "APE") cat("\nAverage Partial Effects (APE):\n")
        else cat("\nConditional Partial Effects (CPE):\n")
        print.default(x$table.info$PE.p)
    }
    cat("\n")
    invisible(x)
}

#' @export
summary.fracreg.reset <- function(object, ...) {
    ti <- attr(object, "table.info")
    if(!is.null(ti)) do.call(fracreg.tests.table, ti)
    invisible(object)
}

#' @export
print.fracreg.reset <- function(x, ...) {
    cat("\nRESET Test for Fractional Regression\n\n")
    print.default(c(x))
    cat("\n")
    invisible(x)
}

#' @export
summary.fracreghet.reset <- function(object, ...) {
    ti <- attr(object, "table.info")
    if(!is.null(ti)) do.call(fracreghet.tests.table, ti)
    invisible(object)
}

#' @export
print.fracreghet.reset <- function(x, ...) {
    cat("\nRESET Test for Fractional Regression with Heteroskedasticity\n\n")
    print.default(c(x))
    cat("\n")
    invisible(x)
}

#' @export
summary.fracreg.ggoff <- function(object, ...) {
    ti <- attr(object, "table.info")
    if(!is.null(ti)) do.call(fracreg.tests.table, ti)
    invisible(object)
}

#' @export
print.fracreg.ggoff <- function(x, ...) {
    cat("\nGOFF Test for Fractional Regression\n\n")
    print.default(c(x))
    cat("\n")
    invisible(x)
}

#' @export
summary.fracreg.ptest <- function(object, ...) {
    ti <- attr(object, "table.info")
    if(!is.null(ti)) do.call(fracreg.tests.table, ti)
    invisible(object)
}

#' @export
print.fracreg.ptest <- function(x, ...) {
    cat("\nP-Test for Fractional Regression\n\n")
    print.default(c(x))
    cat("\n")
    invisible(x)
}

#' @export
print.fracregridge <- function(x, ...) {
    cat("\nFractional Ridge Regression\n")
    if(!is.null(x$call)) {
        cat("\nCall:\n")
        print(x$call)
    }
    cat("\nRidge Coefficients at Target Fractions:\n")
    print(x$coef)
    cat("\n")
    invisible(x)
}

#' @export
summary.fracregridge <- function(object, ...) {
    cat("\n")
    cat(.fracreg.sep(), "\n")
    cat(.fracreg.center("Fractional Ridge Regression"), "\n")
    cat(.fracreg.sep(), "\n")
    
    .fracreg.cat.right("Data type:", "Cross-sectional")
    .fracreg.cat.right("Convergence:", "Successful")
    .fracreg.cat.right("Standard errors:", "homoskedastic")
    cat(.fracreg.sep(), "\n")
    
    for (name in names(object$table.info)) {
        cat(.fracreg.center(paste("Target Fraction:", name)), "\n")
        cat(.fracreg.sep(), "\n")
        if(!is.null(object$stats.info)) {
            stats <- object$stats.info[[name]]
            .fracreg.cat.right("Number of observations:", stats$n_obs)
            .fracreg.cat.right("Pseudo R-squared:", round(stats$R2, 5))
            .fracreg.cat.right("Degrees of freedom:", round(stats$n_obs - stats$df_alpha, 2))
            if (!is.null(stats$W) && !is.na(stats$W)) {
                .fracreg.cat.right(paste0("Wald chi2(", stats$df_W, "):"), round(stats$W, 4))
                .fracreg.cat.right("Prob > chi2:", sprintf("%.4f", stats$p_W))
            }
            cat(.fracreg.sep(), "\n")
        }
        suppressWarnings(stats::printCoefmat(object$table.info[[name]], P.values=TRUE, has.Pvalue=TRUE, digits=4, signif.legend=TRUE))
        cat(.fracreg.sep(), "\n")
    }
    
    cat(.fracreg.center(paste("Run Date:", format(Sys.time(), "%Y-%m-%d %H:%M:%S"))), "\n")
    cat(.fracreg.sep(), "\n\n")
    invisible(object)
}

#' @export
summary.fracregridge.pe <- function(object, ...) {
    cat("\n\n")
    cat(.fracreg.sep(), "\n")
    cat(.fracreg.center("Average partial effects"), "\n")
    cat(.fracreg.sep(), "\n")
    cat(.fracreg.center("Fractional Ridge Regression"), "\n")
    cat(.fracreg.sep(), "\n")
    
    cat("\nNote: Fractional Ridge Regression is a linear model without a link function.\n")
    cat("Therefore, the partial effects are mathematically identical to the coefficients themselves.\n\n")
    
    for (name in names(object$table.info)) {
        cat(.fracreg.center(paste("Target Fraction:", name)), "\n")
        cat(.fracreg.sep(), "\n")
        res <- object$table.info[[name]]
        colnames(res)[1] <- "dy/dx"
        suppressWarnings(stats::printCoefmat(res, P.values=TRUE, has.Pvalue=TRUE, digits=4, signif.legend=TRUE))
        cat(.fracreg.sep(), "\n")
    }
    cat(.fracreg.center(paste("Run Date:", format(Sys.time(), "%Y-%m-%d %H:%M:%S"))), "\n")
    cat(.fracreg.sep(), "\n\n")
    invisible(object)
}

#' @export
print.fracregridge.pe <- function(x, ...) {
    cat("\nPartial Effects for Fractional Ridge Regression\n")
    if(!is.null(x$call)) {
        cat("\nCall:\n")
        print(x$call)
    }
    cat("\n")
    invisible(x)
}

#' @export
print.fracregmlogit <- function(x, ...) {
    cat("\n")
    cat(.fracreg.sep(), "\n")
    cat(.fracreg.center("Fractional multinomial logit model"), "\n")
    cat(.fracreg.sep(), "\n")
    if(!is.null(x$call)) {
        cat("\nCall:\n")
        print(x$call)
    }
    cat("\nCoefficients:\n")
    print.default(x$coefficient)
    cat("\n")
    invisible(x)
}

#' Generate Summary Tables for fracregmlogit Objects
#' 
#' Generate tables of coefficient estimates, partial effects, and willingness to pay from
#' fracregmlogit-type objects. 
#' 
#' @name summary.fracregmlogit
#' @aliases summary.fracregmlogit.pe
#' @aliases summary.fracregmlogit.wtp
#' 
#' @param object an object with class "fracregmlogit", "fracregmlogit.pe", or "fracregmlogit.wtp".
#' @param ... Additional arguments passed to the printCoefmat function.
#' @return Returns the object invisibly.
#' 
#' @details This module provides summary methods for three fracregmlogit objects: \code{fracregmlogit}, \code{fracregmlogit.pe}
#' , and \code{fracregmlogit.wtp}. 
#' 
#' For \code{fracregmlogit} objects, the summary prints the number of observations, log pseudo-likelihood,
#' baseline choice, and the coefficient estimates with standard errors, z-statistics, and p-values
#' for each choice equation. 
#' 
#' For \code{fracregmlogit.pe} objects, it displays the marginal or discrete effects 
#' along with their computed standard errors (if Krinsky-Robb sampling was performed) for each choice.
#' 
#' For \code{fracregmlogit.wtp} objects, it provides a table of the aggregated willingness to pay 
#' along with its standard errors and test statistics.
#' @seealso \code{\link{fracregmlogit}}, \code{\link{fracregmlogit.pe}}
#' @examples
#' data("fracreg_spending")
#' X = fracreg_spending[,2:5]
#' y = fracreg_spending[,6:11]
#' 
#' # generate fracregmlogit summary
#' results1 = fracregmlogit(y, X)
#' summary(results1)
#' 
#' # generate marginal effects summary
#' effects1 = fracregmlogit.pe(results1, effect="marginal", se=FALSE)
#' summary(effects1)
#' 
#' @rdname summary.fracregmlogit
#' @exportS3Method summary fracregmlogit
summary.fracregmlogit <- function(object, ...) {
    cat("\n")
    cat(.fracreg.sep(), "\n")
    cat(.fracreg.center("Fractional multinomial logit model"), "\n")
    cat(.fracreg.sep(), "\n")
    
    .fracreg.cat.right("Data type:", "Cross-sectional")
    .fracreg.cat.right("Convergence:", "Successful")
    .fracreg.cat.right("Number of observations:", object$count["Obs"])
    if (!is.null(object$likelihood)) {
        .fracreg.cat.right("Log pseudolikelihood:", round(object$likelihood, 4))
    }
    if (!is.null(object$pseudo_R2)) {
        .fracreg.cat.right("Pseudo R-squared:", round(object$pseudo_R2, 5))
    }
    .fracreg.cat.right("Baseline choice:", object$baseline)
    
    if (object$reps > 0) {
        .fracreg.cat.right("Standard errors:", "bootstrap")
        .fracreg.cat.right("Bootstrap reps:", object$reps)
    } else {
        .fracreg.cat.right("Standard errors:", "HC0")
    }
    cat("\n")

    for(i in 1:length(object$estimates)){
        cat(.fracreg.sep(), "\n")
        cat(.fracreg.center(paste("Choice:", names(object$estimates)[i])), "\n")
        cat(.fracreg.sep(), "\n")
        
        if (!is.null(object$wald[[i]]) && !is.na(object$wald[[i]]$W)) {
            .fracreg.cat.right(paste0("Wald chi2(", object$wald[[i]]$df, "):"), round(object$wald[[i]]$W, 4))
            .fracreg.cat.right("Prob > chi2:", sprintf("%.4f", object$wald[[i]]$p))
            cat(.fracreg.sep(), "\n")
        }
        
        res <- object$estimates[[i]]
        se_type <- if (object$reps > 0) "Bootstrap" else "Robust"
        colnames(res) <- c("Coefficient", paste(se_type, "Std.Err."), "z value", "Pr(>|z|)")
        rownames(res)[rownames(res) == "(Intercept)"] <- "(Intercept)"
        stats::printCoefmat(res, P.values = TRUE, has.Pvalue = TRUE, digits = 4, signif.legend = TRUE)
        cat("\n")
    }
    
    cat(.fracreg.sep(), "\n")
    cat(.fracreg.center(paste("Run Date:", format(Sys.time(), "%Y-%m-%d %H:%M:%S"))), "\n")
    cat(.fracreg.sep(), "\n")

    invisible(object)
}

#' @export
print.fracregmlogit.pe <- function(x, ...) {
    cat("\n")
    cat(.fracreg.sep(), "\n")
    if(grepl("average across observations", x$expl)) {
        cat(.fracreg.center("Average partial effects"), "\n")
    } else {
        cat(.fracreg.center("Conditional partial effects"), "\n")
    }
    cat(.fracreg.sep(), "\n")
    cat("\nNote:", x$expl, "\n")
    cat("\nEffects:\n")
    print.default(x$effects)
    cat("\n")
    invisible(x)
}

#' @export
#' @exportS3Method summary fracregmlogit.pe
summary.fracregmlogit.pe <- function(object, ...) {
    cat("\n\n")
    cat(.fracreg.sep(), "\n")
    if(grepl("average across observations", object$expl)) {
        cat(.fracreg.center("Average partial effects"), "\n")
    } else {
        cat(.fracreg.center("Conditional partial effects"), "\n")
    }
    cat(.fracreg.sep(), "\n")
    cat(.fracreg.center("Fractional multinomial logit regression"), "\n")
    cat(.fracreg.sep(), "\n")
    cat("\nNote:", object$expl, "\n")
    
    if(!is.null(object$ztable)){
        for(i in 1:length(object$ztable)){
            cat("\n")
            cat(.fracreg.sep(), "\n")
            cat(.fracreg.center(paste("Variable:", names(object$ztable)[i])), "\n")
            cat(.fracreg.sep(), "\n")
            
            res <- object$ztable[[i]]
            colnames(res) <- c("dy/dx", "Std. Error", "z value", "Pr(>|z|)")
            stats::printCoefmat(res, P.values = TRUE, has.Pvalue = TRUE, digits = 4, signif.legend = TRUE)
        }
        cat("\n")
    } else {
        cat("Effects:\n")
        print.default(object$effects)
    }
    
    cat(.fracreg.sep(), "\n")
    cat(.fracreg.center(paste("Run Date:", format(Sys.time(), "%Y-%m-%d %H:%M:%S"))), "\n")
    cat(.fracreg.sep(), "\n")

    invisible(object)
}
############
# generate willingness to pay tables
############

#' @export
summary.fracregmlogit.wtp = function(object, ...) {
  if(!inherits(object, "fracregmlogit.wtp")) stop("Expect an fracregmlogit.wtp object. Wrong object type given.")
  if (is.null(dim(object$wtp))) return(object$wtp)
  if(is.null(colnames(object$wtp)) || colnames(object$wtp)[1]!="Coefficient") return(object$wtp) # no need to summary.
  
  cat("\n")
  cat(.fracreg.sep(), "\n")
  cat(.fracreg.center("Willingness to Pay"), "\n")
  cat(.fracreg.sep(), "\n")
  cat(.fracreg.center("Fractional multinomial logit regression"), "\n")
  cat(.fracreg.sep(), "\n\n")
  
  cat("Note: Krinsky-Robb standard error calculated\n")
  stats::printCoefmat(object$wtp, digits = max(3, getOption("digits") - 2), 
                      signif.stars = TRUE, P.values = TRUE, has.Pvalue = TRUE)
  
  cat("\n")
  cat(.fracreg.sep(), "\n")
  cat(.fracreg.center(paste("Run Date:", format(Sys.time(), "%Y-%m-%d %H:%M:%S"))), "\n")
  cat(.fracreg.sep(), "\n\n")
  
  invisible(object)
}

#' @title Fractional Response Regressions - Partial Effects
#'
#' @description
#' \code{fracreg.pe} is used to compute average and/or conditional partial effects in fractional response models.
#' @param object an object containing the results of an \code{fracreg} command.
#' @param APE a logical value indicating whether average partial effects are to be computed.
#' @param CPE a logical value indicating whether conditional partial effects are to be computed.
#' @param at a numeric vector containing the covariates' values at which the conditional partial effects are to be computed or  the strings \code{"mean"} (the default) or \code{"median"}, in which cases the covariates are evaluated at their  mean or median values (or mode, in case of dummy variables), respectively.
#' @param which.x a vector containing the names of the covariates to which the partial effects are to be computed.
#' @param variance a logical value indicating whether the variance of the estimated partial effects should be calculated. Defaults to  \code{TRUE} whenever \code{table = TRUE}.
#' @param table a logical value indicating whether a summary table with the results should be printed.
#'
#' @details
#' \code{fracreg.pe} calculates partial effects for fractional response models estimated via \code{fracreg}. \code{fracreg.pe} may be used to compute average or conditional partial effects for: (i) one-part fractional response models; (ii) the binary components of two-part and three-part fractional response models; (iii) the fractional components of two-part and three-part fractional response models; and (iv) two-part and three-part fractional response models overall. 
#' 
#' \strong{Partial Effects for Continuous Variables:}
#' For a continuous covariate \eqn{x_k}, the partial effect on the conditional mean \eqn{E(y|x) = G(x\beta)} is the first derivative with respect to \eqn{x_k}:
#' \deqn{PE_k(x) = \frac{\partial E(y|x)}{\partial x_k} = g(x\beta)\beta_k}
#' where \eqn{g(\cdot)} is the probability density function corresponding to the link function \eqn{G(\cdot)}.
#' 
#' \strong{Partial Effects for Discrete Variables:}
#' For a discrete or dummy covariate \eqn{x_k}, the partial effect is calculated as the discrete difference in the expected value when \eqn{x_k} changes from 0 to 1, holding all other variables \eqn{x_{-k}} constant:
#' \deqn{PE_k(x) = G(x_{-k}\beta_{-k} + \beta_k) - G(x_{-k}\beta_{-k})}
#' 
#' \strong{Average vs. Conditional Partial Effects:}
#' - \strong{Average Partial Effects (APE):} Evaluated for each observation \eqn{i} in the sample and then averaged:
#' \deqn{APE_k = \frac{1}{N} \sum_{i=1}^N PE_k(x_i)}
#' - \strong{Conditional Partial Effects (CPE):} Evaluated at a specific vector of covariate values \eqn{x^*} (e.g., the sample mean or median):
#' \deqn{CPE_k = PE_k(x^*)}
#' 
#' For calculating standard errors, it is taken into account the option that was previously chosen for estimating the model. See Ramalho, Ramalho and Murteira (2011) and Fang and Ma (2013) for details on the computation of partial effects in the fractional response framework.
#'
#' @return
#' \code{fracreg.pe} returns a list with the following element:
#'   \item{PE.p}{a named vector of partial effects.
#' }
#' 
#' If \code{variance = TRUE} or \code{table = TRUE}, the previous list also contains the following element:
#'   \item{PE.sd}{a named vector of standard errors of the estimated partial effects.
#' }
#' 
#' When both average and conditional partial effects are requested, two lists containing the previous elements are returned, indexed by the prefixes \code{ape} and \code{cpe}.
#'
#' @references
#' Ramalho, E.A., J.J.S. Ramalho and J.M.R. Murteira (2011), "Alternative
#' estimating and testing empirical strategies for fractional response models",
#' \emph{Journal of Economic Surveys}, 25(1), 19-68.
#' 
#' Fang, K., & Ma, S. (2013), "Three-part model for fractional response variables with application to Chinese household health insurance coverage", \emph{Journal of Applied Statistics}, 40(5), 925-940.
#'
#' @author Sulman Olieko Owili <oliekosulman@gmail.com>
#'
#' @seealso
#' \code{\link{fracreg}}, for fitting fractional response models.\cr
#' \code{\link{fracreg.reset}} and \code{\link{fracreg.ggoff}}, for specification tests.\cr
#' \code{\link{fracreg.ptest}}, for non-nested hypothesis tests.
#'
#' @examples
#' ### Empirical 401(k) Examples
#' data("fracreg_k401k")
#' y <- fracreg_k401k$prate
#' X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age, 
#'            totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole)
#' 
#' m <- fracreg(y, X, type="1P", linkfrac="logit")
#' pe_res <- fracreg.pe(m)
#' summary(pe_res)
#' 
#' ### Simulated Examples
#' 
#' N <- 250
#' u <- rnorm(N)
#' 
#' X <- cbind(rnorm(N),rnorm(N))
#' dimnames(X)[[2]] <- c("X1","X2")
#' 
#' ym <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u))
#' y <- rbeta(N,ym*20,20*(1-ym))
#' y[y > 0.9] <- 1
#' 
#' #Computing average partial effects for a logit fractional response model
#' mod <- fracreg(y,X,linkfrac="logit")
#' pe_res <- fracreg.pe(mod)
#' summary(pe_res)
#' 
#' #Computing average partial effects for a binary logit + fractional probit
#' #two-part model
#' mod <- fracreg(y,X,linkbin="logit",linkfrac="probit",type="2P",inf=1)
#' pe_res <- fracreg.pe(mod)
#' summary(pe_res)
#' 
#' #Computing conditional partial effects for X2 in the logit component
#' #of a two-part fractional response model, with the covariates evaluated
#' #at their median values
#' mod <- fracreg(y,X,linkfrac="logit",type="2Pfrac",inf=1)
#' pe_res <- fracreg.pe(mod,APE=FALSE,CPE=TRUE,at="median",which.x="X2")
#' summary(pe_res)
#' 
#' #Computing average partial effects for a three-part double-inflated model
#' y3p <- y
#' y3p[1:20] <- 0
#' y3p[21:40] <- 1
#' res3p <- fracreg(y3p,X,linkbin=c("logit","probit"),linkfrac="logit",type="3P")
#' pe_res <- fracreg.pe(res3p)
#' summary(pe_res)
#' @export
fracreg.pe <- function(object,APE=TRUE,CPE=FALSE,at=NULL,which.x=NULL,variance=TRUE,table=FALSE)
{
	### 1. Error and warning messages

	if(missing(object)) stop("object is missing")
	if(is.null(object$class)) stop("object is not the output of an fracreg command")
	if(object$class!="fracreg") stop("object is not the output of an fracreg command")

	if(!is.logical(APE)) stop("non-logical value assigned to option APE")
	if(!is.logical(CPE)) stop("non-logical value assigned to option CPE")
	if(!is.logical(variance)) stop("non-logical value assigned to option variance")
	if(!is.logical(table)) stop("non-logical value assigned to option table")

	if(all(c(APE,CPE)==FALSE)) stop("You must specify at least one option: APE and/or CPE")
	if(CPE==FALSE & !is.null(at)) stop("option at is only required for CPE")

	if(object$converged==0) stop("object is not the output of a successful (converged) fracreg command")

	if(object$type!="2P" & object$type!="3P")
	{
		if(is.null(object$p.var)) stop("fracreg command was run with variance = FALSE")
	}
	else if (object$type=="2P")
	{
		if(is.null(object$resBIN$p.var)) stop("fracreg command was run with variance = FALSE")
	}
	else if (object$type=="3P")
	{
		if(is.null(object$resBIN0$p.var)) stop("fracreg command was run with variance = FALSE")
	}

	if(table==TRUE & variance==FALSE)
	{
		variance <- TRUE
		warning("option variance changed from FALSE to TRUE, as required by table=TRUE")
	}

	### 2. Recovering definitions and estimates and other definitions

	type <- object$type

	if(type!="2P" & type!="3P")
	{
		linka <- object$link
		pa <- object$p
		pa.var <- object$p.var
		x <- model.matrix(object$formula)
		x.names <- object$x.names

		if(type=="1P") title <- paste("Fractional",linka,"regression")
		if(type=="2Pbin") title <- paste("Binary",linka,"component of a two-part regression")
		if(type=="2Pfrac") title <- paste("Fractional",linka,"component of a two-part regression")
	}
	if(type=="3P")
	{
		linka <- object$resBIN0$link
		pa <- object$resBIN0$p
		pa.var <- object$resBIN0$p.var
		xa <- model.matrix(object$resBIN0$formula)
		xa.names <- object$resBIN0$x.names

		linkb <- object$resBIN1$link
		pb <- object$resBIN1$p
		pb.var <- object$resBIN1$p.var

		linkc <- object$resFRAC$link
		pc <- object$resFRAC$p
		pc.var <- object$resFRAC$p.var
		xc <- object$x2base
		xc.names <- object$resFRAC$x.names

		if(!identical(xa.names,xc.names)) stop("currently fracreg.pe requires all components of 3P models to use the same covariates")
		x <- xa
		x.names <- xa.names

		title <- paste("Three-part regression - binary",linka,", binary",linkb,"+ fractional",linkc)
	}
	if(type=="2P")
	{
		linka <- object$resBIN$link
		pa <- object$resBIN$p
		pa.var <- object$resBIN$p.var
		xa <- model.matrix(object$resBIN$formula)
		xa.names <- object$resBIN$x.names

		linkb <- object$resFRAC$link
		pb <- object$resFRAC$p
		pb.var <- object$resFRAC$p.var
		xb <- object$x2base
		xb.names <- object$resFRAC$x.names

		if(!identical(xa.names,xb.names)) stop("currently fracreg.pe requires both components of two-part models to use the same covariates")
		x <- xa
		x.names <- xa.names

		title <- paste("Binary",linka,"+ Fractional",linkb,"two-part regression")
	}

	if(any(x.names=="(Intercept)")) xvar.names <- x.names[-1]
	else xvar.names <- x.names

	k <- length(xvar.names)
	npar <- ncol(x)
	n <- nrow(x)

	if(is.null(which.x)) which.x <- xvar.names
	xw.names <- unique(c(xvar.names,which.x))
	if(!identical(xvar.names,xw.names)) stop("option which not appropriately defined")

	### 3. Average partial effects

	if(APE==TRUE)
	{
		PE.type <- "APE"

		if(any(x.names=="(Intercept)")) p.pe <- matrix(rep(pa[-1],each=n),ncol=k)
 		else p.pe <- matrix(rep(pa,each=n),ncol=k)
		dimnames(p.pe) <- list(NULL,xvar.names)

		if(type!="2P" & type!="3P") xbhata <- object$xbhat
		if(type=="2P") xbhata <- object$resBIN$xbhat
		if(type=="3P") xbhata <- object$resBIN0$xbhat

		ga <- fracreg.links(linka)$mu.eta(xbhata)
		PEa.p <- as.matrix(p.pe[,which.x])*ga

		if(type!="2P" & type!="3P") PE.p <- apply(PEa.p,2,mean)

		if(type=="3P")
		{
			yhata <- object$resBIN0$yhat
			if(any(x.names=="(Intercept)")) p.pe2 <- matrix(rep(pb[-1],each=n),ncol=k) else p.pe2 <- matrix(rep(pb,each=n),ncol=k)
			if(any(x.names=="(Intercept)")) p.pe3 <- matrix(rep(pc[-1],each=n),ncol=k) else p.pe3 <- matrix(rep(pc,each=n),ncol=k)
			dimnames(p.pe2) <- list(NULL,xvar.names)
			dimnames(p.pe3) <- list(NULL,xvar.names)
			
			xbhatb <- as.vector(x%*%pb)
			gb <- fracreg.links(linkb)$mu.eta(xbhatb)
			yhatb <- fracreg.links(linkb)$linkinv(xbhatb)
			PEb.p <- as.matrix(p.pe2[,which.x])*gb
			
			xbhatc <- as.vector(x%*%pc)
			gc <- fracreg.links(linkc)$mu.eta(xbhatc)
			yhatc <- fracreg.links(linkc)$linkinv(xbhatc)
			PEc.p <- as.matrix(p.pe3[,which.x])*gc
			
			H <- yhatb + (1-yhatb)*yhatc
			PE.p <- apply( PEa.p*H + PEb.p*yhata*(1-yhatc) + PEc.p*yhata*(1-yhatb), 2, mean )
		}
		else if(type=="2P")
		{
			yhata <- object$resBIN$yhat
			PEa.p <- as.matrix(p.pe[,which.x])*ga

			if(any(x.names=="(Intercept)")) p.pe <- matrix(rep(pb[-1],each=n),ncol=k)
 			else p.pe <- matrix(rep(pb,each=n),ncol=k)
			dimnames(p.pe) <- list(NULL,xvar.names)

			xbhatb <- as.vector(x%*%pb)
			gb <- fracreg.links(linkb)$mu.eta(xbhatb)
			yhatb <- fracreg.links(linkb)$linkinv(xbhatb)
			PEb.p <- as.matrix(p.pe[,which.x])*gb

			PE.p <- apply(PEb.p*yhata+PEa.p*yhatb,2,mean)
		}

		resAPE <- list(PE.p=PE.p)

		if(variance==TRUE)
		{
			if(type!="2P" & type!="3P") PE.sd <- fracreg.pe.var(x,npar,which.x,x.names,xvar.names,type,pa,xbhata,ga,linka,pa.var)
			if(type=="2P") PE.sd <- fracreg.pe.var(x,npar,which.x,x.names,xvar.names,type,pa,xbhata,ga,linka,pa.var,pb,xbhatb,gb,linkb,pb.var,yhata,yhatb)
			if(type=="3P") PE.sd <- fracreg.pe.var(x,npar,which.x,x.names,xvar.names,type,pa,xbhata,ga,linka,pa.var,pb,xbhatb,gb,linkb,pb.var,yhata,yhatb,pc,xbhatc,gc,linkc,pc.var,yhatc)
			resAPE[["PE.sd"]] <- PE.sd
		}

		table.info.APE <- list(PE.p=PE.p,PE.sd=if(variance) PE.sd else NA,PE.type=PE.type,which.x=which.x,xvar.names=xvar.names,title=title,at=at)
		if(table==TRUE) do.call(fracreg.pe.table, table.info.APE)
		resAPE[["table.info"]] <- table.info.APE
	}

	### 4. Conditional partial effects

	if(CPE==TRUE)
	{
		PE.type <- "CPE"

		if(is.null(at)) at <- "mean"

		if(length(at)==1)
		{
			if((!any(at==c("mean","median")) & k!=1)) stop("at not appropriately specified")

			if(any(at==c("mean","median")))
			{
				if(at=="mean") xm <- apply(x,2,mean)
				if(at=="median") xm <- apply(x,2,median)

				xdum <- apply(x,2,function(a) all(a %in% c(0,1)))
				xm[xdum==TRUE] <- round(xm,0)[xdum==TRUE]
			}
			else
			{
				if(is.numeric(at))
				{
					if(any(x.names=="(Intercept)")) xm <- c(1,at)
					else xm <- at
				}
				else stop("at not appropriately specified")
			}
		}
		else
		{
			if(length(at)!=k) stop("at not appropriately specified")
			else
			{
				if(any(x.names=="(Intercept)")) xm <- c(1,at)
				else xm <- at
			}
		}

		xbhata <- as.vector(xm%*%pa)
		ga <- fracreg.links(linka)$mu.eta(xbhata)
		PEa.p <- pa[which.x]*ga
		PE.p <- PEa.p

		if(type=="3P")
		{
			yhata <- fracreg.links(linka)$linkinv(xbhata)
			
			xbhatb <- as.vector(xm%*%pb)
			gb <- fracreg.links(linkb)$mu.eta(xbhatb)
			yhatb <- fracreg.links(linkb)$linkinv(xbhatb)
			PEb.p <- pb[which.x]*gb
			
			xbhatc <- as.vector(xm%*%pc)
			gc <- fracreg.links(linkc)$mu.eta(xbhatc)
			yhatc <- fracreg.links(linkc)$linkinv(xbhatc)
			PEc.p <- pc[which.x]*gc
			
			H <- yhatb + (1-yhatb)*yhatc
			PE.p <- PEa.p*H + PEb.p*yhata*(1-yhatc) + PEc.p*yhata*(1-yhatb)
		}
		else if(type=="2P")
		{
			yhata <- fracreg.links(linka)$linkinv(xbhata)

			xbhatb <- as.vector(xm%*%pb)
			gb <- fracreg.links(linkb)$mu.eta(xbhatb)
			yhatb <- fracreg.links(linkb)$linkinv(xbhatb)
			PEb.p <- pb[which.x]*gb

			PE.p <- PEb.p*yhata+PEa.p*yhatb
		}

		resCPE <- list(PE.p=PE.p)

		if(variance==TRUE)
		{
			if(type!="2P" & type!="3P") PE.sd <- fracreg.pe.var(x,npar,which.x,x.names,xvar.names,type,pa,xbhata,ga,linka,pa.var)
			if(type=="2P") PE.sd <- fracreg.pe.var(x,npar,which.x,x.names,xvar.names,type,pa,xbhata,ga,linka,pa.var,pb,xbhatb,gb,linkb,pb.var,yhata,yhatb)
			if(type=="3P") PE.sd <- fracreg.pe.var(x,npar,which.x,x.names,xvar.names,type,pa,xbhata,ga,linka,pa.var,pb,xbhatb,gb,linkb,pb.var,yhata,yhatb,pc,xbhatc,gc,linkc,pc.var,yhatc)
			resCPE[["PE.sd"]] <- PE.sd
		}

		table.info.CPE <- list(PE.p=PE.p,PE.sd=if(variance) PE.sd else NA,PE.type=PE.type,which.x=which.x,xvar.names=xvar.names,title=title,at=at)
		if(table==TRUE) do.call(fracreg.pe.table, table.info.CPE)
		resCPE[["table.info"]] <- table.info.CPE
	}

	### 5. Return results

	if(APE==TRUE & CPE==TRUE) res <- list(ape=resAPE,cpe=resCPE)
	else if(APE==TRUE & CPE==FALSE) res <- resAPE
	else if(APE==FALSE & CPE==TRUE) res <- resCPE
	
	class(res) <- "fracreg.pe"
	return(invisible(res))
}

library(testthat)
library(fracreg)

# Prepare data for testing
data("fracreg_k401k")
y <- fracreg_k401k$prate
X <- as.matrix(cbind(
  mrate = fracreg_k401k$mrate, 
  age = fracreg_k401k$age, 
  totemp = fracreg_k401k$totemp, 
  sole = fracreg_k401k$sole
))

test_that("fracreg 1P estimator works", {
  model_1p <- fracreg(y = y, x = X, type = "1P", linkfrac = "logit", table = FALSE)
  
  expect_type(model_1p, "list")
  expect_true(all(c("p", "p.var", "yhat") %in% names(model_1p)))
  expect_false(any(is.na(model_1p$p)))
  expect_equal(length(model_1p$p), ncol(X) + 1)
})

test_that("fracreg 2P estimator works", {
  # Modify y slightly to ensure some 0s for the 2P test
  y_2p <- y
  y_2p[1:50] <- 0
  
  model_2p <- fracreg(y = y_2p, x = X, type = "2P", inflation = 0, linkbin = "logit", linkfrac = "logit", table = FALSE)
  
  expect_type(model_2p, "list")
  expect_true(all(c("resBIN", "resFRAC", "yhat2P") %in% names(model_2p)))
  expect_false(any(is.na(model_2p$resBIN$p)))
  expect_false(any(is.na(model_2p$resFRAC$p)))
})

test_that("fracreg 3P estimator works", {
  # Modify y slightly to ensure some 0s and 1s for the 3P test
  y_3p <- y
  y_3p[1:50] <- 0
  y_3p[51:100] <- 1
  
  model_3p <- fracreg(y = y_3p, x = X, type = "3P", linkbin = c("logit", "logit"), linkfrac = "logit", table = FALSE)
  
  expect_type(model_3p, "list")
  expect_true(all(c("resBIN0", "resBIN1", "resFRAC", "yhat3P") %in% names(model_3p)))
})

test_that("fracreg.pe works for 1P", {
  model_1p <- fracreg(y = y, x = X, type = "1P", linkfrac = "logit", table = FALSE)
  pe_1p <- fracreg.pe(model_1p, table = FALSE)
  
  expect_type(pe_1p, "list")
  expect_true(all(c("PE.p", "PE.sd") %in% names(pe_1p)))
  expect_equal(length(pe_1p$PE.p), ncol(X))
})

#' "Willingness to Pay" for fracregmlogit models
#' 
#' Evaluate the "Willingness to Pay" given a set of arbitrary values for outcome variables. 
#' Usually used for policy evaluations where the total magnitude of marginal change matters.
#' 
#' @param object A \code{fracregmlogit.pe} object.
#' @param wtp.vec A numeric vector containing the arbitrary outcome values to be evaluated for each choice j.
#' @param varlist A string vector which provides the names of variables to calculate 
#' the wtp for. If missing, all variables in the object will be calculated.
#' @param indv.obs A logical value indicating whether to return individual observations.
#' @return A "fracregmlogit.wtp" object containing the estimates, standard error, z-stats, and p-value.
#' @details This function calculates the aggregate effect of a variable on the 
#' "willingness to pay" by linearly multiplying the average partial effect with ex-ante (arbitrary) 
#' willingness to pay numbers associated with each choice. 
#' 
#' Suppose there are three choices A, B, C, each with a willingness to pay (or cost, profit, budget),
#' of 100, 200, and 300. The discrete effects of variable X on A, B and C are 0.5, 0.5, and -1, with 
#' standard errors 0.2, 0.3 and 0.5. The aggregated discrete effect of X on the total willingness 
#' to pay (or cost), is thus 100*0.5 + 200*0.5 + 300*(-1) = -150. The standard error can also be
#' calculated to be 162.8, assuming that the standard error is independent. 
#' A simple z-test is provided to test whether the aggregate effect is different from zero. 
#' 
#' Note that if the input \code{fracregmlogit.pe} object has no standard error computation, then no standard error
#' will be computed for the willingness to pay.
#' 
#' @seealso \code{\link{fracregmlogit.pe}}, \code{\link{plot.fracregmlogit}}
#' 
#' @examples
#' \donttest{
#' data("fracreg_spending")
#' X = fracreg_spending[,2:5]
#' y = fracreg_spending[,6:11]
#' results1 = fracregmlogit(y, X)
#' pe = fracregmlogit.pe(results1)
#' # Assuming arbitrary WTP values for the 6 choices
#' wtp_est = wtp(pe, wtp.vec = c(1, 2, 3, 4, 5, 6), varlist = "houseval")
#' summary(wtp_est)
#' }
#' @export wtp
wtp = function(object, wtp.vec, varlist=NULL, indv.obs=FALSE){
  if(!inherits(object, "fracregmlogit.pe")) stop("Expect an fracregmlogit.pe object. Wrong object type given.")
  j=nrow(object$effects); k=ncol(object$effects)
  Xnames = colnames(object$effects); ynames = rownames(object$effects)
  if(length(varlist)==0){
    varlist=Xnames
    var_colNo = c(1:k)
    k = length(var_colNo)
  }else{
    var_colNo = which(varlist %in% Xnames)
    k = length(var_colNo)
  }
  if(length(wtp.vec)!=j) stop("Wrong length of wtp.vec. Please check specification again.")
  # wtp calcs
  betamat = object$effects[,varlist, drop=FALSE]; semat = object$se[,varlist, drop=FALSE]
  wtp_mean = wtp.vec %*% betamat

  if(object$R>0){ # prevent a bug that does not output R in the fracregmlogit.pe module. 
  wtp_se = sqrt(wtp.vec^2 %*% semat^2)
  # output tables
  tabout = matrix(ncol=4,nrow=k)
  tabout[,1] = wtp_mean
  tabout[,2] = wtp_se
  tabout[,3] = tabout[,1] / tabout[,2]
  tabout[,4] = 2*(1-pnorm(abs(tabout[,3])))
  colnames(tabout) = c("Coefficient", "Std. Error", "z value", "Pr(>|z|)")
  rownames(tabout) = varlist
  }else tabout = wtp_mean
  if(indv.obs){
    wtp_mat = matrix(ncol = k, nrow=nrow(object$marg.list[[1]]))
    for(c in var_colNo){
      c1 = which(var_colNo == c)
      wtp_mat[,c1] = as.matrix(object$marg.list[[c1]]) %*% wtp.vec
    }
    colnames(wtp_mat) = varlist
  }
  # output list
  outlist = list()
  outlist$wtp = tabout
  if(indv.obs) outlist$wtp.obs = wtp_mat
  return(structure(outlist,class="fracregmlogit.wtp"))
}
  

#' @title Partial Effects for Fractional Panel Data Regression
#'
#' @description
#' Computes Average Partial Effects (APEs) for fractional panel data models fit using \code{fracregpd}. In correlated random effects (CRE) models, the time-averages of the covariates are used merely to control for unobserved heterogeneity, and as such, they are automatically filtered out when computing the structural partial effects. Standard errors are computed using the Delta method and the bootstrapped parameter variance matrix.
#' @param object An object of class \code{fracregpd}.
#' @param APE logical. Compute Average Partial Effects?
#' @param CPE logical. Compute Conditional Partial Effects? (Not currently supported for panel data models).
#' @param at numeric vector. The values at which to evaluate the CPE.
#' @param which.x character vector. Variables for which to compute partial effects. By default, auxiliary CRE parameters (like \code{_mean} and \code{vhat}) are automatically excluded so that partial effects are only computed for the main structural parameters.
#' @param variance logical. Compute standard errors using the Delta method?
#' @param table logical. Print the resulting partial effects table?
#' @param ... further arguments passed to or from other methods.
#'
#' @return
#' An object of class \code{fracreg.pe} containing the standard coefficient tables with Average Partial Effects.
#'
#' @author Sulman Olieko Owili <oliekosulman@gmail.com>
#'
#' @seealso
#' \code{\link{fracregpd}}, \code{\link{fracreg.pe}}
#' @export
fracregpd.pe <- function(object, APE=TRUE, CPE=FALSE, at=NULL, which.x=NULL, variance=TRUE, table=FALSE, ...) {
    if(missing(object)) stop("object is missing")
    if(is.null(object$class)) {
        if(!inherits(object, "fracregpd")) stop("object is not the output of a fracregpd command")
    } else {
        if(object$class != "fracregpd") stop("object is not the output of a fracregpd command")
    }

    if(!is.logical(APE)) stop("non-logical value assigned to option APE")
    if(!is.logical(CPE)) stop("non-logical value assigned to option CPE")
    if(!is.logical(variance)) stop("non-logical value assigned to option variance")
    if(!is.logical(table)) stop("non-logical value assigned to option table")

    if(all(c(APE,CPE)==FALSE)) stop("You must specify at least one option: APE and/or CPE")
    if(CPE==FALSE & !is.null(at)) stop("option at is only required for CPE")

    if(object$converged==0) stop("object is not the output of a successful (converged) fracregpd command")

    x.names <- object$table.info$x.names
    
    # Filter out intercept and CRE parameters
    aux_vars <- grep("_mean$|^vhat$", x.names, value = TRUE)
    struct_vars <- setdiff(x.names, aux_vars)
    if ("(Intercept)" %in% struct_vars) {
        struct_vars <- setdiff(struct_vars, "(Intercept)")
    }
    if (length(struct_vars) == 0) stop("No structural variables found to compute partial effects.")

    if(is.null(which.x)) which.x <- struct_vars
    xw.names <- unique(c(struct_vars,which.x))
    if(!all(which.x %in% x.names)) stop("option which.x contains variables not in the model")

    xvar.names <- struct_vars
    k <- length(xvar.names)
    npar <- length(object$p)
    n <- nrow(object$x)

    linka <- object$link
    pa <- object$p
    pa.var <- object$table.info$p.var
    x <- object$x
    xbhata <- object$xbhat
    
    title <- paste("Panel Data Fractional", linka, "regression")
    type <- "1P" # Treat structurally as 1P for PE variance computation

    resAPE <- list()
    resCPE <- list()

    if(APE==TRUE) {
        PE.type <- "APE"

        if(any(x.names=="(Intercept)")) p.pe <- matrix(rep(pa[-1],each=n),ncol=npar-1)
        else p.pe <- matrix(rep(pa,each=n),ncol=npar)
        
        # We need to map p.pe columns to x.names (excluding intercept)
        xnames_noint <- setdiff(x.names, "(Intercept)")
        dimnames(p.pe) <- list(NULL, xnames_noint)

        ga <- fracregpd.links(linka)$mu.eta(xbhata)
        PEa.p <- as.matrix(p.pe[,which.x])*ga
        PE.p <- apply(PEa.p,2,mean)

        if(variance==TRUE) {
            # Note: fracreg.pe.var relies on type, p, etc. 
            # We map the inputs directly to fracreg.pe.var expectations
            PE.sd <- fracreg.pe.var(x,npar,which.x,x.names,xnames_noint,"1P",pa,xbhata,ga,linka,pa.var)
        } else {
            PE.sd <- NA
        }

        table.info.APE <- list(PE.p=PE.p,PE.sd=if(variance) PE.sd else NA,PE.type=PE.type,which.x=which.x,xvar.names=xvar.names,title=title)
        if(table==TRUE) do.call(fracreg.pe.table, table.info.APE)
        resAPE[["table.info"]] <- table.info.APE
    }

    if(CPE==TRUE) {
        PE.type <- "CPE"
        # CPE computation similar to fracreg.pe
        # Since it's panel data with CRE, the user needs to provide values for ALL parameters including _mean.
        # This is complex, so we will throw an error if CPE is requested for now unless we fully support it.
        stop("CPE is currently not supported for fracregpd models due to CRE auxiliary parameters. Use APE.")
    }

    if(APE==TRUE & CPE==TRUE) res <- list(ape=resAPE,cpe=resCPE)
    else if(APE==TRUE & CPE==FALSE) res <- resAPE
    else if(APE==FALSE & CPE==TRUE) res <- resCPE
    
    class(res) <- c("fracregpd.pe", "fracreg.pe")
    return(invisible(res))
}

fracregpd.links <- function(link) 
{
	switch(link,
		logit = {
			linkfun <- function(mu) qlogis(mu)
			linkinv <- function(eta) plogis(eta)
			mu.eta <- function(eta) dlogis(eta)
			H1 <- function(dep) dep/(1-dep)
			G2 <- function(eta) exp(eta)
			g2 <- function(eta) exp(eta)
			H2 <- function(uu) log(uu)
			gd <- function(eta) exp(eta)*(1-exp(eta))/((1+exp(eta))^3)
			valideta <- function(eta) TRUE
		},

		probit = {
			linkfun <- function(mu) qnorm(mu)
			linkinv <- function(eta) {
				thresh <- -qnorm(.Machine$double.eps)
				eta <- pmin(pmax(eta, -thresh), thresh)
	 			pnorm(eta)
			}
			mu.eta <- function(eta) pmax(dnorm(eta),.Machine$double.eps)
			H1 <- NULL
			G2 <- NULL
			g2 <- NULL
			H2 <- NULL
			gd <- function(eta) -eta*dnorm(eta)
			valideta <- function(eta) TRUE
		},

		cloglog = {
			linkfun <- function(mu) log(-log(1 - mu))
			linkinv <- function(eta) pmax(pmin(-expm1(-exp(eta)),1-.Machine$double.eps),.Machine$double.eps)
			mu.eta <- function(eta) {
				eta <- pmin(eta, 700)
				pmax(exp(eta) * exp(-exp(eta)),.Machine$double.eps)
			}
			H1 <- function(dep) -log(1-dep)
			G2 <- function(eta) exp(eta)
			g2 <- function(eta) -exp(-eta)
			H2 <- function(uu) log(uu)
			gd <- function(eta) (exp(-exp(eta))*exp(eta))*(1-exp(eta))
			valideta <- function(eta) TRUE
		}
	)

	structure(list(linkfun=linkfun,linkinv=linkinv,mu.eta=mu.eta,H1=H1,G2=G2,g2=g2,H2=H2,gd=gd,valideta=valideta,name=link),class="link-glm")
}

fracregpd.gi <- function(type,id,Ti,Hy,z,XB,link,at,at1)
{
	if(any(type==c("GMMc","GMMww","GMMbgw")) | (type=="GMMpfe" & !any(Hy==0)))
	{
		XBexp <- exp(XB)

		if(type=="GMMc") u <- (XBexp[at1]/XBexp[at])*Hy[at]-Hy[at1]
		if(type=="GMMww")
		{
			u <- Hy/XBexp
			u <- u[at]-u[at1]
		}
		if(type=="GMMbgw")
		{
			Hy.m <- rep(as.vector(by(Hy[at],id,mean)),times=Ti)
			XBexp.m <- rep(as.vector(by(XBexp[at],id,mean)),times=Ti)
			u <- Hy[at]-(XBexp[at]/XBexp.m)*Hy.m
		}
		if(type=="GMMpfe")
		{
			HyXBexp.m <- rep(as.vector(by(Hy[at]/XBexp[at],id,mean)),times=Ti)
			u <- Hy[at]/(XBexp[at]*HyXBexp.m)-1
		}
	}
	if(any(type==c("GMMpre","GMMcre")) | (type=="GMMpfe" & any(Hy==0)))
	{
		G2 <- fracregpd.links(link)$G2(XB)
		u <- Hy[at]/G2[at]-1
	}
	if(type=="QMLcre")
	{
		yhat <- fracregpd.links(link)$linkinv(XB)
		g <- fracregpd.links(link)$mu.eta(XB)
		u <- (Hy-yhat)*g/(yhat*(1-yhat))
	}

	gi <- t(z*u)

	return(list(gi=gi,u=u))
}

fracregpd.Gn <- function(type,x.exogenous,id,Ti,Hy,x,z,XB,link,at,at1,NT,k,p,z.in,u)
{
	if(any(type==c("GMMc","GMMww","GMMbgw")) | (type=="GMMpfe" & !any(Hy==0)))
	{
		XBexp <- exp(XB)

		if(type=="GMMc") Gn <- t(z*((XBexp[at1]/XBexp[at])*Hy[at]))%*%(x[at1,]-x[at,])
		if(type=="GMMww")
		{
			uu <- -Hy/XBexp
			Gn <- t(z)%*%(uu[at]*x[at,]-uu[at1]*x[at1,])
		}
		if(type=="GMMbgw")
		{
			Hy.m <- rep(as.vector(by(Hy[at],id,mean)),times=Ti)
			XBexp.m <- rep(as.vector(by(XBexp[at],id,mean)),times=Ti)

			XBexpX.m <- matrix(NA,nrow=NT,ncol=k)
			for(j in 1:k) XBexpX.m[,j] <- rep(as.vector(by(XBexp[at]*x[at,j],id,mean)),times=Ti)

			Gn <- -t(z*(XBexp[at]*Hy.m/XBexp.m))%*%x[at,]+t(z)%*%((XBexp[at]*Hy.m/((XBexp.m)^2))*XBexpX.m)
		}
		if(type=="GMMpfe")
		{
			HyXBexp.m <- rep(as.vector(by(Hy[at]/XBexp[at],id,mean)),times=Ti)

			HyXBexpX.m <- matrix(NA,nrow=NT,ncol=k)
			for(j in 1:k) HyXBexpX.m[,j] <- rep(as.vector(by((Hy[at]/XBexp[at])*x[at,j],id,mean)),times=Ti)
			Gn <- -t(z*(Hy[at]/(XBexp[at]*HyXBexp.m)))%*%x[at,]+t(z)%*%(Hy[at]/(XBexp[at]*(HyXBexp.m^2))*HyXBexpX.m)
		}
	}
	if(any(type==c("GMMpre","GMMcre")) | (type=="GMMpfe" & any(Hy==0)))
	{
		G2 <- fracregpd.links(link)$G2(XB)
		g2 <- fracregpd.links(link)$g2(XB)
		bet <- Hy*g2/(G2^2)
		Gn <- -t(z*bet[at])%*%x[at,]
	}
	if(type=="QMLcre")
	{
		yhat <- fracregpd.links(link)$linkinv(XB)
		g <- fracregpd.links(link)$mu.eta(XB)
		bet <- g^2/(yhat*(1-yhat))
		Gn <- -t(z*bet)%*%x

		if(x.exogenous==FALSE)
		{
			G12 <- t(x*(bet*p[k]))%*%z.in
			G12[k,] <- G12[k,]-apply(u*z.in,2,sum)

			G21 <- matrix(0,nrow=ncol(z.in),ncol=k)
			G22 <- -t(z.in)%*%z.in

			Gn <- rbind(cbind(Gn,G12),cbind(G21,G22))
		}
	}

	Gn <- Gn/NT

	return(Gn)
}

fracregpd.est <- function(type,x.exogenous,lags,id,Ti,Hy,x,z,link,var.type,start,at,at1,variance,NT,k,kz,gixv,vhat,bootstrap,offset=NULL,...)
{
	if(type=="QMLcre" & x.exogenous==FALSE)
	{
		z.in <- z
		z <- x
		kz <- ncol(z)
	}

	GMM.est <- TRUE
	if(x.exogenous==TRUE & ((any(type==c("GMMpre","GMMcre")) & !any(Hy==0) & lags==FALSE) | type=="QMLcre"))
	{
		if(type=="QMLcre") results <- tryCatch(glm(Hy ~ x-1,family=quasibinomial(link=fracregpd.links(link)),maxit=100,offset=offset),error=function(e) return(NULL))
		if(any(type==c("GMMpre","GMMcre"))) results <- tryCatch(glm(Hy ~ x-1,family=Gamma(link=log),maxit=100,offset=offset),error=function(e) return(NULL))

		if(any(is.null(results))) converged <- FALSE
		else
		{
			p <- results$coefficients
			XB <- results$linear.predictors
			converged <- results$converged*(1-results$boundary)
		}

		if(converged==TRUE) GMM.est <- FALSE
	}
	if(GMM.est==TRUE)
	{
		GMMn <- function(p)
		{
			XB <- as.vector(x%*%p)
			if(!is.null(offset)) XB <- XB + offset
			gi <- fracregpd.gi(type,id,Ti,Hy,z,XB,link,at,at1)$gi
			gn <- as.matrix(apply(gi,1,mean))
			Qn <- t(gn)%*%S%*%gn

			return(Qn)
		}

		S <- diag(kz)
		results <- nlminb(start=start,objective=GMMn,...)
		p <- results$par
		XB <- as.vector(x%*%p)
		if(!is.null(offset)) XB <- XB + offset

		if(type!="QMLcre" & kz>k)
		{
			fi.inv <- fracregpd.var(type,x.exogenous,var.type,id,Ti,Hy,x,z,XB,link,at,at1,NT,k,kz,TRUE,gixv,vhat,p)$fi.inv
			if(!is.character(fi.inv))
			{
				S <- fi.inv
				results <- nlminb(start=start,objective=GMMn,...)
				p <- results$par
				XB <- as.vector(x%*%p)
				if(!is.null(offset)) XB <- XB + offset
			}

			Qn <- results$objective
		}

		converged <- ifelse(results$convergence==0,TRUE,FALSE)
	}

	ret.list <- list(p=p,converged=converged)
	if(type!="QMLcre" & kz>k) ret.list[["Qn"]] <- Qn
	if (type == "QMLcre") {
		yhat <- fracregpd.links(link)$linkinv(XB)
		eps <- 1e-16
		LL <- sum(ifelse(Hy > 0, Hy * log(pmax(yhat, eps)), 0) + ifelse(Hy < 1, (1-Hy) * log(pmax(1-yhat, eps)), 0))
		ret.list[["LL"]] <- LL
	}
	
	ret.list[["xbhat"]] <- XB

	if(variance==FALSE | converged==FALSE | bootstrap==TRUE) return(ret.list)

	if(type=="QMLcre" & x.exogenous==FALSE) z <- z.in
	p.var <- fracregpd.var(type,x.exogenous,var.type,id,Ti,Hy,x,z,XB,link,at,at1,NT,k,kz,FALSE,gixv,vhat,p)$p.var
	ret.list[["p.var"]] <- p.var

	return(ret.list)
}

fracregpd.var <- function(type,x.exogenous,var.type,id,Ti,Hy,x,z,XB,link,at,at1,NT,k,kz,step.one,gixv,vhat,p)
{
	if(type=="QMLcre" & x.exogenous==FALSE)
	{
		z.in <- z
		z <- x
		kz <- ncol(z)
	}
	else z.in <- z

	results <- fracregpd.gi(type,id,Ti,Hy,z,XB,link,at,at1)
	u <- results$u

	if(var.type=="robust")
	{
		gi <- results$gi
		if(type=="QMLcre" & x.exogenous==FALSE) gi <- rbind(gi,gixv)
		fi <- (1/NT)*gi%*%t(gi)
	}

	if(var.type=="cluster")
	{
		fi <- 0

		for(j in unique(id))
		{
			Zi <- matrix(z[id==j,],ncol=kz)
			ui <- u[id==j]
			zu <- t(Zi)%*%ui

			if(type=="QMLcre" & x.exogenous==FALSE)
			{
				Zi <- matrix(z.in[id==j,],ncol=ncol(z.in))
				vi <- vhat[id==j]
				zu2 <- t(Zi)%*%vi
				zu <- rbind(zu,zu2)
			}

			fi <- fi+zu%*%t(zu)
		}

		fi <- fi/NT
	}

	fi.inv <- tryCatch(solve(fi),error=function(e) NaN)
	if(any(is.nan(fi.inv))) fi.inv <- "singular"

	if(step.one==TRUE) return(list(fi.inv=fi.inv))

	Gn <- fracregpd.Gn(type,x.exogenous,id,Ti,Hy,x,z,XB,link,at,at1,NT,k,p,z.in,u)

	if(is.numeric(fi.inv))
	{
		sigma <- NT*t(Gn)%*%fi.inv%*%Gn
		p.var <- tryCatch(solve(sigma),error=function(e) NaN)
		if(any(is.nan(p.var))) p.var <- "singular"
	}
	else p.var <- "singular"

	if(is.character(p.var))
	{
		if(k!=kz) p.var <- "singular"
		else
		{
			Gn.inv <- tryCatch(solve(Gn),error=function(e) NaN)
			if(any(is.nan(Gn.inv))) p.var <- "singular"
			else p.var <- (1/NT)*Gn.inv%*%fi%*%t(Gn.inv)
		}
	}

	ret.list <- list(p.var=p.var)

	return(ret.list)
}

#' @title Fitting Panel Data Fractional Response Regressions
#'
#' @description
#' \code{fracregpd} is used to fit panel data regression models when the dependent variable has a bounded, fractional nature.
#' @param id a numeric vector identifying the cross-sectional units.
#' @param time a numeric vector identifying the time periods in which the cross-sectional units were observed.
#' @param y a numeric vector containing the values of the response variable.
#' @param x a numeric matrix, with column names, containing the values of all covariates (exogenous and endogenous).
#' @param z a numeric matrix, with column names, containing the values of all exogenous variables (covariates and external instrumental variables). Only required in case of endogenous explanatory variables.
#' @param var.endog a numeric vector containing the values of the endogenous covariate (or of some transformation of it), which will be used as dependent variable in the linear reduced form assumed for application of the \code{QMLcre} estimator. Only required for this estimator.
#' @param x.exogenous a logical value indicating whether all explanatory variables are assumed to be exogenous or not.
#' @param lags a logical value indicating whether the first lags of \code{x} or  \code{z} should be used as instruments for \code{x}. Defaults to \code{TRUE} for the GMMww and GMMc estimators and to \code{FALSE} for the remaining estimators. The \code{GMMcre} and \code{QMLcre} estimators do not admit lagged instruments.
#' @param start a numeric vector containing the initial values for the parameters to be optimised. Optional.
#' @param type a description of the estimator to compute: \code{GMMww}, \code{GMMc}, \code{GMMbgw}, \code{GMMpfe}, \code{GMMcre}, \code{GMMpre} or \code{QMLcre}.
#' @param GMMww.cor a logical value indicating whether each explanatory variable should be transformed in deviations from its overall mean before computing the \code{GMMww} estimator.
#' @param link a description of the link function to use. Available options for all GMM estimators: \code{logit} and \code{cloglog}. Only option for the \code{QMLcre} estimator: \code{probit}.
#' @param intercept a logical value indicating whether the model should include a constant term or not. Only relevant for the  \code{GMMpre} estimator.
#' @param table a logical value indicating whether a summary table with the regression results should be printed.
#' @param variance a logical value indicating whether the variance of the estimated parameters should be calculated. Defaults to  \code{TRUE} whenever \code{table = FALSERUE}.
#' @param var.type a description of the type of variance of the estimated parameters to be calculated. Options are \code{cluster}, the default, and \code{robust}. In overidentified models, it also affects the parameter estimates via the GMM weighting matrix.
#' @param tdummies a logical value indicating whether time dummies should be included among the model explanatory variables.
#' @param bootstrap a logical value indicating whether bootstrap should be used in the estimation of the parameter standard errors.
#' @param B the number of bootstrap replications.
#' @param offset an optional numeric vector containing an offset. It must be of the same dimension as the response variable. It specifies that the variable should be included in the model with its coefficient constrained to 1.
#' @param or a logical value indicating whether to report odds ratios. Only valid when the link function is \code{"logit"}. Defaults to \code{FALSE}.
#' @param level a numeric value between 0 and 1 indicating the confidence level for the confidence intervals. Defaults to \code{0.95}.
#' @param na.action A function specifying how to handle missing values, default is \code{stats::na.omit}. If \code{NULL}, no action is taken.
#' @param \dots Arguments to pass to \link[stats]{nlminb}.
#'
#' @details
#' \code{fracregpd} computes the GMM estimators proposed in Ramalho, Ramalho and Coelho (2018) for panel data fractional response models with both time-variant and time-invariant unobserved heterogeneity and endogeneous covariates: GMMww, GMMc, GMMbgw, GMMpfe, GMMcre and GMMpre. In addition, \code{fracregpd} also computes QMLcre, which was proposed by Papke and Wooldridge (2008) and Wooldridge (2019). 
#' 
#' \strong{Correlated Random Effects (CRE) - QMLcre:}
#' In panel data, unobserved individual-specific heterogeneity \eqn{c_i} may be correlated with the covariates \eqn{x_{it}}. The CRE approach (Papke and Wooldridge, 2008) models this dependence by projecting \eqn{c_i} onto the time averages of the strictly exogenous covariates \eqn{\bar{x}_i}:
#' \deqn{c_i = \psi + \bar{x}_i \xi + a_i}
#' where \eqn{a_i} is an error term independent of \eqn{x_i}. Assuming \eqn{a_i | x_i \sim N(0, \sigma_a^2)} and a probit link, integrating out \eqn{a_i} yields the "population-averaged" or scaled conditional mean:
#' \deqn{E(y_{it} | x_i) = G(x_{it} \beta_a + \psi_a + \bar{x}_i \xi_a)}
#' where the parameters with subscript \eqn{a} are scaled by \eqn{(1 + \sigma_a^2)^{-1/2}}. This equation is estimated via pooled Bernoulli QML.
#' 
#' \strong{Generalised Method of Moments (GMM):}
#' For models where strict exogeneity fails or the link function is an exponential-type link, Ramalho et al. (2018) propose GMM estimators based on the following general moment conditions:
#' \deqn{E[Z_{it} (H(y_{it}) - \exp(x_{it}\beta + c_i))] = 0}
#' where \eqn{H(\cdot)} is a transformation function and \eqn{Z_{it}} is a matrix of valid instruments. Estimators such as GMMww, GMMc, and GMMbgw use different transformations to eliminate the unobserved fixed effect \eqn{c_i} before applying GMM.
#' 
#' For overidentified models, \code{fracregpd} calculates Hansen's J statistic to test the validity of the overidentifying restrictions.
#'
#' @return
#' \code{fracregpd} returns a list with the following elements:
#'   \item{type}{the name of the estimator computed.
#' }
#'   \item{link}{the name of the specified link.
#' }
#'   \item{p}{a named vector of coefficients.
#' }
#'   \item{Hy}{the transformed values of the response variable when GMM estimators are computed or the 
#'    values of the response variable in the QML case.
#' }
#'   \item{converged}{logical. Was the algorithm judged to have converged?
#' }
#' 
#' In case of an overidentifying model, the following element is also returned:
#'   \item{J}{the result of Hansen's J test of overidentifying moment conditions.
#' }
#' 
#' If \code{variance = TRUE} or \code{table = FALSERUE} and the algorithm converged successfully, the previous list also contains the following elements:
#'   \item{p.var}{a named covariance matrix.
#' }
#'   \item{var.type}{covariance matrix type.
#' }
#'
#' @section Odds Ratios:
#' When \code{or=TRUE} and the fractional link function (\code{linkfrac} or \code{link}) is \code{"logit"}, the model additionally computes odds ratios for the coefficients. 
#' Odds Ratios are exponentiated coefficients.
#' The corresponding standard errors for the odds ratios are calculated using the Delta method.
#' The confidence intervals for the odds ratios are calculated using the adjusted standard errors and the specified \code{level} (defaulting to 95\%).
#' Odds ratios are particularly useful in fractional logit models as they provide a direct multiplicative interpretation of the independent variable on the odds of the fractional outcome.
#'
#' @references
#' Papke, L. and Wooldridge, J.M. (2008), "Panel data methods for fractional response variables with an application to test pass rates", \emph{Journal of Econometrics}, 145(1-2), 121-133.
#' 
#' Ramalho, E. A., Ramalho, J. J. S., & Coelho, L. M. S. (2018), "Exponential Regression of Fractional-Response Fixed-Effects Models with an Application to Firm Capital Structure", \emph{Journal of Econometric Methods}, 7(1), 20150019.
#' 
#' Wooldridge, J. M. (2019). Correlated random effects models with unbalanced panels. \emph{Journal of Econometrics}, 211(1), 137-150.
#'
#' @author Sulman Olieko Owili <oliekosulman@gmail.com>
#'
#' @seealso
#' \code{\link{fracreg}}, for fitting standard cross-sectional fractional response models.\cr
#' \code{\link{fracreghet}}, for fitting cross-sectional fractional response models with unobserved heterogeneity.
#'
#' @examples
#' ### Empirical 401(k) Examples
#' data("fracreg_k401k")
#' y <- fracreg_k401k$prate
#' X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age, 
#'            totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole)
#' 
#' # Artificial panel data structure for demonstration
#' N_emp <- nrow(X)
#' id_emp <- rep(1:(N_emp/2), each=2)
#' time_emp <- rep(1:2, times=N_emp/2)
#' mod <- fracregpd(id_emp, time_emp, y, X, type="QMLcre", link="probit")
#' summary(mod)
#' 
#' ### Simulated Examples
#' 
#' set.seed(123)
#' # Simulating Panel Data
#' N <- 100
#' T_periods <- 5
#' id <- rep(1:N, each = T_periods)
#' time <- rep(1:T_periods, times = N)
#' x_panel <- rnorm(N * T_periods)
#' 
#' # Unobserved individual effect (CRE)
#' c_i <- rep(rnorm(N), each = T_periods) 
#' y_panel <- exp(x_panel + c_i) / (1 + exp(x_panel + c_i))
#' 
#' X <- cbind(x_panel = x_panel)
#' 
#' # Endogenous variable and instrument simulation
#' z_panel <- rnorm(N * T_periods)
#' u_panel <- 0.5 * z_panel + rnorm(N * T_periods)
#' var_endog <- 0.8 * z_panel + u_panel
#' y_endog <- exp(x_panel + 1.2 * var_endog + c_i + u_panel) / 
#'              (1 + exp(x_panel + 1.2 * var_endog + c_i + u_panel))
#' 
#' X_endog <- cbind(x_panel = x_panel, var_endog = var_endog)
#' Z_inst <- cbind(x_panel = x_panel, z_panel = z_panel)
#' 
#' \donttest{
#' # Estimate a Correlated Random Effects (CRE) Model
#' mod <- fracregpd(id=id, time=time, y=y_panel, x=X, type="QMLcre", link="probit")
#' summary(mod)
#' 
#' # Compute Partial Effects
#' pe_res <- fracregpd.pe(mod)
#' summary(pe_res)
#' 
#' # Exogeneity, no lags, no time dummies, clustered standard errors, GMMbgw estimator
#' mod <- fracregpd(id=id, time=time, y=y_panel, x=X, type="GMMbgw")
#' summary(mod)
#' 
#' # Estimate the GMMww estimator with odds ratios and 99% confidence intervals
#' mod <- fracregpd(id=id, time=time, y=y_panel, x=X, type="GMMww", or=TRUE, level=0.99)
#' summary(mod)
#' 
#' # Lagged covariates and instruments, robust standard errors, GMMww estimator
#' mod <- fracregpd(id=id, time=time, y=y_panel, x=X, lags=TRUE, type="GMMww", var.type="robust")
#' summary(mod)
#' 
#' # Endogeneity, time dummies, GMMpfe estimator
#' mod <- fracregpd(id=id, time=time, y=y_endog, x=X_endog, z=Z_inst,
#'                  x.exogenous=FALSE, type="GMMpfe", tdummies=TRUE)
#' summary(mod)
#' }
#' @export
fracregpd <- function(id,time,y,x,z,var.endog,x.exogenous=TRUE,lags,start,type,GMMww.cor=TRUE,link="logit",intercept=TRUE,table=FALSE,variance=TRUE,var.type="cluster",tdummies=FALSE,bootstrap=FALSE,B=200,offset=NULL,or=FALSE,level=0.95,na.action=stats::na.omit,...)
{
	cl <- match.call()
	
	if (!missing(id) && !missing(time) && !missing(y) && !missing(x)) {
	    args_to_clean <- list(id=id, time=time, y=y, x=x, offset=offset)
	    if (!missing(z)) args_to_clean$z <- z
	    if (!missing(var.endog)) args_to_clean$var.endog <- var.endog
	    
	    args_to_clean$na.action <- na.action
	    cleaned <- do.call(fracreg_clean_data, args_to_clean)
	    
	    id <- cleaned$id
	    time <- cleaned$time
	    y <- cleaned$y
	    x <- cleaned$x
	    offset <- cleaned$offset
	    if (!missing(z)) z <- cleaned$z
	    if (!missing(var.endog)) var.endog <- cleaned$var.endog
	}
	
	### 1. Error and warning messages

	if(missing(id)) stop("variable id is missing")
	if(missing(time)) stop("variable time is missing")
	if(missing(y)) stop("dependent variable is missing")
	if(missing(x)) stop("explanatory variables are missing")

	if(missing(type)) stop("type must be specified")
	if(any(y>1) | any(y<0)) stop("The dependent variable has values outside the unit interval")
	if(!is.logical(x.exogenous)) stop("x.exogenous must be logical")
	if(!is.logical(GMMww.cor)) stop("GMMww.cor must be logical")
	if(!is.logical(intercept)) stop("intercept must be logical")
	if(!is.logical(table)) stop("table must be logical")
	if(!is.logical(variance)) stop("variance must be logical")
	if(!is.logical(tdummies)) stop("tdummies must be logical")
	if(all(link!=c("logit","probit","cloglog"))) stop(sQuote(link)," - link not recognised")
	if(all(type!=c("GMMc","GMMww","GMMbgw","GMMpre","GMMpfe","GMMcre","QMLcre"))) stop(sQuote(type)," - type not recognised")
	if(any(type==c("GMMc","GMMww","GMMbgw","GMMpre","GMMpfe","GMMcre")) & all(link!=c("logit","cloglog"))) stop("type and link not compatible")
	if(any(type==c("GMMc","GMMww","GMMbgw","GMMpre","GMMpfe","GMMcre")) & any(y==1)) stop("estimator does not allow y = 1")
	if(type=="QMLcre" & link!="probit") stop("type and link not compatible")
	if(!missing(z) & x.exogenous==TRUE) stop("z must not be specified in case of exogeneity")
	if(missing(z) & x.exogenous==FALSE) stop("z needs to be specified in case of endogeneity")
	if(type=="QMLcre" & x.exogenous==FALSE & missing(var.endog)) stop("QMLcre under endogeneity requires var.endog to be specified")
	if((type!="QMLcre" | x.exogenous==TRUE) & !missing(var.endog)) stop("var.endog should not be specified for this estimator")
	if(table==TRUE & variance==FALSE)
	{
		variance <- TRUE
		warning("option variance changed from FALSE to TRUE, as required by table=TRUE")
	}
	if(all(var.type!=c("robust","cluster"))) stop(sQuote(var.type)," - var.type not recognised")

	if(any(unlist(by(time,id,duplicated)))) stop("variable time has duplicated entries for the same id")

	### 2. Data and variables preparation - general

	id <- as.vector(id)
	time <- as.vector(time)
	y <- as.vector(y)
	if(missing(z)) z <- x

	if(is.data.frame(x)) x <- as.matrix(x)
	if(is.data.frame(z)) z <- as.matrix(z)

	if(!is.matrix(x)) stop("x is not a matrix")
	if(!is.matrix(z)) stop("z is not a matrix")

	if(any(is.na(id))) stop("id has missing values")
	if(any(is.na(time))) stop("time has missing values")
	if(any(is.na(y))) stop("y has missing values")
	if(any(is.na(x))) stop("x has missing values")
	if(any(is.na(z))) stop("z has missing values")

	x.names <- dimnames(x)[[2]]
	z.names <- dimnames(z)[[2]]

	if(is.null(x.names)) stop("x has no column names")
	if(is.null(z.names)) stop("z has no column names")

	if(any(type==c("GMMc","GMMww","GMMbgw","GMMpfe","GMMcre","QMLcre"))) intercept <- FALSE
	if(intercept==TRUE)
	{
		x <- cbind(1,x)
		z <- cbind(1,z)
		x.names <- c("(Intercept)",x.names)
		z.names <- c("(Intercept)",z.names)
	}

	if(length(x.names)!=length(unique(x.names))) stop("some covariate names in x are identical")
	if(length(z.names)!=length(unique(z.names))) stop("some instrument names in z are identical")
	if(identical(x.names,z.names) & x.exogenous==FALSE) stop("instruments and covariates are identical")
	if(!identical(x.names,z.names) & x.exogenous==TRUE) stop("instruments should not be specified for this estimator")
	if(length(x.names)>length(z.names)) stop("number of instruments not enough")

	if(any(length(y)!=c(nrow(x),nrow(z)))) stop("the number of observations for y, x and/or z is different")
	if(!missing(var.endog))
	{
		if(length(y)!=length(var.endog)) stop("var.endog does not have the appropriate dimension")
	}
	if(!is.null(offset) && length(y)!=length(offset)) stop("offset does not have the appropriate dimension")

	### 3. Data and variables preparation - panel / estimator specifics

	if(missing(lags))
	{
		if(any(type==c("GMMc","GMMww"))) lags <- TRUE
		if(any(type==c("GMMbgw","GMMpre","GMMpfe","GMMcre","QMLcre"))) lags <- FALSE
	}
	if(!is.logical(lags)) stop("lags must be logical")
	if(any(type==c("GMMcre","QMLcre")) & lags==TRUE) stop("GMMcre/QMLcre cannot be used with lagged instruments")

	N.ini <- length(unique(id))
	Ti.ini <- as.vector(by(id,id,length))
	NT.ini <- length(id)

	if(any(type==c("GMMc","GMMww","GMMbgw","GMMpfe","GMMcre","QMLcre")))
	{
		for(j in 1:ncol(x))
		{
			xm <- rep(as.vector(by(x[,j],id,mean)),times=Ti.ini)
			x.dif <- x[,j]-xm

			if(all(x.dif==0)) stop("Time-invariant covariates not allowed")

			if(any(type==c("GMMcre","QMLcre")) & x.exogenous==TRUE)
			{
				if(j==1) x.m <- cbind(xm)
				if(j>1) x.m <- cbind(x.m,xm)
			}
		}
		for(j in 1:ncol(z))
		{
			zm <- rep(as.vector(by(z[,j],id,mean)),times=Ti.ini)
			z.dif <- z[,j]-zm

			if(all(z.dif==0)) stop("Time-invariant instruments not allowed")

			if(any(type==c("GMMcre","QMLcre")) & x.exogenous==FALSE)
			{
				if(j==1) z.m <- cbind(zm)
				if(j>1) z.m <- cbind(z.m,zm)
			}
		}
	}

	TRUE.ord <- rep(NA,NT.ini)

	if(tdummies==TRUE)
	{
		if(lags==TRUE) out <- 2
		else out <- 1

		td <- matrix(0,nrow=NT.ini,ncol=length(unique(time))-out)
	}

	a <- 0
	for(j in sort(unique(time)))
	{
		a <- a+1
		TRUE.ord[time==j] <- a
		if(tdummies==TRUE)
		{
			if(a>out)
			{
				td[time==j,a-out] <- 1
				if(a==out+1) td.names <- paste("time",j,sep=".")
				if(a>(out+1)) td.names <- c(td.names,paste("time",j,sep="."))
			}
		}
	}

	if(type=="GMMpfe" & any(y==0))
	{
		D <- matrix(0,nrow=NT.ini,ncol=N.ini)
		a <- 0
		for(j in sort(unique(id)))
		{
			a <- a+1
			D[id==j,a] <- 1
			if(a==1) D.names <- paste("D",j,sep=".")
			if(a>1) D.names <- c(D.names,paste("D",j,sep="."))
		}

	}

	ord <- order(id,time)
	id <- id[ord]
	TRUE.ord <- TRUE.ord[ord]
	y <- y[ord]
	x <- cbind(x[ord,])
	if(tdummies==TRUE) td <- cbind(td[ord,])
	if(type=="GMMpfe" & any(y==0)) D <- D[ord,]
	z <- cbind(z[ord,])
	if(!is.null(offset)) offset <- offset[ord]

	if(any(type==c("GMMc","GMMww")) | lags==TRUE)
	{
		at <- rep(FALSE,NT.ini)
		at1 <- rep(FALSE,NT.ini)

		if(any(Ti.ini!=max(Ti.ini)))
		{
			keep <- rep(FALSE,NT.ini)

			for(j in 1:NT.ini)
			{
				if(j==1 & (TRUE.ord[2]-TRUE.ord[1])==1 & id[1]==id[2]) at1[1] <- TRUE
				if(j>1 & j<NT.ini)
				{
					if((TRUE.ord[j]-TRUE.ord[j-1])==1 & id[j]==id[j-1]) at[j] <- TRUE
					if((TRUE.ord[j+1]-TRUE.ord[j])==1 & id[j]==id[j+1]) at1[j] <- TRUE
				}
				if(j==NT.ini & (TRUE.ord[NT.ini]-TRUE.ord[NT.ini-1])==1 & id[NT.ini]==id[NT.ini-1]) at[NT.ini] <- TRUE
			}

			keep <- at==TRUE | at1==TRUE
			id <- id[keep==TRUE]
			y <- y[keep==TRUE]
			x <- cbind(x[keep==TRUE,])
			if(tdummies==TRUE) td <- cbind(td[keep==TRUE,])
			if(type=="GMMpfe" & any(y==0)) D <- D[keep==TRUE,]
			z <- cbind(z[keep==TRUE,])
			at <- at[keep==TRUE]
			at1 <- at1[keep==TRUE]
			if(!is.null(offset)) offset <- offset[keep==TRUE]
		}
		else
		{
			at[TRUE.ord!=1] <- TRUE
			at1[TRUE.ord!=max(Ti.ini)] <- TRUE			
		}

		z.full <- z
		id.full <- id

		if(lags==TRUE)
		{
			z <- cbind(z[at1,])
			id <- id[at1]
		}
		if(lags==FALSE)
		{
			z <- cbind(z[at,])
			id <- id[at]
		}
	}
	else
	{
		at <- rep(TRUE,NT.ini)
		at1 <- rep(TRUE,NT.ini)

		z.full <- z
		id.full <- id
	}

	if(tdummies==TRUE)
	{
		x <- cbind(x,td)
		z <- cbind(z,td[at,])
		z.full <- cbind(z.full,td)

		if(any(type==c("GMMcre","QMLcre")))
		{
			xa.names <- x.names
			za.names <- z.names
		}

		x.names <- c(x.names,td.names)
		z.names <- c(z.names,td.names)
	}
	else
	{
		xa.names <- x.names
		za.names <- z.names
	}

	if(type=="GMMpfe" & any(y==0))
	{
		x <- cbind(x,D)
		z <- cbind(z,D[at,])
		z.full <- cbind(z.full,D)
		x.names <- c(x.names,D.names)
		z.names <- c(z.names,D.names)
	}

	if(any(type==c("GMMcre","QMLcre")))
	{
		if(any(Ti.ini!=max(Ti.ini)))
		{
			Ti.unique <- unique(Ti.ini)
			Ti.unique <- Ti.unique[Ti.unique!=1]
			Ti.unique <- sort(Ti.unique)

			TRUE.dum <- matrix(NA,nrow=NT.ini,ncol=length(Ti.unique))
			a <- 0

			Ti.ini.exp <- rep(Ti.ini,Ti.ini)

			for(j in Ti.unique)
			{
				a <- a+1
				TRUE.dum[,a] <- Ti.ini.exp==j
			}

			if(x.exogenous==TRUE)
			{
				x.m.aug <- TRUE.dum*x.m[,1]
				if(ncol(x.m)>1) for(j in 2:ncol(x.m)) x.m.aug <- cbind(x.m.aug,TRUE.dum*x.m[,j])
				x.m <- cbind(TRUE.dum,x.m.aug)
				x.m.names <- c(paste("(Intercept)",Ti.unique,sep="_"),paste(rep(paste(xa.names,"mean",sep="_"),each=length(Ti.unique)),rep(Ti.unique,length(xa.names)),sep="_"))
				x.m <- cbind(x.m[Ti.ini.exp!=1,])
			}
			if(x.exogenous==FALSE)
			{
				z.m.aug <- TRUE.dum*z.m[,1]
				if(ncol(z.m)>1) for(j in 2:ncol(z.m)) z.m.aug <- cbind(z.m.aug,TRUE.dum*z.m[,j])
				z.m <- cbind(TRUE.dum,z.m.aug)
				z.m.names <- c(paste("(Intercept)",Ti.unique,sep="_"),paste(rep(paste(za.names,"mean",sep="_"),each=length(Ti.unique)),rep(Ti.unique,length(za.names)),sep="_"))
				z.m <- cbind(z.m[Ti.ini.exp!=1,])
			}

			id <- id[Ti.ini.exp!=1]
			y <- y[Ti.ini.exp!=1]
			x <- cbind(x[Ti.ini.exp!=1,])
			z <- cbind(z[Ti.ini.exp!=1,])
			at <- at[Ti.ini.exp!=1]
			if(type=="QMLcre" & x.exogenous==FALSE) var.endog <- var.endog[Ti.ini.exp!=1]
			if(!is.null(offset)) offset <- offset[Ti.ini.exp!=1]
		}
		else
		{
			if(x.exogenous==TRUE)
			{
				x.m <- cbind(1,x.m)
				x.m.names <- c("(Intercept)_mean",paste(xa.names,"mean",sep="_"))
			}
			if(x.exogenous==FALSE)	
			{
				z.m <- cbind(1,z.m)
				z.m.names <- c("(Intercept)_mean",paste(za.names,"mean",sep="_"))
			}
		}

		if(x.exogenous==FALSE)
		{
			x <- cbind(x,z.m)
			z <- cbind(z,z.m)
			x.names <- c(x.names,z.m.names)

			if(type=="QMLcre")
			{
				results <- lm(var.endog ~ 0+z)
				PIhat <- results$coefficients
				vhat <- results$residuals

				gixv <- t(z*vhat)

				x <- cbind(x,vhat)
				x1.names <- c(x.names,"vhat")
				x2.names <- paste("Z",c(z.names,z.m.names),sep="_")
				x.names <- c(x1.names,x2.names)
			}
		}
		else
		{
			x <- cbind(x,x.m)
			z <- x
			x.names <- c(x.names,x.m.names)

			gixv <- NA
			vhat <- NA
		}

		z.full <- z
	}
	else
	{
		gixv <- NA
		vhat <- NA
	}

	N <- length(unique(id))
	Ti <- as.vector(by(id,id,length))
	NT <- sum(at)

	k <- ncol(x)
	kz <- ncol(z)
	J <- NA
	dfJ <- kz-k

	### 4. Estimation

	if(any(type==c("GMMc","GMMww","GMMbgw","GMMpre","GMMpfe","GMMcre"))) Hy <- fracregpd.links(link)$H1(y)
	if(type=="QMLcre") Hy <- y

	if(type=="GMMww" & GMMww.cor==TRUE)
	{
		x <- x-matrix(apply(x,2,mean),nrow=nrow(x),ncol=k,byrow=TRUE)
		z <- z-matrix(apply(z.full,2,mean),nrow=nrow(z),ncol=kz,byrow=TRUE)
	}

	if(missing(start)) start <- rep(0,k)
	if(length(start)!=k) stop("start is not of the same dimension as the covariate vector (includes all auxiliary parameters)")

	results <- fracregpd.est(type,x.exogenous,lags,id,Ti,Hy,x,z,link,var.type,start,at,at1,variance,NT,k,kz,gixv,vhat,bootstrap,offset=offset,...)
	p <- results$p
	converged <- results$converged
	LL <- results$LL

	if(type!="QMLcre" & dfJ>0)
	{
		Qn <- results$Qn
		J <- NT*Qn
	}

	if(variance==TRUE & converged==TRUE)
	{
 		if(bootstrap==FALSE) p.var <- results$p.var
		else
		{
			Ti.full <- as.vector(by(id.full,id.full,length))

			pboot <- matrix(NA,nrow=B,ncol=length(x.names))

			for(j in 1:B)
			{
				index.id <- sample(unique(id),N,replace=TRUE)
				a <- 1
				aa <- 1

				for(jj in unique(id))
				{
					n.id <- sum(index.id==jj)
					if(n.id>=1)
					{
						bb.1 <- (a!=1)*sum(Ti.full[1:(a-1)])+1
						bb.2 <- sum(Ti.full[1:a])

						refa <- rep(bb.1:bb.2,n.id)
						refe <- rep(aa:(aa+n.id-1),each=Ti.full[a])

						if(aa==1)
						{
							ref <- refa
							id.B <- refe
						}
						else
						{
							ref <- c(ref,refa)
							id.B <- c(id.B,refe)
						}

						aa <- aa+n.id
					}

					a <- a+1
				}

				Hy.B <- Hy[ref]
				x.B <- cbind(x[ref,])
				z.B <- cbind(z.full[ref,])
				if(type=="QMLcre" & x.exogenous==FALSE) var.endog.B <- var.endog[ref]
				at.B <- at[ref]
				at1.B <- at1[ref]
				if(!is.null(offset)) offset.B <- offset[ref] else offset.B <- NULL

				if(any(type==c("GMMc","GMMww")) | lags==TRUE)
				{
					if(lags==TRUE)
					{
						z.B <- cbind(z.B[at1.B,])
						id.B <- id.B[at1.B]
					}
					if(lags==FALSE)
					{
						z.B <- cbind(z.B[at.B,])
						id.B <- id.B[at.B]
					}
				}

				Ti.B <- as.vector(by(id.B,id.B,length))
				NT.B <- sum(at.B)

				if(type=="QMLcre" & x.exogenous==FALSE)
				{
					results <- lm(var.endog.B ~ 0+z.B)
					PIres <- results$coefficients
				}

				results <- fracregpd.est(type,x.exogenous,lags,id.B,Ti.B,Hy.B,x.B,z.B,link,var.type,start,at.B,at1.B,variance,NT.B,k,kz,NA,NA,bootstrap,offset=offset.B,...)
				if(results$converged==TRUE)
				{
					if(table==TRUE) cat("1")

					pres <- results$p
					if(type=="QMLcre" & x.exogenous==FALSE) pres <- c(pres,PIres)
					pboot[j,] <- pres
				}
				else if(table==TRUE) cat("0")

				if(any(j==seq(50,100000,50)) & table==TRUE) cat("\n"
)			}

			p.var <- matrix(NA,nrow=length(x.names),ncol=length(x.names))
			diag(p.var) <- apply(pboot,2,var,na.rm=TRUE)
		}
	}
	else p.var <- "singular"

	if(type=="QMLcre" & x.exogenous==FALSE) p <- c(p,PIhat)

	table.info <- list(p=p,p.var=p.var,x.names=x.names,x.exogenous=x.exogenous,lags=lags,type=type,link=link,converged=converged,N.ini=N.ini,N=N,NT.ini=NT.ini,NT=NT,J=J,dfJ=dfJ,k=k,var.type=var.type,bootstrap=bootstrap,LL=LL,or=or,level=level)
	if(table==TRUE) do.call(fracregpd.table, table.info)

	names(p) <- x.names
	res <- list(call=cl, type=type,link=link,Hy=Hy,p=p,converged=converged,table.info=table.info,x=x,xbhat=results$xbhat)
	if(!is.null(offset)) res[["offset"]] <- offset
	if(dfJ>0) res[["J"]] <- J

	if(variance==TRUE & converged==TRUE)
	{ 
		if(is.character(p.var)) p.var <- matrix(NA,nrow=length(p),ncol=length(p))
		dimnames(p.var) <- list(x.names,x.names)
		res[["p.var"]] <- p.var
		res[["var.type"]] <- var.type
	}

	### 5. Return results

	class(res) <- "fracregpd"
	return(invisible(res))
}

library(testthat)
library(fracreg)

test_that("fracregpd QMLcre estimator works", {
  set.seed(42)
  N <- 100
  T <- 5
  id <- rep(1:N, each = T)
  time <- rep(1:T, times = N)
  x_panel <- rnorm(N * T)
  c_i <- rep(rnorm(N), each = T) 
  xb <- x_panel + c_i
  y_panel <- exp(xb) / (1 + exp(xb))
  
  X <- cbind(x_panel = x_panel)
  
  model_pd <- fracregpd(
    id = id, 
    time = time, 
    y = y_panel, 
    x = X, 
    type = "QMLcre", 
    link = "probit",
    table = FALSE
  )
  
  expect_type(model_pd, "list")
  expect_true(all(c("p", "p.var", "Hy") %in% names(model_pd)))
})

fracreg.tests.table <- function(test.which,S,Sp,ver,title1,title2=NA,n.ver=NA,test.ggoff=NA)
{
	stars <- rep("",length(S))
	stars[Sp<=0.01] <- "***"
	stars[Sp>0.01 & Sp<=0.05] <- "**"
	stars[Sp>0.05 & Sp<=0.1] <- "*"

	if(all(is.na(S))) stop("NO WALD/LR TEST HAS ACHIEVED CONVERGENCE. USE OTHER TEST VERSIONS INSTEAD")

	res <- cbind(`Statistic` = S[-1], `p-value` = Sp[-1])
	if(test.which != "GGOFF") {
		rownames(res) <- ver[-1]
	} else {
		rownames(res) <- paste(test.ggoff[-1], ver[-1], sep=" - ")
	}

	cat("\n")
	cat(.fracreg.sep(), "\n")
	cat(.fracreg.center(paste(test.which, "test")), "\n")
	cat(.fracreg.sep(), "\n")
	
	if(test.which != "P")
	{
		cat("H0:", title1, "\n")
		cat(.fracreg.sep(), "\n")
		printCoefmat(res, P.values = TRUE, has.Pvalue = TRUE, digits = 4, signif.legend = TRUE)
	}
	else
	{
		cat("H0:", title1, "\n")
		cat("H1:", title2, "\n")
		cat(.fracreg.sep(), "\n")
		stats::printCoefmat(res[1:n.ver, , drop = FALSE], P.values = TRUE, has.Pvalue = TRUE, digits = 4, signif.legend = TRUE)
		
		cat(.fracreg.sep(), "\n")
		cat("H0:", title2, "\n")
		cat("H1:", title1, "\n")
		cat(.fracreg.sep(), "\n")
		stats::printCoefmat(res[(n.ver+1):(2*n.ver), , drop = FALSE], P.values = TRUE, has.Pvalue = TRUE, digits = 4, signif.legend = TRUE)
	}
	cat(.fracreg.sep(), "\n")
	cat(.fracreg.center(paste("Run Date:", format(Sys.time(), "%Y-%m-%d %H:%M:%S"))), "\n")
	cat(.fracreg.sep(), "\n")
}

#' @title P Test for Fractional Response Regressions
#'
#' @description
#' \code{fracreg.ptest} is used to perform the P test to evaluate the specification of alternative, non-nested fractional response models by testing against each other.
#' @param object1 an object containing the results of an \code{fracreg} command.
#' @param object2 an object containing the results of another \code{fracreg} command.
#' @param version a vector containing the test versions to use. Available options: \code{Wald} (the default) and \code{LM}.  Both options may be chosen at the same time and are computed in a robust way.
#' @param table a logical value indicating whether a summary table with the test results should be printed.
#'
#' @details
#' \code{fracreg.ptest} applies the P test statistic proposed by Davidson and
#' MacKinnon (1981) to fractional response models estimated via\code{fracreg}. \code{fracreg.ptest} may be used to test against each other two alternative specifications for the link function  in: (i) one-part fractional response models; (ii) the binary components of two-part and three-part fractional response models; (iii) the fractional components of two-part and three-part fractional response models; and (iv) two-part and three-part fractional response models. 
#' 
#' \strong{P Test Framework:}
#' The P test allows the comparison of non-nested models (e.g., alternative link functions or non-nested regressors). Let model 1 specify \eqn{E_1(y|x) = G(x\beta)} and model 2 specify \eqn{E_2(y|x) = H(z\theta)}. To test model 1 against model 2, the baseline model is augmented with the difference between the fitted values:
#' \deqn{E(y|x) = G\left(x\beta + \gamma \left( \hat{y}_{M2} - \hat{y}_{M1} \right)\right)}
#' where \eqn{\hat{y}_{M1} = G(x\hat{\beta})} and \eqn{\hat{y}_{M2} = H(z\hat{\theta})}. The null hypothesis that model 1 is correct is tested via \eqn{H_0: \gamma = 0}.
#' 
#' In addition, \code{fracreg.ptest} may be used to test one-part models against two-part or three-part models and in cases where the link functions are the same but the regressors are non-nested. See Ramalho, Ramalho and Murteira (2011) for details on the application of the P test in the fractional response framework.
#'
#' @return
#' \code{fracreg.reset} returns a named vector with the test results.
#'
#' @references
#' Davidson, R. and J.G. MacKinnon (1981), "Several tests for model specification
#' on the presence of alternative hypotheses", \emph{Econometrica}, 49(3), 781-793.
#' 
#' Ramalho, E.A., J.J.S. Ramalho and J.M.R. Murteira (2011), "Alternative
#' estimating and testing empirical strategies for fractional response models",
#' \emph{Journal of Economic Surveys}, 25(1), 19-68.
#'
#' @author Sulman Olieko Owili <oliekosulman@gmail.com>
#'
#' @seealso
#' \code{\link{fracreg}}, for fitting fractional response models.\cr
#' \code{\link{fracreg.reset}} and \code{\link{fracreg.ggoff}}, for specification tests.\cr
#' \code{\link{fracreg.pe}}, for computing partial effects.
#'
#' @examples
#' ### Empirical 401(k) Examples
#' data("fracreg_k401k")
#' y <- fracreg_k401k$prate
#' X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age, 
#'            totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole)
#' 
#' m1 <- fracreg(y, X, type="1P", linkfrac="logit")
#' m2 <- fracreg(y, X, type="1P", linkfrac="probit")
#' ptest_res <- fracreg.ptest(m1, m2)
#' summary(ptest_res)
#' 
#' ### Simulated Examples
#' 
#' N <- 250
#' u <- rnorm(N)
#' 
#' X <- cbind(rnorm(N),rnorm(N))
#' dimnames(X)[[2]] <- c("X1","X2")
#' 
#' ym <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u))
#' y <- rbeta(N,ym*20,20*(1-ym))
#' y[y > 0.9] <- 1
#' 
#' #Testing logit versus loglog specifications for standard fractional
#' #regression models using a LM version of the P test
#' res1 <- fracreg(y,X,linkfrac="logit")
#' res2 <- fracreg(y,X,linkfrac="loglog")
#' ptest_res <- fracreg.ptest(res1,res2,"LM")
#' summary(ptest_res)
#' 
#' #Testing a logit one-part fractional response model versus a binary logit +
#' #fractional probit two-part model using a Wald version of the P test
#' res1 <- fracreg(y,X,linkfrac="logit")
#' res2 <- fracreg(y,X,linkbin="logit",linkfrac="probit",type="2P",inf=1)
#' ptest_res <- fracreg.ptest(res1,res2,"Wald")
#' summary(ptest_res)
#' @export
fracreg.ptest <- function(object1,object2,version="Wald",table=FALSE)
{
	### 1. Error and warning messages

	if(missing(object1)) stop("object1 is missing")
	if(missing(object2)) stop("object2 is missing")

	if(is.null(object1$class)) stop("object1 not the output of an fracreg command")
	if(is.null(object2$class)) stop("object2 not the output of an fracreg command")

	if(object1$class!="fracreg") stop("object1 is not the output of an fracreg command")
	if(object2$class!="fracreg") stop("object2 is not the output of an fracreg command")

	if(any(object1$type==c("1P","2P","3P")) & all(object2$type!=c("1P","2P","3P"))) stop("object1 and object2 cannot be compared")
	if((object1$type=="2Pbin" | object1$type=="3Pbin0" | object1$type=="3Pbin1") & object1$type!=object2$type) stop("object1 and object2 cannot be compared")
	if(object1$type=="2Pfrac" & object2$type!="2Pfrac") stop("object1 and object2 cannot be compared")

	if(object1$converged==0) stop("object1 is not the output of a successful (converged) fracreg command")
	if(object2$converged==0) stop("object2 is not the output of a successful (converged) fracreg command")

	if(all(version!="LM") & all(version!="Wald")) stop("test version not correctly specified")

	if(!is.logical(table)) stop("non-logical value assigned to option table")

	### 2. Variables and other information for the tests

	# 2.1. Model 1

	if(any(object1$type==c("1P","2Pbin","2Pfrac","3Pbin0","3Pbin1","3Pfrac")))
	{
		link1 <- object1$link
		type1 <- object1$type

		mf <- model.frame(object1$formula)
		y1 <- as.vector(model.response(mf))
		x1 <- model.matrix(object1$formula)
		dimnames(x1)[[2]] <- object1$x.names

		yhat1 <- object1$yhat
		xbhat1 <- object1$xbhat

		g1 <- fracreg.links(link1)$mu.eta(xbhat1)

		gx1 <- g1*x1

		if(type1=="1P") title1 <- paste("Fractional",link1,"regression")
		if(type1=="2Pbin") title1 <- paste("Binary",link1,"component of a two-part regression")
		if(type1=="3Pbin0") title1 <- paste("First binary",link1,"component of a three-part regression")
		if(type1=="3Pbin1") title1 <- paste("Second binary",link1,"component of a three-part regression")
		if(type1=="2Pfrac") title1 <- paste("Fractional",link1,"component of a two-part regression")
		if(type1=="3Pfrac") title1 <- paste("Fractional",link1,"component of a three-part regression")

		if(type1=="2Pbin") type.both <- "2Pbin"
		else type.both <- "others"
	}
		if(object1$type=="3P")
	{
		y1 <- object1$ybase
		link1a <- object1$resBIN0$link; type1a <- "3Pbin0"
		x1a <- model.matrix(object1$resBIN0$formula)
		yhat1a <- object1$resBIN0$yhat
		xbhat1a <- object1$resBIN0$xbhat
		g1a <- fracreg.links(link1a)$mu.eta(xbhat1a)
		dimnames(x1a)[[2]] <- object1$resBIN0$x.names

		link1b <- object1$resBIN1$link; type1b <- "3Pbin1"
		x1b <- model.matrix(object1$resBIN1$formula)
		yhat1b <- object1$resBIN1$yhat
		xbhat1b <- object1$resBIN1$xbhat
		g1b <- fracreg.links(link1b)$mu.eta(xbhat1b)
		dimnames(x1b)[[2]] <- object1$resBIN1$x.names

		link1c <- object1$resFRAC$link; type1c <- "3Pfrac"
		x1c <- object1$x2base
		xbhat1c <- x1c%*%object1$resFRAC$p
		yhat1c <- fracreg.links(link1c)$linkinv(xbhat1c)
		g1c <- fracreg.links(link1c)$mu.eta(xbhat1c)
		dimnames(x1c)[[2]] <- object1$resFRAC$x.names

		yhat1 <- object1$yhat3P
		H <- yhat1b + (1-yhat1b)*yhat1c
		g1a_part <- as.vector(g1a * H)
		g1b_part <- as.vector(yhat1a * g1b * (1 - yhat1c))
		g1c_part <- as.vector(yhat1a * (1 - yhat1b) * g1c)

		gx1 <- cbind(g1a_part*x1a, g1b_part*x1b, g1c_part*x1c)
		title1 <- paste("Three-part regression - binary",link1a,", binary",link1b,"+ fractional",link1c)
		type.both <- "others"
	}

	if(object1$type=="2P")
	{
		y1 <- object1$ybase

		link1a <- object1$resBIN$link
		type1a <- "2Pbin"

		x1a <- model.matrix(object1$resBIN$formula)
		yhat1a <- object1$resBIN$yhat
		xbhat1a <- object1$resBIN$xbhat
		g1a <- fracreg.links(link1a)$mu.eta(xbhat1a)
		dimnames(x1a)[[2]] <- object1$resBIN$x.names

		link1b <- object1$resFRAC$link
		type1b <- "2Pfrac"

		x1b <- object1$x2base
		xbhat1b <- x1b%*%object1$resFRAC$p
		yhat1b <- fracreg.links(link1b)$linkinv(xbhat1b)
		g1b <- fracreg.links(link1b)$mu.eta(xbhat1b)
		dimnames(x1b)[[2]] <- object1$resFRAC$x.names

		yhat1 <- object1$yhat2P
		g1ab <- as.vector(g1a*yhat1b)
		g1ba <- as.vector(g1b*yhat1a)

		gx1 <- cbind(g1ab*x1a,g1ba*x1b)

		title1 <- paste("Binary",link1a,"+ Fractional",link1b,"two-part regression")

		type.both <- "others"
	}

	# 2.2. Model 2

	if(any(object2$type==c("1P","2Pbin","2Pfrac","3Pbin0","3Pbin1","3Pfrac")))
	{
		link2 <- object2$link
		type2 <- object2$type

		mf <- model.frame(object2$formula)
		y2 <- as.vector(model.response(mf))
		x2 <- model.matrix(object2$formula)
		dimnames(x2)[[2]] <- object2$x.names

		yhat2 <- object2$yhat
		xbhat2 <- object2$xbhat
		g2 <- fracreg.links(link2)$mu.eta(xbhat2)

		gx2 <- g2*x2

		if(type2=="1P") title2 <- paste("Fractional",link2,"regression")
		if(type2=="2Pbin") title2 <- paste("Binary",link2,"component of a two-part regression")
		if(type2=="3Pbin0") title2 <- paste("First binary",link2,"component of a three-part regression")
		if(type2=="3Pbin1") title2 <- paste("Second binary",link2,"component of a three-part regression")
		if(type2=="2Pfrac") title2 <- paste("Fractional",link2,"component of a two-part regression")
		if(type2=="3Pfrac") title2 <- paste("Fractional",link2,"component of a three-part regression")
	}
		if(object2$type=="3P")
	{
		y2 <- object2$ybase
		link2a <- object2$resBIN0$link; type2a <- "3Pbin0"
		x2a <- model.matrix(object2$resBIN0$formula)
		yhat2a <- object2$resBIN0$yhat
		xbhat2a <- object2$resBIN0$xbhat
		g2a <- fracreg.links(link2a)$mu.eta(xbhat2a)
		dimnames(x2a)[[2]] <- object2$resBIN0$x.names

		link2b <- object2$resBIN1$link; type2b <- "3Pbin1"
		x2b <- model.matrix(object2$resBIN1$formula)
		yhat2b <- object2$resBIN1$yhat
		xbhat2b <- object2$resBIN1$xbhat
		g2b <- fracreg.links(link2b)$mu.eta(xbhat2b)
		dimnames(x2b)[[2]] <- object2$resBIN1$x.names

		link2c <- object2$resFRAC$link; type2c <- "3Pfrac"
		x2c <- object2$x2base
		xbhat2c <- x2c%*%object2$resFRAC$p
		yhat2c <- fracreg.links(link2c)$linkinv(xbhat2c)
		g2c <- fracreg.links(link2c)$mu.eta(xbhat2c)
		dimnames(x2c)[[2]] <- object2$resFRAC$x.names

		yhat2 <- object2$yhat3P
		H <- yhat2b + (1-yhat2b)*yhat2c
		g2a_part <- as.vector(g2a * H)
		g2b_part <- as.vector(yhat2a * g2b * (1 - yhat2c))
		g2c_part <- as.vector(yhat2a * (1 - yhat2b) * g2c)

		gx2 <- cbind(g2a_part*x2a, g2b_part*x2b, g2c_part*x2c)
		title2 <- paste("Three-part regression - binary",link2a,", binary",link2b,"+ fractional",link2c)
	}

	if(object2$type=="2P")
	{
		y2 <- object2$ybase

		link2a <- object2$resBIN$link
		type2a <- "2Pbin"

		x2a <- model.matrix(object2$resBIN$formula)
		yhat2a <- object2$resBIN$yhat
		xbhat2a <- object2$resBIN$xbhat
		g2a <- fracreg.links(link2a)$mu.eta(xbhat2a)
		dimnames(x2a)[[2]] <- object2$resBIN$x.names

		link2b <- object2$resFRAC$link
		type2b <- "2Pfrac"

		x2b <- object2$x2base
		xbhat2b <- x2b%*%object2$resFRAC$p
		yhat2b <- fracreg.links(link2b)$linkinv(xbhat2b)
		g2b <- fracreg.links(link2b)$mu.eta(xbhat2b)
		dimnames(x2b)[[2]] <- object2$resFRAC$x.names

		yhat2 <- object2$yhat2P
		g2ab <- as.vector(g2a*yhat2b)
		g2ba <- as.vector(g2b*yhat2a)

		gx2 <- cbind(g2ab*x2a,g2ba*x2b)

		title2 <- paste("Binary",link2a,"+ Fractional",link2b,"two-part regression")
	}

	### 3. Further error and warning messages

	if(!isTRUE(all.equal(y1,y2))) stop("The dependent variable is not the same in the two models")

	if(any(object1$type==c("2Pbin","2Pfrac")) | (object1$type=="1P" & object2$type=="1P"))
	{
		if(object1$link==object2$link)
		{
			x1.names <- dimnames(x1)[[2]]
			x2.names <- dimnames(x2)[[2]]
			x12.names <- c(x1.names,x2.names)
			x12.names <- unique(x12.names)
			x1.len <- length(x1.names)
			x2.len <- length(x2.names)
			x12.len <- length(x12.names)

			if(identical(x1.names,x2.names)) stop("object 1 and object 2 are based on the same link function and covariates")
			if(x1.len==x12.len & x2.len!=x12.len) stop("object 2 is nested in object 1 - no need to use the P test")
			if(x1.len!=x12.len & x2.len==x12.len) stop("object 1 is nested in object 2 - no need to use the P test")
		}
	}
	if(object1$type=="2P" & object2$type=="2P")
	{
		if(object1$resBIN$link==object2$resBIN$link & object1$resFRAC$link==object2$resFRAC$link)
		{
			x1a.names <- dimnames(x1a)[[2]]
			x2a.names <- dimnames(x2a)[[2]]
			x12a.names <- c(x1a.names,x2a.names)
			x12a.names <- unique(x12a.names)
			x1a.len <- length(x1a.names)
			x2a.len <- length(x2a.names)
			x12a.len <- length(x12a.names)

			x1b.names <- dimnames(x1b)[[2]]
			x2b.names <- dimnames(x2b)[[2]]
			x12b.names <- c(x1b.names,x2b.names)
			x12b.names <- unique(x12b.names)
			x1b.len <- length(x1b.names)
			x2b.len <- length(x2b.names)
			x12b.len <- length(x12b.names)

			if(identical(x1a.names,x2a.names) & identical(x1b.names,x2b.names)) stop("object 1 and object 2 are based on the same link function and covariates")
			if(x1a.len==x12a.len & x2a.len==x12a.len & x1b.len==x12b.len & x2b.len!=x12b.len) stop("object 2 is nested in object 1 - no need to use the P test")
			if(x1a.len==x12a.len & x2a.len==x12a.len & x1b.len!=x12b.len & x2b.len==x12b.len) stop("object 1 is nested in object 2 - no need to use the P test")
			if(x1a.len==x12a.len & x2a.len!=x12a.len & x1b.len==x12b.len & x2b.len==x12b.len) stop("object 2 is nested in object 1 - no need to use the P test")
			if(x1a.len!=x12a.len & x2a.len==x12a.len & x1b.len==x12b.len & x2b.len==x12b.len) stop("object 1 is nested in object 2 - no need to use the P test")
			if(x1a.len==x12a.len & x2a.len!=x12a.len & x1b.len==x12b.len & x2b.len!=x12b.len) stop("object 2 is nested in object 1 - no need to use the P test")
			if(x1a.len!=x12a.len & x2a.len==x12a.len & x1b.len!=x12b.len & x2b.len==x12b.len) stop("object 1 is nested in object 2 - no need to use the P test")
		}
	}

	### 4. Tests

	ver <- NA
	S <- NA
	Sp <- NA

	df <- 1
	for(j in 1:2)
	{
		if(j==1)
		{
			yj <- y1
			yhatj <- yhat1
			gxj <- gx1
			gzj <- yhat2-yhat1
		}
		if(j==2)
		{
			yj <- y2
			yhatj <- yhat2
			gxj <- gx2
			gzj <- yhat1-yhat2
		}

		if(any(version=="LM"))
		{
			ver <- c(ver,"LM")

			results <- fracreg.lm(yj,yhatj,gxj,gzj,type.both)
			Sj <- results$LM
			S <- c(S,Sj)
			Sp <- c(Sp,1-pchisq(Sj,df))
		}
		if(any(version=="Wald"))
		{
			ver <- c(ver,"Wald")

			yt <- yj-yhatj
			gxzj <- cbind(gxj,gzj)
			results <- lm(yt ~ gxzj-1)

			gzj.b <- results$coefficients[ncol(gxzj)]
			dfcc <- nrow(gxzj)/(nrow(gxzj)-ncol(gxzj))
			gzj.var <- dfcc*(solve(t(gxzj)%*%gxzj)%*%t(gxzj)%*%diag(results$residuals^2)%*%gxzj%*%solve(t(gxzj)%*%gxzj))[ncol(gxj)+1,ncol(gxj)+1]

			Sj <- as.vector(gzj.b/(gzj.var^0.5))
			S <- c(S,Sj)
			Spj <- 2*(1-pt(abs(Sj),nrow(gxzj)-ncol(gxzj)))
			Sp <- c(Sp,Spj)
		}
	}

	n.ver <- length(ver[-1])/2 
	table.info <- list(test.which="P",S=S,Sp=Sp,ver=ver,title1=title1,title2=title2,n.ver=n.ver)
	if(table==TRUE) do.call(fracreg.tests.table, table.info)

	### 5. Return results

	statistics=S[-1]
	ver <- ver[-1]
	names(statistics) <- paste(c(rep("H0-obj1",n.ver),rep("H0-obj2",n.ver)),ver,sep="-")

	class(statistics) <- c("fracreg.ptest", "numeric")
	attr(statistics, "table.info") <- table.info

	return(invisible(statistics))
}

#' @title RESET Test for Fractional Response Regressions under Neglected Heterogeneity
#'
#' @description
#' \code{fracreghet.reset} is used to test the specification of fractional response models estimated by GMMx or LINx.
#' @param object an object containing the results of an \code{fracreghet} command.
#' @param lastpower.vec a numeric vector containing the maximum powers of the linear predictors to be used in RESET tests.
#' @param version a vector containing the test versions to use. Available options: \code{Wald} (the default) and \code{LM} (only  available for \code{GMMx}).
#' @param table a logical value indicating whether a summary table with the test results should be printed.
#' @param \dots Arguments to pass to \link[stats]{nlminb}, which is used to estimate the model under the alternative hypothesis when \code{version} is equal to \code{"Wald"} and the null model was estimated by \code{GMMx}.
#'
#' @details
#' \code{fracreghet.reset} applies the RESET test statistic to fractional response
#' models estimated via \code{fracreghet} using the options \code{GMMx} or \code{LINx}. \code{fracreghet.reset} may be used to test simultaneously the validity of the link specification and the transformation applied to the response variable by each estimator.  
#' 
#' \strong{RESET Test under Unobserved Heterogeneity:}
#' The test is based on augmenting the original model with powers of the linear predictor \eqn{x\hat{\beta}}. For GMMx, it tests \eqn{H_0: \gamma = 0} in the expanded moment conditions:
#' \deqn{E\left[Z_i \left(H(y_i) - \exp\left(x_i\beta + \sum_{k=2}^P \gamma_k (x_i\hat{\beta})^k\right)E(e^{c_i})\right)\right] = 0}
#' This simultaneously evaluates whether the mean function and the specific heterogeneity transformation \eqn{H(\cdot)} are correctly specified.
#' 
#' It is taken into account the option that was chosen for computing standard errors in the model under evaluation. See Ramalho and Ramalho (2017) for details.
#'
#' @return
#' \code{fracreghet.reset} returns a named vector with the test results.
#'
#' @references
#' Ramalho, E. A., & Ramalho, J. J. S. (2017), "Moment-based estimation of nonlinear regression models with boundary outcomes and endogeneity, with applications to nonnegative and fractional responses", \emph{Econometric Reviews}, 36(4), 397-420.
#' 
#' Ramsey, J.B. (1969), "Tests for Specification Errors in Classical Linear Least-Squares Regression Analysis", \emph{Journal of the Royal Statistical Society: Series B (Methodological)}, 31(2), 350-371.
#'
#' @author Sulman Olieko Owili <oliekosulman@gmail.com>
#'
#' @seealso
#' \code{\link{fracreghet}}, for fitting fractional response models under unobserved heterogeneity.\cr
#' \code{\link{fracreghet.pe}}, for computing partial effects.
#'
#' @examples
#' ### Empirical 401(k) Examples 
#' data("fracreg_k401k") 
#' y <- fracreg_k401k$prate 
#' X_het <- cbind(mrate = fracreg_k401k$mrate, ltotemp = fracreg_k401k$ltotemp)
#'  
#' # fracreghet estimators do not allow exact 1s or 0s
#' y_adj <- y
#' y_adj[y_adj == 1] <- 0.999
#' 
#' # Instrument mrate using age
#' 
#' Z_emp <- cbind(age = fracreg_k401k$age, ltotemp = fracreg_k401k$ltotemp) 
#' res_emp <- fracreghet(y_adj, X_het, type="GMMx", link="logit") 
#' reset_res <- fracreghet.reset(res_emp)
#' summary(reset_res)
#'  
#' ### Simulated Examples
#' 
#' N <- 250
#' u <- rnorm(N)
#' 
#' X <- cbind(rnorm(N),rnorm(N))
#' dimnames(X)[[2]] <- c("X1","X2")
#' 
#' Z <- cbind(rnorm(N),rnorm(N),rnorm(N))
#' dimnames(Z)[[2]] <- c("Z1","Z2","Z3")
#' 
#' y <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u))
#' 
#' mod <- fracreghet(y,X,type="GMMx")
#' 
#' #LM and Wald versions of the RESET test, based on 1 or 2 fitted powers of xb
#' reset_res <- fracreghet.reset(mod,2:3,c("Wald","LM"))
#' summary(reset_res)
#' @export
fracreghet.reset <- function(object,lastpower.vec=3,version="Wald",table=FALSE,...)
{
	### 1. Error and warning messages

	if(missing(object)) stop("object is missing")
	if(is.null(object$class)) stop("object is not the output of an fracreghet command")
	if(object$class!="fracreghet") stop("object is not the output of an fracreghet command")
	if(object$converged==0) stop("object is not the output of a successful (converged) fracreghet command")
	if(any(lastpower.vec<2)) stop(sQuote(lastpower.vec)," - lastpower.vec contains elements lower than 2")
	if(all(object$type!=c("GMMx","LINx"))) stop("fracreghet.reset is only implemented for GMMx and LINx estimators")
	if(all(version!="LM") & all(version!="Wald")) stop("test version not correctly specified")
	if(any(version=="LM") & object$type=="LINx") stop("LM version not implemented for LINx; choose Wald version")
	if(is.null(object$var.type)) stop("fracreghet command was run with variance = FALSE")

	### 2. Recovering definitions and estimates

	mf <- model.frame(object$formula)
	y <- model.response(mf)
	x <- model.matrix(object$formula)

	type <- object$type
	xbhat <- object$xbhat
	Hy <- object$Hy
	link <- object$link
	adjust <- object$adjust

	if(any(version=="Wald") | type=="GMMx")
	{
		var.type <- object$var.type
		if(var.type=="cluster") var.cluster <- object$var.cluster
	}

	title1 <- paste("Fractional",link,"regression")
	title2 <- paste("Estimator:",type)
	if(adjust!=0)
	{
		if(is.numeric(adjust)) title3 <- paste("(adjustment:",adjust,"added to all observations)")
		else title3 <- "(adjustment: all boundary observations dropped)"
		title2 <- paste(title2,title3,sep=" ")
	}

	### 3. Test

	lastpower.vec <- round(lastpower.vec,0)

	N <- length(Hy)
	g <- fracreghet.links(link)$mu.eta(xbhat)
	gx <- g*x

	xx.all <- as.matrix(xbhat^2)
	if(max(lastpower.vec)>2) for(i in 3:max(lastpower.vec)) xx.all <- cbind(xx.all,xbhat^i)

	ver <- NA
	S <- NA
	Sp <- NA

	for(m in lastpower.vec)
	{
		df <- m-1
		xx <- xx.all[,1:(m-1)]

		X <- cbind(x,xx)
		k <- ncol(X)

		if(any(version=="LM"))
		{
			name <- paste("LM(",m,")",sep="")
			ver <- c(ver,name)

			if(type=="GMMx")
			{
				gi <- fracreghet.gi(type,X,Hy,xbhat,link)$gi
				gn <- as.matrix(apply(gi,1,mean))

				if(var.type=="robust") fi <- (1/N)*gi%*%t(gi)
				if(var.type=="cluster")
				{
					fi <- 0
					id <- var.cluster
					u <- Hy-xbhat

					for(j in unique(id))
					{
						Zi <- matrix(X[id==j,],ncol=k)
						ui <- u[id==j]
						zu <- t(Zi)%*%ui

						fi <- fi+zu%*%t(zu)
					}

					fi <- fi/N
				}

				fi.inv <- tryCatch(solve(fi),error=function(e) NaN)
				if(any(is.nan(fi.inv))) fi.inv <- "singular"

				if(!is.character(fi.inv))
				{
					Sj <- N*t(gn)%*%fi.inv%*%gn
					S <- c(S,Sj)
					Sp <- c(Sp,1-pchisq(Sj,df))
				}
				else
				{
					S <- c(S,NA)
					Sp <- c(Sp,NA)
				}
			}
		}

		if(any(version=="Wald"))
		{
			name <- paste("Wald(",m,")",sep="")
			ver <- c(ver,name)

			if(type=="GMMx")
			{
				results <- fracreghet.est(type,X,X,link,c(object$p,rep(0,df)),Hy,TRUE,var.type,var.cluster,NA,NA,...)
				converged <- results$converged
			}
			if(type=="LINx")
			{
				results <- lm(Hy ~ 0+X)
				converged <- TRUE
			}

			if(converged==TRUE)
			{
				if(type=="GMMx")
				{
					p1 <- results$p
					p.var1 <- results$p.var
				}
				if(type=="LINx")
				{ 
					p1 <- results$coefficients
					XB <- results$fitted.values
					p.var1 <- fracreghet.var(type,p1,XB,X,X,link,Hy,var.type,var.cluster,FALSE,NA,NA)$p.var
				}

				if(!is.character(p.var1))
				{
					p.n <- length(p1)
					p1 <- p1[(p.n-df+1):p.n]
					p.var1 <- p.var1[(p.n-df+1):p.n,(p.n-df+1):p.n]

					Sj <- t(p1)%*%solve(p.var1)%*%p1
					S <- c(S,Sj)
					Sp <- c(Sp,1-pchisq(Sj,df))
				}
				else
				{
					S <- c(S,NA)
					Sp <- c(Sp,NA)
				}
			}
			else
			{
				S <- c(S,NA)
				Sp <- c(Sp,NA)
			}
		}
	}

	table.info <- list(test.which="RESET",S=S,Sp=Sp,ver=ver,title1=title1,title2=title2)
	if(any(!is.na(S)) & table==TRUE) do.call(fracreghet.tests.table, table.info)
	if(all(is.na(S))) warning("RESET test could not be computed; either algorithm did not converge (Wald version) or covariance matrix is singular (Wald/LM versions)")

	### 4. Return results

	statistics <- S[-1]
	names(statistics) <- ver[-1]
	
	class(statistics) <- c("fracreghet.reset", "numeric")
	attr(statistics, "table.info") <- table.info

	return(invisible(statistics))
}

#' @title Fractional Response Regressions under Unobserved Heterogeneity - Partial Effects
#'
#' @description
#' \code{fracreghet.pe} is used to compute average and/or conditional partial effects in fractional response models 
#' under unobserved heterogeneity.
#' @param object an object containing the results of an \code{fracreghet} command.
#' @param smearing a logical value indicating whether the smearing correction is to be applied
#' @param APE a logical value indicating whether average partial effects are to be computed.
#' @param CPE a logical value indicating whether conditional partial effects are to be computed.
#' @param at a numeric vector containing the covariates' values at which the conditional partial effects are to be computed or the strings \code{"mean"} (the default) or \code{"median"}, in which cases the covariates are evaluated at their mean or median values (or mode, in case of dummy variables), respectively.
#' @param which.x a vector containing the names of the covariates to which the partial effects are to be computed.
#' @param table a logical value indicating whether a summary table with the results should be printed.
#' @param variance a logical value indicating whether the variance of the estimated partial effects should be calculated. Defaults to \code{TRUE} whenever \code{table = TRUE}.
#'
#' @details
#' \code{fracreghet.pe} calculates partial effects for fractional response models estimated via \code{fracreghet}. \code{fracreghet.pe} may be used to compute average or conditional partial effects. These partial effects may be conditional only on observables, using the smearing estimator, or also on unobservables, setting the error term to zero.
#' 
#' \strong{Partial Effects under Unobserved Heterogeneity:}
#' When unobserved heterogeneity or endogeneity is present, calculating partial effects requires dealing with the unobserved error \eqn{v_i}. Let the conditional mean be \eqn{E(y|x, v) = G(x\beta + \gamma v)}.
#' - \strong{Conditional on Observables (Smearing):} The unobserved heterogeneity is integrated out over its empirical distribution. The average partial effect for a continuous variable \eqn{x_k} is computed as:
#' \deqn{PE_k(x) = \frac{1}{N} \sum_{i=1}^N g(x\beta + \gamma \hat{v}_i) \beta_k}
#' - \strong{Conditional on Unobservables (Error = 0):} The partial effect is evaluated for an individual with the mean level of unobserved heterogeneity (\eqn{v = 0}):
#' \deqn{PE_k(x) = g(x\beta) \beta_k}
#' 
#' For discrete variables, the partial effects are calculated as the discrete differences evaluated using either the smearing approach or setting the error term to zero. 
#' 
#' For calculating standard errors, it is taken into account the option that was previously chosen for estimating the model. See Ramalho and Ramalho (2017) for details on the computation of partial effects for fractional response models under unobserved heterogeneity.
#'
#' @return
#' \code{fracreghet.pe} returns a list with the following element:
#'   \item{PE.p}{a named vector of partial effects.
#' }
#' 
#' If \code{variance = TRUE} or \code{table = TRUE}, the previous list also contains the following element:
#'   \item{PE.sd}{a named vector of standard errors of the estimated partial effects.
#' }
#' 
#' When both average and conditional partial effects are requested, two lists containing the previous elements are returned, indexed by the prefixes \code{ape} and \code{cpe}.
#'
#' @references
#' Ramalho, E. A., & Ramalho, J. J. S. (2017), "Moment-based estimation of nonlinear regression models with boundary outcomes and endogeneity, with applications to nonnegative and fractional responses", \emph{Econometric Reviews}, 36(4), 397-420.
#'
#' @author Sulman Olieko Owili <oliekosulman@gmail.com>
#'
#' @seealso
#' \code{\link{fracreghet}}, for fitting fractional response models under unobserved heterogeneity.\cr
#' \code{\link{fracreghet.reset}}, for the RESET test.\cr
#'
#' @examples
#' ### Empirical 401(k) Examples 
#' data("fracreg_k401k") 
#' y <- fracreg_k401k$prate 
#' X_het <- cbind(mrate = fracreg_k401k$mrate, ltotemp = fracreg_k401k$ltotemp)
#'  
#' # fracreghet estimators do not allow exact 1s or 0s
#' y_adj <- y
#' y_adj[y_adj == 1] <- 0.999
#' 
#' # Instrument mrate using age
#' 
#' Z_emp <- cbind(age = fracreg_k401k$age, ltotemp = fracreg_k401k$ltotemp) 
#' res_emp <- fracreghet(y_adj, X_het, Z_emp, var.endog = X_het[, "mrate"], 
#'                       type="QMLxv", link="logit") 
#' pe_res <- fracreghet.pe(res_emp, which.x="mrate")
#' summary(pe_res)
#'  
#' ### Simulated Examples
#' 
#' N <- 250
#' u <- rnorm(N)
#' 
#' X <- cbind(rnorm(N),rnorm(N))
#' dimnames(X)[[2]] <- c("X1","X2")
#' 
#' Z <- cbind(rnorm(N),rnorm(N),rnorm(N))
#' dimnames(Z)[[2]] <- c("Z1","Z2","Z3")
#' 
#' y <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u))
#' 
#' mod <- fracreghet(y,X,type="GMMx")
#' 
#' #Smearing estimator of average partial effects for variable X1
#' pe_res <- fracreghet.pe(mod,which.x="X1")
#' summary(pe_res)
#' 
#' #Naive estimator of conditional partial effects for all covariates,
#' #which are evaluated at X1=1 and X2=-1
#' pe_res <- fracreghet.pe(mod,smearing=FALSE,APE=FALSE,CPE=TRUE,at=c(1,-1))
#' summary(pe_res)
#' @export
fracreghet.pe <- function(object,smearing=TRUE,APE=TRUE,CPE=FALSE,at=NULL,which.x=NULL,table=FALSE,variance=TRUE)
{
	### 1. Error and warning messages

	if(missing(object)) stop("object is missing")
	if(is.null(object$class)) stop("object is not the output of an fracreghet command")
	if(object$class!="fracreghet") stop("object is not the output of an fracreghet command")

	if(all(c(APE,CPE)==FALSE)) stop("You must specify at least one option: APE and/or CPE")
	if(CPE==FALSE & !is.null(at)) stop("option at is only required for cpe")

	if(object$converged==0) stop("object is not the output of a successful (converged) fracreghet command")

	if(table==TRUE & variance==FALSE)
	{
		variance <- TRUE
		warning("option variance changed from FALSE to TRUE, as required by table=TRUE")
	}

	### 2. Recovering definitions and estimates and other definitions

	type <- object$type
	link <- object$link
	adjust <- object$adjust
	p <- object$p
	p.var <- object$p.var
	x <- model.matrix(object$formula)
	x.names <- object$x.names
	Hy <- object$Hy

	if(any(x.names=="(Intercept)")) xvar.names <- x.names[-1]
	else xvar.names <- x.names

	k <- length(xvar.names)
	npar <- ncol(x)
	n <- nrow(x)

	if(any(type==c("QMLxv"))) pv <- p[npar]
	if(any(type==c("GMMxv","LINxv","QMLxv")))
	{
		npar <- npar-1
		p <- p[1:npar]
		p.var <- p.var[1:npar,1:npar]
	}

	if(all(type!=c("GMMxv","LINxv","QMLxv"))) xbhat <- object$xbhat
	if(any(type==c("GMMxv","LINxv","QMLxv"))) xbhat <- as.vector(x[,-(npar+1)]%*%as.matrix(p))

	if(is.null(which.x)) which.x <- xvar.names
	xw.names <- unique(c(xvar.names,which.x))
	if(!identical(xvar.names,xw.names)) stop("option which not appropriately defined")

	### 3. Average partial effects

	if(smearing==TRUE) title1 <- "(conditional only on observables, based on the smearing estimator)"
	else title1 <- "(conditional on both observables and unobservables, with error term = 0)"

	title2 <- paste("Fractional",link,"regression")
	title3 <- paste("Estimator:",type)
	title <- c(title1,title2,title3)
	if(adjust!=0)
	{
		if(is.numeric(adjust)) title4 <- paste("Adjustment:",adjust,"added to all observations")
		else title4 <- "Adjustment: all boundary observations dropped"
		title <- c(title,title4)
	}

	if(APE==TRUE)
	{
		PE.type <- "APE"

		if(any(x.names=="(Intercept)")) p.pe <- matrix(rep(p[-1],each=n),ncol=k)
 		else p.pe <- matrix(rep(p,each=n),ncol=k)
		dimnames(p.pe) <- list(NULL,xvar.names)

		if(smearing==FALSE)
		{
			g <- fracreghet.links(link)$mu.eta(xbhat)
			what <- NA
		}
		if(smearing==TRUE)
		{
			if(type=="QMLxv") what <- x[,npar+1]*pv

			if(any(type==c("LINx","LINxv","LINz"))) what <- Hy-xbhat
			if(any(type==c("GMMx","GMMxv","GMMz")))
			{
				G2 <- fracreghet.links(link)$G2(xbhat)
				uhat.star <- Hy/G2-1
				what <- fracreghet.links(link)$H2(uhat.star+1)
			}

			g <- rep(NA,n)
			for(j in 1:n) g[j] <- mean(fracreghet.links(link)$mu.eta(xbhat[j]+what))
		}

		PE.p <- as.matrix(p.pe[,which.x])*g
		PE.p <- apply(PE.p,2,mean)

		resAPE <- list(PE.p=PE.p)

		if(variance==TRUE)
		{
			PE.sd <- fracreghet.pe.var(x,npar,which.x,x.names,xvar.names,type,p,xbhat,g,link,p.var,smearing,what,n)
			resAPE[["PE.sd"]] <- PE.sd
		}

		table.info.APE <- list(PE.p=PE.p,PE.sd=if(variance) PE.sd else NA,PE.type=PE.type,which.x=which.x,xvar.names=xvar.names,title=title,adjust=adjust,at=at)
		if(table==TRUE) do.call(fracreghet.pe.table, table.info.APE)
		resAPE[["table.info"]] <- table.info.APE
	}

	### 4. Conditional partial effects

	if(CPE==TRUE)
	{
		PE.type <- "CPE"

		if(is.null(at)) at <- "mean"

		if(length(at)==1)
		{
			if((!any(at==c("mean","median")) & k!=1)) stop("at not appropriately specified")

			if(any(at==c("mean","median")))
			{
				if(at=="mean") xm <- apply(x,2,mean)
				if(at=="median") xm <- apply(x,2,median)

				xdum <- apply(x,2,function(a) all(a %in% c(0,1)))

				if(any(type==c("GMMxv","LINxv","QMLxv")))
				{
					xm <- xm[-npar]
					xdum <- xdum[-npar]
				}
				xm[xdum==TRUE] <- round(xm,0)[xdum==TRUE]
			}
			else
			{
				if(is.numeric(at))
				{
					if(any(x.names=="(Intercept)")) xm <- c(1,at)
					else xm <- at
				}
				else stop("at not appropriately specified")
			}
		}
		else
		{
			if(length(at)!=k) stop("at not appropriately specified")
			else
			{
				if(any(x.names=="(Intercept)")) xm <- c(1,at)
				else xm <- at
			}
		}

		xmbhat <- as.vector(xm%*%p)

		if(smearing==FALSE) g <- fracreghet.links(link)$mu.eta(xmbhat)

		if(smearing==TRUE)
		{
			if(type=="QMLxv") what <- x[,npar]*pv
			if(any(type==c("LINx","LINxv","LINz"))) what <- Hy-xbhat
			if(any(type==c("GMMx","GMMxv","GMMz")))
			{
				G2 <- fracreghet.links(link)$G2(xbhat)
				uhat.star <- Hy/G2-1
				what <- fracreghet.links(link)$H2(uhat.star+1)
			}

			g <- mean(fracreghet.links(link)$mu.eta(xmbhat+what))
		}

		PE.p <- p[which.x]*g

		resCPE <- list(PE.p=PE.p)

		if(variance==TRUE)
		{
			PE.sd <- fracreghet.pe.var(x,npar,which.x,x.names,xvar.names,type,p,xbhat,g,link,p.var,smearing,what,n)
			resCPE[["PE.sd"]] <- PE.sd
		}

		table.info.CPE <- list(PE.p=PE.p,PE.sd=if(variance) PE.sd else NA,PE.type=PE.type,which.x=which.x,xvar.names=xvar.names,title=title,adjust=adjust,at=at)
		if(table==TRUE) do.call(fracreghet.pe.table, table.info.CPE)
		resCPE[["table.info"]] <- table.info.CPE
	}

	### 5. Return results

	if(APE==TRUE & CPE==TRUE) res <- list(ape=resAPE,cpe=resCPE)
	else if(APE==TRUE & CPE==FALSE) res <- resAPE
	else if(APE==FALSE & CPE==TRUE) res <- resCPE
	
	class(res) <- "fracreghet.pe"
	return(invisible(res))
}

fracreghet.pe.var <- function(x,npar,which.x,x.names,xvar.names,type,p,xbhat,g,link,p.var,smearing,what,N)
{
	if(smearing==FALSE) gd <- fracreghet.links(link)$gd(xbhat)
	if(smearing==TRUE) 
	{
		gd <- rep(NA,N)
		for(j in 1:N) gd[j] <- mean(fracreghet.links(link)$gd(xbhat[j]+what))
	}

	PE.sd <- matrix(NA,nrow=npar,ncol=npar)

	for(i in 1:npar)
	{
		for(j in 1:npar) PE.sd[i,j] <- mean(p[i]*gd*x[,j]+(i==j)*g)
	}

	PE.sd <- PE.sd%*%p.var%*%t(PE.sd)

	PE.sd <- diag(PE.sd)^0.5
	if(any(x.names=="(Intercept)")) PE.sd <- PE.sd[-1]

	names(PE.sd) <- xvar.names
	PE.sd <- PE.sd[which.x]

	return(PE.sd)
}

#' 401(k) Plan Participation Data
#'
#' A cross-sectional dataset on 401(k) plan participation rates and firm characteristics, 
#' widely used in empirical applications of fractional response models 
#' (e.g., Papke & Wooldridge, 1996). The data is derived from the 'wooldridge' package.
#'
#' @format A data frame with 1,534 observations and 10 variables:
#' \describe{
#'   \item{prate}{Participation rate: proportion of eligible employees participating in the 401(k) plan (0 to 1).}
#'   \item{mrate}{Match rate: the firm's contribution matching rate per dollar.}
#'   \item{totpart}{Total number of participants.}
#'   \item{totelg}{Total number of eligible employees.}
#'   \item{age}{Age of the 401(k) plan in years.}
#'   \item{totemp}{Total number of firm employees.}
#'   \item{sole}{Indicator variable: 1 if the 401(k) is the sole retirement plan offered.}
#'   \item{ltotemp}{Natural log of total employees.}
#'   \item{age_sq}{Square of plan age.}
#'   \item{mrate_sq}{Square of match rate.}
#' }
#' @source Papke, L. E., & Wooldridge, J. M. (1996). "Econometric Methods for Fractional Response Variables with an Application to 401(k) Plan Participation Rates." Journal of Applied Econometrics, 11(6), 619-632.
#' @usage data("fracreg_k401k")
#' @examples
#' data("fracreg_k401k")
#' summary(fracreg_k401k$prate)
"fracreg_k401k"

#' Government Spending Data
#'
#' Spending on different categories by Dutch cities in 2005. 
#' This dataset is commonly used to demonstrate fractional multinomial logit models.
#'
#' @format A data frame with 429 observations and 12 variables:
#' \describe{
#'   \item{muni}{Name of municipality}
#'   \item{houseval}{Average value of a house in 100,000 euros}
#'   \item{popdens}{Population density in 1000s of persons per square km}
#'   \item{noleft}{No left party in city government}
#'   \item{minorityleft}{Minority left party in city government}
#'   \item{governing}{Fraction of spending on governing}
#'   \item{safety}{Fraction of spending on safety}
#'   \item{education}{Fraction of spending on education}
#'   \item{recreation}{Fraction of spending on recreation}
#'   \item{social}{Fraction of spending on social services}
#'   \item{urbanplanning}{Fraction of spending on urban planning}
#'   \item{tot}{Total spending or population}
#' }
#' @source <http://fmwww.bc.edu/repec/bocode/c/citybudget.dta>
#' @usage data("fracreg_spending")
#' @examples
#' data("fracreg_spending")
#' head(fracreg_spending)
"fracreg_spending"

fracreg.table <- function(y,yhat,p,p.var,x.names,type,link,converged,var.type,R2=NA,LL=NULL,method=NULL,var.cluster=NULL,dfc=NULL,or=FALSE,level=0.95)
{
	if(converged==TRUE)
	{
		R2 <- cor(y,yhat)^2

		if(type!="2P" & type!="3P")
		{
			n <- length(y)
			
			Wald <- NA
			p_wald <- NA
			df_wald <- NA
			if(!is.character(p.var)) {
				if(length(p) > 1 && x.names[1] == "(Intercept)") {
					p_idx <- 2:length(p)
					W <- tryCatch(t(p[p_idx]) %*% solve(p.var[p_idx, p_idx]) %*% p[p_idx], error = function(e) NA)
					if (!is.na(W)) {
						Wald <- as.numeric(W)
						df_wald <- length(p_idx)
						p_wald <- 1 - pchisq(Wald, df = df_wald)
					}
				} else if(length(p) > 0 && x.names[1] != "(Intercept)") {
					W <- tryCatch(t(p) %*% solve(p.var) %*% p, error = function(e) NA)
					if (!is.na(W)) {
						Wald <- as.numeric(W)
						df_wald <- length(p)
						p_wald <- 1 - pchisq(Wald, df = df_wald)
					}
				}
			}

			p.sd <- diag(p.var)^0.5
			z.ratio <- p/p.sd
			p.value <- 2*pnorm(abs(z.ratio), lower.tail=FALSE)
			qval <- qnorm((1 + level) / 2)
			ci.low <- p - qval * p.sd
			ci.high <- p + qval * p.sd

			if(or && link == "logit") {
				p <- exp(p)
				p.sd <- p.sd * p
				ci.low <- exp(ci.low)
				ci.high <- exp(ci.high)
			}
			
			conf_str <- paste0("[", round(level * 100), "% Conf.")
			res <- cbind(Coefficient = p, `Std.Err.` = p.sd, `z value` = z.ratio, `ci_low` = ci.low, `ci_high` = ci.high, `Pr(>|z|)` = p.value)
			colnames(res)[4] <- conf_str
			colnames(res)[5] <- "Interval]"

			if(or && link == "logit") {
				colnames(res)[1] <- "Odds Ratio"
			}

			se_type <- var.type
			if(se_type == "standard") se_type <- "EIM"
			colnames(res)[2] <- paste0(tools::toTitleCase(se_type), " Std.Err.")
			rownames(res) <- x.names
			rownames(res)[rownames(res) == "(Intercept)"] <- "(Intercept)"

			cat("\n")
			cat(.fracreg.sep(), "\n")
			if(type=="2Pbin") cat(.fracreg.center(paste("Part 1: Binary", link, "regression")), "\n")
			if(type=="1P") cat(.fracreg.center(paste("Fractional", link, "regression")), "\n")
			if(type=="2Pfrac") cat(.fracreg.center(paste("Part 2: Fractional", link, "regression")), "\n")
			if(type=="3Pbin0") cat(.fracreg.center(paste("Part 1: Binary", link, "regression")), "\n")
			if(type=="3Pbin1") cat(.fracreg.center(paste("Part 2: Binary", link, "regression")), "\n")
			if(type=="3Pfrac") cat(.fracreg.center(paste("Part 3: Fractional", link, "regression")), "\n")
			cat(.fracreg.sep(), "\n")

			.fracreg.cat.right("Data type:", "Cross-sectional")
			if(!is.null(method)) .fracreg.cat.right("Estimator:", method)
			.fracreg.cat.right("Convergence:", "Successful")
			.fracreg.cat.right("Number of observations:", n)
			if(!is.null(var.cluster) && var.type=="cluster") .fracreg.cat.right("Number of clusters:", length(unique(var.cluster)))
			if(!is.null(LL)) {
				if(!is.null(method) && method == "ML") {
					.fracreg.cat.right("Log-likelihood:", round(LL, 4))
				} else {
					.fracreg.cat.right("Log pseudolikelihood:", round(LL, 4))
				}
			}
			.fracreg.cat.right("Pseudo R-squared:", round(R2, 5))
			if(!is.na(Wald)) {
				.fracreg.cat.right(paste0("Wald chi2(", df_wald, "):"), round(Wald, 4))
				.fracreg.cat.right("Prob > chi2:", sprintf("%.4f", p_wald))
			}
			if(var.type!="standard") {
				if(var.type == "robust") {
					robust_type <- if(is.null(dfc) || !dfc) "HC0" else "HC1"
					.fracreg.cat.right("Standard errors:", robust_type)
				} else if(var.type == "cluster") {
					.fracreg.cat.right("Standard errors:", "CRVE")
				} else {
					.fracreg.cat.right("Standard errors:", var.type)
				}
			}
			if(is.null(dfc) || !dfc) {
				.fracreg.cat.right("Small sample correction:", "FALSE")
			} else {
				.fracreg.cat.right("Small sample correction:", "TRUE")
			}
			
			cat(.fracreg.sep(), "\n")
			method_full <- method
			if(!is.null(method)) {
				if(method == "QML") method_full <- "Quasi-Maximum Likelihood"
				else if(method == "ML") method_full <- "Maximum Likelihood"
			}
			cat(.fracreg.center(if(!is.null(method_full)) paste("Final", method_full, "estimates") else "Final estimates"), "\n")
			cat(.fracreg.sep(), "\n")

			stats::printCoefmat(res, P.values = TRUE, has.Pvalue = TRUE, digits = 4, signif.legend = TRUE)
			
			cat(.fracreg.sep(), "\n")
			if(type == "1P") cat(.fracreg.center(paste("Run Date:", format(Sys.time(), "%Y-%m-%d %H:%M:%S"))), "\n")
			if(type == "1P") cat(.fracreg.sep(), "\n")
		}
		else
		{
			cat("\n")
			cat(.fracreg.sep(), "\n")
			if(type=="2P") cat(.fracreg.center(paste("Two-part fractional regression: binary", link[1], "+ fractional", link[2])) , "\n")
			if(type=="3P") cat(.fracreg.center(paste("Three-part fractional regression: binary", link[1], ", binary", link[2], "+ fractional", link[3])) , "\n")
			cat(.fracreg.sep(), "\n")
			.fracreg.cat.right("Data type:", "Cross-sectional")
			.fracreg.cat.right("Convergence:", "Successful")
			.fracreg.cat.right("Pseudo R-squared:", round(R2, 5))
			cat(.fracreg.sep(), "\n")
			cat(.fracreg.center(paste("Run Date:", format(Sys.time(), "%Y-%m-%d %H:%M:%S"))), "\n")
			cat(.fracreg.sep(), "\n")
		}
	}
	else {
		cat(.fracreg.sep(), "\n")
		.fracreg.cat.right("Convergence:", "FAILED OR STOPPED AT BOUNDARY")
		cat(.fracreg.sep(), "\n")
		if(type %in% c("1P", "2P", "3P")) cat(.fracreg.center(paste("Run Date:", format(Sys.time(), "%Y-%m-%d %H:%M:%S"))), "\n")
		if(type %in% c("1P", "2P", "3P")) cat(.fracreg.sep(), "\n")
	}
	cat("\n")
}

#' @title Fractional Response Regressions
#'
#' @description
#' Provides comprehensive tools for the estimation and specification analysis of fractional response models. It supports univariate one-part, hurdle two-part, and double-inflated three-part models. The package also incorporates estimators for panel data settings (CRE, GMM, QML) and addresses unobserved heterogeneity and endogeneity via correlated random effects and control function approaches. It supports various link functions, calculates average and conditional partial effects analytically across all model types, and provides robust specification tests such as RESET, P test, and GGOFF tests.
#'
#' @details
#' \tabular{ll}{
#' Package: \tab fracreg\cr
#' Type: \tab Package\cr
#' Version: \tab 1.0.0\cr
#' Date: \tab 2026-07-05\cr
#' License: \tab GPL-3\cr
#' }
#'
#' @section Acknowledgements:
#' This package builds upon, consolidates, and modernises the fractional regression frameworks originally implemented in the \code{frm}, \code{frmhet}, and \code{frmpd} R packages developed by Joaquim J.S. Ramalho. As those original packages have been deprecated and removed from the active CRAN repository, \code{fracreg} serves as an actively maintained successor, ensuring these econometric tools remain available to the R community.
#' 
#' Furthermore, we acknowledge James Ji (@f1kidd) and A. John Woodill (@johnwoodill), the authors of the \code{fmlogit} R package on GitHub, whose foundational work on fractional multinomial logit models inspired the implementation of \code{fracregmlogit}. We also extend our gratitude to Ariel Rokem and Kendrick Kay, the authors of the \code{fracridge} package, whose methodological contributions to fractional ridge regression are incorporated into the \code{fracregridge} functionalities of this package.
#'
#' @references
#' Ramalho, J. J. S. \emph{frm: Fractional Regression Models}. R package. Formerly available on CRAN, currently archived.
#' 
#' Ramalho, J. J. S. \emph{frmhet: Fractional Regression Models under Heterogeneity}. R package. Formerly available on CRAN, currently archived.
#' 
#' Ramalho, J. J. S. \emph{frmpd: Fractional Regression Models for Panel Data}. R package. Formerly available on CRAN, currently archived.
#' 
#' Papke, L. E. and Wooldridge, J. M. (1996), "Econometric methods for fractional response variables with an application to 401(k) plan participation rates", \emph{Journal of Applied Econometrics}, 11(6), 619-632.
#' 
#' Ramalho, E.A., J.J.S. Ramalho and J.M.R. Murteira (2011), "Alternative estimating and testing empirical strategies for fractional response models", \emph{Journal of Economic Surveys}, 25(1), 19-68.
#' 
#' Ramalho, E. A., Ramalho, J. J. S., and Murteira, J. M. R. (2014), "A two-part fractional response model for the spatial distribution of vineyards in Portugal", \emph{Journal of Applied Econometrics}, 29(4), 607-630.
#' 
#' Fang, K., & Ma, S. (2013), "Three-part model for fractional response variables with application to Chinese household health insurance coverage", \emph{Journal of Applied Statistics}, 40(5), 925-940.
#' 
#' Ji, J., and Woodill, A. J., \emph{fmlogit: Fractional Multinomial Logit}. R package repository. <https://github.com/f1kidd/fmlogit>.
#' 
#' Rokem, A., and Kay, K., \emph{fracridge: Fractional Ridge Regression}. Package repository. <https://github.com/nrdg/fracridge>.
#'
#' @author Sulman Olieko Owili <oliekosulman@gmail.com>
#' @importFrom graphics par
#' @importFrom stats Gamma as.formula binomial coef cor dcauchy dlogis dnorm fitted glm lm logLik median model.frame model.matrix model.response na.omit nlminb nobs pcauchy pchisq plogis pnorm predict printCoefmat pt qcauchy qlogis qnorm quasibinomial residuals sd var vcov
#' @importFrom ggplot2 ggplot aes_string geom_point geom_hline theme_classic ggtitle theme element_text scale_color_discrete
#' @importFrom grid pushViewport viewport grid.layout
"_PACKAGE"

#' @title Partial Effects for Fractional Ridge Regression
#'
#' @description
#' Because Fractional Ridge Regression fits a linear model without a link function, the partial effects are mathematically identical to the estimated ridge coefficients. This function serves as a wrapper to maintain API compatibility with the rest of the \code{fracreg} package, printing a brief notification and returning the standard coefficient tables.
#' @param object An object of class \code{fracregridge}.
#' @param APE logical. Ignored for ridge regression.
#' @param CPE logical. Ignored for ridge regression.
#' @param at numeric vector. Ignored for ridge regression.
#' @param variance logical. Ignored for ridge regression.
#' @param table logical. Ignored for ridge regression.
#' @param ... further arguments passed to or from other methods.
#'
#' @return
#' An object of class \code{fracreg.pe} containing the standard coefficient tables.
#'
#' @author Sulman Olieko Owili <oliekosulman@gmail.com>
#'
#' @seealso
#' \code{\link{fracregridge}}, \code{\link{fracreg.pe}}
#'
#' @examples
#' # Generate random data
#' set.seed(123)
#' y <- rnorm(100)
#' X <- matrix(rnorm(1000), 100, 10)
#' colnames(X) <- paste0("X", 1:10)
#' 
#' # Fit Fractional Ridge Regression
#' mod <- fracregridge(y, X, fracs = c(0.3, 0.5))
#' 
#' # Compute Partial Effects (identical to coefficients)
#' pe <- fracregridge.pe(mod)
#' print(pe)
#' @export
fracregridge.pe <- function(object, APE=TRUE, CPE=FALSE, at=NULL, variance=TRUE, table=FALSE, ...) {
    if(missing(object)) stop("object is missing")
    if(is.null(object$class) || object$class != "fracregridge") stop("object is not the output of a fracregridge command")
    
    cat("\nNote: Fractional Ridge Regression is a linear model without a link function.\n")
    cat("Therefore, the partial effects are mathematically identical to the coefficients themselves.\n\n")
    
    res <- list(
        call = match.call(),
        type = "fracregridge.pe",
        class = "fracregridge.pe",
        table.info = object$table.info
    )
    
    class(res) <- c("fracregridge.pe", "fracregridge")
    return(res)
}

#' @title Extract Model Coefficients for fracregmlogit
#' @description Extracts the estimated coefficients from a fitted \code{fracregmlogit} model.
#' @param object A fitted model object of class \code{fracregmlogit}.
#' @param ... Further arguments passed to or from other methods.
#' @return A matrix of coefficients for each choice equation.
#' @exportS3Method coef fracregmlogit
coef.fracregmlogit <- function(object, ...) {
    return(object$coefficient)
}

#' Extract Fitted Values, Residuals, and Predictions
#' 
#' @name fitted.fracregmlogit
#' @aliases residuals.fracregmlogit
#' @aliases predict.fracregmlogit
#' Extract fitted dependent variable, residuals, or predictions from a fractional multinomial logit model. 
#' @param object A "fracregmlogit" object.
#' @param newdata A new X matrix to perform model prediction. If NULL, defaults to the original dataset. 
#' X can be a vector with length k, or a matrix with k columns, where k is the number of explanatory 
#' variables in the original model. 
#' @param newbeta A new augmented matrix of coefficients that can be used to predict outcome variables. 
#' Feeds into \code{object$coefficient}, which contains the baseline coefficient. Useful for constructing
#' confidence intervals via simulation or bootstrapping. 
#' @param ... Additional arguments.
#' @return An object of class \code{data.frame} containing numeric values where each column corresponds to one of the choice alternatives in the response variable matrix and each row corresponds to an observation. Specifically:
#' \itemize{
#'   \item \code{fitted}: Returns the estimated fitted fractional response values (choice shares or predicted conditional probabilities).
#'   \item \code{residuals}: Returns the response residuals (the actual observed shares minus the estimated fitted shares).
#'   \item \code{predict}: Returns the predicted choice shares or conditional probabilities computed from the specified model object and \code{newdata} or \code{newbeta}.
#' }
#' @seealso \code{\link{fracregmlogit}}
#' @examples
#' data("fracreg_spending")
#' df <- na.omit(fracreg_spending)
#' X = df[,2:5]
#' y = df[,6:11]
#' results1 = fracregmlogit(y, X)
#' 
#' # Extract fitted values
#' fit = fitted(results1)
#' 
#' # Extract residuals
#' res = residuals(results1)
#' 
#' # Predict using the first observation from the original dataset
#' pred = predict(results1, newdata = X[1,])
#' @rdname fitted.fracregmlogit
#' @exportS3Method fitted fracregmlogit
fitted.fracregmlogit <- function(object, ...) {
  j=length(object$estimates)+1; k=dim(object$estimates[[1]])[1]; N=dim(object$y)[1]
  betamat_aug = object$coefficient; X=object$X; y=object$y
  sum_expxb = rowSums(exp(X %*% t(betamat_aug))) # sum of the exp(x'b)s
  yhat = y
  for(i in 1:j){
    expxb = exp(X %*% betamat_aug[i,]) # individual exp(x'b)
    yhat[,i] = expxb / sum_expxb
  }
  return(as.data.frame(yhat))
}

#' @rdname fitted.fracregmlogit
#' @exportS3Method residuals fracregmlogit
residuals.fracregmlogit <- function(object, ...) {
  yhat = fitted(object)
  return(as.data.frame(object$y-yhat))
}

#' @rdname fitted.fracregmlogit
#' @exportS3Method predict fracregmlogit
predict.fracregmlogit <- function(object, newdata=NULL, newbeta = NULL, ...) {
  if(length(newdata)==0) return(fitted(object))
  if(length(newbeta)>0) object$coefficient = newbeta
  j=length(object$estimates)+1; k=dim(object$estimates[[1]])[1]; N=dim(object$y)[1]
  betamat_aug = object$coefficient;
  newdata = as.matrix(newdata)
  if(length(newdata) == dim(newdata)[1]) newdata = t(newdata) # vector
  if(k != dim(newdata)[2]+1) stop(paste("Dimension of newdata is wrong. Should be",k-1,"instead of",dim(newdata)[2]))
  X = cbind(newdata,1); N = dim(X)[1]
  yhat = matrix(ncol=j,nrow=N); colnames(yhat) = colnames(object$y)
  sum_expxb = rowSums(exp(X %*% t(betamat_aug))) # sum of the exp(x'b)s
  for(i in 1:j){
    expxb = exp(X %*% betamat_aug[i,]) # individual exp(x'b)
    yhat[,i] = expxb / sum_expxb
  }
  return(as.data.frame(yhat))
}

#' @title Extract the Number of Observations for fracregmlogit
#' @description Extracts the number of observations used to estimate a \code{fracregmlogit} model.
#' @param object A fitted model object of class \code{fracregmlogit}.
#' @param ... Further arguments passed to or from other methods.
#' @return An integer denoting the number of observations.
#' @exportS3Method nobs fracregmlogit
nobs.fracregmlogit <- function(object, ...) {
    return(unname(object$count["Obs"]))
}

#' @title Extract Log-Likelihood for fracregmlogit
#' @description Extracts the log-pseudolikelihood from a fitted \code{fracregmlogit} model.
#' @param object A fitted model object of class \code{fracregmlogit}.
#' @param ... Further arguments passed to or from other methods.
#' @return An object of class \code{logLik}.
#' @exportS3Method logLik fracregmlogit
logLik.fracregmlogit <- function(object, ...) {
    if(is.null(object$likelihood)) {
        return(NA)
    }
    val <- object$likelihood
    # df is roughly number of choices * number of params
    k <- dim(object$coefficient)[2]
    j <- dim(object$coefficient)[1] - 1
    attr(val, "df") <- k * j
    attr(val, "nobs") <- nobs(object)
    class(val) <- "logLik"
    return(val)
}

#' @title Extract Covariance Matrix for fracregmlogit
#' @description Extracts the estimated variance-covariance matrices of the parameters.
#' @param object A fitted model object of class \code{fracregmlogit}.
#' @param ... Further arguments passed to or from other methods.
#' @return A list of covariance matrices for each choice equation.
#' @exportS3Method vcov fracregmlogit
vcov.fracregmlogit <- function(object, ...) {
    return(object$vcov)
}

#' @title GGOFF Tests for Fractional Response Regressions
#'
#' @description
#' \code{fracreg.ggoff} is used to perform Generalised Goodness-Of-Functional-Form (GGOFF) tests to check the adequacy of the functional form and link specification of fractional response models.
#' @param object an object containing the results of an \code{fracreg} command.
#' @param version a vector containing the test versions to use. Available options: \code{Wald}, \code{LM} (the default) and, only for the binary component of two-part models, \code{LR}. More than one option may be chosen.
#' @param table a logical value indicating whether a summary table with the test results should be printed.
#' @param \dots Arguments to pass to \link[stats]{glm}, which is used to estimate the model under the alternative hypothesis when \code{version} is a vector containing \code{"Wald"} or \code{"LR"}.
#'
#' @details
#' \code{fracreg.ggoff} applies the GGOFF, GOFF1 and GOOFF2 test statistics to fractional response
#' models estimated via \code{fracreg}. \code{fracreg.ggoff} may be used to test the link
#' specification of: (i) one-part fractional response models; (ii) the binary
#' component of two-part fractional response models; and (iii) the fractional
#' component of two-part fractional response models.
#' 
#' \strong{GGOFF Test Framework:}
#' The Generalised Goodness-of-Functional Form (GGOFF) test evaluates the adequacy of the link function \eqn{G(\cdot)}. It is based on augmenting the baseline model with specific directions of departure. The auxiliary testing equation takes the form:
#' \deqn{E(y|x) = G\left(x\beta + \gamma_1 \frac{g'(x\hat{\beta})}{g(x\hat{\beta})} + \gamma_2 x\hat{\beta} \right)}
#' where \eqn{g(\cdot)} and \eqn{g'(\cdot)} are the first and second derivatives of \eqn{G(\cdot)} evaluated at the linear predictor \eqn{x\hat{\beta}}. The test checks \eqn{H_0: \gamma_1 = 0, \gamma_2 = 0}. GOFF1 and GOFF2 are variants testing individual components.
#' 
#' When the \code{Wald} version is implemented, it is taken into account the option that was chosen for computing standard errors in the model under evaluation. For the \code{LM} version, a robust version is computed in cases (i) and (iii) and a conventional version in case (ii). See Ramalho, Ramalho and Murteira (2014) for details on the application of the GGOFF, GOFF1 and GOOFF2 tests in the fractional response framework.
#'
#' @return
#' \code{fracreg.ggoff} returns a named vector with the test results.
#'
#' @references
#' Ramalho, E.A., J.J.S. Ramalho and J.M.R. Murteira (2014), "A generalized goodness-of-functional form test for binary and fractional response models", \emph{Manchester School}, 82(4), 488-507.
#' 
#' Pregibon, D. (1980), "Goodness of Link Tests for Generalized Linear Models", \emph{Journal of the Royal Statistical Society: Series C (Applied Statistics)}, 29(1), 15-24.
#'
#' @author Sulman Olieko Owili <oliekosulman@gmail.com>
#'
#' @seealso
#' \code{\link{fracreg}}, for fitting fractional response models.\cr
#' \code{\link{fracreg.reset}}, for asymptotically equivalent specification tests.\cr
#' \code{\link{fracreg.ptest}}, for non-nested hypothesis tests.\cr
#' \code{\link{fracreg.pe}}, for computing partial effects.
#'
#' @examples
#' ### Empirical 401(k) Examples
#' data("fracreg_k401k")
#' y <- fracreg_k401k$prate
#' X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age, 
#'            totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole)
#' 
#' m <- fracreg(y, X, type="1P", linkfrac="logit")
#' ggoff_res <- fracreg.ggoff(m)
#' summary(ggoff_res)
#' 
#' ### Simulated Examples
#' 
#' N <- 250
#' u <- rnorm(N)
#' 
#' X <- cbind(rnorm(N),rnorm(N))
#' dimnames(X)[[2]] <- c("X1","X2")
#' 
#' ym <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u))
#' y <- rbeta(N,ym*20,20*(1-ym))
#' y[y > 0.9] <- 1
#' 
#' #Testing the logit specification of a standard fractional response model
#' #using LM and Wald versions of the GGOFF test, based on 1 or 2 fitted powers of
#' #the linear predictor
#' mod <- fracreg(y,X,linkfrac="logit")
#' ggoff_res <- fracreg.ggoff(mod,c("Wald","LM"))
#' summary(ggoff_res)
#' 
#' #Testing the probit specification of the binary component of a two-part fractional
#' #regression model using a LR-based GGOFF test
#' mod <- fracreg(y,X,linkbin="probit",type="2Pbin",inf=1)
#' ggoff_res <- fracreg.ggoff(mod,"LR")
#' summary(ggoff_res)
#' @export
fracreg.ggoff <- function(object,version="LM",table=FALSE,...)
{
	### 1. Error and warning messages

	if(missing(object)) stop("object is missing")
	if(is.null(object$class)) stop("object is not the output of an fracreg command")
	if(object$class!="fracreg") stop("object is not the output of an fracreg command")
	if(object$type=="2P" | object$type=="3P") stop("GGOFF tests are not applicable to two-part or three-part models")
	if(object$converged==0) stop("object is not the output of a successful (converged) fracreg command")
	if(object$method!="ML" & any(version=="LR")) stop("LR tests require ML estimation")
	if(all(version!="LM") & all(version!="Wald") & all(version!="LR")) stop("test version not correctly specified")
	if(!is.logical(table)) stop("non-logical value assigned to option table")

	### 2. Recovering definitions and estimates

	mf <- model.frame(object$formula)
	y <- model.response(mf)
	x <- model.matrix(object$formula)

	yhat <- object$yhat
	xbhat <- object$xbhat
	method <- object$method
	link <- object$link
	type <- object$type

	if(any(version=="Wald"))
	{
		if(is.null(object$var.type)) stop("Wald test required but fracreg command was run with variance = FALSE")
		var.type <- object$var.type
		var.eim <- object$var.eim
		dfc <- object$dfc
		if(var.type=="cluster") var.cluster <- object$var.cluster
	}
	if(any(version=="LR")) LL0 <- object$LL
	if(type=="1P") title <- paste("Fractional",link,"regression")
	if(type=="2Pbin") title <- paste("Binary",link,"component of a two-part regression")
	if(type=="2Pfrac") title <- paste("Fractional",link,"component of a two-part regression")

	### 3. Tests

	ver <- NA
	S <- NA
	Sp <- NA
	test <- NA
	tests <- c("GOFF1","GOFF2","GGOFF")

	g <- fracreg.links(link)$mu.eta(xbhat)
	gx <- g*x
	if(link!="loglog") z1 <- yhat*log(yhat)/g
	if(link!="cloglog") z2 <- (1-yhat)*log(1-yhat)/g

	for(j in 1:3)
	{
		if((j==1 & link!="loglog") | (j==2 & link!="cloglog") | j==3)
		{
			if(j==1) z <- z1
			if(j==2) z <- z2
			if(j==3)
			{
				if(all(link!=c("loglog","cloglog"))) z <- cbind(z1,z2)
				if(link=="loglog") z <- z2
				if(link=="cloglog") z <- z1
			}
			z <- as.matrix(z)

			df <- ncol(z)

			if(any(version=="LM"))
			{
				test <- c(test,tests[j])
				ver <- c(ver,"LM")

				gz <- g*z
				results <- fracreg.lm(y,yhat,gx,gz,type)
				Sj <- results$LM
				S <- c(S,Sj)
				Sp <- c(Sp,1-pchisq(Sj,df))
			}
			if(any(version=="LR") | any(version=="Wald"))
			{
				if(any(version=="Wald")) results <- fracreg.est(y,cbind(x,z),link,method,TRUE,var.type,var.eim,var.cluster,dfc,...)
				else results <- fracreg.est(y,cbind(x,z),link,method,FALSE,...)

				if(any(version=="LR"))
				{
					test <- c(test,tests[j])
					ver <- c(ver,"LR")

					if(results$converged==TRUE)
					{
						LL1 <- results$LL

						Sj <- 2*(LL1-LL0)
						S <- c(S,Sj)
						Sp <- c(Sp,1-pchisq(Sj,df))
					}
					else
					{
						S <- c(S,NA)
						Sp <- c(Sp,NA)
					}
				}
				if(any(version=="Wald"))
				{
					test <- c(test,tests[j])
					ver <- c(ver,"Wald")

					if(results$converged==TRUE)
					{
						p1 <- results$p
						p.var1 <- results$p.var

						p.n <- length(p1)
						p1 <- p1[(p.n-df+1):p.n]
						p.var1 <- p.var1[(p.n-df+1):p.n,(p.n-df+1):p.n]

						Sj <- t(p1)%*%solve(p.var1)%*%p1

						S <- c(S,Sj)
						Sp <- c(Sp,1-pchisq(Sj,df))
					}
					else
					{
						S <- c(S,NA)
						Sp <- c(Sp,NA)
					}
				}
			}
		}
	}

	table.info <- list(test.which="GGOFF",S=S,Sp=Sp,ver=ver,title1=title,test.ggoff=test)
	if(table==TRUE) do.call(fracreg.tests.table, table.info)

	### 4. Return results

	statistics <- S[-1]
	names(statistics) <- paste(test[-1],ver[-1],sep="-")

	class(statistics) <- c("fracreg.ggoff", "numeric")
	attr(statistics, "table.info") <- table.info

	return(invisible(statistics))
}

fracreg.pe.table <- function(PE.p,PE.sd,PE.type,which.x,xvar.names,title,at)
{
	z.ratio <- PE.p/PE.sd
	p.value <- 2*(1-pnorm(abs(z.ratio)))

	res <- cbind(`dy/dx` = PE.p, `Std. Error` = PE.sd, `z value` = z.ratio, `Pr(>|z|)` = p.value)
	rownames(res) <- which.x
	rownames(res)[rownames(res) == "(Intercept)"] <- "(Intercept)"

	cat("\n\n")
	cat(.fracreg.sep(), "\n")
	if(PE.type=="APE") cat(.fracreg.center("Average partial effects"), "\n")
	if(PE.type=="CPE") cat(.fracreg.center("Conditional partial effects"), "\n")
	cat(.fracreg.sep(), "\n")
	
	cat(.fracreg.center(title), "\n")
	cat(.fracreg.sep(), "\n")
	
	if(!is.na(PE.sd[1])) cat("\nNote: Standard errors computed using the Delta method\n")
	stats::printCoefmat(res, P.values = TRUE, has.Pvalue = TRUE, digits = 4, signif.legend = TRUE)
	
	cat(.fracreg.sep(), "\n")
	cat(.fracreg.center(paste("Run Date:", format(Sys.time(), "%Y-%m-%d %H:%M:%S"))), "\n")
	cat(.fracreg.sep(), "\n")
	
	if(PE.type=="CPE")
	{
		if(length(at)==1)
		{
			if(any(at==c("mean","median"))) cat("\nNote: covariates evaluated at",at,"(or mode, for dummies) values\n")
			else
			{
				names(at) <- xvar.names
				cat("\nNote: covariates evaluated at the following values:\n\n")
				print(at)
			}
		}
		else
		{
			names(at) <- xvar.names
			cat("\nNote: covariates evaluated at the following values:\n\n")
			print(at)
		}
	}
}

#' Estimate Fractional Multinomial Logit Models
#' 
#' Used to estimate fractional multinomial logit models using quasi-maximum
#' likelihood estimation following Papke and Wooldridge (1996).
#' @param y the dependent variable (N*J). Can be a matrix or a named data frame.
#'   The first column of the matrix is automatically treated as the baseline.
#' @param X independent variable (N*K). Can be a matrix or a named data frame.
#'   If there is no intercept term in the X, then an intercept term is
#'   automatically added.
#' @param beta0 Initial value for beta used in optimisation. Uses a 1*K(J-1)
#'   vector. Default to a vector of zeros.
#' @param MLEmethod Method of optimisation. Goes into
#'   \code{maxLik(method=MLEmethod))}. Choose from "NR","BFGS","CG","BHHH","SANN",or "NM".  
#'   Default to "CG", the conjugate gradients method. See Details. 
#' @param maxit Maximum number of iterations.
#' @param abstol Tolerance.
#' @param cluster A vector of clusters to be used for clustered standard error computation. 
#' Default to NULL, no cluster computed. 
#' @param reps Number of bootstrap replications to be computed for clustered standard errors.
#' @param na.action A function specifying how to handle missing values, default is \code{stats::na.omit}. If \code{NULL}, no action is taken.
#' @param ... additional parameters that go into \code{maxLik()}
#' @return The function returns an object of class "fracregmlogit". Use \code{fracregmlogit.pe}, \code{predict}, 
#'  \code{residuals}, \code{fitted} to extract various useful features of the value returned by 
#' \code{fracregmlogit}. 
#' @return An object of class "fracregmlogit" contains the following components: 
#' @return \code{estimates}   A list of matrices containing parameter estimates,
#'   standard errors, and hypothesis testing results.
#' @return \code{baseline}    The baseline choice
#' @return \code{likelihood}  The likelihood value
#' @return \code{conv_code}   Convergence diagnostics code. 
#' @return \code{convergence} Convergence messages. 
#' @return \code{count}       Provides dataset information
#' @return \code{y}           The dependent variable data frame.
#' @return \code{X}           The independent variable data frame. Augmented by factor dummy transformation
#' , constant term added. 
#' @return \code{rowNo}       A vector of row numbers from the original X and y that is used for estimation.
#' @return \code{coefficient} Matrix of estimated coefficients. Augmented with the baseline coefficient
#' (which is a vector of zeros). 
#' @return \code{vcov}        A list of matrices containing the robust variance covariance matrix for each choice
#' variable. 
#' @return \code{cluster}     The vector of clusters. 
#' @return \code{reps}        Number of bootstrap replications for clustered standard error
#' @details The fractional multinomial logit model is the expansion of the multinomial
#' logit to fractional responses. Unlike standard multinomial logit models,
#' which only consider 0-1 responses, the fractional multinomial logit model considers the
#' case where the response variable is fractions that sum up to one. Examples
#' of this type of data include percentages of budget spent in education, defence,
#' public health; fractions of a population that have middle school, high
#' school, college, or post-college education, etc.
#' 
#' This function follows Papke and Wooldridge (1996)'s paper, in which they
#' proposed a quasi-maximum likelihood estimator for fractional response data.
#' The likelihood function used here is a standard multinomial likelihood
#' function, see Buis (2008) and <http://maartenbuis.nl/software/likelihoodFmlogit.pdf> for
#' the likelihood used here. Robust standard errors are provided following Papke
#' and Wooldridge (1996), in which they proposed an asymptotically consistent
#' estimator of variance.
#' 
#' Maximisation is done by calling \code{\link[maxLik]{maxLik}}. maxLik is a wrapper function 
#' for different maximisation methods in R. These include most methods provided by \code{\link[maxLik]{maxLik}},  
#' but also other methods such as BHHH (Berndt-Hall-Hall-Hausman). 
#' 
#' MLE convergence can be a problem in R, especially if the dataset is large with many explanatory variables. 
#' It is recommended to call CG (Conjugate Gradients) or BHHH (Berndt-Hall-Hall-Hausman).
#' The conjugate gradients method is usually faster, but could lead to non-convergence under 
#' certain scenarios. BHHH is slower, but has better convergence properties.
#' 
#' @seealso \code{\link{fracregmlogit.pe}} for computing partial effects, \code{\link{plot.fracregmlogit.pe}} for plotting effects, \code{\link{fitted.fracregmlogit}} for residuals and predictions.
#' 
#' @examples
#' data("fracreg_spending")
#' X = fracreg_spending[,2:5]
#' y = fracreg_spending[,6:11]
#' 
#' # Fit the fractional multinomial logit model
#' results1 = fracregmlogit(y, X)
#' 
#' # View estimates
#' summary(results1)
#' 
#' # Compute marginal effects
#' pe = fracregmlogit.pe(results1, effect="marginal", marg.type="aveacr", se=TRUE, R=50)
#' summary(pe)
#' 
#' # Plot effects for 'houseval'
#' plot(pe, varlist="houseval")
#' 
#' @references Papke, L. E. and Wooldridge, J. M. (1996), Econometric methods
#'   for fractional response variables with an application to 401(k) plan
#'   participation rates. J. Appl. Econ., 11: 619-632.
#' @references Buis, M. L. (2008), fmlogit: Stata module fitting a fractional 
#'   multinomial logit model by quasi maximum likelihood. Statistical Software Components, 
#'   Boston College Department of Economics.
#' @references Mullahy, J. (2015), Multivariate fractional regression estimation of 
#'   econometric share models. Journal of Econometric Methods, 4(1): 71-100.
#' @references Murteira, J. M. R., and Ramalho, J. J. S. (2016), Regression analysis of 
#'   multivariate fractional data. Econometric Reviews, 35(4): 515-552.
#' @references Ji, J., and Woodill, A. J., fmlogit: Fractional Multinomial Logit. R package repository. <https://github.com/f1kidd/fmlogit>.
#' @importFrom maxLik maxLik
#' @export fracregmlogit




fracregmlogit=function(y, X, beta0 = NULL, MLEmethod = "CG", maxit = 5e+05, 
                          abstol = 1e-05,cluster=NULL,reps=1000, na.action=stats::na.omit, ...){
  start.time = proc.time()
  
  if (!missing(y) && !missing(X)) {
      args_to_clean <- list(y=y, X=X)
      if (!is.null(cluster)) args_to_clean$cluster <- cluster
      
      args_to_clean$na.action <- na.action
      cleaned <- do.call(fracreg_clean_data, args_to_clean)
      
      y <- cleaned$y
      X <- cleaned$X
      if (!is.null(cluster)) cluster <- cleaned$cluster
  }
  
  if(length(cluster)!=nrow(y) & !is.null(cluster)){
    warning("Length of the cluster does not match the data. Cluster is ignored.")
    cluster = NULL
  }
  Xclass = sapply(X, class)
  Xfac = which(Xclass %in% c("factor", "character"))
  if (length(Xfac) > 0) {
    Xfacnames = colnames(X)[Xfac]
    strformFac = paste(Xfacnames, collapse = "+")
    Xdum = model.matrix(as.formula(paste("~", strformFac, 
                                         sep = "")), data = X)[, -1]
    X = cbind(X, Xdum)
    X = X[, -Xfac]
  }
  Xnames = colnames(X)
  ynames = colnames(y)
  X = as.matrix(X)
  y = as.matrix(y)
  n = dim(X)[1]
  j = dim(y)[2]
  k = dim(X)[2]
  xy = cbind(X, y)
  xy = na.omit(xy)
  row.remain = setdiff(1:n, attr(xy, "na.action"))
  X = xy[, 1:k]
  y = xy[, (k + 1):(k + j)]
  n = dim(y)[1]
  remove(xy)
  # adding in the constant term
  if(k==1){
    # check if the input X is constant
    if(length(unique(X))==1){ # X is constant
      Xnames = "constant"
      X = as.matrix(as.numeric(X),nrow=1)
      colnames(X) = Xnames
      X = as.matrix(X)
      k=0
    }else{ # one single variable of input
      Xnames = "X1"
      X = as.matrix(X)
      k = dim(X)[2]
      X = cbind(X, rep(1, n))
      Xnames = c(Xnames, "constant")
      colnames(X) = Xnames
    }
  }else{ # normal cases
    X = X[, apply(X, 2, function(x) length(unique(x)) != 1)]
    Xnames = colnames(X)
    k = dim(X)[2]
    X = cbind(X, rep(1, n))
    Xnames = c(Xnames, "constant")
    colnames(X) = Xnames
  }
  
  
  testcols <- function(X) {
    m = crossprod(as.matrix(X))
    ee = eigen(m)
    evecs <- split(zapsmall(ee$vectors), col(ee$vectors))
    mapply(function(val, vec) {
      if (val != 0) 
        NULL
      else which(vec != 0)
    }, zapsmall(ee$values), evecs)
  }
  collinear = unique(unlist(testcols(X)))
  while (length(collinear) > 0) {
    if (qr(X)$rank == dim(X)[2]) 
      print("Model may suffer from multicollinearity problems.")
    break
    if ((k + 1) %in% collinear) 
      collinear = collinear[-length(collinear)]
    X = X[, -collinear[length(collinear)]]
    Xnames = colnames(X)
    k = k - 1
    collinear = unique(unlist(testcols(X)))
  }

  QMLE_Obs <- function(betas) {
    betas = matrix(betas, nrow = j - 1, byrow = TRUE)
    betamat = rbind(rep(0, k + 1), betas)
    
    # Pre-calculate to save time
    Xbeta = X %*% t(betamat)
    # log_sum_exp vector of length n
    log_sum_exp = log(rowSums(exp(Xbeta)))
    
    # L is n x j matrix
    L = y * (Xbeta - log_sum_exp)
    
    # log likelihood for each observation
    llf = rowSums(L)
    return(llf)
  }
  
  grad_Obs <- function(betas) {
    betas = matrix(betas, nrow = j - 1, byrow = TRUE)
    betamat = rbind(rep(0, k + 1), betas)
    
    Xbeta = X %*% t(betamat)
    # G is n x j matrix
    G = exp(Xbeta)
    sum_G = rowSums(G)
    G = G / sum_G
    
    # gradient for each observation n and parameter i (for i = 2..j)
    grad = matrix(0, nrow = n, ncol = (j - 1) * (k + 1))
    
    for (i in 2:j) {
      grad[, ((i-2)*(k+1) + 1):((i-1)*(k+1))] = X * (y[, i] - G[, i])
    }
    
    return(grad)
  }

  if (length(beta0) == 0){
    beta0 = rep(0, (k + 1) * (j - 1))
  }
  if (length(beta0) != (k + 1) * (j - 1)) {
    beta0 = rep(0, (k + 1) * (j - 1))
    warning("Wrong length of beta0 given. Use default setting instead.")
  }
  opt <- maxLik::maxLik(QMLE_Obs, grad = grad_Obs, start = beta0, method = MLEmethod, 
                control = list(iterlim = maxit, tol = abstol), ...)
  betamat = matrix(opt$estimate, ncol = k + 1, byrow = TRUE)
  betamat_aug = rbind(rep(0, k + 1), betamat)
  colnames(betamat_aug) = Xnames
  rownames(betamat_aug) = ynames
  sigmat = matrix(nrow = j - 1, ncol = k + 1)
  vcov = list()
  
  if(is.null(cluster)==FALSE){
    cluster = cluster[row.remain]
    clusters <- names(table(cluster))
    for (i in 1:j) {
      sterrs <- matrix(NA, nrow=reps, ncol=k + 1)
      vcov_j_list=list()
      
      b=1
      no_singular_error=c()
      while(b<=reps){
        index <- sample(1:length(clusters), length(clusters), replace=TRUE)
        aa <- clusters[index]
        bb <- table(aa)
        bootdat <- NULL
        dat=cbind(y,X)
        for(b1 in 1:max(bb)){
          cc <- dat[cluster %in% names(bb[bb %in% b1]),]
          for(b2 in 1:b1){
            bootdat <- rbind(bootdat, cc)
          }
        }
        
        bootdatX=matrix(bootdat[,(j+1):ncol(bootdat)],nrow=nrow(bootdat))
        bootdaty=bootdat[,1:j]
        
        sum_expxb = rowSums(exp(bootdatX %*% t(betamat_aug)))
        expxb = exp(bootdatX %*% betamat_aug[i, ])
        G = expxb/sum_expxb
        g = (expxb * sum_expxb - expxb^2)/sum_expxb^2
        X_a = bootdatX * as.vector(sqrt(g^2/(G * (1 - G))))
        A = t(X_a) %*% X_a
        mu = bootdaty[, i] - G
        X_b = bootdatX * as.vector(mu * g/G/(1 - G))
        B = t(X_b) %*% X_b
        
        a_solve_error = tryCatch(solve(A),error=function(e){NULL})
        if(is.null(a_solve_error)){
          no_singular_error=c(no_singular_error,b)
          next
        }
        
        Var_b = solve(A) %*% B %*% solve(A)
        std_b = sqrt(diag(Var_b))
        sterrs[b,]=std_b
        vcov_j_list[[b]]=Var_b
        
        b=b+1
      }
      if(length(no_singular_error)>0){warning(paste('Error in solve.default(A) : Lapack routine dgesv: system is exactly singular: U[28,28] = 0" Appeared',length(no_singular_error),'times within cluster bootstrap for outcome #',i))}
      std_b=apply(sterrs,2,mean)
      vcov[[i]] = Reduce("+", vcov_j_list) / length(vcov_j_list)
      if (i > 1) 
        sigmat[i - 1, ] = std_b
    }
  }else{
    for(i in 1:j){
      sum_expxb = rowSums(exp(X %*% t(betamat_aug))) 
      expxb = exp(X %*% betamat_aug[i,]) 
      G = expxb / sum_expxb 
      g = (expxb * sum_expxb - expxb^2) / sum_expxb^2 
      
      X_a = X * as.vector(sqrt(g^2/(G*(1-G))))
      A = t(X_a) %*% X_a
      
      mu = y[,i] - G
      X_b = X * as.vector(mu * g / G / (1-G))
      B = t(X_b) %*% X_b
      Var_b = solve(A) %*% B %*% solve(A)
      std_b = sqrt(diag(Var_b))
      vcov[[i]] = Var_b
      if(i>1) sigmat[i-1,] = std_b
    }
  }
  
  listmat = list()
  wald_list = list()
  for (i in 1:(j - 1)) {
    tabout = matrix(ncol = 4, nrow = k + 1)
    tabout[, 1:2] = t(rbind(betamat[i, ], sigmat[i, ]))
    tabout[, 3] = tabout[, 1]/tabout[, 2]
    tabout[, 4] = 2 * (1 - pnorm(abs(tabout[, 3])))
    colnames(tabout) = c("estimate", "std", "z", "p-value")
    if (length(Xnames) > 0) 
      rownames(tabout) = Xnames
    listmat[[i]] = tabout
    
    if (k > 0) {
      beta_slope = betamat[i, 1:k]
      V_slope = vcov[[i + 1]][1:k, 1:k]
      W = tryCatch(as.numeric(t(beta_slope) %*% solve(V_slope) %*% beta_slope), error = function(e) NA)
      p_W = if(!is.na(W)) 1 - pchisq(W, df = k) else NA
      wald_list[[i]] = list(W = W, p = p_W, df = k)
    } else {
      wald_list[[i]] = list(W = NA, p = NA, df = 0)
    }
  }
  
  y_bar = colMeans(y)
  LL0 = n * sum(y_bar * log(y_bar + 1e-16))
  pseudo_R2 = 1 - opt$maximum / LL0

  if (length(ynames) > 0) {
    names(listmat) = ynames[2:j]
    names(wald_list) = ynames[2:j]
  }
  outlist = list()
  outlist$estimates = listmat
  outlist$wald = wald_list
  outlist$pseudo_R2 = pseudo_R2
  outlist$baseline = ynames[1]
  outlist$likelihood = opt$maximum
  outlist$conv_code = opt$code
  outlist$convergence = paste(opt$type, paste(as.character(opt$iterations), 
                                              "iterations"), opt$message, sep = ",")
  outlist$count = c(Obs = n, Explanatories = k, Choices = j)
  outlist$y = y
  outlist$X = X
  outlist$rowNo = row.remain
  outlist$coefficient = betamat_aug
  names(vcov) = ynames
  outlist$vcov = vcov
  outlist$cluster = cluster
  outlist$reps=ifelse(is.null(cluster),0,reps)
  
  print(paste("Fractional logit model estimation completed. Time:", 
              round(proc.time()[3] - start.time[3], 1), "seconds"))
  return(structure(outlist, class = "fracregmlogit"))
}

---
title: "A Comprehensive Guide to Fractional Response Regressions with fracreg"
author: "Sulman Olieko Owili"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{A Comprehensive Guide to Fractional Response Regressions with fracreg}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  eval      = TRUE
)

local({
  hook_output <- knitr::knit_hooks$get("output")
  knitr::knit_hooks$set(output = function(x, options) {
    paste0("\n<details><summary>Toggle to see the output</summary>\n\n",
           hook_output(x, options),
           "\n</details>\n")
  })
})

library(fracreg)
```

## Introduction

The **`fracreg`** package is the most comprehensive tool available in R for estimating, diagnosing, and interpreting fractional response models. Originally focused on standard cross-sectional models, it has been substantially expanded to provide a complete suite of fractional response methodologies. The package supports:

- **Cross-Sectional Models (`fracreg`)**: Includes one-part models (1P) for standard $[0, 1]$ fractions, two-part hurdle models (2P) for fractions with boundaries at 0 or 1, and three-part double-inflated models (3P) for inflation at both boundaries.
- **Panel Data Models (`fracregpd`)**: Utilises Correlated Random Effects (CRE), Quasi-Maximum Likelihood (QML), and Generalised Method of Moments (GMM) approaches for longitudinal data.
- **Endogeneity Correction (`fracreghet`)**: Corrects for endogenous covariates using Control Function and IV-GMM approaches.
- **Fractional Multinomial Logit (`fracregmlogit`)**: Models multivariate fractional outcomes where multiple fractions sum to 1.
- **Fractional Ridge Regression (`fracregridge`)**: Incorporates L2 regularization dynamically controlled by the fraction of the unregularized coefficient vector length.

This vignette walks you through comprehensive empirical examples (using the built-in 401(k) and spending datasets) and simulated examples for each estimator. We provide detailed explanations of why each model is used and how to interpret the associated outputs.

```{r load-package}
library(fracreg)
```

## 1. Data Description and Preparation

We use the built-in `fracreg_k401k` dataset to demonstrate univariate fractional models. This is the canonical firm-level 401(k) plan participation data used in **Papke and Wooldridge (1996)** (*"Econometric methods for fractional response variables with an application to 401(k) plan participation rates"*, Journal of Applied Econometrics). The dataset contains 1,534 observations of 401(k) plans.

The primary variables we use are:
- `prate`: The plan participation rate (the fraction of eligible employees who are active participants). It strictly falls in the $[0, 1]$ interval and serves as our dependent variable ($y$). 
- `mrate`: The firm's matching rate (the firm's contribution per $1 of employee contribution). 
- `age`: The age of the 401(k) plan. 
- `totemp`: Total number of employees at the firm. 
- `sole`: A binary indicator equal to 1 if the 401(k) plan is the sole retirement plan offered by the firm.

---

## 2. Cross-Sectional Fractional Models (`fracreg`)

The core `fracreg()` function is designed for univariate models where the dependent variable is bounded between 0 and 1 inclusive ($0 \le y \le 1$). We explore how to fit 1P, 2P, and 3P models using both the empirical 401(k) data and simulated data with strict boundaries.

### 2.1 Empirical and Simulated Fractional Regressions

In this section, we start with a standard 1-part (1P) fractional logit model to estimate participation rates in the 401(k) data. Since we might also be interested in multiplicative effects, we show how to extract odds ratios (`or=TRUE`). Next, we demonstrate how to tackle boundary inflation. For example, some firms might have 100% participation (`prate = 1`), so we fit a 2-part model (`inflation=1`).

The simulated section further demonstrates generating artificial inflation at exactly 0 and 1. We then isolate the components of the 2-part model (using `type="2Pbin"` and `type="2Pfrac"`) and fit full 3-part models.

```{r fracreg}
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age,  
           totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole) 
 
# 1P Model 
mod <- fracreg(y, X, type="1P", linkfrac="logit") 
summary(mod)

# 1P Model reporting odds ratios and 99% confidence intervals
mod <- fracreg(y, X, type="1P", linkfrac="logit", or=TRUE, level=0.99)
summary(mod)
 
# 2P Model (modelling mass at 1) 
mod <- fracreg(y, X, type="2P", inflation=1, linkbin="logit", linkfrac="logit") 
summary(mod)
 
# 3P Model (inject artificial 0s for demonstration) 
y_3p <- y; y_3p[1:50] <- 0 
mod <- fracreg(y_3p, X, type="3P", linkbin=c("logit","logit"), linkfrac="logit") 
summary(mod)
 
### Simulated Examples 
 
set.seed(123) 
N <- 1000 
x1 <- rnorm(N) 
x2 <- runif(N) 
 
# Generating a fractional dependent variable with inflation at 0 and 1 
XB <- -0.5 + 0.8 * x1 + 1.2 * x2 + rnorm(N) 
y_latent <- exp(XB) / (1 + exp(XB)) 
 
y <- y_latent 
# Inflate at boundaries 
y[y_latent < 0.2] <- 0 
y[y_latent > 0.8] <- 1 
 
X <- cbind(x1 = x1, x2 = x2) 
 
# fracreg estimation of a logit fractional response model 
mod <- fracreg(y, X, type="1P", linkfrac="logit") 
summary(mod)
 
```
        
#### 2.1.2 Binary Logit Component of a Two-Part Model

Here, we estimate the binary (hurdle) component of a two-part fractional regression model. We specify `type="2Pbin"`, which instructs the function to model the probability of observing a non-boundary value versus the boundary value (in this case, 0). The `inflation=0` argument explicitly sets 0 as the relevant boundary.

```{r fracreg_2pbin}
# Estimate the binary logit component 
mod <- fracreg(y, X, type="2Pbin", inflation=0, linkbin="logit") 
summary(mod)
 
```
        
#### 2.1.3 Fractional Component of a Two-Part Model

After modelling the hurdle, we estimate the continuous fractional component using `type="2Pfrac"`. This component only uses observations that are strictly bounded away from the inflated boundary ($y > 0$). In this example, we apply a **probit** link function to model the fractional values.

```{r fracreg_2pfrac}
# Estimate the fractional component using a probit link 
mod <- fracreg(y, X, type="2Pfrac", inflation=0, linkfrac="probit") 
summary(mod)
 
```

#### 2.1.4 Full Two-Part Model (Joint Estimation)

Alternatively, we can estimate both the binary and fractional components simultaneously by specifying `type="2P"`. Here we use different link functions for each part: a **cloglog** link for the binary hurdle (which is useful for asymmetric data) and a **logit** link for the fractional continuous part.

```{r fracreg_2p_joint}
# Estimate both components jointly 
mod <- fracreg(y, X, type="2P", inflation=0, linkbin="cloglog", linkfrac="logit") 
summary(mod)
 
```

#### 2.1.5 Three-Part Double-Inflated Model

When the data exhibits inflation at **both** 0 and 1, a three-part model (`type="3P"`) is required. This models the probability of observing 0, the probability of observing 1, and the continuous fractional outcomes in between. We pass a vector of link functions `c("logit", "probit")` for the binary components, alongside the `logit` fractional link.

```{r fracreg_3p_simulated}
# Three-part double-inflated model 
mod <- fracreg(y, X, type="3P", linkbin=c("logit","probit"), linkfrac="logit") 
summary(mod)
```

### 2.2 Partial Effects (`fracreg.pe`)

Because fractional models are inherently non-linear, raw coefficients cannot be interpreted directly as constant marginal effects. The `fracreg.pe()` function computes the actual marginal effects. We can compute Average Partial Effects (APE) across all observations or Conditional Partial Effects (CPE) evaluated at specific covariate values (e.g., holding covariates at their median). The examples below demonstrate how to extract these effects across standard, 2-part, and 3-part models.

```{r fracreg-pe}
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age,  
           totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole) 
 
m <- fracreg(y, X, type="1P", linkfrac="logit") 
pe_res <- fracreg.pe(m) 
summary(pe_res)
 
### Simulated Examples 
 
N <- 250 
u <- rnorm(N) 
 
X <- cbind(rnorm(N),rnorm(N)) 
dimnames(X)[[2]] <- c("X1","X2") 
 
ym <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u)) 
y <- rbeta(N,ym*20,20*(1-ym)) 
y[y > 0.9] <- 1 
 
#Computing average partial effects for a logit fractional response model 
mod <- fracreg(y,X,linkfrac="logit",table=FALSE) 
pe_res <- fracreg.pe(mod) 
summary(pe_res)
 
```

#### 2.2.2 APEs for a Two-Part Model

When the data has boundary inflation (e.g., at 1), partial effects must account for both the hurdle process and the fractional continuous process. The `fracreg.pe()` function automatically handles this composite calculation for `type="2P"`.

```{r fracreg_pe_2p}
# Compute average partial effects for a binary logit + fractional probit two-part model 
mod <- fracreg(y,X,linkbin="logit",linkfrac="probit",type="2P",inf=1,table=FALSE) 
pe_res <- fracreg.pe(mod) 
summary(pe_res)
 
```

#### 2.2.3 Conditional Partial Effects (CPE)

Instead of averaging the effects over all observations (APE), you may want the effect for a "typical" observation. By setting `APE=FALSE` and `CPE=TRUE`, and providing `at="median"`, we compute the Conditional Partial Effect evaluated at the median values of all covariates. We isolate the effect of `X2` using `which.x="X2"`.

```{r fracreg_pe_cpe}
# Compute conditional partial effects for X2 at median values 
mod <- fracreg(y,X,linkfrac="logit",type="2Pfrac",inf=1,table=FALSE) 
pe_res <- fracreg.pe(mod,APE=FALSE,CPE=TRUE,at="median",which.x="X2") 
summary(pe_res)
 
```

#### 2.2.4 APEs for a Three-Part Model

For double-inflated models (`type="3P"`), the partial effect calculation incorporates both boundary probabilities and the interior fractional distribution.

```{r fracreg_pe_3p}
# Compute average partial effects for a three-part double-inflated model 
y3p <- y 
y3p[1:20] <- 0 
y3p[21:40] <- 1 
res3p <- fracreg(y3p,X,linkbin=c("logit","probit"),linkfrac="logit",type="3P",table=FALSE) 
pe_res <- fracreg.pe(res3p) 
summary(pe_res)
```

---

## 3. Hypothesis Testing and Specification Diagnostics

`fracreg` includes state-of-the-art specification tests to validate your model's functional form and link function assumptions.

### 3.1 Generalised Goodness-Of-Functional-Form (`fracreg.ggoff`)

The GGOFF test checks if the chosen link function (e.g., `logit`, `probit`) is appropriate for the data. A significant test statistic suggests that the link function may be misspecified and an alternative should be explored. Here, we run the LM, Wald, and LR variations of the test on our fitted fractional models.

```{r fracreg-ggoff}
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age,  
           totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole) 
 
m <- fracreg(y, X, type="1P", linkfrac="logit") 
ggoff_res <- fracreg.ggoff(m) 
summary(ggoff_res)
 
### Simulated Examples 
 
N <- 250 
u <- rnorm(N) 
 
X <- cbind(rnorm(N),rnorm(N)) 
dimnames(X)[[2]] <- c("X1","X2") 
 
ym <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u)) 
y <- rbeta(N,ym*20,20*(1-ym)) 
y[y > 0.9] <- 1 
 
#Testing the logit specification of a standard fractional response model 
#using LM and Wald versions of the GGOFF test, based on 1 or 2 fitted powers of 
#the linear predictor 
mod <- fracreg(y,X,linkfrac="logit",table=FALSE) 
ggoff_res <- fracreg.ggoff(mod,c("Wald","LM")) 
summary(ggoff_res)
 
```

#### 3.1.2 GGOFF Test for the Binary Component

The GGOFF test can also be applied to individual components of hurdle models. Here, we test the **probit** specification of the binary component (`2Pbin`) using a Likelihood Ratio (LR) test approach.

```{r fracreg_ggoff_2pbin}
# Test the probit specification of the binary component 
mod <- fracreg(y,X,linkbin="probit",type="2Pbin",inf=1,table=FALSE) 
ggoff_res <- fracreg.ggoff(mod,"LR") 
summary(ggoff_res)
```

### 3.2 RESET Test (`fracreg.reset`)

The RESET test detects general functional form misspecification by testing whether powers of the fitted values have explanatory power. Testing $H_0: \gamma = 0$ provides a robust diagnostic check against omitted non-linearities.

```{r fracreg-reset}
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age,  
           totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole) 
 
m <- fracreg(y, X, type="1P", linkfrac="logit") 
reset_res <- fracreg.reset(m) 
summary(reset_res)
 
### Simulated Examples 
 
N <- 250 
u <- rnorm(N) 
 
X <- cbind(rnorm(N),rnorm(N)) 
dimnames(X)[[2]] <- c("X1","X2") 
 
ym <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u)) 
y <- rbeta(N,ym*20,20*(1-ym)) 
y[y > 0.9] <- 1 
 
#Testing the logit specification of a standard fractional response model 
#using LM and Wald versions of the RESET test, based on 1 or 2 fitted powers of 
#the linear predictor 
mod <- fracreg(y,X,linkfrac="logit",table=FALSE) 
reset_res <- fracreg.reset(mod,2:3,c("Wald","LM")) 
summary(reset_res)
 
```

#### 3.2.2 RESET Test for the Binary Component

Similar to GGOFF, the RESET test can validate individual hurdle components. Here, we use the Likelihood Ratio (LR) version of the RESET test on the binary probit component, checking up to cubic fitted powers of the linear predictor.

```{r fracreg_reset_2pbin}
# Test the probit specification of the binary component using LR RESET 
mod <- fracreg(y,X,linkbin="probit",type="2Pbin",inf=1,table=FALSE) 
reset_res <- fracreg.reset(mod,3,"LR") 
summary(reset_res)
```

### 3.3 P-Test for Non-Nested Models (`fracreg.ptest`)

When you need to choose between two non-nested model specifications (for example, a `logit` link vs. a `probit` or `loglog` link, or a 1P vs. a 2P model), the P-test compares them directly to see if one statistically dominates the other.

```{r fracreg-ptest}
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age,  
           totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole) 
 
m1 <- fracreg(y, X, type="1P", linkfrac="logit") 
m2 <- fracreg(y, X, type="1P", linkfrac="probit") 
ptest_res <- fracreg.ptest(m1, m2) 
summary(ptest_res)
 
### Simulated Examples 
 
N <- 250 
u <- rnorm(N) 
 
X <- cbind(rnorm(N),rnorm(N)) 
dimnames(X)[[2]] <- c("X1","X2") 
 
ym <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u)) 
y <- rbeta(N,ym*20,20*(1-ym)) 
y[y > 0.9] <- 1 
 
#Testing logit versus loglog specifications for standard fractional 
#regression models using a LM version of the P test 
res1 <- fracreg(y,X,linkfrac="logit",table=FALSE) 
res2 <- fracreg(y,X,linkfrac="loglog",table=FALSE) 
ptest_res <- fracreg.ptest(res1,res2,"LM") 
summary(ptest_res)
 
```

#### 3.3.2 P-Test: 1P vs. 2P Model

The P-test is exceptionally useful for deciding whether a simple 1-part model suffices, or if the extra complexity of a 2-part model is statistically justified. We test the `1P` logit model against a `2P` (binary logit + fractional probit) model using the Wald test statistic.

```{r fracreg_ptest_1p_vs_2p}
# Test 1P logit versus 2P logit-probit using Wald P-test 
res1 <- fracreg(y,X,linkfrac="logit",table=FALSE) 
res2 <- fracreg(y,X,linkbin="logit",linkfrac="probit",type="2P",inf=1,table=FALSE) 
ptest_res <- fracreg.ptest(res1,res2,"Wald") 
summary(ptest_res)
```

---

## 4. Endogeneity & Heteroscedasticity (`fracreghet`)

When a covariate is endogenous (correlated with the error term) or heteroscedasticity is present, standard maximum likelihood estimation becomes inconsistent. `fracreghet()` provides Instrumental Variable (IV) corrections using Control Function (CF) and Generalised Method of Moments (GMM) approaches.

In the empirical example below, we suspect `mrate` (matching rate) might be endogenous. We use `age` and `ltotemp` (log of total employees) as instruments. We estimate the model using a Quasi-Maximum Likelihood control function approach (`QMLxv`). We also show how to extract odds ratios. In the simulated examples, we explore `GMMx`, `GMMz`, and `GMMxv` estimators which relax the need for a linear first-stage relationship in different ways.

```{r fracreghet}
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X_het <- cbind(mrate = fracreg_k401k$mrate, ltotemp = fracreg_k401k$ltotemp)
 
# fracreghet estimators do not allow exact 1s or 0s
y_adj <- y
y_adj[y_adj == 1] <- 0.999

# Instrument mrate using age

Z_emp <- cbind(age = fracreg_k401k$age, ltotemp = fracreg_k401k$ltotemp) 
mod <- fracreghet(y_adj, X_het, Z_emp, var.endog = X_het[, "mrate"], type="QMLxv", link="logit") 
summary(mod)

# Compute the same QMLxv estimator reporting Odds Ratios with 90% confidence intervals
mod <- fracreghet(y_adj, X_het, Z_emp, var.endog = X_het[, "mrate"], type="QMLxv", link="logit", or=TRUE, level=0.90)
summary(mod)
 
### Simulated Examples 
 
set.seed(123) 
N <- 1000 
x1 <- rnorm(N) 
 
# Simulating an endogenous variable (var.endog) and an instrument (z1) 
z1 <- rnorm(N) 
u <- 0.5 * z1 + rnorm(N) 
var.endog <- 0.8 * z1 + u 
y_endog <- exp(0.5 * x1 + 1.2 * var.endog + u) / (1 + exp(0.5 * x1 + 1.2 * var.endog + u)) 
 
# Avoid exact 0 or 1 boundaries for some estimators 
y_endog[y_endog <= 0] <- 0.01 
y_endog[y_endog >= 1] <- 0.99 
 
X <- cbind(x1 = x1, var.endog = var.endog) 
Z <- cbind(x1 = x1, z1 = z1) 
 
# Exogeneity (assuming var.endog is exogenous for comparison), GMMx estimator 
mod <- fracreghet(y = y_endog, x = X, type = "GMMx", link = "logit") 
summary(mod)
 
```

#### 4.1.2 GMMz Estimator

The **GMMz** estimator handles endogeneity directly through moment conditions using the instruments $Z$, without requiring an explicit linear reduced form equation for the endogenous variable. This approach is more robust to first-stage misspecification.

```{r fracreghet_gmmz}
# Endogeneity, GMMz estimator 
mod <- fracreghet(y = y_endog, x = X, z = Z, type = "GMMz", link = "logit") 
summary(mod)
 
```

#### 4.1.3 GMMxv Estimator

The **GMMxv** estimator is a variation that does assume a linear reduced form for the endogenous variable, utilizing the first-stage residuals within the generalized method of moments framework to correct for endogeneity.

```{r fracreghet_gmmxv}
# Endogeneity, GMMxv estimator 
mod <- fracreghet(y = y_endog, x = X, z = Z, var.endog = var.endog, type = "GMMxv", link = "logit") 
summary(mod)
 
```

#### 4.1.4 QMLxv Control Function Estimator

The **QMLxv** approach utilizes a Control Function (CF). It explicitly estimates the first-stage linear reduced form, extracts the residuals, and includes them as control variables in the second-stage Quasi-Maximum Likelihood estimation to absorb the endogeneity.

```{r fracreghet_qmlxv}
# Endogeneity, QMLxv control function approach 
mod <- fracreghet(y = y_endog, x = X, z = Z, var.endog = var.endog, type = "QMLxv", link = "logit") 
summary(mod)
```

### 4.1 Partial Effects for Endogenous Models (`fracreghet.pe`)

Partial effects can also be calculated for endogenous models. When using a control function approach (`QMLxv`), `fracreghet.pe` accurately integrates over the estimated first-stage residuals, offering a corrected smearing estimation.

```{r fracreghet-pe}
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X_het <- cbind(mrate = fracreg_k401k$mrate, ltotemp = fracreg_k401k$ltotemp)
 
# fracreghet estimators do not allow exact 1s or 0s
y_adj <- y
y_adj[y_adj == 1] <- 0.999

# Instrument mrate using age

Z_emp <- cbind(age = fracreg_k401k$age, ltotemp = fracreg_k401k$ltotemp) 
res_emp <- fracreghet(y_adj, X_het, Z_emp, var.endog = X_het[, "mrate"], type="QMLxv", link="logit", table=FALSE) 
pe_res <- fracreghet.pe(res_emp, which.x="mrate")
summary(pe_res)

### Simulated Examples 
 
N <- 250 
u <- rnorm(N) 
 
X <- cbind(rnorm(N),rnorm(N)) 
dimnames(X)[[2]] <- c("X1","X2") 
 
Z <- cbind(rnorm(N),rnorm(N),rnorm(N)) 
dimnames(Z)[[2]] <- c("Z1","Z2","Z3") 
 
y <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u)) 
 
mod <- fracreghet(y,X,type="GMMx",table=FALSE) 
 
#Smearing estimator of average partial effects for variable X1 
pe_res <- fracreghet.pe(mod,which.x="X1") 
summary(pe_res)
 
```

#### 4.1.2 Naive Estimator for CPEs

For a targeted interpretation, we can calculate Conditional Partial Effects (CPE) evaluated at specific fixed values (e.g., $X1=1$ and $X2=-1$). By setting `smearing=FALSE`, we use a naive estimator that does not integrate over the estimated unobserved heterogeneity.

```{r fracreghet_pe_cpe}
# Naive estimator of CPE evaluated at fixed values 
pe_res <- fracreghet.pe(mod,smearing=FALSE,APE=FALSE,CPE=TRUE,at=c(1,-1)) 
summary(pe_res)
```

### 4.2 RESET Test for Endogenous Models (`fracreghet.reset`)

Just as with `fracreg`, we can run RESET specification tests on our heteroscedasticity and endogeneity-robust models to assure proper functional form.

```{r fracreghet-reset}
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X_het <- cbind(mrate = fracreg_k401k$mrate, ltotemp = fracreg_k401k$ltotemp)
 
# fracreghet estimators do not allow exact 1s or 0s
y_adj <- y
y_adj[y_adj == 1] <- 0.999

# Instrument mrate using age

Z_emp <- cbind(age = fracreg_k401k$age, ltotemp = fracreg_k401k$ltotemp) 
res_emp <- fracreghet(y_adj, X_het, type="GMMx", link="logit", table=FALSE) 
reset_res <- fracreghet.reset(res_emp)
summary(reset_res)

### Simulated Examples 
 
N <- 250 
u <- rnorm(N) 
 
X <- cbind(rnorm(N),rnorm(N)) 
dimnames(X)[[2]] <- c("X1","X2") 
 
Z <- cbind(rnorm(N),rnorm(N),rnorm(N)) 
dimnames(Z)[[2]] <- c("Z1","Z2","Z3") 
 
y <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u)) 
 
mod <- fracreghet(y,X,type="GMMx",table=FALSE) 
 
#LM and Wald versions of the RESET test, based on 1 or 2 fitted powers of xb 
reset_res <- fracreghet.reset(mod,2:3,c("Wald","LM")) 
summary(reset_res)
```

---

## 5. Panel Data Fractional Models (`fracregpd`)

When you observe the same entities over multiple time periods, unobserved individual heterogeneity must be accounted for. `fracregpd()` provides fixed-T panel estimators. We explore Correlated Random Effects (CRE) which includes time-averages of the covariates to proxy the unobserved effect, as well as various panel GMM estimators (like `GMMbgw`, `GMMww`, and `GMMpfe`) which can handle lagged dependent variables, endogenous covariates, and time dummies.

```{r fracregpd}
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age,  
           totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole) 
 
# Artificial panel data structure for demonstration 
N_emp <- nrow(X) 
id_emp <- rep(1:(N_emp/2), each=2) 
time_emp <- rep(1:2, times=N_emp/2) 
mod <- fracregpd(id_emp, time_emp, y, X, type="QMLcre", link="probit") 
summary(mod)
 
### Simulated Examples 
 
set.seed(123) 
# Simulating Panel Data 
N <- 100 
T_periods <- 5 
id <- rep(1:N, each = T_periods) 
time <- rep(1:T_periods, times = N) 
x_panel <- rnorm(N * T_periods) 
 
# Unobserved individual effect (CRE) 
c_i <- rep(rnorm(N), each = T_periods)  
y_panel <- exp(x_panel + c_i) / (1 + exp(x_panel + c_i)) 
 
X <- cbind(x_panel = x_panel) 
 
# Endogenous variable and instrument simulation 
z_panel <- rnorm(N * T_periods) 
u_panel <- 0.5 * z_panel + rnorm(N * T_periods) 
var_endog <- 0.8 * z_panel + u_panel 
y_endog <- exp(x_panel + 1.2 * var_endog + c_i + u_panel) /  
             (1 + exp(x_panel + 1.2 * var_endog + c_i + u_panel)) 
 
X_endog <- cbind(x_panel = x_panel, var_endog = var_endog) 
Z_inst <- cbind(x_panel = x_panel, z_panel = z_panel) 
 
# Estimate a Correlated Random Effects (CRE) Model 
mod <- fracregpd(id=id, time=time, y=y_panel, x=X, type="QMLcre", link="probit") 
summary(mod)
 
```

#### 5.1.2 Standard Panel GMM (GMMbgw)

The **GMMbgw** estimator is a standard GMM panel approach that doesn't explicitly model unobserved effects via time-averages, but relies on moment conditions. Here we apply it with clustered standard errors.

```{r fracregpd_gmmbgw}
# Exogeneity, GMMbgw estimator 
mod <- fracregpd(id=id, time=time, y=y_panel, x=X, type="GMMbgw") 
summary(mod)

```

#### 5.1.3 Generalized Method of Moments (GMMww)

The **GMMww** estimator extends panel GMM capabilities. Below, we estimate the model and demonstrate how to extract odds ratios (`or=TRUE`) with a 99% confidence level.

```{r fracregpd_gmmww}
# Estimate GMMww estimator with odds ratios
mod <- fracregpd(id=id, time=time, y=y_panel, x=X, type="GMMww", or=TRUE, level=0.99)
summary(mod)
 
```

#### 5.1.4 GMMww with Lagged Covariates

Dynamic panel data often require the inclusion of lagged covariates as instruments. By setting `lags=TRUE`, the model automatically configures the moment conditions using lagged values, employing robust standard errors to handle potential heteroscedasticity across time periods.

```{r fracregpd_gmmww_lags}
# Lagged covariates and instruments 
mod <- fracregpd(id=id, time=time, y=y_panel, x=X, lags=TRUE, type="GMMww", var.type="robust") 
summary(mod)
 
```

#### 5.1.5 Endogenous Panel GMM (GMMpfe)

When covariates are endogenous in a panel setting, we combine instruments ($Z$) with panel structure. The **GMMpfe** estimator allows for endogenous variables (`x.exogenous=FALSE`) and includes time dummies (`tdummies=TRUE`) to control for macroeconomic or aggregate time shocks.

```{r fracregpd_gmmpfe}
# Endogeneity, time dummies, GMMpfe estimator 
mod <- fracregpd(id=id, time=time, y=y_endog, x=X_endog, z=Z_inst,  
          x.exogenous=FALSE, type="GMMpfe", tdummies=TRUE) 
summary(mod)
```

---

## 6. Fractional Ridge Regression

The `fracreg` package also includes an implementation of Fractional Ridge Regression (`fracregridge`). Standard maximum likelihood approaches can suffer from multicollinearity or overfitting when the number of covariates is large relative to the sample size. `fracregridge` implements Fractional Ridge Regression, which applies L2 regularization to shrink coefficients.

Unlike standard ridge regression where the penalty term $\alpha$ is chosen directly, `fracregridge` allows you to specify the desired *fraction* of the unregularized OLS coefficient vector length (e.g. `0.2` implies shrinking to 20% of the full OLS length). The algorithm then automatically determines the corresponding $\alpha$ penalties.

### 6.1 Empirical 401(k) Example

We can apply this to the 401(k) participation rate data to observe how the coefficients dynamically shrink across different target vector length fractions (from 20% to 100%). We also demonstrate how to compute partial effects on the regularized coefficients.

```{r fracregridge-empirical}
data("fracreg_k401k")
y_401k <- fracreg_k401k$prate
X_401k <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age,
                totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole)

# Fit fractional ridge regression
mod_401k <- fracregridge(y = y_401k, x = X_401k, fracs = seq(0.2, 1.0, by = 0.2))

# View full detailed summary showing the chosen alphas
summary(mod_401k)

# Compute Average Partial Effects for Ridge
pe_401k <- fracregridge.pe(mod_401k)
summary(pe_401k)
```

### 6.2 Simulated Data Example

Using a random matrix of 10 predictors on 100 observations, we can see how the fraction parameters precisely constrain the output coefficients to 30%, 50%, and 80% lengths.

```{r fracregridge-simulated}
# Generate random data
set.seed(123)
n <- 100
p <- 10
y_sim <- rnorm(n)
X_sim <- matrix(rnorm(n * p), n, p)
colnames(X_sim) <- paste0("X", 1:p)

# Fit Fractional Ridge Regression for 30%, 50%, and 80% fractions
mod_sim <- fracregridge(y = y_sim, x = X_sim, fracs = c(0.3, 0.5, 0.8))

# View brief summary
print(mod_sim)

# Compute Partial Effects
pe_sim <- fracregridge.pe(mod_sim)
summary(pe_sim)
```

---

## 7. Fractional Multinomial Logit

The `fracreg` package incorporates the fractional multinomial logit (`fracregmlogit`) to estimate fractional response data where the response variable consists of fractions that sum up to one across multiple categories (e.g., budget allocations across departments). A univariate model is insufficient because the bounded choices are perfectly collinear in sum.

### 7.1 Estimating Budget Shares

We use the built-in `fracreg_spending` dataset, which records the budget share allocations of local governments across 6 distinct sectors. The sum of these 6 sectoral fractions for any given government is exactly 1. We compute both the model summary and the discrete partial effects for specific explanatory variables.

```{r fracregmlogit_example}
# Load the empirical spending data
data("fracreg_spending")

# Define covariates and fractional responses
X <- fracreg_spending[, c("houseval", "popdens", "noleft", "minorityleft", "tot")]
y <- fracreg_spending[, c("governing", "safety", "education", "recreation", "social", "urbanplanning")]

# Fit the Fractional Multinomial Logit model
mn_fit <- fracregmlogit(y, X)

# View estimates
summary(mn_fit)

# Compute Average Partial Effects (discrete)
mn_pe <- fracregmlogit.pe(mn_fit, effect = "discrete", varlist = c("noleft", "minorityleft"))
summary(mn_pe)
```

### 7.2 Willingness to Pay (WTP)

In policy or consumer choice applications, you might want to calculate the aggregate effect of a variable scaled by arbitrary outcome weights (such as cost, budget, or willingness to pay associated with each choice). The `wtp()` function multiplies the average partial effects by these arbitrary values and tests if the aggregate effect differs from zero. We define a vector of length 6 (to match the 6 categories) and plot the effect.

```{r fracregmlogit_wtp}
# Calculate Willingness to Pay for the 'noleft' variable using a hypothetical WTP vector
# Assuming WTP = 1, 2, 3, 4, 5, 6 for each of the 6 choices
wtp_est <- wtp(mn_pe, wtp.vec = 1:6, varlist = "noleft")
summary(wtp_est)

# Plot the Willingness to Pay effect across observations
plot(mn_fit, wtp.vec = 1:6, varlist = "noleft")
```

## Conclusion

With `fracreg`, `fracreghet`, `fracregpd`, `fracregmlogit`, and `fracregridge`, you have a complete toolkit for modelling fractional responses bounded between $[0,1]$, regardless of inflation, endogeneity, unobserved panel effects, multinomial structure, or multicollinearity.


## Acknowledgements

This package builds upon, consolidates, and modernises the fractional
regression frameworks originally implemented in the `frm`, `frmhet`, and
`frmpd` R packages developed by Joaquim J.S. Ramalho. As those original
packages have been deprecated and removed from the active CRAN
repository, `fracreg` serves as an actively maintained successor,
ensuring these econometric tools remain available to the R community.

Furthermore, we acknowledge James Ji (@f1kidd) and A. John Woodill (@johnwoodill), the authors of the `fmlogit` R package on GitHub, whose foundational work on fractional multinomial logit models inspired the implementation of `fracregmlogit`. We also extend our gratitude to Ariel Rokem and Kendrick Kay, the authors of the `fracridge` package, whose methodological contributions to fractional ridge regression are incorporated into the `fracregridge` functionalities of this package.

## References

- **Ji, J., and Woodill, A. J.** *fmlogit: Fractional Multinomial Logit*. R package repository. <https://github.com/f1kidd/fmlogit>
- **Rokem, A., and Kay, K.** *fracridge: Fractional Ridge Regression*. Package repository. <https://github.com/nrdg/fracridge>

- **Ramalho, J. J. S. (2022).** *frm: Fractional Regression Models*. R
  package. Formerly available on CRAN, currently archived.
- **Ramalho, J. J. S. (2023).** *frmhet: Fractional Regression Models
  under Heterogeneity*. R package. Formerly available on CRAN, currently
  archived.
- **Ramalho, J. J. S. (2023).** *frmpd: Fractional Regression Models for
  Panel Data*. R package. Formerly available on CRAN, currently
  archived.
- **Buis, M. L. (2008).** "fmlogit: Stata module fitting a fractional multinomial logit model by quasi maximum likelihood", *Statistical Software Components*, Boston College Department of Economics.
- **Davidson, R. and MacKinnon, J.G. (1981).** "Several tests for model specification in the presence of alternative hypotheses", *Econometrica*, 49(3), 781-793.
- **Fang, K., & Ma, S. (2013).** "Three-part model for fractional response variables with application to Chinese household health insurance coverage", *Journal of Applied Statistics*, 40(5), 925-940.
- **Mullahy, J. (2015).** "Multivariate fractional regression estimation of econometric share models", *Journal of Econometric Methods*, 4(1), 71-100.
- **Murteira, J. M. R., and Ramalho, J. J. S. (2016).** "Regression analysis of multivariate fractional data", *Econometric Reviews*, 35(4), 515-552.
- **Papke, L. E. and Wooldridge, J. M. (1996).** "Econometric methods for fractional response variables with an application to 401(k) plan participation rates", *Journal of Applied Econometrics*, 11(6), 619-632.
- **Papke, L. and Wooldridge, J.M. (2008).** "Panel data methods for fractional response variables with an application to test pass rates", *Journal of Econometrics*, 145(1-2), 121-233.
- **Pregibon, D. (1980).** "Goodness of Link Tests for Generalized Linear Models", *Journal of the Royal Statistical Society: Series C (Applied Statistics)*, 29(1), 15-24.
- **Ramalho, E. A., & Ramalho, J. J. S. (2017).** "Moment-based estimation of nonlinear regression models with boundary outcomes and endogeneity, with applications to nonnegative and fractional responses", *Econometric Reviews*, 36(4), 397-420.
- **Ramalho, E.A., Ramalho, J.J.S. and Murteira, J.M.R. (2011).** "Alternative estimating and testing empirical strategies for fractional response models", *Journal of Economic Surveys*, 25(1), 19-68.
- **Ramalho, E.A., Ramalho, J.J.S. and Murteira, J.M.R. (2014).** "A generalized goodness-of-functional form test for binary and fractional response models", *Manchester School*, 82(4), 488-507.
- **Ramsey, J.B. (1969).** "Tests for Specification Errors in Classical Linear Least-Squares Regression Analysis", *Journal of the Royal Statistical Society: Series B (Methodological)*, 31(2), 350-371.
- **Rokem, A., and Kay, K. (2020).** "Fractional ridge regression: a fast, interpretable reparameterization of ridge regression", *GigaScience*, 9(12), giaa133.

For more information, please visit the [package website](https://sulmanolieko.github.io/fracreg/) or file an issue on [GitHub](https://github.com/SulmanOlieko/fracreg/issues).

To cite this package in your research:

```r
citation("fracreg")
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracreghet.R
\name{fracreghet}
\alias{fracreghet}
\title{Fitting Fractional Response Regressions under Unobserved Heterogeneity}
\usage{
fracreghet(
  y,
  x,
  z = x,
  var.endog,
  start,
  type = "GMMx",
  link = "logit",
  intercept = TRUE,
  table = TRUE,
  variance = TRUE,
  var.type = "robust",
  var.cluster,
  adjust = 0,
  offset = NULL,
  or = FALSE,
  level = 0.95,
  na.action = stats::na.omit,
  ...
)
}
\arguments{
\item{y}{a numeric vector containing the values of the response variable.}

\item{x}{a numeric matrix, with column names, containing the values of all covariates (exogenous and endogenous).}

\item{z}{a numeric matrix, with column names, containing the values of all exogenous variables (covariates and instrumental  variables). Defaults to \code{x}.}

\item{var.endog}{a numeric vector containing the values of the endogenous covariate (or of some transformation of it), which will be used as dependent variable in the linear reduced form assumed for application of xv-type estimators.}

\item{start}{a numeric vector containing the initial values for the parameters to be optimised. Optional.}

\item{type}{a description of the estimator to compute: \code{GMMx} (the default), \code{GMMxv}, \code{GMMz}, \code{LINx}, \code{LINxv}, \code{LINz} or \code{QMLxv}.}

\item{link}{a description of the link function to use. Available options for all estimators: \code{logit} and \code{cloglog}. Additional available options for QML and LIN estimators: \code{probit}, \code{cauchit} and \code{loglog}.}

\item{intercept}{a logical value indicating whether the model should include a constant term or not.}

\item{table}{a logical value indicating whether a summary table with the regression results should be printed.}

\item{variance}{a logical value indicating whether the variance of the estimated parameters should be calculated. Defaults to \code{TRUE} whenever \code{table = TRUE}.}

\item{var.type}{a description of the type of variance of the estimated parameters to be calculated. Options are \code{robust}, the default, and \code{cluster}.}

\item{var.cluster}{a numeric vector containing the values of the variable that specifies to which cluster each observation belongs.}

\item{adjust}{the numeric value to be added to the response variable in case of boundary observations when the LIN estimators are applied or the string \code{drop}, which implies that the boundary observations are dropped.}

\item{offset}{an optional numeric vector containing an offset. It must be of the same dimension as the response variable. It specifies that the variable should be included in the model with its coefficient constrained to 1.}

\item{or}{a logical value indicating whether to report odds ratios. Only valid when the link function is \code{"logit"}. Defaults to \code{FALSE}.}

\item{level}{a numeric value between 0 and 1 indicating the confidence level for the confidence intervals. Defaults to \code{0.95}.}

\item{na.action}{A function specifying how to handle missing values, default is \code{stats::na.omit}. If \code{NULL}, no action is taken.}

\item{\dots}{Arguments to pass to \link[stats]{nlminb}.}
}
\value{
\code{fracreghet} returns a list with the following elements:
  \item{class}{"fracreghet".
}
  \item{formula}{the model formula.
}
  \item{type}{the name of the estimator computed.
}
  \item{link}{the name of the specified link.
}
  \item{adjust}{The value or the type of the adjustment applied to LIN estimators.
}
  \item{p}{a named vector of coefficients.
}
  \item{Hy}{the transformed values of the response variable when GMM or LIN estimators are computed or the 
   values of the response variable in the QML case.
}
  \item{xbhat}{the fitted mean values of the linear predictor (for xv-type estimators, includes the term relative to the first-stage residual).
}
  \item{converged}{logical. Was the algorithm judged to have converged?
}
  \item{x.names}{a vector containing the names of the covariates.
}

In case of an overidentifying model, the following element is also returned:
  \item{J}{the result of Hansen's J test of overidentifying moment conditions.
}

If \code{variance = TRUE} or \code{table = TRUE} and the algorithm converged successfully, the previous list also contains the following elements:

  \item{p.var}{a named covariance matrix.
}
  \item{var.type}{covariance matrix type.
}

If \code{var.type = "cluster"}, the list also contains the following element:
  \item{var.cluster}{the variable that specifies to which cluster each observation belongs.
}
}
\description{
\code{fracreghet} is used to fit fractional response models under unobserved heterogeneity, i.e. regression models for proportions, percentages or fractions that suffer from neglected heterogeneity and/or endogeneity issues.
}
\details{
\code{fracreghet} computes the GMM estimators proposed in Ramalho and Ramalho (2017) for fractional response models with unobserved heterogeneity: GMMx, which allows for neglected heterogeneity but not for endogeneity; GMMxv, which allows both issues and assumes a linear reduced form for the endogeneous covariate (or for a transformation of it); and GMMz, which also allows for both issues but does not require the assumption of a reduced form for the endogenous covariate. In addition, \code{fracreghet} also computes three linearised estimators (LINx, LINxv and LINz) that have similar features to their GMM counterparts. It also provides a QML estimator (QMLxv) that addresses endogeneity using a Control Function (CF) approach, which includes the first-stage reduced-form residuals as an additional regressor in the main fractional equation, providing a Hausman-type test for endogeneity. 

\strong{Control Function (CF) Approach - QMLxv:}
When a continuous regressor \eqn{y_{2i}} is endogenous, the CF approach (Papke and Wooldridge, 2008; Terza et al., 2008) uses a two-stage procedure. First, a linear reduced form is estimated:
\deqn{y_{2i} = z_i \pi + v_i}
where \eqn{z_i} includes all exogenous variables and external instruments. The residuals \eqn{\hat{v}_i} are then included in the fractional response model:
\deqn{E(y_{1i} | z_i, y_{2i}, v_i) = G(x_i \beta + \gamma \hat{v}_i)}
A test of \eqn{H_0: \gamma = 0} serves as a robust Hausman-type test for endogeneity.

\strong{Generalised Method of Moments (GMM):}
For estimators like GMMz, which do not strictly require a linear reduced form, the estimation relies on population orthogonality conditions between the instruments \eqn{Z_i} and the model residuals:
\deqn{E[Z_{i} (y_i - G(x_i \beta))] = 0}
or via specific transformations of the dependent variable to eliminate unobserved heterogeneity (Ramalho and Ramalho, 2017).

For overidentified models, \code{fracreghet} calculates Hansen's J statistic. For \code{GMMx} and \code{LINx}, \code{fracreghet} stores the information needed to implement the RESET test (\link{fracreghet.reset}). For all estimators, \code{fracreghet} stores the information needed to calculate partial effects (\link{fracreghet.pe}).
}
\section{Odds Ratios}{

When \code{or=TRUE} and the fractional link function (\code{linkfrac} or \code{link}) is \code{"logit"}, the model additionally computes odds ratios for the coefficients. 
Odds Ratios are exponentiated coefficients.
The corresponding standard errors for the odds ratios are calculated using the Delta method.
The confidence intervals for the odds ratios are calculated using the adjusted standard errors and the specified \code{level} (defaulting to 95\%).
Odds ratios are particularly useful in fractional logit models as they provide a direct multiplicative interpretation of the independent variable on the odds of the fractional outcome.
}

\examples{
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X_het <- cbind(mrate = fracreg_k401k$mrate, ltotemp = fracreg_k401k$ltotemp)
 
# fracreghet estimators do not allow exact 1s or 0s
y_adj <- y
y_adj[y_adj == 1] <- 0.999

# Instrument mrate using age

Z_emp <- cbind(age = fracreg_k401k$age, ltotemp = fracreg_k401k$ltotemp) 
mod <- fracreghet(y_adj, X_het, Z_emp, var.endog = X_het[, "mrate"], type="QMLxv", link="logit")
summary(mod)

# Compute the same QMLxv estimator reporting Odds Ratios with 90\% confidence intervals
mod <- fracreghet(y_adj, X_het, Z_emp, var.endog = X_het[, "mrate"], type="QMLxv", 
           link="logit", or=TRUE, level=0.90) 
 
### Simulated Examples 

set.seed(123)
N <- 1000
x1 <- rnorm(N)

# Simulating an endogenous variable (var.endog) and an instrument (z1)
z1 <- rnorm(N)
u <- 0.5 * z1 + rnorm(N)
var.endog <- 0.8 * z1 + u
y_endog <- exp(0.5 * x1 + 1.2 * var.endog + u) / (1 + exp(0.5 * x1 + 1.2 * var.endog + u))

# Avoid exact 0 or 1 boundaries for some estimators
y_endog[y_endog <= 0] <- 0.01
y_endog[y_endog >= 1] <- 0.99

X <- cbind(x1 = x1, var.endog = var.endog)
Z <- cbind(x1 = x1, z1 = z1)

# Exogeneity (assuming var.endog is exogenous for comparison), GMMx estimator
mod <- fracreghet(y = y_endog, x = X, type = "GMMx", link = "logit")
summary(mod)

# Endogeneity, GMMz estimator (does not require reduced form for endog)
mod <- fracreghet(y = y_endog, x = X, z = Z, type = "GMMz", link = "logit")
summary(mod)

# Endogeneity, GMMxv estimator (assumes linear reduced form for var.endog)
mod <- fracreghet(y = y_endog, x = X, z = Z, var.endog = var.endog, type = "GMMxv", link = "logit")
summary(mod)

# Endogeneity, QMLxv control function approach
mod <- fracreghet(y = y_endog, x = X, z = Z, var.endog = var.endog, type = "QMLxv", link = "logit")
summary(mod)
}
\references{
Papke, L. E. and Wooldridge, J. M. (2008), "Panel data methods for fractional response variables with an application to test pass rates", \emph{Journal of Econometrics}, 145, 121-133.

Ramalho, E. A., & Ramalho, J. J. S. (2017), "Moment-based estimation of nonlinear regression models with boundary outcomes and endogeneity, with applications to nonnegative and fractional responses", \emph{Econometric Reviews}, 36(4), 397-420.

Terza, J. V., Basu, A., and Rathouz, P. J. (2008), "Two-stage residual inclusion estimation: addressing endogeneity in health econometric modeling", \emph{Journal of Health Economics}, 27(3), 531-543.
}
\seealso{
\code{\link{fracreghet.reset}}, for the RESET test.\cr
\code{\link{fracreghet.pe}}, for computing partial effects.\cr
\code{\link{fracreg}}, for fitting standard cross-sectional fractional response models.\cr
\code{\link{fracregpd}}, for fitting panel data fractional response models.
}
\author{
Sulman Olieko Owili <oliekosulman@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregmlogit.plot.R
\name{plot.fracregmlogit}
\alias{plot.fracregmlogit}
\title{Plot Marginal or Discrete Effects of Willingness to Pay}
\usage{
\method{plot}{fracregmlogit}(
  x,
  wtp.vec = NULL,
  varlist = NULL,
  against = NULL,
  mfrow = NULL,
  t = 500,
  effect = c("discrete", "marginal"),
  type = NULL,
  plot.show = TRUE,
  ...
)
}
\arguments{
\item{x}{A "fracregmlogit" object.}

\item{wtp.vec}{A numeric vector for willingness to pay.}

\item{varlist}{A string vector which provides the names of variables to plot the effect for.
If missing, all variables in the object will be plotted.}

\item{against}{A vector with the same length as the number of observations in the model, or the name of a variable. 
Serves as the x-axis in the plots.}

\item{mfrow}{A numeric vector with two elements. Specifies the number of rows and columns in a panel.
Similar to par(mfrow=c()). Default to NULL, and the program will choose a square panel.}

\item{t}{Number of points to be used for smoothing.}

\item{effect}{The type of effect ("marginal" or "discrete").}

\item{type}{Plot type.}

\item{plot.show}{If TRUE, the plot will be created. Otherwise, the function returns raw data that can be
used to create user-specified (custom) plots.}

\item{...}{Additional arguments.}
}
\value{
Panel plots of effects vs. chosen variables.
}
\description{
Plot marginal or discrete effects of willingness to pay, potentially against another variable.
}
\details{
This function provides a visualisation tool for potentially heterogeneous marginal and discrete effects of willingness to pay.
The function allows the user to plot marginal effects to detect any patterns in the effects, in itself
and against other variables. The plot also allows visualisation of sub-groups in data, which can be
very useful to visualise categorical and dummy variables. 

The function takes a \code{fracregmlogit} object, and internally calls \code{wtp} and \code{fracregmlogit.pe} 
to compute the willingness to pay at different data points.

Additional parameters include \code{varlist}, a vector of string variable names to be plotted. 
 
\code{against} allows a different variable to be chosen
as the x-axis. \code{against} can supply the column name of a variable in the original dataset to plot against.
}
\examples{
data("fracreg_spending")
X = fracreg_spending[,2:5]
y = fracreg_spending[,6:11]
results1 = fracregmlogit(y, X)

# Define a willingness to pay vector
wtp.vec = c(1, 1, 1, 1, 1, 1)

# Plot WTP for 'popdens'
plot(results1, wtp.vec=wtp.vec, varlist="popdens")
}
\seealso{
\code{\link{wtp}}, \code{\link{fracregmlogit.pe}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregridge.R
\name{fracregridge}
\alias{fracregridge}
\title{Fractional Ridge Regression}
\usage{
fracregridge(
  y,
  x,
  fracs = seq(0.1, 1, by = 0.1),
  tol = 1e-10,
  intercept = TRUE,
  na.action = stats::na.omit,
  ...
)
}
\arguments{
\item{y}{A numeric vector or matrix of the dependent variable(s).}

\item{x}{A numeric matrix of the explanatory variables.}

\item{fracs}{A numeric vector indicating the desired fractions of the unregularized coefficient vector length. Default is \code{seq(0.1, 1.0, by=0.1)}. Values must be sorted in ascending order.}

\item{tol}{A numeric tolerance under which singular values of the \code{x} matrix are considered to be zero. Default is \code{1e-10}.}

\item{intercept}{logical. If \code{TRUE}, an intercept is included in the model. Default is \code{TRUE}.}

\item{na.action}{A function specifying how to handle missing values, default is \code{stats::na.omit}. If \code{NULL}, no action is taken.}

\item{...}{further arguments passed to or from other methods.}
}
\value{
An object of class \code{fracregridge} containing:
\item{coef}{The estimated ridge coefficients for each requested fraction.}
\item{alphas}{The corresponding \eqn{\alpha} penalty values.}
\item{fracs}{The grid of fractions.}
\item{call}{The matched call.}
}
\description{
\code{fracregridge} implements Fractional Ridge Regression (Rokem & Kay, 2020), which is an approach to regularized linear regression.
Unlike standard ridge regression where the penalty term \eqn{\alpha} is chosen directly, \code{fracregridge} allows you to specify the desired \emph{fraction} of the unregularized OLS coefficient vector length. The algorithm then automatically determines the corresponding \eqn{\alpha} penalties.
}
\details{
Standard ridge regression minimizes the following objective function:
\deqn{L = (y - X\beta)'(y - X\beta) + \alpha \beta'\beta}

The penalty \eqn{\alpha} shrinks the coefficients towards zero, reducing the length of the coefficient vector \eqn{||\beta||_2}. However, choosing \eqn{\alpha} can be unintuitive. 
\code{fracregridge} re-parameterizes the problem so the user specifies \code{fracs}, the fraction of the unregularized Ordinary Least Squares (OLS) vector length \eqn{\gamma = \frac{||\beta(\alpha)||_2}{||\beta(0)||_2}}.
The function automatically determines the \eqn{\alpha} values corresponding to these fractions.
}
\examples{
# Empirical 401(k) Example
data("fracreg_k401k")
y_401k <- fracreg_k401k$prate
X_401k <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age,
                totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole)

# Fit fractional ridge regression for the 401(k) participation rates
mod_401k <- fracregridge(y = y_401k, x = X_401k, fracs = seq(0.2, 1.0, by = 0.2))

# View full detailed summary
summary(mod_401k)

# Compute Average Partial Effects for Ridge
pe_401k <- fracregridge.pe(mod_401k)
summary(pe_401k)

# Simulated Data Example
set.seed(123)
n <- 100
p <- 10
y <- rnorm(n)
X <- matrix(rnorm(n * p), n, p)
colnames(X) <- paste0("X", 1:p)

# Fit Fractional Ridge Regression
# We want the coefficients that correspond to 30\\\%, 50\\\%, and 80\\\% of the OLS length
mod_sim <- fracregridge(y, X, fracs = c(0.3, 0.5, 0.8))

# View brief summary
print(mod_sim)

# Compute Partial Effects
pe_sim <- fracregridge.pe(mod_sim)
summary(pe_sim)
set.seed(123)
y <- rnorm(100)
x <- matrix(rnorm(1000), 100, 10)
colnames(x) <- paste0("x", 1:10)

# Fit fractional ridge regression
mod <- fracregridge(y, x, fracs = c(0.3, 0.5, 0.8))
print(mod)
summary(mod)
}
\references{
Rokem, A., & Kay, K. (2020). Fractional ridge regression: a fast, interpretable reparameterization of ridge regression. \emph{GigaScience}, 9(12).

Rokem, A., and Kay, K., fracridge: Fractional Ridge Regression. Package repository. <https://github.com/nrdg/fracridge>.
}
\seealso{
\code{\link{fracreg}}
}
\author{
Sulman Olieko Owili <oliekosulman@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregridge.extractors.R
\name{nobs.fracregridge}
\alias{nobs.fracregridge}
\title{Extract the Number of Observations for fracregridge}
\usage{
\method{nobs}{fracregridge}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracregridge}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
An integer denoting the number of observations.
}
\description{
Extracts the number of observations used to estimate a \code{fracregridge} model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregridge.extractors.R
\name{fitted.fracregridge}
\alias{fitted.fracregridge}
\title{Extract Fitted Values for fracregridge}
\usage{
\method{fitted}{fracregridge}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracregridge}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A matrix or array of fitted values.
}
\description{
Extracts the fitted conditional mean values from a \code{fracregridge} model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracreg.extractors.R
\name{logLik.fracreg}
\alias{logLik.fracreg}
\title{Extract Log-Likelihood for fracreg}
\usage{
\method{logLik}{fracreg}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracreg}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
An object of class \code{logLik}.
}
\description{
Extracts the log-pseudolikelihood or log-likelihood from a fitted \code{fracreg} model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregpd.extractors.R
\name{fitted.fracregpd}
\alias{fitted.fracregpd}
\title{Extract Fitted Values for fracregpd}
\usage{
\method{fitted}{fracregpd}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracregpd}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A numeric vector of fitted values.
}
\description{
Extracts the fitted conditional mean values from a \code{fracregpd} model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracreghet.extractors.R
\name{coef.fracreghet}
\alias{coef.fracreghet}
\title{Extract Model Coefficients for fracreghet}
\usage{
\method{coef}{fracreghet}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracreghet}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A named vector of coefficients.
}
\description{
Extracts the estimated coefficients from a fitted \code{fracreghet} model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracreg.reset.R
\name{fracreg.reset}
\alias{fracreg.reset}
\title{RESET Test for Fractional Response Regressions}
\usage{
fracreg.reset(object, lastpower.vec = 3, version = "LM", table = FALSE, ...)
}
\arguments{
\item{object}{an object containing the results of an \code{fracreg} command.}

\item{lastpower.vec}{a numeric vector containing the maximum powers of the linear predictors to be used in RESET tests.}

\item{version}{a vector containing the test versions to use. Available options: \code{Wald}, \code{LM} (the default) and, only for the binary component of two-part models, \code{LR}. More than one option may be chosen.}

\item{table}{a logical value indicating whether a summary table with the test results should be printed.}

\item{\dots}{Arguments to pass to \link[stats]{glm}, which is used to estimate the model under the alternative hypothesis when \code{version} is a vector containing \code{"Wald"} or \code{"LR"}.}
}
\value{
\code{fracreg.reset} returns a named vector with the test results.
}
\description{
\code{fracreg.reset} is used to perform the Regression Equation Specification Error Test (RESET) to check the functional form and specification of fractional response models.
}
\details{
\code{fracreg.reset} applies the RESET test statistic to fractional response
models estimated via \code{fracreg}. \code{fracreg.reset} may be used to test the link specification of: (i) one-part fractional response models; (ii) the binary
components of two-part and three-part fractional response models; and (iii) the fractional components of two-part and three-part fractional response models. 

\strong{RESET Test Framework:}
The Regression Equation Specification Error Test (RESET) assesses whether the link function \eqn{G(\cdot)} and the linear index \eqn{x\beta} are correctly specified. It tests the null hypothesis \eqn{H_0: \gamma = 0} in the augmented model:
\deqn{E(y|x) = G(x\beta + \sum_{k=2}^P \gamma_k (x\hat{\beta})^k)}
where \eqn{P} is the maximum power of the linear predictor (specified by \code{lastpower.vec}) and \eqn{\hat{\beta}} are the estimated parameters from the baseline model.

When the \code{Wald} version is implemented, it is taken into account the option that was chosen for computing standard errors in the model under evaluation. For the \code{LM} version, a robust version is computed in cases (i) and (iii) and a conventional version in case (ii). See Ramalho, Ramalho and Murteira (2011) for details on the application of the RESET test in the fractional response framework.
}
\examples{
### Empirical 401(k) Examples
data("fracreg_k401k")
y <- fracreg_k401k$prate
X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age, 
           totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole)

m <- fracreg(y, X, type="1P", linkfrac="logit")
reset_res <- fracreg.reset(m)
summary(reset_res)

### Simulated Examples

N <- 250
u <- rnorm(N)

X <- cbind(rnorm(N),rnorm(N))
dimnames(X)[[2]] <- c("X1","X2")

ym <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u))
y <- rbeta(N,ym*20,20*(1-ym))
y[y > 0.9] <- 1

#Testing the logit specification of a standard fractional response model
#using LM and Wald versions of the RESET test, based on 1 or 2 fitted powers of
#the linear predictor
mod <- fracreg(y,X,linkfrac="logit")
reset_res <- fracreg.reset(mod,2:3,c("Wald","LM"))
summary(reset_res)

#Testing the probit specification of the binary component of a two-part fractional
#regression model using LR-based RESET tests with quadratic and cubic fitted 
#powers of the linear predictor
mod <- fracreg(y,X,linkbin="probit",type="2Pbin",inf=1)
reset_res <- fracreg.reset(mod,3,"LR")
summary(reset_res)
}
\references{
Ramalho, E.A., J.J.S. Ramalho and J.M.R. Murteira (2011), "Alternative
estimating and testing empirical strategies for fractional response models",
\emph{Journal of Economic Surveys}, 25(1), 19-68.

Ramsey, J.B. (1969), "Tests for Specification Errors in Classical Linear Least-Squares Regression Analysis", \emph{Journal of the Royal Statistical Society: Series B (Methodological)}, 31(2), 350-371.
}
\seealso{
\code{\link{fracreg}}, for fitting fractional response models.\cr
\code{\link{fracreg.ggoff}}, for asymptotically equivalent specification tests.\cr
\code{\link{fracreg.ptest}}, for non-nested hypothesis tests.\cr
\code{\link{fracreg.pe}}, for computing partial effects.
}
\author{
Sulman Olieko Owili <oliekosulman@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracreghet.pe.R
\name{fracreghet.pe}
\alias{fracreghet.pe}
\title{Fractional Response Regressions under Unobserved Heterogeneity - Partial Effects}
\usage{
fracreghet.pe(
  object,
  smearing = TRUE,
  APE = TRUE,
  CPE = FALSE,
  at = NULL,
  which.x = NULL,
  table = FALSE,
  variance = TRUE
)
}
\arguments{
\item{object}{an object containing the results of an \code{fracreghet} command.}

\item{smearing}{a logical value indicating whether the smearing correction is to be applied}

\item{APE}{a logical value indicating whether average partial effects are to be computed.}

\item{CPE}{a logical value indicating whether conditional partial effects are to be computed.}

\item{at}{a numeric vector containing the covariates' values at which the conditional partial effects are to be computed or the strings \code{"mean"} (the default) or \code{"median"}, in which cases the covariates are evaluated at their mean or median values (or mode, in case of dummy variables), respectively.}

\item{which.x}{a vector containing the names of the covariates to which the partial effects are to be computed.}

\item{table}{a logical value indicating whether a summary table with the results should be printed.}

\item{variance}{a logical value indicating whether the variance of the estimated partial effects should be calculated. Defaults to \code{TRUE} whenever \code{table = TRUE}.}
}
\value{
\code{fracreghet.pe} returns a list with the following element:
  \item{PE.p}{a named vector of partial effects.
}

If \code{variance = TRUE} or \code{table = TRUE}, the previous list also contains the following element:
  \item{PE.sd}{a named vector of standard errors of the estimated partial effects.
}

When both average and conditional partial effects are requested, two lists containing the previous elements are returned, indexed by the prefixes \code{ape} and \code{cpe}.
}
\description{
\code{fracreghet.pe} is used to compute average and/or conditional partial effects in fractional response models 
under unobserved heterogeneity.
}
\details{
\code{fracreghet.pe} calculates partial effects for fractional response models estimated via \code{fracreghet}. \code{fracreghet.pe} may be used to compute average or conditional partial effects. These partial effects may be conditional only on observables, using the smearing estimator, or also on unobservables, setting the error term to zero.

\strong{Partial Effects under Unobserved Heterogeneity:}
When unobserved heterogeneity or endogeneity is present, calculating partial effects requires dealing with the unobserved error \eqn{v_i}. Let the conditional mean be \eqn{E(y|x, v) = G(x\beta + \gamma v)}.
- \strong{Conditional on Observables (Smearing):} The unobserved heterogeneity is integrated out over its empirical distribution. The average partial effect for a continuous variable \eqn{x_k} is computed as:
\deqn{PE_k(x) = \frac{1}{N} \sum_{i=1}^N g(x\beta + \gamma \hat{v}_i) \beta_k}
- \strong{Conditional on Unobservables (Error = 0):} The partial effect is evaluated for an individual with the mean level of unobserved heterogeneity (\eqn{v = 0}):
\deqn{PE_k(x) = g(x\beta) \beta_k}

For discrete variables, the partial effects are calculated as the discrete differences evaluated using either the smearing approach or setting the error term to zero. 

For calculating standard errors, it is taken into account the option that was previously chosen for estimating the model. See Ramalho and Ramalho (2017) for details on the computation of partial effects for fractional response models under unobserved heterogeneity.
}
\examples{
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X_het <- cbind(mrate = fracreg_k401k$mrate, ltotemp = fracreg_k401k$ltotemp)
 
# fracreghet estimators do not allow exact 1s or 0s
y_adj <- y
y_adj[y_adj == 1] <- 0.999

# Instrument mrate using age

Z_emp <- cbind(age = fracreg_k401k$age, ltotemp = fracreg_k401k$ltotemp) 
res_emp <- fracreghet(y_adj, X_het, Z_emp, var.endog = X_het[, "mrate"], 
                      type="QMLxv", link="logit") 
pe_res <- fracreghet.pe(res_emp, which.x="mrate")
summary(pe_res)
 
### Simulated Examples

N <- 250
u <- rnorm(N)

X <- cbind(rnorm(N),rnorm(N))
dimnames(X)[[2]] <- c("X1","X2")

Z <- cbind(rnorm(N),rnorm(N),rnorm(N))
dimnames(Z)[[2]] <- c("Z1","Z2","Z3")

y <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u))

mod <- fracreghet(y,X,type="GMMx")

#Smearing estimator of average partial effects for variable X1
pe_res <- fracreghet.pe(mod,which.x="X1")
summary(pe_res)

#Naive estimator of conditional partial effects for all covariates,
#which are evaluated at X1=1 and X2=-1
pe_res <- fracreghet.pe(mod,smearing=FALSE,APE=FALSE,CPE=TRUE,at=c(1,-1))
summary(pe_res)
}
\references{
Ramalho, E. A., & Ramalho, J. J. S. (2017), "Moment-based estimation of nonlinear regression models with boundary outcomes and endogeneity, with applications to nonnegative and fractional responses", \emph{Econometric Reviews}, 36(4), 397-420.
}
\seealso{
\code{\link{fracreghet}}, for fitting fractional response models under unobserved heterogeneity.\cr
\code{\link{fracreghet.reset}}, for the RESET test.\cr
}
\author{
Sulman Olieko Owili <oliekosulman@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregmlogit.extractors.R
\name{fitted.fracregmlogit}
\alias{fitted.fracregmlogit}
\alias{residuals.fracregmlogit}
\alias{predict.fracregmlogit}
\alias{Extract}
\alias{fitted}
\alias{dependent}
\alias{variable,}
\alias{residuals,}
\alias{or}
\alias{predictions}
\alias{from}
\alias{a}
\alias{fractional}
\alias{multinomial}
\alias{logit}
\alias{model.}
\title{Extract Fitted Values, Residuals, and Predictions}
\usage{
\method{fitted}{fracregmlogit}(object, ...)

\method{residuals}{fracregmlogit}(object, ...)

\method{predict}{fracregmlogit}(object, newdata = NULL, newbeta = NULL, ...)
}
\arguments{
\item{object}{A "fracregmlogit" object.}

\item{...}{Additional arguments.}

\item{newdata}{A new X matrix to perform model prediction. If NULL, defaults to the original dataset. 
X can be a vector with length k, or a matrix with k columns, where k is the number of explanatory 
variables in the original model.}

\item{newbeta}{A new augmented matrix of coefficients that can be used to predict outcome variables. 
Feeds into \code{object$coefficient}, which contains the baseline coefficient. Useful for constructing
confidence intervals via simulation or bootstrapping.}
}
\value{
An object of class \code{data.frame} containing numeric values where each column corresponds to one of the choice alternatives in the response variable matrix and each row corresponds to an observation. Specifically:
\itemize{
  \item \code{fitted}: Returns the estimated fitted fractional response values (choice shares or predicted conditional probabilities).
  \item \code{residuals}: Returns the response residuals (the actual observed shares minus the estimated fitted shares).
  \item \code{predict}: Returns the predicted choice shares or conditional probabilities computed from the specified model object and \code{newdata} or \code{newbeta}.
}
}
\description{
Extract Fitted Values, Residuals, and Predictions
}
\examples{
data("fracreg_spending")
df <- na.omit(fracreg_spending)
X = df[,2:5]
y = df[,6:11]
results1 = fracregmlogit(y, X)

# Extract fitted values
fit = fitted(results1)

# Extract residuals
res = residuals(results1)

# Predict using the first observation from the original dataset
pred = predict(results1, newdata = X[1,])
}
\seealso{
\code{\link{fracregmlogit}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregmlogit.R
\name{fracregmlogit}
\alias{fracregmlogit}
\title{Estimate Fractional Multinomial Logit Models}
\usage{
fracregmlogit(
  y,
  X,
  beta0 = NULL,
  MLEmethod = "CG",
  maxit = 5e+05,
  abstol = 1e-05,
  cluster = NULL,
  reps = 1000,
  na.action = stats::na.omit,
  ...
)
}
\arguments{
\item{y}{the dependent variable (N*J). Can be a matrix or a named data frame.
The first column of the matrix is automatically treated as the baseline.}

\item{X}{independent variable (N*K). Can be a matrix or a named data frame.
If there is no intercept term in the X, then an intercept term is
automatically added.}

\item{beta0}{Initial value for beta used in optimisation. Uses a 1*K(J-1)
vector. Default to a vector of zeros.}

\item{MLEmethod}{Method of optimisation. Goes into
\code{maxLik(method=MLEmethod))}. Choose from "NR","BFGS","CG","BHHH","SANN",or "NM".  
Default to "CG", the conjugate gradients method. See Details.}

\item{maxit}{Maximum number of iterations.}

\item{abstol}{Tolerance.}

\item{cluster}{A vector of clusters to be used for clustered standard error computation. 
Default to NULL, no cluster computed.}

\item{reps}{Number of bootstrap replications to be computed for clustered standard errors.}

\item{na.action}{A function specifying how to handle missing values, default is \code{stats::na.omit}. If \code{NULL}, no action is taken.}

\item{...}{additional parameters that go into \code{maxLik()}}
}
\value{
The function returns an object of class "fracregmlogit". Use \code{fracregmlogit.pe}, \code{predict}, 
 \code{residuals}, \code{fitted} to extract various useful features of the value returned by 
\code{fracregmlogit}.

An object of class "fracregmlogit" contains the following components:

\code{estimates}   A list of matrices containing parameter estimates,
  standard errors, and hypothesis testing results.

\code{baseline}    The baseline choice

\code{likelihood}  The likelihood value

\code{conv_code}   Convergence diagnostics code.

\code{convergence} Convergence messages.

\code{count}       Provides dataset information

\code{y}           The dependent variable data frame.

\code{X}           The independent variable data frame. Augmented by factor dummy transformation
, constant term added.

\code{rowNo}       A vector of row numbers from the original X and y that is used for estimation.

\code{coefficient} Matrix of estimated coefficients. Augmented with the baseline coefficient
(which is a vector of zeros).

\code{vcov}        A list of matrices containing the robust variance covariance matrix for each choice
variable.

\code{cluster}     The vector of clusters.

\code{reps}        Number of bootstrap replications for clustered standard error
}
\description{
Used to estimate fractional multinomial logit models using quasi-maximum
likelihood estimation following Papke and Wooldridge (1996).
}
\details{
The fractional multinomial logit model is the expansion of the multinomial
logit to fractional responses. Unlike standard multinomial logit models,
which only consider 0-1 responses, the fractional multinomial logit model considers the
case where the response variable is fractions that sum up to one. Examples
of this type of data include percentages of budget spent in education, defence,
public health; fractions of a population that have middle school, high
school, college, or post-college education, etc.

This function follows Papke and Wooldridge (1996)'s paper, in which they
proposed a quasi-maximum likelihood estimator for fractional response data.
The likelihood function used here is a standard multinomial likelihood
function, see Buis (2008) and <http://maartenbuis.nl/software/likelihoodFmlogit.pdf> for
the likelihood used here. Robust standard errors are provided following Papke
and Wooldridge (1996), in which they proposed an asymptotically consistent
estimator of variance.

Maximisation is done by calling \code{\link[maxLik]{maxLik}}. maxLik is a wrapper function 
for different maximisation methods in R. These include most methods provided by \code{\link[maxLik]{maxLik}},  
but also other methods such as BHHH (Berndt-Hall-Hall-Hausman). 

MLE convergence can be a problem in R, especially if the dataset is large with many explanatory variables. 
It is recommended to call CG (Conjugate Gradients) or BHHH (Berndt-Hall-Hall-Hausman).
The conjugate gradients method is usually faster, but could lead to non-convergence under 
certain scenarios. BHHH is slower, but has better convergence properties.
}
\examples{
data("fracreg_spending")
X = fracreg_spending[,2:5]
y = fracreg_spending[,6:11]

# Fit the fractional multinomial logit model
results1 = fracregmlogit(y, X)

# View estimates
summary(results1)

# Compute marginal effects
pe = fracregmlogit.pe(results1, effect="marginal", marg.type="aveacr", se=TRUE, R=50)
summary(pe)

# Plot effects for 'houseval'
plot(pe, varlist="houseval")

}
\references{
Papke, L. E. and Wooldridge, J. M. (1996), Econometric methods
  for fractional response variables with an application to 401(k) plan
  participation rates. J. Appl. Econ., 11: 619-632.

Buis, M. L. (2008), fmlogit: Stata module fitting a fractional 
  multinomial logit model by quasi maximum likelihood. Statistical Software Components, 
  Boston College Department of Economics.

Mullahy, J. (2015), Multivariate fractional regression estimation of 
  econometric share models. Journal of Econometric Methods, 4(1): 71-100.

Murteira, J. M. R., and Ramalho, J. J. S. (2016), Regression analysis of 
  multivariate fractional data. Econometric Reviews, 35(4): 515-552.

Ji, J., and Woodill, A. J., fmlogit: Fractional Multinomial Logit. R package repository. <https://github.com/f1kidd/fmlogit>.
}
\seealso{
\code{\link{fracregmlogit.pe}} for computing partial effects, \code{\link{plot.fracregmlogit.pe}} for plotting effects, \code{\link{fitted.fracregmlogit}} for residuals and predictions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregmlogit.extractors.R
\name{vcov.fracregmlogit}
\alias{vcov.fracregmlogit}
\title{Extract Covariance Matrix for fracregmlogit}
\usage{
\method{vcov}{fracregmlogit}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracregmlogit}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A list of covariance matrices for each choice equation.
}
\description{
Extracts the estimated variance-covariance matrices of the parameters.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{summary.fracregmlogit}
\alias{summary.fracregmlogit}
\alias{summary.fracregmlogit.pe}
\alias{summary.fracregmlogit.wtp}
\title{Generate Summary Tables for fracregmlogit Objects}
\usage{
\method{summary}{fracregmlogit}(object, ...)
}
\arguments{
\item{object}{an object with class "fracregmlogit", "fracregmlogit.pe", or "fracregmlogit.wtp".}

\item{...}{Additional arguments passed to the printCoefmat function.}
}
\value{
Returns the object invisibly.
}
\description{
Generate tables of coefficient estimates, partial effects, and willingness to pay from
fracregmlogit-type objects.
}
\details{
This module provides summary methods for three fracregmlogit objects: \code{fracregmlogit}, \code{fracregmlogit.pe}
, and \code{fracregmlogit.wtp}. 

For \code{fracregmlogit} objects, the summary prints the number of observations, log pseudo-likelihood,
baseline choice, and the coefficient estimates with standard errors, z-statistics, and p-values
for each choice equation. 

For \code{fracregmlogit.pe} objects, it displays the marginal or discrete effects 
along with their computed standard errors (if Krinsky-Robb sampling was performed) for each choice.

For \code{fracregmlogit.wtp} objects, it provides a table of the aggregated willingness to pay 
along with its standard errors and test statistics.
}
\examples{
data("fracreg_spending")
X = fracreg_spending[,2:5]
y = fracreg_spending[,6:11]

# generate fracregmlogit summary
results1 = fracregmlogit(y, X)
summary(results1)

# generate marginal effects summary
effects1 = fracregmlogit.pe(results1, effect="marginal", se=FALSE)
summary(effects1)

}
\seealso{
\code{\link{fracregmlogit}}, \code{\link{fracregmlogit.pe}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregridge.extractors.R
\name{logLik.fracregridge}
\alias{logLik.fracregridge}
\title{Extract Log-Likelihood for fracregridge}
\usage{
\method{logLik}{fracregridge}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracregridge}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
An object of class \code{logLik}.
}
\description{
Extracts the log-likelihood from a fitted \code{fracregridge} model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{fracreg_clean_data}
\alias{fracreg_clean_data}
\title{Clean Data for Fractional Regression Models}
\usage{
fracreg_clean_data(..., na.action = stats::na.omit)
}
\arguments{
\item{...}{A variable number of vectors, matrices, or data frames.}

\item{na.action}{A function specifying how to handle missing values, default is \code{stats::na.omit}. If \code{NULL}, no action is taken.}
}
\value{
A named list containing the subsets of the provided arrays without missing values.
}
\description{
An internal helper to gracefully drop missing values across an arbitrary number of vectors and matrices, replicating the functionality of na.action = na.omit for multi-array model inputs.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracreghet.reset.R
\name{fracreghet.reset}
\alias{fracreghet.reset}
\title{RESET Test for Fractional Response Regressions under Neglected Heterogeneity}
\usage{
fracreghet.reset(
  object,
  lastpower.vec = 3,
  version = "Wald",
  table = FALSE,
  ...
)
}
\arguments{
\item{object}{an object containing the results of an \code{fracreghet} command.}

\item{lastpower.vec}{a numeric vector containing the maximum powers of the linear predictors to be used in RESET tests.}

\item{version}{a vector containing the test versions to use. Available options: \code{Wald} (the default) and \code{LM} (only  available for \code{GMMx}).}

\item{table}{a logical value indicating whether a summary table with the test results should be printed.}

\item{\dots}{Arguments to pass to \link[stats]{nlminb}, which is used to estimate the model under the alternative hypothesis when \code{version} is equal to \code{"Wald"} and the null model was estimated by \code{GMMx}.}
}
\value{
\code{fracreghet.reset} returns a named vector with the test results.
}
\description{
\code{fracreghet.reset} is used to test the specification of fractional response models estimated by GMMx or LINx.
}
\details{
\code{fracreghet.reset} applies the RESET test statistic to fractional response
models estimated via \code{fracreghet} using the options \code{GMMx} or \code{LINx}. \code{fracreghet.reset} may be used to test simultaneously the validity of the link specification and the transformation applied to the response variable by each estimator.  

\strong{RESET Test under Unobserved Heterogeneity:}
The test is based on augmenting the original model with powers of the linear predictor \eqn{x\hat{\beta}}. For GMMx, it tests \eqn{H_0: \gamma = 0} in the expanded moment conditions:
\deqn{E\left[Z_i \left(H(y_i) - \exp\left(x_i\beta + \sum_{k=2}^P \gamma_k (x_i\hat{\beta})^k\right)E(e^{c_i})\right)\right] = 0}
This simultaneously evaluates whether the mean function and the specific heterogeneity transformation \eqn{H(\cdot)} are correctly specified.

It is taken into account the option that was chosen for computing standard errors in the model under evaluation. See Ramalho and Ramalho (2017) for details.
}
\examples{
### Empirical 401(k) Examples 
data("fracreg_k401k") 
y <- fracreg_k401k$prate 
X_het <- cbind(mrate = fracreg_k401k$mrate, ltotemp = fracreg_k401k$ltotemp)
 
# fracreghet estimators do not allow exact 1s or 0s
y_adj <- y
y_adj[y_adj == 1] <- 0.999

# Instrument mrate using age

Z_emp <- cbind(age = fracreg_k401k$age, ltotemp = fracreg_k401k$ltotemp) 
res_emp <- fracreghet(y_adj, X_het, type="GMMx", link="logit") 
reset_res <- fracreghet.reset(res_emp)
summary(reset_res)
 
### Simulated Examples

N <- 250
u <- rnorm(N)

X <- cbind(rnorm(N),rnorm(N))
dimnames(X)[[2]] <- c("X1","X2")

Z <- cbind(rnorm(N),rnorm(N),rnorm(N))
dimnames(Z)[[2]] <- c("Z1","Z2","Z3")

y <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u))

mod <- fracreghet(y,X,type="GMMx")

#LM and Wald versions of the RESET test, based on 1 or 2 fitted powers of xb
reset_res <- fracreghet.reset(mod,2:3,c("Wald","LM"))
summary(reset_res)
}
\references{
Ramalho, E. A., & Ramalho, J. J. S. (2017), "Moment-based estimation of nonlinear regression models with boundary outcomes and endogeneity, with applications to nonnegative and fractional responses", \emph{Econometric Reviews}, 36(4), 397-420.

Ramsey, J.B. (1969), "Tests for Specification Errors in Classical Linear Least-Squares Regression Analysis", \emph{Journal of the Royal Statistical Society: Series B (Methodological)}, 31(2), 350-371.
}
\seealso{
\code{\link{fracreghet}}, for fitting fractional response models under unobserved heterogeneity.\cr
\code{\link{fracreghet.pe}}, for computing partial effects.
}
\author{
Sulman Olieko Owili <oliekosulman@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregmlogit.extractors.R
\name{logLik.fracregmlogit}
\alias{logLik.fracregmlogit}
\title{Extract Log-Likelihood for fracregmlogit}
\usage{
\method{logLik}{fracregmlogit}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracregmlogit}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
An object of class \code{logLik}.
}
\description{
Extracts the log-pseudolikelihood from a fitted \code{fracregmlogit} model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{fracreg_k401k}
\alias{fracreg_k401k}
\title{401(k) Plan Participation Data}
\format{
A data frame with 1,534 observations and 10 variables:
\describe{
  \item{prate}{Participation rate: proportion of eligible employees participating in the 401(k) plan (0 to 1).}
  \item{mrate}{Match rate: the firm's contribution matching rate per dollar.}
  \item{totpart}{Total number of participants.}
  \item{totelg}{Total number of eligible employees.}
  \item{age}{Age of the 401(k) plan in years.}
  \item{totemp}{Total number of firm employees.}
  \item{sole}{Indicator variable: 1 if the 401(k) is the sole retirement plan offered.}
  \item{ltotemp}{Natural log of total employees.}
  \item{age_sq}{Square of plan age.}
  \item{mrate_sq}{Square of match rate.}
}
}
\source{
Papke, L. E., & Wooldridge, J. M. (1996). "Econometric Methods for Fractional Response Variables with an Application to 401(k) Plan Participation Rates." Journal of Applied Econometrics, 11(6), 619-632.
}
\usage{
data("fracreg_k401k")
}
\description{
A cross-sectional dataset on 401(k) plan participation rates and firm characteristics, 
widely used in empirical applications of fractional response models 
(e.g., Papke & Wooldridge, 1996). The data is derived from the 'wooldridge' package.
}
\examples{
data("fracreg_k401k")
summary(fracreg_k401k$prate)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregmlogit.plot.R
\name{plot.fracregmlogit.pe}
\alias{plot.fracregmlogit.pe}
\title{Plot Marginal or Discrete Effects}
\usage{
\method{plot}{fracregmlogit.pe}(
  x,
  varlist = NULL,
  X = NULL,
  y = NULL,
  against = NULL,
  against.x = NULL,
  against.y = NULL,
  group.x = NULL,
  group.algebra = NULL,
  mfrow = NULL,
  ...
)
}
\arguments{
\item{x}{A "fracregmlogit.pe" object.}

\item{varlist}{A string vector which provides the names of variables to plot the effect for.
If missing, all variables in the object will be plotted.}

\item{X}{A matrix of independent variables.}

\item{y}{A matrix of dependent variables.}

\item{against}{A vector with the same length as the number of observations in the model. 
Serves as the x-axis in the plots.}

\item{against.x}{A character string, supply the column name in the X matrix to plot against.}

\item{against.y}{A character string, supply the column name in the y matrix to plot against.}

\item{group.x}{A character string. Supply the column name in the X matrix to group upon.}

\item{group.algebra}{A character string. Supply additional algebra imposed on the group variable.}

\item{mfrow}{A numeric vector with two elements. Specifies the number of rows and columns in a panel.
Similar to par(mfrow=c()). Default to NULL, and the program will choose a square panel.}

\item{...}{Additional arguments.}
}
\value{
Panel plots of effects vs. chosen variables.
}
\description{
Plot the desired effect at each observed value for each choice.
}
\details{
This function provides a visualisation tool for potentially heterogeneous marginal and discrete effects.
The function allows the user to plot marginal effects to detect any patterns in the effects, in itself
and against other variables. The plot also allows visualisation of sub-groups in data, which can be
very useful to visualise categorical and dummy variables. 

The function takes a \code{fracregmlogit.pe} object, created by the \code{fracregmlogit.pe()} function. Note that since 
the plotting requires marginal effects for all observations, the object should be created by choosing 
\code{marg.type="aveacr"}, the average across method for effects calculation. 

Additional parameters include \code{varlist}, a vector of string variable names to be plotted. \code{X}
and \code{y} are the dependent and independent variable matrices in the original regression model. 
 
\code{against}, \code{against.x}, and \code{against.y} allow different variables to be chosen
as the x-axis. \code{against} directly supplies the vector to be plotted against, whereas \code{against.x}
and \code{against.y} supply variable names in the original dataset. Note that the user has to provide
\code{X} and \code{y} in order to use the column name options, respectively. 
 
\code{group.x} supplies the column name in the X matrix to group by. The plot will be able to 
differentiate different groups by colours. Additionally, the user can supply a string to \code{group.algebra},
which provides an algebra operation that will be evaluated on the group vector. For example, choosing 
\code{group.x = "a"} and \code{group.algebra = ">0"} will create two groups, one with X$a > 0, and one with X$a <= 0.
}
\examples{
\donttest{
data("fracreg_spending")
X = fracreg_spending[,2:5]
y = fracreg_spending[,6:11]
results1 = fracregmlogit(y, X)

# Calculate marginal effects with marg.type="aveacr" (no standard errors for speed)
effect1 = fracregmlogit.pe(results1, effect="marginal", marg.type="aveacr", se=FALSE)

# Plot effects
plot(effect1, X=results1$X, against.x = "houseval", group.x = "popdens", group.algebra = ">10")
}
}
\seealso{
\code{\link{fracregmlogit.pe}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracreg.ptest.R
\name{fracreg.ptest}
\alias{fracreg.ptest}
\title{P Test for Fractional Response Regressions}
\usage{
fracreg.ptest(object1, object2, version = "Wald", table = FALSE)
}
\arguments{
\item{object1}{an object containing the results of an \code{fracreg} command.}

\item{object2}{an object containing the results of another \code{fracreg} command.}

\item{version}{a vector containing the test versions to use. Available options: \code{Wald} (the default) and \code{LM}.  Both options may be chosen at the same time and are computed in a robust way.}

\item{table}{a logical value indicating whether a summary table with the test results should be printed.}
}
\value{
\code{fracreg.reset} returns a named vector with the test results.
}
\description{
\code{fracreg.ptest} is used to perform the P test to evaluate the specification of alternative, non-nested fractional response models by testing against each other.
}
\details{
\code{fracreg.ptest} applies the P test statistic proposed by Davidson and
MacKinnon (1981) to fractional response models estimated via\code{fracreg}. \code{fracreg.ptest} may be used to test against each other two alternative specifications for the link function  in: (i) one-part fractional response models; (ii) the binary components of two-part and three-part fractional response models; (iii) the fractional components of two-part and three-part fractional response models; and (iv) two-part and three-part fractional response models. 

\strong{P Test Framework:}
The P test allows the comparison of non-nested models (e.g., alternative link functions or non-nested regressors). Let model 1 specify \eqn{E_1(y|x) = G(x\beta)} and model 2 specify \eqn{E_2(y|x) = H(z\theta)}. To test model 1 against model 2, the baseline model is augmented with the difference between the fitted values:
\deqn{E(y|x) = G\left(x\beta + \gamma \left( \hat{y}_{M2} - \hat{y}_{M1} \right)\right)}
where \eqn{\hat{y}_{M1} = G(x\hat{\beta})} and \eqn{\hat{y}_{M2} = H(z\hat{\theta})}. The null hypothesis that model 1 is correct is tested via \eqn{H_0: \gamma = 0}.

In addition, \code{fracreg.ptest} may be used to test one-part models against two-part or three-part models and in cases where the link functions are the same but the regressors are non-nested. See Ramalho, Ramalho and Murteira (2011) for details on the application of the P test in the fractional response framework.
}
\examples{
### Empirical 401(k) Examples
data("fracreg_k401k")
y <- fracreg_k401k$prate
X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age, 
           totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole)

m1 <- fracreg(y, X, type="1P", linkfrac="logit")
m2 <- fracreg(y, X, type="1P", linkfrac="probit")
ptest_res <- fracreg.ptest(m1, m2)
summary(ptest_res)

### Simulated Examples

N <- 250
u <- rnorm(N)

X <- cbind(rnorm(N),rnorm(N))
dimnames(X)[[2]] <- c("X1","X2")

ym <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u))
y <- rbeta(N,ym*20,20*(1-ym))
y[y > 0.9] <- 1

#Testing logit versus loglog specifications for standard fractional
#regression models using a LM version of the P test
res1 <- fracreg(y,X,linkfrac="logit")
res2 <- fracreg(y,X,linkfrac="loglog")
ptest_res <- fracreg.ptest(res1,res2,"LM")
summary(ptest_res)

#Testing a logit one-part fractional response model versus a binary logit +
#fractional probit two-part model using a Wald version of the P test
res1 <- fracreg(y,X,linkfrac="logit")
res2 <- fracreg(y,X,linkbin="logit",linkfrac="probit",type="2P",inf=1)
ptest_res <- fracreg.ptest(res1,res2,"Wald")
summary(ptest_res)
}
\references{
Davidson, R. and J.G. MacKinnon (1981), "Several tests for model specification
on the presence of alternative hypotheses", \emph{Econometrica}, 49(3), 781-793.

Ramalho, E.A., J.J.S. Ramalho and J.M.R. Murteira (2011), "Alternative
estimating and testing empirical strategies for fractional response models",
\emph{Journal of Economic Surveys}, 25(1), 19-68.
}
\seealso{
\code{\link{fracreg}}, for fitting fractional response models.\cr
\code{\link{fracreg.reset}} and \code{\link{fracreg.ggoff}}, for specification tests.\cr
\code{\link{fracreg.pe}}, for computing partial effects.
}
\author{
Sulman Olieko Owili <oliekosulman@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracreg.pe.R
\name{fracreg.pe}
\alias{fracreg.pe}
\title{Fractional Response Regressions - Partial Effects}
\usage{
fracreg.pe(
  object,
  APE = TRUE,
  CPE = FALSE,
  at = NULL,
  which.x = NULL,
  variance = TRUE,
  table = FALSE
)
}
\arguments{
\item{object}{an object containing the results of an \code{fracreg} command.}

\item{APE}{a logical value indicating whether average partial effects are to be computed.}

\item{CPE}{a logical value indicating whether conditional partial effects are to be computed.}

\item{at}{a numeric vector containing the covariates' values at which the conditional partial effects are to be computed or  the strings \code{"mean"} (the default) or \code{"median"}, in which cases the covariates are evaluated at their  mean or median values (or mode, in case of dummy variables), respectively.}

\item{which.x}{a vector containing the names of the covariates to which the partial effects are to be computed.}

\item{variance}{a logical value indicating whether the variance of the estimated partial effects should be calculated. Defaults to  \code{TRUE} whenever \code{table = TRUE}.}

\item{table}{a logical value indicating whether a summary table with the results should be printed.}
}
\value{
\code{fracreg.pe} returns a list with the following element:
  \item{PE.p}{a named vector of partial effects.
}

If \code{variance = TRUE} or \code{table = TRUE}, the previous list also contains the following element:
  \item{PE.sd}{a named vector of standard errors of the estimated partial effects.
}

When both average and conditional partial effects are requested, two lists containing the previous elements are returned, indexed by the prefixes \code{ape} and \code{cpe}.
}
\description{
\code{fracreg.pe} is used to compute average and/or conditional partial effects in fractional response models.
}
\details{
\code{fracreg.pe} calculates partial effects for fractional response models estimated via \code{fracreg}. \code{fracreg.pe} may be used to compute average or conditional partial effects for: (i) one-part fractional response models; (ii) the binary components of two-part and three-part fractional response models; (iii) the fractional components of two-part and three-part fractional response models; and (iv) two-part and three-part fractional response models overall. 

\strong{Partial Effects for Continuous Variables:}
For a continuous covariate \eqn{x_k}, the partial effect on the conditional mean \eqn{E(y|x) = G(x\beta)} is the first derivative with respect to \eqn{x_k}:
\deqn{PE_k(x) = \frac{\partial E(y|x)}{\partial x_k} = g(x\beta)\beta_k}
where \eqn{g(\cdot)} is the probability density function corresponding to the link function \eqn{G(\cdot)}.

\strong{Partial Effects for Discrete Variables:}
For a discrete or dummy covariate \eqn{x_k}, the partial effect is calculated as the discrete difference in the expected value when \eqn{x_k} changes from 0 to 1, holding all other variables \eqn{x_{-k}} constant:
\deqn{PE_k(x) = G(x_{-k}\beta_{-k} + \beta_k) - G(x_{-k}\beta_{-k})}

\strong{Average vs. Conditional Partial Effects:}
- \strong{Average Partial Effects (APE):} Evaluated for each observation \eqn{i} in the sample and then averaged:
\deqn{APE_k = \frac{1}{N} \sum_{i=1}^N PE_k(x_i)}
- \strong{Conditional Partial Effects (CPE):} Evaluated at a specific vector of covariate values \eqn{x^*} (e.g., the sample mean or median):
\deqn{CPE_k = PE_k(x^*)}

For calculating standard errors, it is taken into account the option that was previously chosen for estimating the model. See Ramalho, Ramalho and Murteira (2011) and Fang and Ma (2013) for details on the computation of partial effects in the fractional response framework.
}
\examples{
### Empirical 401(k) Examples
data("fracreg_k401k")
y <- fracreg_k401k$prate
X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age, 
           totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole)

m <- fracreg(y, X, type="1P", linkfrac="logit")
pe_res <- fracreg.pe(m)
summary(pe_res)

### Simulated Examples

N <- 250
u <- rnorm(N)

X <- cbind(rnorm(N),rnorm(N))
dimnames(X)[[2]] <- c("X1","X2")

ym <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u))
y <- rbeta(N,ym*20,20*(1-ym))
y[y > 0.9] <- 1

#Computing average partial effects for a logit fractional response model
mod <- fracreg(y,X,linkfrac="logit")
pe_res <- fracreg.pe(mod)
summary(pe_res)

#Computing average partial effects for a binary logit + fractional probit
#two-part model
mod <- fracreg(y,X,linkbin="logit",linkfrac="probit",type="2P",inf=1)
pe_res <- fracreg.pe(mod)
summary(pe_res)

#Computing conditional partial effects for X2 in the logit component
#of a two-part fractional response model, with the covariates evaluated
#at their median values
mod <- fracreg(y,X,linkfrac="logit",type="2Pfrac",inf=1)
pe_res <- fracreg.pe(mod,APE=FALSE,CPE=TRUE,at="median",which.x="X2")
summary(pe_res)

#Computing average partial effects for a three-part double-inflated model
y3p <- y
y3p[1:20] <- 0
y3p[21:40] <- 1
res3p <- fracreg(y3p,X,linkbin=c("logit","probit"),linkfrac="logit",type="3P")
pe_res <- fracreg.pe(res3p)
summary(pe_res)
}
\references{
Ramalho, E.A., J.J.S. Ramalho and J.M.R. Murteira (2011), "Alternative
estimating and testing empirical strategies for fractional response models",
\emph{Journal of Economic Surveys}, 25(1), 19-68.

Fang, K., & Ma, S. (2013), "Three-part model for fractional response variables with application to Chinese household health insurance coverage", \emph{Journal of Applied Statistics}, 40(5), 925-940.
}
\seealso{
\code{\link{fracreg}}, for fitting fractional response models.\cr
\code{\link{fracreg.reset}} and \code{\link{fracreg.ggoff}}, for specification tests.\cr
\code{\link{fracreg.ptest}}, for non-nested hypothesis tests.
}
\author{
Sulman Olieko Owili <oliekosulman@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracreg.ggoff.R
\name{fracreg.ggoff}
\alias{fracreg.ggoff}
\title{GGOFF Tests for Fractional Response Regressions}
\usage{
fracreg.ggoff(object, version = "LM", table = FALSE, ...)
}
\arguments{
\item{object}{an object containing the results of an \code{fracreg} command.}

\item{version}{a vector containing the test versions to use. Available options: \code{Wald}, \code{LM} (the default) and, only for the binary component of two-part models, \code{LR}. More than one option may be chosen.}

\item{table}{a logical value indicating whether a summary table with the test results should be printed.}

\item{\dots}{Arguments to pass to \link[stats]{glm}, which is used to estimate the model under the alternative hypothesis when \code{version} is a vector containing \code{"Wald"} or \code{"LR"}.}
}
\value{
\code{fracreg.ggoff} returns a named vector with the test results.
}
\description{
\code{fracreg.ggoff} is used to perform Generalised Goodness-Of-Functional-Form (GGOFF) tests to check the adequacy of the functional form and link specification of fractional response models.
}
\details{
\code{fracreg.ggoff} applies the GGOFF, GOFF1 and GOOFF2 test statistics to fractional response
models estimated via \code{fracreg}. \code{fracreg.ggoff} may be used to test the link
specification of: (i) one-part fractional response models; (ii) the binary
component of two-part fractional response models; and (iii) the fractional
component of two-part fractional response models.

\strong{GGOFF Test Framework:}
The Generalised Goodness-of-Functional Form (GGOFF) test evaluates the adequacy of the link function \eqn{G(\cdot)}. It is based on augmenting the baseline model with specific directions of departure. The auxiliary testing equation takes the form:
\deqn{E(y|x) = G\left(x\beta + \gamma_1 \frac{g'(x\hat{\beta})}{g(x\hat{\beta})} + \gamma_2 x\hat{\beta} \right)}
where \eqn{g(\cdot)} and \eqn{g'(\cdot)} are the first and second derivatives of \eqn{G(\cdot)} evaluated at the linear predictor \eqn{x\hat{\beta}}. The test checks \eqn{H_0: \gamma_1 = 0, \gamma_2 = 0}. GOFF1 and GOFF2 are variants testing individual components.

When the \code{Wald} version is implemented, it is taken into account the option that was chosen for computing standard errors in the model under evaluation. For the \code{LM} version, a robust version is computed in cases (i) and (iii) and a conventional version in case (ii). See Ramalho, Ramalho and Murteira (2014) for details on the application of the GGOFF, GOFF1 and GOOFF2 tests in the fractional response framework.
}
\examples{
### Empirical 401(k) Examples
data("fracreg_k401k")
y <- fracreg_k401k$prate
X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age, 
           totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole)

m <- fracreg(y, X, type="1P", linkfrac="logit")
ggoff_res <- fracreg.ggoff(m)
summary(ggoff_res)

### Simulated Examples

N <- 250
u <- rnorm(N)

X <- cbind(rnorm(N),rnorm(N))
dimnames(X)[[2]] <- c("X1","X2")

ym <- exp(X[,1]+X[,2]+u)/(1+exp(X[,1]+X[,2]+u))
y <- rbeta(N,ym*20,20*(1-ym))
y[y > 0.9] <- 1

#Testing the logit specification of a standard fractional response model
#using LM and Wald versions of the GGOFF test, based on 1 or 2 fitted powers of
#the linear predictor
mod <- fracreg(y,X,linkfrac="logit")
ggoff_res <- fracreg.ggoff(mod,c("Wald","LM"))
summary(ggoff_res)

#Testing the probit specification of the binary component of a two-part fractional
#regression model using a LR-based GGOFF test
mod <- fracreg(y,X,linkbin="probit",type="2Pbin",inf=1)
ggoff_res <- fracreg.ggoff(mod,"LR")
summary(ggoff_res)
}
\references{
Ramalho, E.A., J.J.S. Ramalho and J.M.R. Murteira (2014), "A generalized goodness-of-functional form test for binary and fractional response models", \emph{Manchester School}, 82(4), 488-507.

Pregibon, D. (1980), "Goodness of Link Tests for Generalized Linear Models", \emph{Journal of the Royal Statistical Society: Series C (Applied Statistics)}, 29(1), 15-24.
}
\seealso{
\code{\link{fracreg}}, for fitting fractional response models.\cr
\code{\link{fracreg.reset}}, for asymptotically equivalent specification tests.\cr
\code{\link{fracreg.ptest}}, for non-nested hypothesis tests.\cr
\code{\link{fracreg.pe}}, for computing partial effects.
}
\author{
Sulman Olieko Owili <oliekosulman@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregpd.pe.R
\name{fracregpd.pe}
\alias{fracregpd.pe}
\title{Partial Effects for Fractional Panel Data Regression}
\usage{
fracregpd.pe(
  object,
  APE = TRUE,
  CPE = FALSE,
  at = NULL,
  which.x = NULL,
  variance = TRUE,
  table = FALSE,
  ...
)
}
\arguments{
\item{object}{An object of class \code{fracregpd}.}

\item{APE}{logical. Compute Average Partial Effects?}

\item{CPE}{logical. Compute Conditional Partial Effects? (Not currently supported for panel data models).}

\item{at}{numeric vector. The values at which to evaluate the CPE.}

\item{which.x}{character vector. Variables for which to compute partial effects. By default, auxiliary CRE parameters (like \code{_mean} and \code{vhat}) are automatically excluded so that partial effects are only computed for the main structural parameters.}

\item{variance}{logical. Compute standard errors using the Delta method?}

\item{table}{logical. Print the resulting partial effects table?}

\item{...}{further arguments passed to or from other methods.}
}
\value{
An object of class \code{fracreg.pe} containing the standard coefficient tables with Average Partial Effects.
}
\description{
Computes Average Partial Effects (APEs) for fractional panel data models fit using \code{fracregpd}. In correlated random effects (CRE) models, the time-averages of the covariates are used merely to control for unobserved heterogeneity, and as such, they are automatically filtered out when computing the structural partial effects. Standard errors are computed using the Delta method and the bootstrapped parameter variance matrix.
}
\seealso{
\code{\link{fracregpd}}, \code{\link{fracreg.pe}}
}
\author{
Sulman Olieko Owili <oliekosulman@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregmlogit.wtp.R
\name{wtp}
\alias{wtp}
\title{"Willingness to Pay" for fracregmlogit models}
\usage{
wtp(object, wtp.vec, varlist = NULL, indv.obs = FALSE)
}
\arguments{
\item{object}{A \code{fracregmlogit.pe} object.}

\item{wtp.vec}{A numeric vector containing the arbitrary outcome values to be evaluated for each choice j.}

\item{varlist}{A string vector which provides the names of variables to calculate 
the wtp for. If missing, all variables in the object will be calculated.}

\item{indv.obs}{A logical value indicating whether to return individual observations.}
}
\value{
A "fracregmlogit.wtp" object containing the estimates, standard error, z-stats, and p-value.
}
\description{
Evaluate the "Willingness to Pay" given a set of arbitrary values for outcome variables. 
Usually used for policy evaluations where the total magnitude of marginal change matters.
}
\details{
This function calculates the aggregate effect of a variable on the 
"willingness to pay" by linearly multiplying the average partial effect with ex-ante (arbitrary) 
willingness to pay numbers associated with each choice. 

Suppose there are three choices A, B, C, each with a willingness to pay (or cost, profit, budget),
of 100, 200, and 300. The discrete effects of variable X on A, B and C are 0.5, 0.5, and -1, with 
standard errors 0.2, 0.3 and 0.5. The aggregated discrete effect of X on the total willingness 
to pay (or cost), is thus 100*0.5 + 200*0.5 + 300*(-1) = -150. The standard error can also be
calculated to be 162.8, assuming that the standard error is independent. 
A simple z-test is provided to test whether the aggregate effect is different from zero. 

Note that if the input \code{fracregmlogit.pe} object has no standard error computation, then no standard error
will be computed for the willingness to pay.
}
\examples{
\donttest{
data("fracreg_spending")
X = fracreg_spending[,2:5]
y = fracreg_spending[,6:11]
results1 = fracregmlogit(y, X)
pe = fracregmlogit.pe(results1)
# Assuming arbitrary WTP values for the 6 choices
wtp_est = wtp(pe, wtp.vec = c(1, 2, 3, 4, 5, 6), varlist = "houseval")
summary(wtp_est)
}
}
\seealso{
\code{\link{fracregmlogit.pe}}, \code{\link{plot.fracregmlogit}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregpd.R
\name{fracregpd}
\alias{fracregpd}
\title{Fitting Panel Data Fractional Response Regressions}
\usage{
fracregpd(
  id,
  time,
  y,
  x,
  z,
  var.endog,
  x.exogenous = TRUE,
  lags,
  start,
  type,
  GMMww.cor = TRUE,
  link = "logit",
  intercept = TRUE,
  table = FALSE,
  variance = TRUE,
  var.type = "cluster",
  tdummies = FALSE,
  bootstrap = FALSE,
  B = 200,
  offset = NULL,
  or = FALSE,
  level = 0.95,
  na.action = stats::na.omit,
  ...
)
}
\arguments{
\item{id}{a numeric vector identifying the cross-sectional units.}

\item{time}{a numeric vector identifying the time periods in which the cross-sectional units were observed.}

\item{y}{a numeric vector containing the values of the response variable.}

\item{x}{a numeric matrix, with column names, containing the values of all covariates (exogenous and endogenous).}

\item{z}{a numeric matrix, with column names, containing the values of all exogenous variables (covariates and external instrumental variables). Only required in case of endogenous explanatory variables.}

\item{var.endog}{a numeric vector containing the values of the endogenous covariate (or of some transformation of it), which will be used as dependent variable in the linear reduced form assumed for application of the \code{QMLcre} estimator. Only required for this estimator.}

\item{x.exogenous}{a logical value indicating whether all explanatory variables are assumed to be exogenous or not.}

\item{lags}{a logical value indicating whether the first lags of \code{x} or  \code{z} should be used as instruments for \code{x}. Defaults to \code{TRUE} for the GMMww and GMMc estimators and to \code{FALSE} for the remaining estimators. The \code{GMMcre} and \code{QMLcre} estimators do not admit lagged instruments.}

\item{start}{a numeric vector containing the initial values for the parameters to be optimised. Optional.}

\item{type}{a description of the estimator to compute: \code{GMMww}, \code{GMMc}, \code{GMMbgw}, \code{GMMpfe}, \code{GMMcre}, \code{GMMpre} or \code{QMLcre}.}

\item{GMMww.cor}{a logical value indicating whether each explanatory variable should be transformed in deviations from its overall mean before computing the \code{GMMww} estimator.}

\item{link}{a description of the link function to use. Available options for all GMM estimators: \code{logit} and \code{cloglog}. Only option for the \code{QMLcre} estimator: \code{probit}.}

\item{intercept}{a logical value indicating whether the model should include a constant term or not. Only relevant for the  \code{GMMpre} estimator.}

\item{table}{a logical value indicating whether a summary table with the regression results should be printed.}

\item{variance}{a logical value indicating whether the variance of the estimated parameters should be calculated. Defaults to  \code{TRUE} whenever \code{table = FALSERUE}.}

\item{var.type}{a description of the type of variance of the estimated parameters to be calculated. Options are \code{cluster}, the default, and \code{robust}. In overidentified models, it also affects the parameter estimates via the GMM weighting matrix.}

\item{tdummies}{a logical value indicating whether time dummies should be included among the model explanatory variables.}

\item{bootstrap}{a logical value indicating whether bootstrap should be used in the estimation of the parameter standard errors.}

\item{B}{the number of bootstrap replications.}

\item{offset}{an optional numeric vector containing an offset. It must be of the same dimension as the response variable. It specifies that the variable should be included in the model with its coefficient constrained to 1.}

\item{or}{a logical value indicating whether to report odds ratios. Only valid when the link function is \code{"logit"}. Defaults to \code{FALSE}.}

\item{level}{a numeric value between 0 and 1 indicating the confidence level for the confidence intervals. Defaults to \code{0.95}.}

\item{na.action}{A function specifying how to handle missing values, default is \code{stats::na.omit}. If \code{NULL}, no action is taken.}

\item{\dots}{Arguments to pass to \link[stats]{nlminb}.}
}
\value{
\code{fracregpd} returns a list with the following elements:
  \item{type}{the name of the estimator computed.
}
  \item{link}{the name of the specified link.
}
  \item{p}{a named vector of coefficients.
}
  \item{Hy}{the transformed values of the response variable when GMM estimators are computed or the 
   values of the response variable in the QML case.
}
  \item{converged}{logical. Was the algorithm judged to have converged?
}

In case of an overidentifying model, the following element is also returned:
  \item{J}{the result of Hansen's J test of overidentifying moment conditions.
}

If \code{variance = TRUE} or \code{table = FALSERUE} and the algorithm converged successfully, the previous list also contains the following elements:
  \item{p.var}{a named covariance matrix.
}
  \item{var.type}{covariance matrix type.
}
}
\description{
\code{fracregpd} is used to fit panel data regression models when the dependent variable has a bounded, fractional nature.
}
\details{
\code{fracregpd} computes the GMM estimators proposed in Ramalho, Ramalho and Coelho (2018) for panel data fractional response models with both time-variant and time-invariant unobserved heterogeneity and endogeneous covariates: GMMww, GMMc, GMMbgw, GMMpfe, GMMcre and GMMpre. In addition, \code{fracregpd} also computes QMLcre, which was proposed by Papke and Wooldridge (2008) and Wooldridge (2019). 

\strong{Correlated Random Effects (CRE) - QMLcre:}
In panel data, unobserved individual-specific heterogeneity \eqn{c_i} may be correlated with the covariates \eqn{x_{it}}. The CRE approach (Papke and Wooldridge, 2008) models this dependence by projecting \eqn{c_i} onto the time averages of the strictly exogenous covariates \eqn{\bar{x}_i}:
\deqn{c_i = \psi + \bar{x}_i \xi + a_i}
where \eqn{a_i} is an error term independent of \eqn{x_i}. Assuming \eqn{a_i | x_i \sim N(0, \sigma_a^2)} and a probit link, integrating out \eqn{a_i} yields the "population-averaged" or scaled conditional mean:
\deqn{E(y_{it} | x_i) = G(x_{it} \beta_a + \psi_a + \bar{x}_i \xi_a)}
where the parameters with subscript \eqn{a} are scaled by \eqn{(1 + \sigma_a^2)^{-1/2}}. This equation is estimated via pooled Bernoulli QML.

\strong{Generalised Method of Moments (GMM):}
For models where strict exogeneity fails or the link function is an exponential-type link, Ramalho et al. (2018) propose GMM estimators based on the following general moment conditions:
\deqn{E[Z_{it} (H(y_{it}) - \exp(x_{it}\beta + c_i))] = 0}
where \eqn{H(\cdot)} is a transformation function and \eqn{Z_{it}} is a matrix of valid instruments. Estimators such as GMMww, GMMc, and GMMbgw use different transformations to eliminate the unobserved fixed effect \eqn{c_i} before applying GMM.

For overidentified models, \code{fracregpd} calculates Hansen's J statistic to test the validity of the overidentifying restrictions.
}
\section{Odds Ratios}{

When \code{or=TRUE} and the fractional link function (\code{linkfrac} or \code{link}) is \code{"logit"}, the model additionally computes odds ratios for the coefficients. 
Odds Ratios are exponentiated coefficients.
The corresponding standard errors for the odds ratios are calculated using the Delta method.
The confidence intervals for the odds ratios are calculated using the adjusted standard errors and the specified \code{level} (defaulting to 95\%).
Odds ratios are particularly useful in fractional logit models as they provide a direct multiplicative interpretation of the independent variable on the odds of the fractional outcome.
}

\examples{
### Empirical 401(k) Examples
data("fracreg_k401k")
y <- fracreg_k401k$prate
X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age, 
           totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole)

# Artificial panel data structure for demonstration
N_emp <- nrow(X)
id_emp <- rep(1:(N_emp/2), each=2)
time_emp <- rep(1:2, times=N_emp/2)
mod <- fracregpd(id_emp, time_emp, y, X, type="QMLcre", link="probit")
summary(mod)

### Simulated Examples

set.seed(123)
# Simulating Panel Data
N <- 100
T_periods <- 5
id <- rep(1:N, each = T_periods)
time <- rep(1:T_periods, times = N)
x_panel <- rnorm(N * T_periods)

# Unobserved individual effect (CRE)
c_i <- rep(rnorm(N), each = T_periods) 
y_panel <- exp(x_panel + c_i) / (1 + exp(x_panel + c_i))

X <- cbind(x_panel = x_panel)

# Endogenous variable and instrument simulation
z_panel <- rnorm(N * T_periods)
u_panel <- 0.5 * z_panel + rnorm(N * T_periods)
var_endog <- 0.8 * z_panel + u_panel
y_endog <- exp(x_panel + 1.2 * var_endog + c_i + u_panel) / 
             (1 + exp(x_panel + 1.2 * var_endog + c_i + u_panel))

X_endog <- cbind(x_panel = x_panel, var_endog = var_endog)
Z_inst <- cbind(x_panel = x_panel, z_panel = z_panel)

\donttest{
# Estimate a Correlated Random Effects (CRE) Model
mod <- fracregpd(id=id, time=time, y=y_panel, x=X, type="QMLcre", link="probit")
summary(mod)

# Compute Partial Effects
pe_res <- fracregpd.pe(mod)
summary(pe_res)

# Exogeneity, no lags, no time dummies, clustered standard errors, GMMbgw estimator
mod <- fracregpd(id=id, time=time, y=y_panel, x=X, type="GMMbgw")
summary(mod)

# Estimate the GMMww estimator with odds ratios and 99\% confidence intervals
mod <- fracregpd(id=id, time=time, y=y_panel, x=X, type="GMMww", or=TRUE, level=0.99)
summary(mod)

# Lagged covariates and instruments, robust standard errors, GMMww estimator
mod <- fracregpd(id=id, time=time, y=y_panel, x=X, lags=TRUE, type="GMMww", var.type="robust")
summary(mod)

# Endogeneity, time dummies, GMMpfe estimator
mod <- fracregpd(id=id, time=time, y=y_endog, x=X_endog, z=Z_inst,
                 x.exogenous=FALSE, type="GMMpfe", tdummies=TRUE)
summary(mod)
}
}
\references{
Papke, L. and Wooldridge, J.M. (2008), "Panel data methods for fractional response variables with an application to test pass rates", \emph{Journal of Econometrics}, 145(1-2), 121-133.

Ramalho, E. A., Ramalho, J. J. S., & Coelho, L. M. S. (2018), "Exponential Regression of Fractional-Response Fixed-Effects Models with an Application to Firm Capital Structure", \emph{Journal of Econometric Methods}, 7(1), 20150019.

Wooldridge, J. M. (2019). Correlated random effects models with unbalanced panels. \emph{Journal of Econometrics}, 211(1), 137-150.
}
\seealso{
\code{\link{fracreg}}, for fitting standard cross-sectional fractional response models.\cr
\code{\link{fracreghet}}, for fitting cross-sectional fractional response models with unobserved heterogeneity.
}
\author{
Sulman Olieko Owili <oliekosulman@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregridge.pe.R
\name{fracregridge.pe}
\alias{fracregridge.pe}
\title{Partial Effects for Fractional Ridge Regression}
\usage{
fracregridge.pe(
  object,
  APE = TRUE,
  CPE = FALSE,
  at = NULL,
  variance = TRUE,
  table = FALSE,
  ...
)
}
\arguments{
\item{object}{An object of class \code{fracregridge}.}

\item{APE}{logical. Ignored for ridge regression.}

\item{CPE}{logical. Ignored for ridge regression.}

\item{at}{numeric vector. Ignored for ridge regression.}

\item{variance}{logical. Ignored for ridge regression.}

\item{table}{logical. Ignored for ridge regression.}

\item{...}{further arguments passed to or from other methods.}
}
\value{
An object of class \code{fracreg.pe} containing the standard coefficient tables.
}
\description{
Because Fractional Ridge Regression fits a linear model without a link function, the partial effects are mathematically identical to the estimated ridge coefficients. This function serves as a wrapper to maintain API compatibility with the rest of the \code{fracreg} package, printing a brief notification and returning the standard coefficient tables.
}
\examples{
# Generate random data
set.seed(123)
y <- rnorm(100)
X <- matrix(rnorm(1000), 100, 10)
colnames(X) <- paste0("X", 1:10)

# Fit Fractional Ridge Regression
mod <- fracregridge(y, X, fracs = c(0.3, 0.5))

# Compute Partial Effects (identical to coefficients)
pe <- fracregridge.pe(mod)
print(pe)
}
\seealso{
\code{\link{fracregridge}}, \code{\link{fracreg.pe}}
}
\author{
Sulman Olieko Owili <oliekosulman@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{fracreg_spending}
\alias{fracreg_spending}
\title{Government Spending Data}
\format{
A data frame with 429 observations and 12 variables:
\describe{
  \item{muni}{Name of municipality}
  \item{houseval}{Average value of a house in 100,000 euros}
  \item{popdens}{Population density in 1000s of persons per square km}
  \item{noleft}{No left party in city government}
  \item{minorityleft}{Minority left party in city government}
  \item{governing}{Fraction of spending on governing}
  \item{safety}{Fraction of spending on safety}
  \item{education}{Fraction of spending on education}
  \item{recreation}{Fraction of spending on recreation}
  \item{social}{Fraction of spending on social services}
  \item{urbanplanning}{Fraction of spending on urban planning}
  \item{tot}{Total spending or population}
}
}
\source{
<http://fmwww.bc.edu/repec/bocode/c/citybudget.dta>
}
\usage{
data("fracreg_spending")
}
\description{
Spending on different categories by Dutch cities in 2005. 
This dataset is commonly used to demonstrate fractional multinomial logit models.
}
\examples{
data("fracreg_spending")
head(fracreg_spending)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracreg.extractors.R
\name{fitted.fracreg}
\alias{fitted.fracreg}
\title{Extract Fitted Values for fracreg}
\usage{
\method{fitted}{fracreg}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracreg}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A numeric vector of fitted values.
}
\description{
Extracts the fitted conditional mean values from a \code{fracreg} model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregridge.extractors.R
\name{residuals.fracregridge}
\alias{residuals.fracregridge}
\title{Extract Model Residuals for fracregridge}
\usage{
\method{residuals}{fracregridge}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracregridge}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A matrix or array of residuals.
}
\description{
Extracts the response residuals from a fitted \code{fracregridge} model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregridge.extractors.R
\name{predict.fracregridge}
\alias{predict.fracregridge}
\title{Predict Method for fracregridge}
\usage{
\method{predict}{fracregridge}(object, newdata = NULL, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracregridge}.}

\item{newdata}{An optional data frame or matrix in which to look for variables with which to predict. If omitted, the fitted values are used.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A matrix or array of predicted values.
}
\description{
Predicts conditional mean values from a fitted \code{fracregridge} model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracreg.R
\name{fracreg}
\alias{fracreg}
\title{Fitting Fractional Response Regressions}
\usage{
fracreg(
  y,
  x,
  x2 = x,
  linkbin,
  linkfrac,
  type = "1P",
  inflation = 0,
  intercept = TRUE,
  table = FALSE,
  variance = TRUE,
  var.type = "default",
  var.eim = TRUE,
  var.cluster,
  dfc = FALSE,
  offset = NULL,
  or = FALSE,
  level = 0.95,
  na.action = stats::na.omit,
  ...
)
}
\arguments{
\item{y}{a numeric vector containing the values of the response variable.}

\item{x}{a numeric matrix, with column names, containing the values of the covariates.}

\item{x2}{a numeric matrix, with column names, containing the values of the covariates in the fractional component of two-part models if option \code{type = "2P"} is defined. Defaults to \code{x}.}

\item{linkbin}{a description of the link function to use in the binary component of a two-part fractional response model, or a vector of two link functions for the two binary components of a three-part model (e.g. \code{c("logit", "probit")}). Available options: \code{logit}, \code{probit}, \code{cauchit}, \code{loglog}, \code{cloglog}.}

\item{linkfrac}{a description of the link function to use in standard fractional response models or in the fractional component of a two-part fractional response model. Available options: \code{logit}, \code{probit}, \code{cauchit}, \code{loglog}, \code{cloglog}.}

\item{type}{a description of the model to estimate: a standard one-part model (\code{1P}, the default), a two-part model (\code{2P}), the binary component of a two-part model (\code{2Pbin}), the fractional component of a two-part model (\code{2Pfrac}), or a three-part model (\code{3P}) for double boundary inflation.}

\item{inflation}{a numeric value indicating which of the extreme values of \code{0} (the default) or \code{1} is the relevant boundary value for defining two-part fractional response models.}

\item{intercept}{a logical value indicating whether the model should include a constant term or not.}

\item{table}{a logical value indicating whether a summary table with the regression results should be printed.}

\item{variance}{a logical value indicating whether the variance of the estimated parameters should be calculated. Defaults to \code{TRUE} whenever \code{table = TRUE}.}

\item{var.type}{a description of the type of variance of the estimated parameters to be calculated. Options are \code{standard} (recommended for models estimated by maximum likelihood, such as the binary component of two-part models), \code{robust} (recommended for models estimated by quasi-maximum likelihood, such as standard fractional response models or the fractional component of a two-part fractional response model), \code{cluster} (recommended in the case of panel data) and \code{default} (implements the \code{standard} or \code{robust} versions as appropriate).}

\item{var.eim}{a logical value indicating whether the expected information matrix should be used in the calculation of the variance. When false, the observation information matrix will be used. Defaults to \code{TRUE}.}

\item{var.cluster}{a numeric vector containing the values of the variable that specifies to which cluster each observation belongs.}

\item{dfc}{a logical value indicating whether a degrees of freedom correction should be applied to the covariance matrix. Defaults to \code{FALSE}.}

\item{offset}{an optional numeric vector containing an offset. It must be of the same dimension as the response variable. It specifies that the variable should be included in the model with its coefficient constrained to 1.}

\item{or}{a logical value indicating whether to report odds ratios. Only valid when the link function is \code{"logit"}. Defaults to \code{FALSE}.}

\item{level}{a numeric value between 0 and 1 indicating the confidence level for the confidence intervals. Defaults to \code{0.95}.}

\item{na.action}{A function specifying how to handle missing values, default is \code{stats::na.omit}. If \code{NULL}, no action is taken.}

\item{\dots}{Arguments to pass to \link[stats]{glm}.}
}
\value{
When \code{type = "1P" or "2Pfrac"}, \code{fracreg} returns a list with the following elements:
  \item{class}{"fracreg".
}
  \item{formula}{the model formula.
}
  \item{type}{the name of the estimated model.
}
  \item{link}{the name of the specified link.
}
  \item{method}{estimation method. Currently, "QML" (quasi-maximum likelihood) for fractional components or models and"ML" (maximum likelihood) for the binary component of two-part models.
}
  \item{p}{a named vector of coefficients.
}
  \item{yhat}{the fitted mean values.
}
  \item{xbhat}{the fitted mean values of the linear predictor.
}
  \item{converged}{logical. Was the algorithm judged to have converged?
}
  \item{x.names}{a vector containing the names of the covariates.
}

If \code{variance = TRUE} or \code{table = TRUE}, the previous list also contains the following elements:
  \item{p.var}{a named covariance matrix.
}
  \item{var.type}{covariance matrix type.
}
  \item{var.eim}{logical. Was the expected information matrix used in the computation of the covariance matrix?
}
  \item{dfc}{logical. Was a degrees of freedom correction used for the computation of the covariance matrix?
}

If \code{var.type = "cluster"}, the list also contains the following element:
  \item{var.cluster}{the variable that specifies to which cluster each observation belongs.
}

When \code{type = "2Pbin"}, \code{fracreg} returns a similar list with the following additional element:
  \item{LL}{the value of the log-likelihood.
}

When \code{type = "2P"}, \code{fracreg} returns the previous lists, indexed by the prefixes \code{resBIN} and \code{resFRAC}, and the following additional elements:
  \item{class}{"fracreg".
}
  \item{type}{"2P".
}
  \item{ybase}{a numeric vector containing the values of the response variable.
}
  \item{x2base}{a numeric matrix containing the values of the covariates.
}
  \item{yhat2P}{the overall fitted mean values.
}
  \item{converged}{logical. Were the algorithms judged to have converged in both parts of the model?
}

When \code{type = "3P"}, \code{fracreg} returns the previous lists, indexed by the prefixes \code{resBIN0}, \code{resBIN1}, and \code{resFRAC}, and the following additional elements:
  \item{class}{"fracreg".
}
  \item{type}{"3P".
}
  \item{ybase}{a numeric vector containing the values of the response variable.
}
  \item{x2base}{a numeric matrix containing the values of the covariates.
}
  \item{yhat3P}{the overall fitted mean values.
}
  \item{converged}{logical. Were the algorithms judged to have converged in all parts of the model?
}
}
\description{
\code{fracreg} is used to fit fractional response models, which are appropriate for responses that are proportions, percentages, or fractions restricted to the [0, 1] interval. It supports standard one-part models, two-part hurdle models for modelling boundary values at 0 or 1, and three-part models for double inflation at both 0 and 1.
}
\details{
\code{fracreg} estimates one-part, two-part hurdle, and three-part double-inflated fractional response models; see Ramalho, Ramalho and Murteira (2011) and Fang and Ma (2013) for details on those models. 

\strong{One-Part Fractional Response Regressions (\code{type = "1P"}):}
The standard one-part model assumes that the conditional expectation of the fractional response \eqn{y_i \in [0,1]} is given by:
\deqn{E(y_i|x_i) = G(x_i \beta)}
where \eqn{G(\cdot)} is a known non-linear link function mapping the linear predictor to the unit interval (e.g., logit, probit). The parameters \eqn{\beta} are estimated by maximising the Bernoulli-based quasi-log-likelihood function:
\deqn{\ln L_i(\beta) = y_i \ln[G(x_i \beta)] + (1 - y_i) \ln[1 - G(x_i \beta)]}
This estimator requires only the correct specification of the conditional mean to yield consistent parameter estimates (Papke and Wooldridge, 1996).

\strong{Two-Part Hurdle Models (\code{type = "2P"}):}
When the data exhibits a boundary mass (e.g., at \eqn{y_i = 0}), the two-part hurdle model handles the boundary values separately from the interior fractional values. Let \eqn{y_i^*} be a binary indicator such that \eqn{y_i^* = 1} if \eqn{y_i > 0} and \eqn{y_i^* = 0} otherwise. The probability of observing a boundary value is modelled as:
\deqn{P(y_i = 0 | x_{1i}) = 1 - F(x_{1i} \gamma_1)}
\deqn{P(y_i > 0 | x_{1i}) = F(x_{1i} \gamma_1)}
where \eqn{F(\cdot)} is a binary link function. Conditional on observing an interior fractional value, the response is modelled as:
\deqn{E(y_i | x_{2i}, y_i > 0) = G(x_{2i} \beta_2)}
The unconditional mean of the response is therefore:
\deqn{E(y_i|x_i) = F(x_{1i} \gamma_1) \times G(x_{2i} \beta_2)}

\strong{Three-Part Double Inflated Models (\code{type = "3P"}):}
For data containing boundary mass at both \eqn{0} and \eqn{1}, the three-part model estimates two separate binary mechanisms for each boundary and a fractional component for the interior values \eqn{(0, 1)}, extending the two-part logic to double inflation (Fang and Ma, 2013).

\code{fracreg} uses the standard \link[stats]{glm} command to perform the estimations. Therefore, \code{fracreg} is essentially a convenience command, allowing estimation of several alternative fractional response models using the same command. In addition, \code{fracreg} provides an R-squared measure for all models (calculated as the square of the correlation coefficient between the actual and fitted values of the dependent variable), calculates the fitted values of the dependent variable in two-part models and stores the information needed to implement some very useful commands for fractional response models: \link{fracreg.reset} (RESET test), \link{fracreg.ptest} (P test), \link{fracreg.ggoff} (GGOFF tests) and \link{fracreg.pe} (partial effects).
}
\section{Odds Ratios}{

When \code{or=TRUE} and the fractional link function (\code{linkfrac} or \code{link}) is \code{"logit"}, the model additionally computes odds ratios for the coefficients. 
Odds Ratios are exponentiated coefficients.
The corresponding standard errors for the odds ratios are calculated using the Delta method.
The confidence intervals for the odds ratios are calculated using the adjusted standard errors and the specified \code{level} (defaulting to 95\%).
Odds ratios are particularly useful in fractional logit models as they provide a direct multiplicative interpretation of the independent variable on the odds of the fractional outcome.
}

\examples{
### Empirical 401(k) Examples
data("fracreg_k401k")
y <- fracreg_k401k$prate
X <- cbind(mrate = fracreg_k401k$mrate, age = fracreg_k401k$age, 
           totemp = fracreg_k401k$totemp, sole = fracreg_k401k$sole)

# 1P Model
mod <- fracreg(y, X, type="1P", linkfrac="logit")
summary(mod)

# 1P Model reporting odds ratios and 99\% confidence intervals
mod <- fracreg(y, X, type="1P", linkfrac="logit", or=TRUE, level=0.99)
summary(mod)

# 2P Model (modelling mass at 1)
mod <- fracreg(y, X, type="2P", inflation=1, linkbin="logit", linkfrac="logit")
summary(mod)

# 3P Model (inject artificial 0s for demonstration)
y_3p <- y; y_3p[1:50] <- 0
mod <- fracreg(y_3p, X, type="3P", linkbin=c("logit","logit"), linkfrac="logit")
summary(mod)

### Simulated Examples

set.seed(123)
N <- 1000
x1 <- rnorm(N)
x2 <- runif(N)

# Generating a fractional dependent variable with inflation at 0 and 1
XB <- -0.5 + 0.8 * x1 + 1.2 * x2 + rnorm(N)
y_latent <- exp(XB) / (1 + exp(XB))

y <- y_latent
# Inflate at boundaries
y[y_latent < 0.2] <- 0
y[y_latent > 0.8] <- 1

X <- cbind(x1 = x1, x2 = x2)

# fracreg estimation of a logit fractional response model
mod <- fracreg(y, X, type="1P", linkfrac="logit")
summary(mod)

# fracreg estimation of the binary logit component of the two-part fractional
# regression model with y=0 as the relevant boundary value
mod <- fracreg(y, X, type="2Pbin", inflation=0, linkbin="logit")
summary(mod)

# fracreg estimation of the fractional component of the two-part fractional
# regression model with y=0 as the relevant boundary value and using a
# probit link function
mod <- fracreg(y, X, type="2Pfrac", inflation=0, linkfrac="probit")
summary(mod)

# fracreg estimation of both components of a two-part fractional response model
# with y=0 as the relevant boundary value and using a cloglog binary link
# function and a logit fractional link function
mod <- fracreg(y, X, type="2P", inflation=0, linkbin="cloglog", linkfrac="logit")
summary(mod)

# Three-part double-inflated model (y has both 0s and 1s)
mod <- fracreg(y, X, type="3P", linkbin=c("logit","probit"), linkfrac="logit")
summary(mod)
}
\references{
Papke, L. E. and Wooldridge, J. M. (1996), "Econometric methods for fractional response variables with an application to 401(k) plan participation rates", \emph{Journal of Applied Econometrics}, 11(6), 619-632.

Ramalho, E.A., J.J.S. Ramalho and J.M.R. Murteira (2011), "Alternative
estimating and testing empirical strategies for fractional response models",
\emph{Journal of Economic Surveys}, 25(1), 19-68.

Fang, K., & Ma, S. (2013), "Three-part model for fractional response variables with application to Chinese household health insurance coverage", \emph{Journal of Applied Statistics}, 40(5), 925-940.
}
\seealso{
\code{\link{fracreg.reset}} and \code{\link{fracreg.ggoff}}, for specification tests.\cr
\code{\link{fracreg.ptest}}, for non-nested hypothesis tests.\cr
\code{\link{fracreg.pe}}, for computing partial effects.\cr
\code{fracreghet}, for fitting cross-sectional fractional response models with unobserved heterogeneity.\cr
\code{fracregpd}, for fitting panel data fractional response models.
}
\author{
Sulman Olieko Owili <oliekosulman@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregmlogit.pe.R
\name{fracregmlogit.pe}
\alias{fracregmlogit.pe}
\title{Fractional Multinomial Logit Average Partial Effects}
\usage{
fracregmlogit.pe(
  object,
  effect = c("marginal", "discrete"),
  marg.type = "atmean",
  se = FALSE,
  varlist = NULL,
  at = NULL,
  R = 1000
)
}
\arguments{
\item{object}{A "fracregmlogit" object.}

\item{effect}{Can be "marginal", for marginal effects; or "discrete", for discrete changes from
the min to the max.}

\item{marg.type}{Type of marginal or discrete effects to be computed. Default to "atmean", the effect at 
the mean of all covariates. Also takes "aveacr", the averaged effects across all observations. See details.}

\item{se}{Whether to calculate standard errors for those margins. See details.}

\item{varlist}{A string vector which provides the names of variables to calculate 
the marginal effect for. If missing, all variables except the constant will be calculated. 
Use "constant" if you wish to compute the marginal effect of the constant.}

\item{at}{Specify values of the X-matrix at which the partial effect will be retrieved. Expects a vector input
of length K-1. Only supported for \code{marg.type="atmean"}. See \code{predict.fracregmlogit(newdata)}.}

\item{R}{Number of times to sample for the Krinsky-Robb standard error. Default to 1000.}
}
\value{
The function returns an object of class "fracregmlogit.pe". It contains the following components:

\code{effects} A matrix of calculated effects.

\code{se} A matrix of standard errors corresponding to the effects. Shows up if se=TRUE for the 
input parameter.

\code{ztable} A list of matrices containing effects, standard errors, z-stats and p-values.

\code{R} Number of simulation times for Krinsky-Robb standard error calculation. Null if se=FALSE.

\code{expl} String message explaining the effects calculated.
}
\description{
Calculate average partial effects (APE) of independent variables from a fractional multinomial logit model.
}
\details{
This module calculates the average partial effects (APEs) from a fractional multinomial logit model.
Partial effects are the counterpart of the marginal effects in a linear model setting. In linear models, 
usually the parameter estimate itself represents the marginal effect (if the variable in question is continuous). 
In logit models, however, the parameter estimate at hand is the effect on the log-ratio between the choice variable
and the baseline variable. This function is intended to extract APEs from the 
coefficient estimates computed from the fractional multinomial logit models.

This function allows for two types of partial effects: marginal effects, and discrete effects.
A marginal effect represents how a unit change in one continuous variable x may influence the choice variable y. 
The estimation of marginal effects is very straightforward. However, special care is needed when averaging 
the marginal effect across observations to acquire the APE. One approach is to use the estimate of the marginal effect while setting
other explanatory variables at the mean. We call this the marginal effect at the mean (MEM), which corresponds
to the option \code{marg.type="atmean"}. Another approach is to take the average of marginal effects for each
individual. We call this the average marginal effect (AME), which corresponds to the option \code{marg.type="aveacr"}. 

The discrete effect represents how a discrete change in one specific x, discrete or continuous, influences the choice variable y. 
This is more useful for categorical variables, as calculating the "marginal effect" makes little sense
for them. In this function, we calculate the discrete effect by changing the explanatory variable from 
its minimum to its maximum. For a binary variable, this is just the difference between 0 and 1. Similar 
to the marginal effect case, we also have the discrete effect at the mean (DEM), corresponding to \code{marg.type="atmean"}
and the average discrete effect (ADE), corresponding to \code{marg.type="aveacr"}.

Standard errors are provided for the effects by using the Krinsky-Robb (KR) method. Krinsky-Robb is a simulation-based
method that calculates the empirical value of a function given a known distribution of its variables. Here 
we provide Krinsky-Robb standard errors for MEM and DEM, and the user can specify how many times of 
simulation \code{R} the Krinsky-Robb algorithm should run. 

The user can also specify a subset of explanatory variables when calculating effects. This is done through
specifying string vectors containing the column names of the explanatory variables to \code{varlist}. As the
KR standard error can be computationally intensive, it is advised to calculate it only for the variables of interest.
}
\examples{
data("fracreg_spending")
X = fracreg_spending[,2:5]
y = fracreg_spending[,6:11]
results1 = fracregmlogit(y, X)

# Calculate marginal effects at the mean (without standard errors for speed)
pe_marg = fracregmlogit.pe(results1, effect="marginal", se=FALSE)

# Calculate discrete effects for specific variables with standard errors
pe_disc = fracregmlogit.pe(results1, effect="discrete", 
                           varlist = colnames(results1$X)[c(1,3)], 
                           se=TRUE, R=50)
summary(pe_disc)
}
\seealso{
\code{\link{fracregmlogit}} for the model estimation, \code{\link{plot.fracregmlogit.pe}} for plotting effects.
}
